;
; atapi.asm - lowlevel ATAPI device control
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;---------------------------
;Execute ATAPI command, C=1 if error
;
		#assert_cfg_open

OP_atapi_command_multiplex .proc
		cmp #1
                php
		jsr atapi_select_drive_A
                bcc ok
err             plp
		sec
                rts
ok	      				;Set up the controller registers.
		.if FAST_DATA_TRANSFERS
      		#stz_a feat_g		;No OVL/DMA
		#stz_aa nsector_g      	;No tag
		#stz_aa cyllow_g       	;Maximum packet size
		.else
		#stz_a low_reg
		sta feat_g		;No OVL/DMA
		sta nsector_g		;No tag
		sta cyllow_g		;Maximum packet size
                .fi
		lda #>2048
		#sta_low_reg
		sta cylhi_g

		lda #packet_d		;Packet Command
		jsr ata_data_command
		bcs err

		lda #ATAPI_PH_CMDOUT
		jsr atapi_ireason
		bcs err

		ldx #11
                .if FAST_DATA_TRANSFERS
lp		lda packet_buff_exp-1,x
		sta hig_reg
                lda packet_buff_exp,x
                sta data_g
                dex
                .else
lp		txa
		lsr
		lda packet_buff_exp,x	;Send the command to the device.
		bcs u
		sta hig_reg
		sta data_g
u		sta low_reg
		.fi
		dex
		bpl lp
                jsr defaultwait
retry           jsr OP_atapi_waitme
                bcs err

                lda #ATAPI_PH_DONE
                plp
                php
                beq +
                #lda0
                rol
                asl			;ATAPI_PH_DATAOUT=CMD_ATAPI_DATAOUT=0, ATAPI_PH_DATAIN=CMD_ATAPI_DATAIN=2
+		jsr atapi_ireason
		bcs retry

		lda #%00001000		;DRQ==1?
                plp
		beq nondata

                bit status_g
                .if !FAST_DATA_TRANSFERS
		bit low_reg
                .fi
                beq unexp

nondata         clc
		rts

atapi_ireason	.if !FAST_DATA_TRANSFERS
		bit ireason_g
		eor low_reg
                .else
                eor ireason_g
                .fi
		and #%00000111
unexp		cmp #1
		lda #EUNEXP
		rts
		.pend
