;
; boot.asm - IDEDOS booter
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;***** Header for the cartridge *****
;OP    Must be in open_cfg
;LW    $8000-$9fff
;NO    No open_cfg

		#assert_8000romioonly
		.word cart_reset,$fe5e
		.text "CBM80"
		.byte $20
;---------------------------
		.text "ide64"
copyright	.text " 1998-2006 j. soucek, t. pribyl",13
                .null "idedos (c) 2001-2006 kajtar zsolt"
chksum		.word 0
uuid		.include uuid
;---------------------------
;Boot
;
                .cpu 6502
cart_reset	.proc
		#assert_kernal_io
kernal_fd8c	= $fd8c
kernal_fddd	= $fddd
kernal_e453	= $e453
kernal_e3bf	= $e3bf
kernal_ff5e	= $ff5e
kernal_e429	= $e429

		#assert_basic_kernalio
basic_a408	= $a408
basic_a644	= $a644

		sei
		sta cfg_8
                #assert_cfg_8
		ldx #0
		stx $dc03
		dex
		stx $dc02
                txs
		lda #%11111101
		sta $dc00
dye		dex
		beq th
		lda $dc01
		cmp $dc01
		bne dye
		eor #%01111111		;shift
		bne th
		sta $dc02
		sta $dc03
		sta $dc00
		ldx #5
nn		jsr LW_waitretrace      ;Wait stable machine state
		dex
		bne nn
                bit $d0bc		;Test for cpu type
		.if SUPERCPU=1
		bmi selftestx
		.else
		bpl selftestx
		.fi
		sta cfg_open		;to be sure, reset buttons are not ideal
                #assert_cfg_open
		jsr OP_check_sign	;First boot/reset?
                beq sok
                #assert_8000romioonly
selftestx     	jmp selftest

		#assert_cfg_open
sok		jsr OP_external_init
;                jsr LW_palntsctest
;		jsr crtc_init_if128
		dec $318
                .if SUPERCPU=1
                lda $d246
                clc
                adc #5
                tcs
       		lda $d241	;PCH
                pha
                lda $d240	;PCL
                pha
                lda $d24e	;SR
                pha
                lda $d248	;A
                pha
                lda $d24a	;X
                pha
                lda $d24c	;Y
                pha
                .else
		#psr $fce3
		lda #%00100100
		pha
                ldx #$f9
		txs
                .fi
		lda #BANK+128
		jmp monitor

		#assert_8000romioonly
th		sta cfg_8
		#assert_cfg_8
		jsr LWNO_init_io	;$fda3

		lda #0
		tay
zacyk		sta $0002,y		;Clear memory
		sta $0200,y
		sta $0300,y
		iny
		bne zacyk

		ldy #<$033c
		sty $b2
		ldy #>$033c
		sty $b3
		ldy #$a0
		sty $c2
		tax
		jsr kernal_fd8c		;Set memory pointers

		jsr LWNO_set_old314vect
		lda #<$fec1
		sta $318                ;No RUN/STOP+RESTORE until fully initialized

		jsr LWNO_init_vic	;$e518
                jsr exit_c128key

		jsr LW_palntsctest
;                pha
                asl
                rol $2a6

		lda kernal_ff5e		;kernal rev 1?
                eor #$aa
                beq +
		jsr kernal_fddd       	;Machine type, etc.
+		lda #7
		sta $dd00

;		pla
;		jsr crtc_init_if128

;       	cli

		.if SUPERCPU=1
                lda #<$e502
                sta $b0
                lda #>$e502		;JiffyDOS F-keys patch
                sta $b1
                .else
		jsr LWNO_detect_jiffydos
                .fi

		jsr kernal_e453		;Init $0300 vectors
		jsr kernal_e3bf		;Init basic
		lda $2b
		ldy $2c
		jsr basic_a408		;check memory overlap
		jsr basic_a644		;basic NEW

		.if DE32_EXISTS
		ldx #messde32-mess
		lda ideconf_port
		cmp #$12		;wrong dos for cartridge without
		bne wrong_hardware      ;cartconfig register
		.fi

		ldx #messcpu-mess
		bit $d0bc		;Teszt for cpu type
		.if SUPERCPU=1
		bpl cpu_type_ok
		.else
		bmi cpu_type_ok
		.fi
;---------------------------
;Wrong CPU or no DE32
;
		#assert_8000romioonly
wrong_hardware 	#sta_cfg_8
		ldy #4
                sty 198
-		lda mess-1,x
                sta 630,y
                dex
                dey
                bne -
		jsr LWNO_set_old314vect
		jsr LW_set_screen_cold	;not the right cpu for this dos

		jsr kernal_e429
		jmp cart_reset_end	;sorry, wrong cpu

		#assert_8000romonly
mess            .text "cpu "
messcpu         .text "ram "
messram         .text "rom "
messrom         .if DE32_EXISTS
		.text "de32"
messde32
		.fi

		.assert CFG_RAM | CFG_8000_ROM | CFG_A000_BASIC | CFG_D000_IO | CFG_E000_ROM, CFG_RAM | CFG_8000_ROM | CFG_A000_BASIC | CFG_D000_IO | CFG_E000_ROM, CFG_OPEN_RAM | CFG_8000_RAM | CFG_A000_RAM | CFG_A000_ROM | CFG_D000_ROM | CFG_E000_OPEN
;---------------------------
;Right CPU
;
                .cpu default

cpu_type_ok	jsr LWNO_set_new314vect

                sec
		jsr LWNO_basic_restore.set
                #assert_8000romioonly
retry   	sta cfg_open
		#assert_cfg_open
		.if SUPERCPU=1
		jsr scpu_default_opt
                .fi

		jsr OP_external_init

		ldx #6
se              lda chrgpatch-1,x          ;patch chrget/chrgot, to make it 2 cycles faster ;)
                sta $84-1,x
                .if SUPERCPU=1
                lda brkpatch-1,x		;brk with hwreg enabled?
                sta $17ffa-1,x
                .fi
                lda fpx-1,x
                sta monitor_freezepoint_stuff_exp,x
                dex
                bne se
                .if SUPERCPU=1
                txa
		sta monitor_ripper_exp
                sta monitor_freezepoint_stuff_exp
                .else
		stx monitor_ripper_exp
                stx monitor_freezepoint_stuff_exp
                .fi

		.if SUPERCPU=1
		ldx #6
-		lda memtrashpatch,x     ;patch kernal to not trash memory
		sta $1fd20,x
		dex
		bpl -

                ldx #3
-		lda jiffypatch,x	;JiffyDOS CONTROL+D hack
		sta $162a6,x
                dex
                bpl -

                ldx #10
-		lda shiftcpatch,x	;shift+c= repeat
		sta $1eaf2,x
                dex
                bpl -
                lda #$5a
                sta $1ea99
		.fi

		jsr OP_check_sign	;First boot/reset?
		beq ini_allready
;---------------------------
;Poweron detected
;
		jsr OP_kill_drives     	;No IDE devices
		lda setup_boot_drive_exp
		sta monitor_dr_exp

                .if SUPERCPU=0
                lda $dc0e
                beq not

                sec
                ldx #0
                stx $d011
                stx $d020
                dex
                jsr extramtest
                clc
                ldx #0
                jsr extramtest
                lda #$aa
                sta $03
		ldx #messram-mess
                bcs wrong_hardware
                sty $dc0e
                gcc retry
not
                .fi

		#sta_cfg_16

		sei
                sec
		ror msgmode		;enable messages
		.if SUPERCPU=0
		lda #<__brk
		sta $fffa
		sta $fffe
		lda #>__brk
		sta $fffb
		sta $ffff
		.fi
		lda #$33
		sta $01
                ldx #0
hthee		lda $d000,x
		sta $2000,x
                lda $d100,x
		sta $2100,x
                lda $d800,x
                sta $2200,x
		inx
		bne hthee
		ldy #$37
		sty $01
if              asl $22
                bne kf
                lda logo-$37,y
                iny
                sta $22
                sec
kf              bcc thd
                lda logo-$37,y
                iny
thd     	sta $2300,x
                inx
                bne if
		stx $d020
		stx $d021		;Black screen

                jsr LWNO_print_imediate ;cfg_16 after this
                .text 147,8,154,224,225,226,227,228,229,230,231,232
                .null 13,233,234,235,236,237,238,239,240,241
	     	#assert_cfg_16

		lda #24
		sta $d018

		lda #<copyright
		ldy #>copyright
		jsr LWNO_print_ay
		lda #27
		sta $d011

		sei
		.if SUPERCPU=0
                lda #1
                sta $dc0e		;Enable IRQ
		.fi

                jsr romtest
                beq vx
                ldx #messrom-mess
                gne wrong_hardware

vx		#sta_cfg_open
		ldx #keysend-keys-1
-               lda keys,x
                sta function_keys_exp,x
                dex
                bpl -
		clc
		jsr OP_check_sign.set		;Now we fully initialized...
		jsr LWNO_print_imediate ;cfg_16 after this
		.text 13,13
                .text "idedos COMES with absolutely no warranty"
		.text "tHIS  IS  FREE  SOFTWARE,  AND  YOU  ARE"
		.text "WELLCOME   TO   REDISTRIBUTE  IT   UNDER"
		.text "CERTAIN CONDITIONS. sEE copying! (gpl-2)"
                .text 13,"fLAGS: "
                .if PCLINK_TYPE=1
                .text "xpcLINK"
                .elsif PCLINK_TYPE=2
                .text "ppcLINK"
                .elsif (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
                .text "spcLINK"
                .fi
                .if PCLINK_USECRC
                .text "+CRC"
                .fi
                .if JIFFYDOS_ACCEL=1
                .text " jIFFY"
                .fi
                .if DOLPHINDOS_ACCEL=1
                .text " dOLPHIN"
                .fi
		.text 13,13,"bUILT: "
		.include date
		.null 13,13,"dETECTING DEVICES...",13,13
                #assert_cfg_16

;		lda $d030		;C128?
;		lsr
;		bcs n128_		;no
;		jsr crtc_logo
;n128_
		inc $318		;Enable RUN/STOP+RESTORE
		jsr bashdinit
		dec $318		;kill nmi
		#sta_cfg_open

		asl msgmode		;clear messages

-		lda $dc01
		cmp #%11111011
                beq -

		lda setup_atribut_exp	;Test BOOT Power up
		bpl no_load_boot
                clc

read_c_key	lda $dc01
;		eor #%01111111		;Test 'RUN/STOP'
;		beq no_load_boot
;		eor #%11011111^%01111111;Test 'C='
		eor #%11011111		;Test 'C='
		beq no_load_boot
                rol $22

		ldx #12
		stx ActualFNLength
-		lda bf-1,x
		sta $1ff,x
		dex
		bne -
		stx ActualFilename
		lda #2
		sta ActualSecAddr
		sta ActualFilename+1
                sta load_autostart_exp

		#psr loadback
		#psr switch_16k
		txa
		#jmps newload

loadback	#sta_cfg_open
                bcs no_load_boot
		lda Status
		eor #$40
		bne no_load_boot
                lsr $22
                rol
                sta Status
		stx $2d
		sty $2e			;set basic end pointer
		gcc no_load_boot
;---------------------------
;Simple reset
;
ini_allready	jsr check_drive_status

		jsr OP_hdinit_drivenumbers

		lda setup_atribut_exp	;Test BOOT Always
		cmp #%11000000
		bcs read_c_key

no_load_boot    lda #CMOS_SET_TI        ;Test SET BASIC CLOCK ENABLE/DISABLE
                jsr check_setup_atribut_exp
		bcc skip_basic_clock

		jsr LWOP_read_clock
                jsr OP_save_zeropage
		#stz_a szam
		#stz_aa szam+1
		#stz_aa szam+2
                ldx #2
-		#phx_a
		lda sec_exp,x
		jsr bcd_sum_mul_60
                #plx_a
                dex
                bpl -
		lda szam
		sta $a2
		lda szam+1
		sta $a1
		lda szam+2
		sta $a0
		jsr OP_restore_zeropage

skip_basic_clock

		lda #CMOS_KEYREPEAT     ;Test keyboard repeat ENABLE/DISABLE
                jsr check_setup_atribut_exp
		ror 650			;Keyboard repeat

		jsr LWOP_init_c128key

                lda setup_chars_exp
                sta $286
                lda #147
                jsr newchrout_t

		jsr LW_waitretrace
                #sta_cfg_open
		jsr LWOP_set_screen_col

		#sta_cfg_16
		lda #<$e474
		ldy #>$e474
		jsr LWNO_print_ay
		jsr LWNO_print_imediate

		.if SUPERCPU=1
		.text 157,"idedos "
		#version
		.text "s"
		.else
		.text "idedos "
		#version
		.fi
		.null " expanded",13,9
;       	.null " idedos 0.90-rc4 another  test  version",13,9

		lda status_g		;Last chance for status...

		inc $318		;Enable RUN/STOP+RESTORE

		#assert_8000romioonly
cart_reset_end	#psro $e39d
		jmp exit2basic		;Return to basic

		#assert_8000romonly
bf		.text "1//:boot,prg"

                .if DEFAULT_FUNCTION_KEYS_STYLE=1
keys		.text "^!*,p",13    	;f1
		.text 147,"@$",13      	;f3
		.text "lI",13		;f5
		.text "rU:",13		;f7
		.text "%:*",13       	;f2
		.text 147,"@$*=p",157,0	;f4
		.text "ll",13		;f6
		.text "mA",13       	;f8
                .elsif DEFAULT_FUNCTION_KEYS_STYLE=2
keys		.text 147,"lI",13    	;f1
		.text 147,"rU",13      	;f3
		.text "/0:*",13		;f5
		.text 147,"@$:*",13    	;f7
		.text "sY.",13       	;f2
		.text 0			;f4
		.text "sY.",13		;f6
		.text "kI",13       	;f8
                .elsif DEFAULT_FUNCTION_KEYS_STYLE=3
keys		.text "lI:",13    	;f1
		.text "rU:",13      	;f3
		.text "/:*",13		;f5
		.text 147,"@$",13    	;f7
		.text "sY.",13       	;f2
		.text 0			;f4
		.text "sA",34,0		;f6
		.text "@",34,0       	;f8
                .elsif DEFAULT_FUNCTION_KEYS_STYLE=4
keys		.text 147,"lI",13    	;f1
		.text 147,"rU",13      	;f3
		.text "lO",13		;f5
		.text 147,"@$",13    	;f7
		.text "sY$0",157,0     	;f2
		.text "vE",13  		;f4
		.text "sA",34,"@:",0   	;f6
		.text 0		       	;f8
                .elsif DEFAULT_FUNCTION_KEYS_STYLE=5
keys		.text "@$:*",13    	;f1
		.text "/",0      	;f3
		.text 94,0		;f5
		.text "%",0	    	;f7
		.text 0			;f2
		.text 0  		;f4
		.text 95,0	   	;f6
		.text "@  ",34,"s:",0 	;f8
                .elsif DEFAULT_FUNCTION_KEYS_STYLE=6
keys		.text "%0:*",0    	;f1
		.text "@$:*",13      	;f3
		.text "lI",13		;f5
		.text "rU:",13	    	;f7
		.text "/0:*",0 		;f2
		.text 0  		;f4
		.text "sY.",13	   	;f6
		.text "sY.",13 		;f8
                .elsif DEFAULT_FUNCTION_KEYS_STYLE=7
keys		.text "/*",13    	;f1
		.text "@$",13      	;f3
		.text "rU:",13		;f5
		.text "lI:",13	    	;f7
		.text 0 		;f2
		.text "@$:",0 		;f4
		.text 29,20,0	   	;f6
		.text "sY.",13 		;f8
                .fi
keysend

;---------------------------
;ROM fix
;
fpx		.byte $de,$f7,$20
		.byte $de,$fa,$20

		.if SUPERCPU=1
memtrashpatch	lda ($c3),y
		bcc +
		lda $314,y
+
		.assert CFG_OPEN_RAM | CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO | CFG_E000_OPEN, CFG_8000_ROM, CFG_8000_RAM
brkpatch        .word __brk,__brk,__brk
		#assert_8000romonly

jiffypatch	jsr $ffc3
		.byte $80

shiftcpatch	.logical $eaf2
		cmp #$7f
		beq $eb26
                bit $28a
                bmi $eb0d
                bvs $eb42
                .here

		.fi

chrgpatch     	eor #$30
		cmp #$10
		eor #$30
;---------------------------
;IDE64 logo
;
logo		.byte $01,$f3,$21,$ff,$c0,$d1,$c1,$f1
		.byte $f9,$79,$25,$ff,$e0,$ff,$25,$fc
		.byte $00,$fc,$e5,$1f,$7f,$ff,$f0,$ff
		.byte $f7,$f0,$fc,$fe,$1e,$00,$f0,$fc
		.byte $ff,$00,$01,$03,$07,$0f,$1e,$3c
		.byte $78,$31,$fc,$bc,$3c,$03,$f3,$00
		.byte $13,$c0,$ff,$00,$2f,$79,$f9,$f1
		.byte $c1,$00,$93,$ff,$e0,$ff,$00,$93
		.byte $fc,$00,$fc,$00,$9f,$ff,$f0,$ff
		.byte $7f,$1f,$00,$9f,$fe,$1e,$fe,$fc
		.byte $f0,$00,$91,$70,$7f,$00,$93,$3c
		.byte $ff,$3c,$00
		.pend
;---------------------------
;Pal or ntsc?
;
                .cpu 6502
		#assert_8000romioonly
LW_palntsctest  .proc
		jsr LW_waitretrace.in
-		lda $d011
		bpl -
		lda #20
-		cmp $d012		;PAL/NTSC?
		bcs -
		lda $d011
                and #$80
                rts
                .pend
;---------------------------
;Wait retrace
;
                #assert_8000romioonly
LW_waitretrace 	.proc
-		bit $d011
		bpl -
in		bit $d011
		bmi in
		rts
                .pend
;---------------------------
;Set screen colors and font
;
		#assert_8000romioonly
LW_set_screen_cold .proc
                clc
in		lda #$0e
		ldx #$e6       		;Default colors
                gne LWOP_set_screen_col.in

                .cpu default
		#assert_cfg_open
LWOP_set_screen_col .proc
		lda #CMOS_LOWERCASE     ;Test lowercase ENABLE/DISABLE
                jsr check_setup_atribut_exp
		#ldatax setup_paper_border_exp
		eor setup_chars_exp
                and #$0f
		beq LW_set_screen_cold.in;Same paper and char color is invalid...

		lda setup_chars_exp
                #assert_8000romioonly

                .cpu 6502
in      	sta $286
		lda #$05
                rol
                asl
		sta $d018
                txa
                lsr
                lsr
                lsr
                lsr
                stx $d021
		sta $d020
		lda #27
		sta $d011
                rts
                .pend
;---------------------------
;Manage kernal vectors
;
		.assert CFG_8000_ROM | CFG_E000_ROM,CFG_8000_ROM | CFG_E000_ROM,CFG_8000_RAM | CFG_E000_OPEN
LWNO_set_old314vect .proc
		sec
		.byte SKIP1BCC
LWNO_set_new314vect .proc
		clc
		ldy #0
		ldx #15
loop		bcs old
		lda new314vect,x
		beq old
		sta $314,y
		lda #$de		;All between $de60-$deff...
in		sta $315,y
		iny
		iny
		dex
		bpl loop
		rts

old		lda $fd30,y
		sta $314,y
		lda $fd31,y
		gne in			;High address never on zp

                .assert CFG_8000_ROM | CFG_D000_IO,CFG_8000_ROM,CFG_8000_RAM
new314vect	.byte <__save,<__load,0,<__clall,<__getin,0
		.byte <__chrout,<__chrin,<__clrchn,<__chkout
		.byte <__chkin,<__close,<__open,<block_nmi,<__brk,0
		.pend
                .cpu default
;---------------------------
;Manage basic vectors
;
		.assert CFG_8000_ROM | CFG_E000_ROM,CFG_8000_ROM | CFG_E000_ROM,CFG_8000_RAM | CFG_E000_OPEN
LWNO_basic_restore .proc
		clc
set		ldy #7			;Restore basic vectors
loop		lda $e44b,y
		bcc old
		lda newbasic,y
old		sta $0304,y
		dey
		bpl loop
		rts

                .assert CFG_8000_ROM | CFG_D000_IO,CFG_8000_ROM,CFG_8000_RAM
newbasic	.word __mytoken
		.word __mytprt
		.word __mydoit
		.word __myexpr
		.pend
;---------------------------
;Init IO ($fda3)
;
                .cpu 6502
		.assert CFG_RAM | CFG_8000_ROM | CFG_D000_IO | CFG_E000_ROM,CFG_RAM | CFG_8000_ROM | CFG_D000_IO | CFG_E000_ROM,CFG_OPEN_RAM | CFG_8000_RAM | CFG_D000_ROM | CFG_E000_OPEN
LWNO_init_io	.proc
		#assert_kernal_io
kernal_fda3	= $fda3

		.assert CFG_RAM | CFG_8000_ROM | CFG_D000_IO | CFG_E000_ROM,CFG_RAM | CFG_8000_ROM | CFG_D000_IO | CFG_E000_ROM,CFG_OPEN_RAM | CFG_8000_RAM | CFG_D000_ROM | CFG_E000_OPEN
		jsr kernal_fda3
		ldy #0
		.if (SECOND_SID >> 8)=$d6
		lda $d030		;C128?
		lsr
		bcc +
		.fi
		sty SECOND_SID+$18	;Second SID
+		lda #7
		sta $dd00		;Release serial bus
		rts
		.pend
                .cpu default
;---------------------------
;Check setup attribut
;
                #assert_cfg_open
check_setup_atribut_exp .proc
		and setup_atribut_exp
		cmp #1
		rts
		.pend
;---------------------------
;C128 keyboard init
;
                #assert_cfg_open
LWOP_init_c128key .proc
		.if (DE32_EXISTS=1) & (SUPERCPU=0)
		lda $d030
		lsr
		bcs exit_c128key
		lda #CMOS_C128KEYS
                jsr check_setup_atribut_exp
		ldx #<__keyb
		lda #>__keyb
		ldy #%11111000
		bcs exit_c128key.in
		.fi
;---------------------------
;C128 keyboard exit
;
                .cpu 6502
		#assert_8000romioonly
exit_c128key	.proc
		ldx #<$eb48
		lda #>$eb48
		ldy #$ff
in		stx $028f
		sta $0290
                sty $d02f
                rts
                .pend
                .cpu default
;---------------------------
;Init VIC ($e518)
;
;Used before cpu detection!
;
                .cpu 6502
		#assert_8000romioonly
LWNO_init_vic	.proc
		ldx #3
		stx ActualOutput	;Standard output
		inx
		stx $028b		;keyboard repeat
                ldx #$2f
tntj		lda vicdefault-$11,x
		bmi +
		sta $d001,x
+               dex
		cpx #$10
		bne tntj
		lda #0
		sta ActualInput		;Standard input
		sta $0291
		sta $cf
-		sta $d000,x		;Initialize VIC
		dex
		bpl -
                ror
		ora $d011
		sta $d011

		lda #$0a
		sta $0289		;Keyboard buffer size
		sta $028c		;Keyboard first repeat

		lda #$0c
		sta $cd			;timer for cursor
		sta $cc			;no cursor blink
		rts

vicdefault	.byte         $37,$00,$00,$00,$08,$00;d010
		.byte $80,$0f,$00,$00,$00,$00,$00,$00;d018
		.byte $80,$80,$01,$02,$03,$04,$00,$01;d020
		.byte $02,$03,$04,$05,$06,$07,$4c,$ff;d028
                .byte $fc		;d030 C128
		.pend
                .cpu default
;---------------------------
;Init external variables
;
                #assert_cfg_open
OP_external_init .proc
		lda #INFO_KERNAL
		sta buffer_info_exp+STDIN
		sta buffer_info_exp+STDOUT
		sta buffer_info_exp+MISCIN
		sta buffer_info_exp+MISCOUT
		lda #INFO_IGNORE
		sta buffer_info_exp+IGNOREINOUT
		lda #3
		sta buffer_drive_exp+STDOUT;stdout
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		ldx #92
-		lda newchrin_ram_from,x
		sta newchrin_ram_exp,x
		dex
		bpl -
		.fi
		lda #STDIN
		sta buffer_input_exp	;Set In/Output to standard
		lda #STDOUT
		sta buffer_output_exp
		#lda0
		sta zp_valid_exp
		sta direct_write_exp	;no direct write
		sta kill_port		;Just to be sure...
                #sta_low_reg
		sta load_autostart_exp	;no autostart
		sta buffer_drive_exp+STDIN;stdin
		sta buffer_logical_exp+STDIN
		sta buffer_logical_exp+STDOUT
                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                sta dos_accel_internal_exp
                .fi
		ldx #MAX_OPEN_FILES-1
-		sta open_files_logical_exp,x;All files closed
		cpx #MAX_BUFFERS
                bcs +
                sta buffer_logical_exp,x;All buffers free
+		dex
		bpl -
		lda $02a6		;PAL(1)/NTSC(0)
		sta TV_machine_type_exp
                .if SUPERCPU=0
		lda $b0			;jiffydos
		sta JiffyDos_present_exp
                .fi

		.if PCLINK_USECRC
                ldx #0
cu2             ldy #7
                #lda0
                stx $2
-		lsr $2
		bcc +
                eor crcgen,y
+               dey
                bpl -
                sta crc8_table_exp,x
                inx
                bne cu2
		.fi

		jmp LWOP_get_cmos_setup

		.if PCLINK_USECRC
crcgen		.byte $8c,$46,$23,$9d,$c2,$61,$bc,$5e
		.fi

		.pend
;---------------------------
;Chrin, chrout, getin ram
;
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)

                #assert_cfg_open
chrin_data_sector_last__
		stx help_x_exp
		ldx buffer_input_exp
		lda newchrin_ram.tmpp+1
		sta $a4
		lda buffer_address_hi_exp,x
		jmp newchrin.chrin_data_sector_last

chrin_next_bank__
		stx help_x_exp
		ldx buffer_input_exp
		jmp newchrin.chrin_next_bank

chrin_not_data_sector__
		stx help_x_exp
		ldx buffer_input_exp
		jmp newchrin.chrin_not_data_sector

newchrin_ram_from
		.logical newchrin_ram_exp
newchrin_ram	.proc
		lda buffer_info_exp+STDIN;3
		bmi chrin_not_data_sector_;5

getin		asr #0			;7

		isb buffer_address_low_exp	;10
		eor #$ff		;12
		beq chrin_next_bank_	;14
		sta tmpp+1		;17
		lda buffer_address_hi_exp;20
		bmi chrin_data_sector_last_;22
		sta tmpp+2		;25

tmpp		lda $ffff		;28
		jmp exit2basic		;31

chrin_not_data_sector_
		jmp chrin_not_data_sector__;34
chrin_next_bank_
		jmp chrin_next_bank__	;37
chrin_data_sector_last_
		jmp chrin_data_sector_last__;40

newgetin_ram	.proc
		lda buffer_info_exp+STDIN;43
		bpl newchrin_ram.getin	;45
		jmp newgetin_back__	;48

newchrout_ram	.proc
		sta bce+1		;51
		lda buffer_info_exp+STDOUT;54
		asr #$01       		;56
		bcs chrout_not_data_sector_;58

		isb buffer_address_low_exp	;61
		eor #$ff		;63
		beq chrout_nxt_bank_	;65
		sta tmppp+1		;68
		lda buffer_address_hi_exp;71
		bmi chrout_data_sector_last_;73
		sta tmppp+2		;76

bce		lda #0			;78
tmppp		sta $ffff		;81
		jmp exit2basic		;84

chrout_not_data_sector_
		jmp chrout_not_data_sector__;87
chrout_nxt_bank_
		jmp chrout_nxt_bank__	;90
chrout_data_sector_last_
		jmp chrout_data_sector_last__;93
		.pend

		.here

newgetin_back__	stx help_x_exp
		ldx buffer_input_exp
		jmp newgetin.back

chrout_not_data_sector__
		stx help_x_exp
		ldx buffer_output_exp
		lda newchrout_ram.bce+1
		pha
		jmp newchrout.chrout_not_data_sector

chrout_nxt_bank__
		stx help_x_exp
		ldx buffer_output_exp
		lda newchrout_ram.bce+1
		pha
		lda #0
		jmp newchrout.chrout_nxt_bank

chrout_data_sector_last__
		stx help_x_exp
		ldx buffer_output_exp
		lda newchrout_ram.bce+1
		pha
		lda newchrout_ram.tmppp+1
		sta $a4
		lda buffer_address_hi_exp,x
		jmp newchrout.chrout_data_sector_last
		.fi
;---------------------------
;Check JiffyDos (Z=1)
;
                .cpu 6502
		.if SUPERCPU=0
		.assert CFG_8000_ROM | CFG_E000_ROM,CFG_8000_ROM | CFG_E000_ROM,CFG_8000_RAM | CFG_E000_OPEN
LWNO_detect_jiffydos .proc
		ldy #250
		ldx #184
j11x		iny
		beq yes_jiffy		;JiffyDos found
j1x		inx
		beq no_jiffy		;No Jiffydos
		lda $e45f-185,x
		cmp jiffytext-251,y
		beq j11x
cce		cpy #251
		beq j1x
		dex
		dey
		gne cce
yes_jiffy	lda #<$f672		;Fix jiffydos F-keys
		sta $b0
		lda #>$f672
		sta $b1
no_jiffy	rts

jiffytext	.text "jiffy"
		.pend
                .fi
                .cpu default
;---------------------------
;Check/set ram sign
;
                #assert_cfg_open
OP_check_sign	.proc
		sec
set		ldx #38
loop		lda uuid-3,x		;chksum!
		bcc se
		cmp ident_label_exp-1,x
		bne sorry
se		sta ident_label_exp-1,x
		dex
		bne loop                ;Z=1
sorry		rts
		.pend
;---------------------------
;Check drive status
;
                #assert_cfg_open
check_drive_status .proc
                lda status_g
                #lda_low_reg
                and #%10001000
                beq ok

                ldx #255
                stx hig_reg
		lda #idle_imediate1_d
                #sta_low_reg
		sta comm_g
                ldy #3
-		jsr BACK_with_C1_on_timeout_or_C0_status
		.if FAST_DATA_TRANSFERS
                lda status_g
                .else
		lda low_reg
                .fi
		bmi -
ok		rts
		.pend
;--------------------------------------
                .if SUPERCPU=1
ofs_fs_ident	.text "c64-ide v0.02-a "
cfs_fs_ident	.text "c64 cfs v 0.11b "
cdstr           .text "cd001"                   ;from dos.asm
powerconst	.byte 0,12,24,36,48,60,72,84,96,120,180,240,241,242,244
jotyp		.byte $01,$04,$06,$0b,$0c,$0e,$0f
featc		.byte %10000000,$33,$99		;retry
		.byte %01000000,$82,$02		;writecache
                .byte %00100000,$55,$aa         ;readahead
                .byte %00010000,$ff,$01         ;cdromspeed
                .fi
