;
; chkin.asm - CHKIN kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Chkin (X=logical file numbe)
;
; Used registers: A X (Y preserved)
; Status: C
; C=1:
; A=3 file not open
; A=5 device not present
; A=6 (not input file)
; Status byte response:
; Cleared!
; ST=$03 timeout
; ST=$80 if device not present
; Secondary address 128..255 then communication on serial bus is a bit different...
;**************************************************
		#assert_cfg_open

newchkin	.proc
		jsr search_logical_X_chkinout
		beq chkin_cont3		;Ok found buffer for file

                jsr open_file_open
		beq chkin_cont4

		lda #STDIN
                ldx ActualDev
		beq +			;keyboard?
		#stx_l_a buffer_drive_exp+MISCIN
                lda #MISCIN
+               .if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
                jsr chkin_doch2
                .else
		sta buffer_input_exp
                .fi
		cpx #4
		bcs serial      	;serial device?
		#psr $f219
		gcc exit2		;keyboard, tape, rs232, screen

serial  	jsr chkinout_driveconv
                sta manager_chkin_exp

		#jsr_OP_cfg16up
                #assert_cfg_16
		txa
                .if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr iectalk
                .else

                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                php
                bcs nx
                jsr $ed09		;talk
                #jmps nx2
nx              jsr iectalk
nx2		plp
                .else
                jsr $ed09		;talk
                .fi

                .fi			;JIFFYDOS ACCEL CHRINOUT

		lda ActualSecAddr
		bpl sfc
		jsr $edcc		;wait for clock
		gpl xf
sfc
		.if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr iectksa
                .else

                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                bcs nx3
                jsr $edc7		;tksa
                #jmps nx4
nx3             jsr iectksa
nx4
                .else
                jsr $edc7		;tksa
                .fi

                .fi		;JIFFYDOS ACCEL CHRINOUT

xf		#jsr_OP_cfg16down
		#assert_cfg_open
		txa
		bit Status
                bpl exit                ;to $f21b
                gmi exit_device_not_present

chkin_cont4    	ldx #IGNOREINOUT
chkin_cont3	.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jsr chkin_doch
		.else
		#stx_l_a buffer_input_exp
		.fi

		lda ActualDev
exit		sta ActualInput
		tax             	;buggy software

		clc
exit2		jmp exit2basic
		.pend

		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
chkin_doch	.proc
		txa
chkin_doch2	.proc
		sta buffer_input_exp
		clc
		adc #<buffer_address_low_exp
		sta newchrin_ram_at2_exp
		adc #buffer_address_hi_exp-buffer_address_low_exp
		sta newchrin_ram_at3_exp
		adc #buffer_info_exp-buffer_address_hi_exp
		sta newchrin_ram_at_exp
		sta newgetin_ram_at_exp
		rts
		.pend
		.fi

chkinout_driveconv .proc
		ldx #3
		lda ActualDev
cy		cmp current_floppy8_exp,x
		bne xg
		txa
		ora #8
		ldx #0
xg		dex
		bpl cy

		tax

                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                lda dos_accel_internal_exp
                cmp #1
                .fi

		#lda0
                rts
                .pend
