;
; chrin.asm - CHRIN, GETIN kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Chrin & getin
;
; Used registers: A (X Y preserved)
; Status: C (N Z for Accu)
; C=1:
; A=6 (not input file)
; Status response:
; ST=$00 if no error
; ST=$40 if last byte of file, or end of line on channel #15, or end of record.
; ST=$42 if no more bytes (open error, output file, device...)
; ST=$52 if no more bytes, because of readerror (open error, output file, device...)
; ST=$80 if device not present
;
;Note:
;Directory summ at end will be incorrect if multiple read at same time...
;And directory is half done anyway.
;**************************************************
;Buffer:
;
;current file position: (buffer_address_low_exp + buffer_address_hi_exp << 8 + 1) & 0x7ff | buffer_filepos2_exp <<8 ...
;if buffer_address_end_hi_exp == 0 then
;       buffer contains no data, have to read first
;       buffer_address_low/hi_exp must point before the buffer
;	buffer_address_end_low_exp must be 0
;
;if buffer_address_end_hi_exp is after the end of buffer then
;	the whole buffer is needed and there are more sectors to come
;
;if buffer_address_hi_exp & 0x80 then
;       file ends in this memory page
;       buffer_address_end_low_exp selects last byte of buffer
;
;buffer_address_low/hi_exp is only before buffer if it contains no data! so buffer_address_end_low/hi_exp=0
;buffer_address_low/hi_exp points to the previous byte read from buffer if any
;
; file position pointer does not advance if eof...
;--------------------------------------
; GETIN
;--------------------------------------
		#assert_cfg_open

		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
fake		= newchrin_ram
fake2		= newgetin
		.fi

newgetin	.proc
		.if SUPERCPU=1
		phx
		.else
		stx help_x_exp
		.fi
		#ldx_l_a buffer_input_exp
		lda buffer_info_exp,x
		bpl newchrin.getin
back		cmp #INFO_KERNAL
		bne newchrin.chrin_not_data_sector
                .if SUPERCPU=1
                pla
                .fi
		#psr $f140
		.if SUPERCPU=1
                pha
                .fi
		cpx #STDIN
		bne nonstandard
		lda buffer_output_exp
		eor #STDOUT
		bne nonstandard
		.if SUPERCPU=1
                plx
                .fi
		#psr switch_norm
		#psr gotobank3
                .if SUPERCPU=0
                lda #0
                .fi
bbb             .if SUPERCPU=0
		ldx help_x_exp
                .fi
		ora #$00                ;Now we switch to bank3 to prevent
                jmp exit2basic          ;raster bugs when calling $ffe4
                                        ;instantly to scan keyboard...
nonstandard	lda buffer_drive_exp,x
		.if SUPERCPU=1
                plx
                .fi
		cmp #4
                bcc bbb
                .if SUPERCPU=0
		ldx help_x_exp
                .fi
		pla
                pla
	        gcs newchrin.getinen
;--------------------------------------
; CHRIN
;--------------------------------------

                #assert_a000romioonly
newchrin_t       .proc
                #sta_cfg_open
		#psr switch_16k

		#assert_cfg_open

newchrin	.proc
		.if SUPERCPU=1
		phx
		.else
		stx help_x_exp
		.fi
		#ldx_l_a buffer_input_exp
		lda buffer_info_exp,x
		bmi chrin_not_data_sector

;BLK_RW_DIRTY, BLK_READ, BLK_RW

getin		.if SUPERCPU=1
		lda $a6
		pha
		lda #1
		sta $a6			;SCPU Page 1
		clc
hard_chrin	lda buffer_address_low_exp,x
		inc a
		sta buffer_address_low_exp,x
		.else
hard_chrin	asr #$00
		isb buffer_address_low_exp,x
		eor #$ff
		.fi
                .if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		beq chrin_next_bank
		sta $a4
                .else
                sta $a4
                beq chrin_next_bank
                .fi
		lda buffer_address_hi_exp,x
		bmi chrin_data_sector_last
chrin_bank_nen2	sta $a5
chrin_bank_nend

		.check CFG_OPEN_RAM, CFG_RAM
		.if SUPERCPU=1		;Get byte
chrin_lastb
		lda [$a4]
		plx			;Restore $a6
		stx $a6
chrin_v		plx
		.else
chrin_lastb	ldx #0
		lda ($a4,x)
chrin_v		ldx help_x_exp
		.fi
		ora #$00		;Set NZ flags
exit2		jmp exit2basic
;---------------------------------
chrin_next_bank .if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		sta $a4
                .fi
		.if SUPERCPU=1
		lda buffer_address_hi_exp,x
		inc a
		sta buffer_address_hi_exp,x
		.else
		isb buffer_address_hi_exp,x
		eor #$ff
		.fi
		cmp buffer_address_end_hi_exp,x
		bcc chrin_bank_nen2
		ora #$80
		sta buffer_address_hi_exp,x;Mark next bank

chrin_data_sector_last
		and #$7f
		sta $a5
		lda $a4
		cmp buffer_address_end_low_exp,x
		bcc chrin_bank_nend
		gcs chrin_next
;---------------------------------------

;BLK_WRITE_DIRTY, BLK_RW_EOF_DIRTY, BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, STATUS, KERNAL, IGNORE

noread2		.if SUPERCPU=1
                phx
		.fi
noread		lda #$0d
		clc
		gcc chrin_v

chrin_not_data_sector
		cmp #INFO_KERNAL
		bne nchrin_stand

		lda buffer_drive_exp,x
		cmp #4
		.if SUPERCPU=1
                plx
		.else
		ldx help_x_exp
                .fi
		bcc vew			;serial device?

getinen
                lda Status
                bne noread2

		.if JIFFYDOS_ACCEL_CHRINOUT=1
		#jsr_OP_cfg16up
                #assert_cfg_16
		jsr iecin
		#jsr_OP_cfg16down
                #assert_cfg_open
                lda SerialInBuf
                .if SUPERCPU=0
                jmp exit2basic
                .else
                bra exit2
                .fi
                .else

                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                lda dos_accel_internal_exp
                beq nx
                #jsr_OP_cfg16up
                #assert_cfg_16
		jsr iecin
		#jsr_OP_cfg16down
                #assert_cfg_open
                lda SerialInBuf
                .if SUPERCPU=0
                jmp exit2basic
                .else
                bra exit2
                .fi
                .fi

nx              #psr $ee13
                .if SUPERCPU=0
                jmp exit2basic
                .else
                bra exit2
                .fi
                .fi		;JIFFYDOS ACCEL CHRINOUT

vew		sta ActualInput
		ora #0
		bne bvb
		.if SUPERCPU=0
                lda $315
                eor #$ea
                bne bvb
		.fi
                lda #CMOS_FUNCTION_KEYS
                and setup_atribut2_exp
                bne functionkeys

bvb		#psr $f157
                .if SUPERCPU=0
		jmp exit2basic
                .else
                bra exit2
                .fi
;---------------------------------------

;BLK_WRITE_DIRTY, BLK_RW_EOF_DIRTY, BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, STATUS, IGNORE

nchrin_stand
                .if SUPERCPU=1
                xba
                .fi
                lda Status
                bne noread

                .if SUPERCPU=1
                xba
                .else
		lda buffer_info_exp,x
                .fi
		cmp #INFO_STATUS
		bne nchrin_status

		lda buffer_type_exp,x
                cmp #BUFFERMODE_PCLINK
                bne norm

                lda status_pclink_pos_exp
                bpl fe
                #phy_a
                jsr OP_save_zeropage_scpu01up
                .if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
                php
                sei
                .fi
		jsr pclink_status
                bcc +			;timeout and crc
                ldy #0
+
		.if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
                plp
                .fi
		#sty_l_a status_pclink_size_exp
                jsr OP_restore_zeropage_scpu01down
                #ply_a
                #lda0

fe              tax
                #adc1c
		cmp status_pclink_size_exp
                bcc readb
                lda #$40
                sta Status
                asl

readb           sta status_pclink_pos_exp
		lda pclink_status_exp,x
chrin_vx        gcc chrin_v

norm		lda buffer_drive_exp,x
		sta current_drive_exp
                tax
		lda status_pos_exp,x
		bpl ok_st		;No pending request

		.if SUPERCPU=1
		phy
                tay
		.else
		tya
		pha
		ldy status_pos_exp,x
		.fi
		jsr OP_save_zeropage_scpu01up
		tya
		jsr LWOP_disk_status
		jsr OP_restore_zeropage_scpu01down
		#ldx_l_a current_drive_exp
		#ply_a

ok_st		lda status_pos_exp,x
		cmp status_size_exp,x
                bcs gon
		txa
		lsr
		ror			;*64
		ror
		ora status_pos_exp,x
		.if SUPERCPU=1
		pha
		and #$3f
		inc a
		sta status_pos_exp,x
		plx
		.else
		inc status_pos_exp,x
		tax
		.fi
		lda drive_status_exp,x
		gcc chrin_vx		;end of message? (c=0)

gon
		#OP_scpu01up
		jsr LWOP_reporterror_ok
		#OP_scpu01down
reteof          lda #$40        	;simple eof, value CR
retsom          sta Status
                #jmps noread
;---------------------------------------

;BLK_WRITE_DIRTY, BLK_RW_EOF_DIRTY, BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, IGNORE

nchrin_status   cmp #INFO_RELATIVE
		beq chrin_relative
		cmp #INFO_RELATIVE_DIRTY
		beq chrin_relative
                lda #$42
retsomx         gne retsom                   ;no more data, value unspecified
;---------------------------------------
chrin_relative  ;sec
		sbc #INFO_RELATIVE-INFO_RELATIVE_READ
                .cerror (INFO_RELATIVE-INFO_RELATIVE_READ)!=(INFO_RELATIVE_DIRTY-INFO_RELATIVE_READ_DIRTY)
                pha
                #phx_a
                jsr OP_save_zeropage
                .if SUPERCPU=1
                plx
                phx
                .else
                pla
                tax
                pha
                .fi
                #lda0
                jsr search_end_of_record
                jsr OP_restore_zeropage
                #plx_a
                pla
                sta buffer_info_exp,x
                #jmps getin
;---------------------------------------
chrin_next	#stx_l_a buffer_currentp_exp
		#phy_a
		jsr OP_save_zeropage_scpu01up

		jsr XIT_do_next

		jsr OP_restore_zeropage_scpu01down
		#ply_a
                #ldx_l_a buffer_currentp_exp

                bcc noerr
                .if SUPERCPU=1
                pla
                sta $a6
                .fi
                lda #$52
                gcs retsomx

noerr		bit Status
		bvs chrin_lastb
		gvc hard_chrin
;---------------------------------------
;BLK_RW_DIRTY, BLK_READ, BLK_RW
;
XIT_do_next	#xit_init

		#ldx_l_a buffer_currentp_exp
		lda buffer_type_exp,x
                and #BUFFERMODE_MSK
                tay
                cpy #hi-lo
                bcs ill
                jsr fileop_prepare
                pha
                lda hi,y
                pha
                lda lo,y
                pha
                lda buffer_address_end_hi_exp,x
                cmp #1
                lda $a5                 ;buffer_address_hi_exp,x
                eor buffer_address_max_hi_exp,x
ill		rts

lo		.byte <(ill-1)
		.byte <(XIT_do_filenext-1)
		.byte <(XIT_do_dirnext-1)
		.byte <(XIT_do_direct-1)
                .byte <(XIT_do_pclink-1)
                .byte <(XIT_do_isofilenext-1)
                .byte <(XIT_do_ofsfilenext-1)
                .byte <(XIT_do_fatfilenext-1)
                .byte <(XIT_do_link-1)
		.byte <(XIT_do_relnext-1)

hi		.byte >(ill-1)
		.byte >(XIT_do_filenext-1)
		.byte >(XIT_do_dirnext-1)
		.byte >(XIT_do_direct-1)
                .byte >(XIT_do_pclink-1)
                .byte >(XIT_do_isofilenext-1)
                .byte >(XIT_do_ofsfilenext-1)
                .byte >(XIT_do_fatfilenext-1)
                .byte >(XIT_do_link-1)
		.byte >(XIT_do_relnext-1)

;---------------------------------------
XIT_do_relnext	.proc                   ;no problem with dirty buffer, as it can't happen
		bcc dc
                beq next

                lda #$40
                sta Status
                pla
                adc #INFO_RELATIVE-INFO_RELATIVE_READ-1
                pha
                .cerror (INFO_RELATIVE-INFO_RELATIVE_READ)!=(INFO_RELATIVE_DIRTY-INFO_RELATIVE_READ_DIRTY)

                jsr position_to_next_record

                #jmps meg

next            pla
                cmp #INFO_RELATIVE_READ_DIRTY
                bne v1
		jsr XIT_flush_buf
                lda #INFO_RELATIVE_READ
v1              pha

		jsr incfileposhi512

dc		jsr XIT_load_tree	;-=> error

                jsr XIT_relative_read_in_buffer;-=> error

meg             lda #1
		jsr search_end_of_record

                jsr check_fp
		lda buffer_filesize4_exp,x
		sbc buffer_filepos4_exp,x
                bcs +
                lda #$80+50             ;record not present hack
                jsr set_rel_error
+
		#jmps chrin_chrout_do_file_end.be
                .pend
;---------------------------------------
XIT_do_filenext	.proc
		bcc dc
eof		bne XIT_do_pclink.eof	;eof mark?

		pla
		cmp #INFO_BLK_RW_DIRTY
		bne v1
		jsr XIT_flush_buf	;-=> error
		lda #INFO_BLK_RW
v1		pha

		jsr incfileposhi512

dc		jsr XIT_load_tree	;-=> error

		sec			;force clean
		jsr XIT_read_in_buffer	;-=> error

		#jmps chrin_chrout_do_file_end
                .pend
;---------------------------------------
XIT_do_pclink	.proc
		bcc dc
eof		bne XIT_do_isofilenext.eof     	;eof mark?

dc              clc
		.if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
                php
                sei
                .fi
		jsr pclink_chrinout
                .if PCLINK_USECRC
                bcc noerr
                .if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
                plp
                .fi
                pla
                rts				;crc error
noerr
		.fi

		.if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
                plp
                .fi
                pha
                jsr OP_init_buffer_pointers
                pla
                beq chrin_chrout_do_file_end.more
                tya
                bne +
                dec pom2
+               dey
		lda pom2
                #jmps chrin_chrout_do_file_end.fer
		.pend
;---------------------------------------
XIT_do_isofilenext .proc
		bcc dc
eof		bne XIT_do_ofsfilenext.eof     	;eof mark?

		jsr incfileposhi2048

dc      	lda buffer_filepos2_exp,x
                sta next_sector+3
		lda buffer_filepos3_exp,x
                sta next_sector+2
                lda buffer_filepos4_exp,x

		ldy #3
ry              lsr
                ror next_sector+2
                ror next_sector+3
                dey
                bne ry
                sta next_sector+1
                sty next_sector

                txa
                asl
                asl
                tax
                ldy #3
ro              lda next_sector,y
                adc buffer_isofile_start_exp,x
                sta next_sector,y
                inx
                dey
                bpl ro

		jsr OP_buffer_start
                jsr XIT_read_a_sector

		#jmps chrin_chrout_do_file_end
		.pend
;---------------------------------------
XIT_do_ofsfilenext .proc
		bcc dc2
eof		bne XIT_do_fatfilenext.eof      	;eof mark?

dc2
		lda #3
		sta buffer_address_low_exp,x
		lda buffer_start,x
                sta buffer_address_hi_exp,x
                jsr init_pom1_y

		lda #$3f
eos             #and_zp_y pom1
                ldx oldfsbyteorder,y
                sta next_sector,x
                lda #$ff
                iny
                cpy #4
                bcc eos

		lda pom2
                jsr XIT_read_a_sector

                jsr OP_buffer_start_pom1_y

                #lda_zp_y pom1
                asl
                asl
		lda buffer_address_max_hi_exp,x
                bcc ne
                iny
                #lda_zp_y pom1
                adc #3-1
		sta buffer_address_end_low_exp,x;Auto read first sector
                dey
                #lda0
                #adc_zp_y pom1
                and #$3f
                bne ne
                lda pom2
                ora #$80
                sta buffer_address_hi_exp,x
                #lda0
kf              adc pom2
ne		sta buffer_address_end_hi_exp,x

		#jmps chrin_chrout_do_file_end.be
		.pend
;---------------------------------------
XIT_do_fatfilenext .proc
		bcc dc
eof		bne read_file_eof      	;eof mark?

		jsr incfileposhi512
dc
		lda #>b_fat_current_exp
		jsr buffer_subpart_cp	;restore

                jsr cptons
                ldy #4
                #lda_zp_y current_path
                jsr cluster2sector

		jsr OP_buffer_start
                jsr XIT_read_a_sector

		lda #>b_fat_current_exp
		jsr buffer_subpart_cp	;restore
                #ldx_l_a partition_currentp_exp;ok
                ldy #4
                #lda_zp_y current_path
                #adc1c
                #sta_zp_y current_path
                eor partition_fat_sectperclust_exp,x
                bne ok
                #sta_zp_y current_path

                jsr cptons

                jsr XIT_nextcluster

		lda #>b_fat_current_exp
		jsr buffer_subpart_cp	;restore

                ldy #3
-               lda next_sector,y
                #sta_zp_y current_path
                dey
                bpl -

ok		#jmps chrin_chrout_do_file_end

cptons          ldy #3
-               #lda_zp_y current_path
                sta next_sector,y
                dey
                bpl -
                rts
		.pend
;---------------------------------------
XIT_do_link	.proc
		bcs read_file_eof	;no more sectors...

                jsr XIT_load_tree

                jsr OP_buffer_start_pom1_y
                lda #>b_cfs_link_data_exp
                jsr buffer_subpart_cp

-               #lda_zp_y current_path
                #sta_zp_y pom1
                iny
                bne -
                inc pom1+1
                inc current_path+1
                lda pom1+1
                lsr
                bcs -
                gcc chrin_chrout_do_file_end
;---------------------------------------
; Common end of both chrin and chrout
;
read_file_eof  	.proc
		pla
		cmp #INFO_BLK_READ
		bne +
		lda #INFO_BLK_R_EOF
+		cmp #INFO_BLK_RW
		bne +
		lda #INFO_BLK_RW_EOF
+		cmp #INFO_BLK_RW_DIRTY
		bne +
		lda #INFO_BLK_RW_EOF_DIRTY
+               pha
                lda #$40
		sta Status
		gne chrin_chrout_do_file_end.more
                .pend

chrin_chrout_do_file_end .proc
		jsr OP_init_buffer_pointers

                jsr check_fp
		bne more
		lda buffer_filesize4_exp,x
		sbc buffer_filepos4_exp,x
		bcc more
		bne more

		lda buffer_address_max_hi_exp,x
;		sec
		sbc buffer_start,x
		clc
		sbc pom2
		bcc more

		lda pom2
		ora buffer_start,x
		gcs fer

more		ldy #0
		lda buffer_address_max_hi_exp,x
fer		sta buffer_address_end_hi_exp,x
		tya
		sta buffer_address_end_low_exp,x
be
		pla			;be safe
be2		sta buffer_info_exp,x
		clc
		rts			;-=> ok
		.pend

check_fp        .proc
		lda buffer_filesize1_exp,x
		sec
		sbc #1
		tay
		lda buffer_filesize2_exp,x
		sbc buffer_filepos2_exp,x
		sta pom2
		lda buffer_filesize3_exp,x
		sbc buffer_filepos3_exp,x
                rts
                .pend
;---------------------------------------
XIT_do_dirnext	.proc
		ldx #write_byte_r2end-write_byte_r2-1
htdo2		lda write_byte_r2,x
		sta start_zeropage,x
		dex
		bpl htdo2

		jsr OP_init_buffer_pointers

		lda buffer_start,x
		sta start_zeropage+6

		lda #>b_dir_misc_exp
		jsr buffer_subpart_cp

		.if <b_dir_misc_exp
                ldy #<b_dir_misc_exp
                .fi
		ldx #direntry_backup_end_exp-direntry_backup_start_exp-1
-		#lda_zp_y current_path
		sta direntry_backup_start_exp,x
		iny
		dex
		bpl -

                #cnext direntry_next_sector_exp , next_sector

		lda direntry_first_exp
		bpl ndirlastbyte

		pla			;INFO_IGNORE
do_eof		lda #$40
		sta Status
		clc
		rts			;-=> last byte of file

ndirlastbyte	lda #>b_dir_searchstring_exp
		jsr buffer_subpart_cp	;restore searchstring

		lda buffer_start,x
                ora #>b_dir_limit_exp
		jsr XIT_dirlp2		;-=> error

		lda direntry_first_exp
		bpl hq
		lda start_zeropage+5
		bne +
		dec start_zeropage+6
+		dec start_zeropage+5
hq
		#ldx_l_a buffer_currentp_exp
		lda start_zeropage+5
		sta buffer_address_end_low_exp,x
		lda start_zeropage+6
		sta buffer_address_end_hi_exp,x

		jsr next_sector_to_direntry_next_sector_exp

		lda #>b_dir_misc_exp
		jsr buffer_subpart_cp

;		ldy #<b_dir_misc_exp
		ldx #direntry_backup_end_exp-direntry_backup_start_exp-1
-		lda direntry_backup_start_exp,x
		#sta_zp_y current_path
		iny
		dex
		bpl -

		#ldx_l_a buffer_currentp_exp
		pla			;be safe
		sta buffer_info_exp,x
		clc			;-=> ok
		rts
                .pend
;---------------------------------------
XIT_do_direct	.proc
		jsr OP_init_buffer_pointers
		pla			;be safe
		sta buffer_info_exp,x
		#jmps XIT_do_dirnext.do_eof
                .pend
;---------------------------------------
search_end_of_record .proc
                sta pom3
                lsr
                bcs +
		lda buffer_address_end_hi_exp,x
                beq empt
+               #phy_a
		lda buffer_address_low_exp,x
		sta pom1
                cmp #$ff
		lda buffer_address_hi_exp,x
                and #$7f
		sta pom2
                adc #0
                eor buffer_address_max_hi_exp,x
                bne nend
                .if SUPERCPU=1
                inc a
                .else
                lda #1
                .fi
;                clc
                gcc all

nend            lda buffer_recordlen_exp,x
                sec
                sbc buffer_recordpos_exp,x
                bit pom3
                bmi all
                .if SUPERCPU=1
		ldy #1
		sty pom3       		;SCPU Page 1
                .fi
                bvc search
                lda buffer_address_end_low_exp,x
                sec
                sbc pom1
                beq jo
                tay
                #lda0
-               #sta_zp_y pom1
                dey
                bne -
                geq jo

search          tay
-               #lda_zp_y pom1
                bne megvan
                dey
                bne -
                iny
megvan          tya
all		sta buffer_recordclen_exp,x
                clc
                adc pom1
                sta buffer_address_end_low_exp,x
                #lda0
                php
                adc pom2
                sta buffer_address_end_hi_exp,x
                plp
                lda pom2
                bcs +
                ora #$80
+               sta buffer_address_hi_exp,x
jo
                #ply_a
empt            rts
                .pend
;---------------------------------------
position_to_next_record .proc
                lda buffer_address_end_hi_exp,x
                beq meg
                lda buffer_recordlen_exp,x
                sec
                sbc buffer_recordpos_exp,x
                sbc buffer_recordclen_exp,x
                clc
                adc buffer_address_end_low_exp,x
                sta buffer_address_low_exp,x
                #lda0
                sta buffer_recordpos_exp,x
                adc buffer_address_end_hi_exp,x
                sta buffer_address_hi_exp,x
meg             rts
                .pend
;---------------------------------------
XIT_relative_read_in_buffer .proc
		sec			;force clean
		jsr XIT_read_in_buffer	;-=> error

                jsr OP_buffer_start
                tay
                lda buffer_recordpad_exp,x
                sec
                sbc #1
                sta buffer_address_low_exp,x
                tya
                sbc #0
                sta buffer_address_hi_exp,x
                rts
                .pend
;---------------------------------------
set_rel_error   .proc
		.if SUPERCPU=1
                xba
                txy
                lda buffer_drive_exp,x
                tax
                xba			;record not present hack
                sta status_pos_exp,x
                tyx
                .else
		ldy buffer_drive_exp,x
                sta status_pos_exp,y
                .fi
                .if DRIVES=4
                bit errledon_g
                .fi
                rts
                .pend
;---------------------------------------
write_byte_r2	.proc
		.logical start_zeropage
		.if SUPERCPU=1
		txa			;1
		tyx			;2
		tay			;3
lp		pla			;4
		sta $1ff00,x		;8
		dex			;9
		bpl lp			;11
		tyx			;12
		rts			;13
		.else
		jmp lp			;3
lp		pla			;4
		sta $ff00,y		;7
		dey			;8
		bpl lp			;10
		rts			;11
		.fi
		.here
		.pend
write_byte_r2end
