;
; chrout.asm - CHROUT kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Chrout (A=character out)
;
; Used registers: - (all preserved)
; Status: C (N Z for Accu)
; C=1:
; A=7 (not output file)
; Status response:
; ST=$00 if no error
; ST=$03 timeout
; ST=$80 if device not present
;Note:
;Currently buffer is 256 bytes long and only used for error channel...
;**************************************************
		#assert_a000romioonly
newchrout_t	.proc
		#sta_cfg_open
                #assert_cfg_open
		.if SUPERCPU=1
		xba
		.else
		sta help_trap_accu_exp
                lda #>(switch_16k-1)
                pha
		.fi

		lda buffer_output_exp
		eor #STDOUT
		bne nonstandard
		lda buffer_input_exp
		cmp #STDIN
		bne nonstandard
                .if SUPERCPU=1
		#psr gotobank3back
		gcs standard

nonstandard	#psr switch_16k
standard
		xba
		.else
		lda #<(gotobank3back-1)
                .byte SKIP2
nonstandard    	lda #<(switch_16k-1)
                pha
		lda help_trap_accu_exp
		.fi

		#assert_cfg_open
newchrout	.proc
		pha
		#stx_l_a help_x_exp
		#ldx_l_a buffer_output_exp
		lda buffer_info_exp,x
dback
		.if SUPERCPU=1
		lsr
		bcs chrout_not_data_sector
chrout
;BLK_WRITE_DIRTY, BLK_RW_DIRTY, BLK_RW_EOF_DIRTY

		lda $a6
		pha
		lda #1
		sta $a6			;SCPU Page 1

hard_chrout	lda buffer_address_low_exp,x
		inc a
		sta buffer_address_low_exp,x
		.else
		asr #$01
		bcs chrout_not_data_sector
chrout
hard_chrout	isb buffer_address_low_exp,x
		eor #$ff
		.fi
                .if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		beq chrout_nxt_bank
		sta $a4
                .else
		sta $a4
		beq chrout_nxt_bank
                .fi
		lda buffer_address_hi_exp,x
		bmi chrout_data_sector_last
chrout_bnk_nen2	sta $a5
chrout_bnk_nend
		.check CFG_OPEN_RAM, CFG_RAM
		.if SUPERCPU=1		;Store byte
		plx
		pla
		sta [$a4]
		stx $a6			;Restore $a6
chrout_v       	xba
		lda help_x_exp
		tax
		xba			;Set NZ flags
		.else
		pla
		ldx #0
		sta ($a4,x)
chrout_v       	ldx help_x_exp
		ora #$00		;Set NZ flags
		.fi
exit2		jmp exit2basic
;---------------------------------
chrout_nxt_bank .if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		sta $a4
                .fi
		.if SUPERCPU=1
		lda buffer_address_hi_exp,x
		inc a
		sta buffer_address_hi_exp,x
		.else
		isb buffer_address_hi_exp,x
		eor #$ff
		.fi
		cmp buffer_address_end_hi_exp,x
		bcc chrout_bnk_nen2
		ora #$80
		sta buffer_address_hi_exp,x;Mark next bank

chrout_data_sector_last
		and #$7f
		sta $a5
		lda $a4
		cmp buffer_address_end_low_exp,x
		bcc chrout_bnk_nend
		gcs chrout_next

newconsoljmp	jmp newconsole

;BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, BLK_READ, BLK_RW, KERNAL, IGNORE

chrout_not_data_sector = *
		cpx #STDOUT
		beq newconsoljmp

		.if SUPERCPU=1
                sec
                rol
                .else
		lda buffer_info_exp,x
                .fi
		cmp #INFO_KERNAL
		bne nchrout_stand

		lda buffer_drive_exp,x
		cmp #4
                .if SUPERCPU=1
                sta ActualOutput
		#ldx_l_a help_x_exp
                .fi
		bcc vew			;serial device?

                .if JIFFYDOS_ACCEL_CHRINOUT=1
                .if SUPERCPU=0
                ldx help_x_exp
                .fi
		#jsr_OP_cfg16up
                #assert_cfg_16
		pla
		jsr iecout
                #jsr_OP_cfg16down
                #assert_cfg_open
                lda SerialOutBuf
                .if SUPERCPU=0
                jmp exit2basic
                .else
                bra exit2
                .fi
                .else         	;JIFFYDOS_ACCEL_CHRINOUT

                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                lda dos_accel_internal_exp
                beq nx
                .if SUPERCPU=0
                ldx help_x_exp
                .fi
		#jsr_OP_cfg16up
                #assert_cfg_16
		pla
		jsr iecout
                #jsr_OP_cfg16down
                #assert_cfg_open
                lda SerialOutBuf
                .if SUPERCPU=0
                jmp exit2basic
                .else
                bra exit2
                .fi
                .fi

nx              pla
		.if SUPERCPU=0
                tax
                .fi
		#psr $eddd
		.if SUPERCPU=0
                txa
                ldx help_x_exp
		jmp exit2basic
                .else
                bra exit2
                .fi

                .fi		;JIFFYDOS ACCEL CHRINOUT

vew		.if SUPERCPU=0
		sta ActualOutput
                .fi
		#psr $f1cb
                .if SUPERCPU=0
		jmp exit2basic
                .else
                bra exit2
                .fi

;BLK_R_EOF, BLK_RW_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_WRITE, BLK_READ, BLK_RW, IGNORE

nchrout_stand
                cmp #INFO_BLK_WRITE
                beq markdirt
                cmp #INFO_BLK_RW
                beq markdirt
                cmp #INFO_BLK_RW_EOF
                beq markdirt
                cmp #INFO_RELATIVE_WRITE_DIRTY_FULL
                beq rfull
                and #<~INFO_RELDIRTY
                cmp #INFO_RELATIVE
                beq chrout_relative

nchrout_err	lda #$83		;BLK_R_EOF, BLK_RW_EOF_FULL, BLK_WRITE_FULL, BLK_READ, IGNORE
		sta Status
		pla
		clc
		gcc chrout_v

markdirt        ;sec
		adc #INFO_BLK_WRITE_DIRTY-INFO_BLK_WRITE-1
                .cerror (INFO_BLK_WRITE_DIRTY-INFO_BLK_WRITE)!=(INFO_BLK_RW_DIRTY-INFO_BLK_RW)
                .cerror (INFO_BLK_RW_DIRTY-INFO_BLK_RW)!=(INFO_BLK_RW_EOF_DIRTY-INFO_BLK_RW_EOF)
dback2		sta buffer_info_exp,x
		gcc dback

rfull           #phy_a
                lda #$80+51             ;overflow in record hack
		jsr set_rel_error
                #ply_a
                gcs nchrout_err
;---------------------------------------
chrout_relative #phx_a
		jsr OP_save_zeropage
                .if SUPERCPU=1
                plx
                phx
                .else
                pla
                tax
                pha
                .fi
                lda #128
		jsr search_end_of_record
                jsr OP_restore_zeropage
                #plx_a
                lda #INFO_RELATIVE_WRITE_DIRTY
                clc
                gcc dback2
;---------------------------------------
chrout_next	#stx_l_a buffer_currentp_exp
		#phy_a
		jsr OP_save_zeropage_scpu01up

		jsr XIT_do_next

		jsr OP_restore_zeropage_scpu01down
		#ply_a
		#ldx_l_a buffer_currentp_exp
		.if SUPERCPU=0
		lda #0
		.fi
		bcc hard_chrout
                .if SUPERCPU=1
                pla
                sta $a6
                .fi
		gcs nchrout_err
;---------------------------------------
XIT_do_next	#xit_init

		#ldx_l_a buffer_currentp_exp
		lda buffer_type_exp,x
                .if BUFFERMODE_ERROR
                cmp #BUFFERMODE_ERROR
                .fi
                beq XIT_do_errnext
		cmp #BUFFERMODE_DIRECT
		beq XIT_do_direct
		cmp #BUFFERMODE_RELATIVE
		beq XIT_do_relative
		cmp #BUFFERMODE_FILE
		beq XIT_do_filenext
		cmp #BUFFERMODE_PCLINK
		beq XIT_do_pclink
		cmp #BUFFERMODE_LINK
		bne unknown

XIT_do_link     jsr fileop_prepare       ;be safe
                pha
		lda buffer_address_end_hi_exp,x
		beq +
                pla
                lda #INFO_BLK_WRITE_FULL        ;buffer overflow...
                pla
+		jmp chrin_chrout_do_file_end

XIT_do_direct	jsr OP_init_buffer_pointers
yy              pla
                pla
                jsr OP_restore_zeropage_scpu01down
		#ply_a
                clc
		gcc chrout_bnk_nend

XIT_do_pclink
;                .if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
;                php                    ;not needed, one byte received only
;                sei
;                .fi
                jsr fileop_prepare       ;be safe
                pha

		lda buffer_address_end_hi_exp,x
		beq vdc
                pla
                pha
                cmp #INFO_STATUS
                beq vdc			;overflow of command buffer?!

		sec
                ldx #<$0800
                ldy #>$0800
		jsr pclink_chrinout
                beq vdc
                pla
;                .if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
;                plp                    ;not needed, one byte received only
;                .fi
unknown		sec
                rts			;problem while send
vdc
;                .if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
;                pla
;                plp			;not needed, one byte received only
;                pha
;                .fi
		jsr OP_init_buffer_pointers
		#jmps chrin_chrout_do_file_end.more

XIT_do_errnext	lda #254
		sta buffer_address_low_exp,x
yy2             gne yy
;---------------------------------------
;INFO_BLK_WRITE_DIRTY, INFO_BLK_RW_DIRTY, INFO_BLK_RW_EOF_DIRTY
;
XIT_do_relative	jsr fileop_prepare       ;be safe

		lda buffer_address_end_hi_exp,x
		beq dck
		lda $a5			;buffer_address_hi_exp,x
		eor buffer_address_max_hi_exp,x
		beq dck2       		;eof mark?

                lda #INFO_RELATIVE_WRITE_DIRTY_FULL
                sta buffer_info_exp,x
                gne yy2
                .cerror !INFO_RELATIVE_WRITE_DIRTY_FULL

dck2		jsr XIT_flush_buf

                jsr incfileposhi512

dck             jsr XIT_save_tree	;-=> error

                jsr XIT_relative_read_in_buffer;-=> error

                lda #129
                jsr search_end_of_record

                lda #INFO_RELATIVE_WRITE_DIRTY
	     	gne chrin_chrout_do_file_end.be2
                .cerror !INFO_RELATIVE_WRITE_DIRTY

XIT_do_filenext	jsr fileop_prepare       ;be safe
                pha

		lda buffer_address_end_hi_exp,x
		beq dc
		lda $a5			;buffer_address_hi_exp,x
		cmp buffer_address_max_hi_exp,x
		beq dc2			;eof mark?
                ldy $a4
                cpy #1
                sbc #0
                sta buffer_address_hi_exp,x
                dey
		tya
                sta buffer_address_low_exp,x

		pla
		lda #INFO_BLK_RW_EOF_DIRTY
		pha
		#jmps chrin_chrout_do_file_end.more

dc2		jsr XIT_flush_buf

                jsr incfileposhi512

dc		jsr XIT_save_tree	;-=> error

		clc			;no clean
		jsr XIT_read_in_buffer	;-=> error

		#jmps chrin_chrout_do_file_end
		.pend

XIT_flush_buf	.proc
		jsr buffer_tree_pom1_y

                lda buffer_type_exp,x
                eor #BUFFERMODE_LINK
                pha
                beq go          	;force buffer fill for links

		#lda_zp_y pom1
		and #%01001111
		ldy #3
-		#ora_zp_y pom1
		bne allre		;is it already allocated?
                dey
                bne -

                pla
                pha
                eor #BUFFERMODE_RELATIVE ^ BUFFERMODE_LINK
                beq aad                 ;no buffer fill for relative files

		tya
go              sec
		adc buffer_address_low_exp,x
		tay
                #lda0
		sta pom1
		adc buffer_address_hi_exp,x
		and #$7f
                cmp buffer_address_max_hi_exp,x
                bcs aad
		sta pom2
                lsr

		#lda0			;fill with 0
                jsr fill_pom1
                bcs aad
                jsr fill_pom1_inc

aad             pla
                pha
                beq allre               ;no allocation for links

		jsr restore_file_state

                .if FIND_FREE_DATA
                lda #FIND_FREE_DATA
                .else
                #lda0
                .fi
		jsr find_free_space	;allocate data sector

                jsr save_file_state

		jsr buffer_tree_pom1_y
                lda #$80		;dirty
                sta buffer_treestate_exp,x

		ldy #3
-		lda next_sector,y
		#sta_zp_y pom1
		dey
		bne -
		#lda_zp_y pom1		;fill in
		and #%10110000
		ora next_sector
		#sta_zp_y pom1

allre           jsr update_filelen

                pla
		bne XIT_write_out_buffer

		jsr OP_buffer_start_pom1_y
                lda #>b_cfs_link_data_exp
                jsr buffer_subpart_cp
-               #lda_zp_y pom1
                #sta_zp_y current_path
                iny
                bne -
                inc pom1+1
                inc current_path+1
                lda pom1+1
                lsr
                bcs -
                rts
		.pend

fill_pom1_inc   .proc
		inc pom2
fill_pom1       .proc
-		#sta_zp_y pom1
		iny
		bne -
		rts
                .pend
