;
; clrchn.asm - CLRCHN, CLALL kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Clall & Clrchn
;
; Used registers: A X (Y preserved, A=0!!!)
; Status: not useful
; Status byte response:
; ST=$00 if no error
; ST=$03 timeout
; ST=$80 if device not present
;**************************************************
		#assert_cfg_open
newclall	.proc
                ldx #2
-               lda ActualLogical,x
                pha
                dex
                bpl -
		lda #MAX_OPEN_FILES-1
-               pha
                tax
		lda open_files_logical_exp,x
                beq +
                jsr search_buffer_A_readfiletables
                bne +
                jsr newclose.close_ide64  ;close IDE64 files
+               pla
                #sbc1c
		bpl -
                #lda0
		ldx #MAX_OPEN_FILES-1
-		sta open_files_logical_exp,x;All files closed
		cpx #MAX_BUFFERS
                bcs +
                sta buffer_logical_exp,x;All buffers free
+		dex
		bpl -
                ldx #$7d
-               pla
		sta ActualLogical-$7d,x
                inx
                bpl -

		#assert_cfg_open
newclrchn	.proc
		.if SUPERCPU=0
		tsx
                inx
                lda $100,x
                cmp #$ce
                bne xx
                inx
                lda $100,x		;Yet another nice JiffyDOS hack
                cmp #$f2		;to make CONTROL+D work ;)
                bne xx
                #psr newclose+1
                php
                lda #$6f
                pha
                #psr nmi_return
                .fi

xx		#ldatax buffer_output_exp
		sta buffer_currentp_exp	;OP_do_command_channel
		lda buffer_info_exp,x
                jsr check_rel
		cmp #INFO_STATUS
		bne clrchn_restore.clrchn_ano1
                jsr chkoutclrchncmdchannel
		#jmps clrchn_restore.clrchn_ano2

clrchn_restore	.proc
		#psr switch_16k
                .if SUPERCPU=1
		lda buffer_input_exp
                .else
		ldx buffer_input_exp
                .fi
                jsr kill_channel
                .if SUPERCPU=1
		lda buffer_output_exp
                .else
                ldx buffer_output_exp
                .fi
                jsr kill_channel
		lda buffer_info_exp,x
clrchn_ano1	cmp #INFO_KERNAL
		bne clrchn_ano2
		lda #3
		cmp buffer_drive_exp,x
		bcs clrchn_ano2

		lda #STDOUT
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
                jsr chkout_doch2
		.else
		sta buffer_output_exp	;to fix unlisten loop
                .fi

                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                lda dos_accel_internal_exp
                cmp #1
                .fi

		#jsr_OP_cfg16up
                #assert_cfg_16
                .if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr iecunlisten
                .else

                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                bcs nx
                jsr $edfe
                #jmps nx2
nx              jsr iecunlisten
nx2
                .else
                jsr $edfe
                .fi

                .fi		;JIFFYDOS ACCEL CHRINOUT

		#jsr_OP_cfg16down
                #assert_cfg_open

clrchn_ano2	lda #STDOUT
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
                jsr chkout_doch2
		.else
		sta buffer_output_exp
                .fi

		#stz_a ActualInput
		lda #3
		sta ActualOutput

		#ldx_l_a buffer_input_exp
		lda buffer_info_exp,x
		cmp #INFO_KERNAL
		bne clrchn_ano3
		lda #3
		cmp buffer_drive_exp,x
		bcs clrchn_ano3x

		lda #STDIN
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
                jsr chkin_doch2
		.else
		sta buffer_input_exp	;to fix untalk loop
                .fi

                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                lda dos_accel_internal_exp
                cmp #1
                .fi

		#jsr_OP_cfg16up
                #assert_cfg_16
                .if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr iecuntalk
                .else

                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                bcs nx3
                jsr $edef
                #jmps nx4
nx3             jsr iecuntalk
nx4
                .else
                jsr $edef
                .fi

                .fi		;JIFFYDOS ACCEL CHRINOUT

		#jsr_OP_cfg16down
                #assert_cfg_open
clrchn_ano3	clc
clrchn_ano3x
		lda #STDIN
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
                php
                jsr chkin_doch2
                plp
		.else
		sta buffer_input_exp
                .fi

		#lda0			;Lame kernal (see $f6f4...)
		jmp exit2basic

kill_channel
		.if SUPERCPU=1
                tax
                .fi
		cpx #MAX_BUFFERS
                bge +
                lda #INFO_IGNORE
                sta buffer_info_exp,x
+               rts
		.pend

check_rel       .proc
                cmp #INFO_RELATIVE_WRITE_DIRTY_FULL
                beq +
		cmp #INFO_RELATIVE_WRITE_DIRTY
                bne un
+
                jsr OP_save_zeropage
                #ldx_l_a buffer_currentp_exp
                jsr rel1
                jsr OP_restore_zeropage

rel2            #ldx_l_a buffer_currentp_exp
                lda #INFO_RELATIVE_DIRTY
                sta buffer_info_exp,x
un              rts

rel1		lda #64
		jsr search_end_of_record
                #jmps position_to_next_record
                .pend

check_rel2      .proc
                cmp #INFO_RELATIVE_WRITE_DIRTY_FULL
                beq +
                cmp #INFO_RELATIVE_WRITE_DIRTY
                bne check_rel.un
+               jsr check_rel.rel1
                #jmps check_rel.rel2
                .pend

chkoutclrchncmdchannel .proc
                #phy_a
		jsr OP_save_zeropage_scpu01up
                jsr XIT_commandchnl
		#OP_scpu01down
		#ply_a
		#jmps OP_restore_zeropage
                .pend

XIT_commandchnl .proc
		#ldx_l_a buffer_currentp_exp
in              lda buffer_type_exp,x
                cmp #BUFFERMODE_PCLINK
		bne OP_XIT_do_command_channel;IDE64 command_channel?
;                .if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
;                php			;not needed, one byte received only
;                sei
;                .fi
		jsr flush_pclink
;                .if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
;                plp                    ;not needed, one byte received only
;                .fi
		#jmps OP_init_buffer_pointers
		.pend
