;
; dir.asm - directory handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
		#assert_cfg_open

XIT_dir_first	.proc
		lsr
		sta direntry_first_exp
                .if SUPERCPU=1
                ror
                sta direntry_raw_exp
                .else
                ror direntry_raw_exp
                .fi

                .if SUPERCPU=1
                tdc
                sta direntry_datemode_exp
                sta direntry_fat_dirsectors_exp
                dec a
                sta direntry_skip_exp
                sta direntry_longmode_exp
                .else
                ldx #0
                stx direntry_datemode_exp
                stx direntry_fat_dirsectors_exp
                dex
                stx direntry_skip_exp
                stx direntry_longmode_exp
                .fi

specloop       	jsr OP_short_path
		cmp #"="
		bne nospecial
		jsr OP_short_path
		eor #"p"
		beq partlist
                cmp #"p"^"t"
                bne nospecial
		sta direntry_datemode_exp
                jsr OP_short_path
                #jmps nospecial

partlist        sta direntry_partnum_exp
		jsr OP_short_path
                lda #RESOLV_PARTITION_MODE
                jsr OP_XIT_resolv_path_part_init
		jsr OP_get_current_drive_type
                asl
		ldy #0
		sty next_sector
		sty next_sector+1
		sty next_sector+2
		bcs +			;LBA?
                iny
+		sty next_sector+3

		and #FS_MASK << 1
		eor #FS_CFS << 1
		cmp #1
		jsr LWOP_reporterror_71_xit;-=> partition listing only for CFS!

		lda #DIRMODE_PARTITION_START
		sta direntry_mode_exp
		lda #255
		gne cke

nospecial	lda #RESOLV_DIR_MODE
		jsr OP_XIT_resolv_path_part_init;-=> error	DIR
		lda path_partition_exp
		bne cke
                .if SUPERCPU=1
		lda current_drive_exp
                tax
		lda drive_workpartition_exp,x
                inc a
                .else
                ldx current_drive_exp
                sec
                adc drive_workpartition_exp,x
                .fi
cke		sta direntry_partition_exp
                lda setup_atribut2_exp
                and #CMOS_BLOCKSFREE
                beq +
                lda #255
+		sta direntry_used_exp
		sta direntry_used_exp+1

		lda path_exp
		eor #"*"
		ora path_exp+1
                cmp #1
		lda path_searchtype_exp
		eor #"*"
		ora path_searchtype_exp+1
		beq +
		lda #2			;no type wildcard
+               ror
		sta direntry_wildcard_exp
		rts
		.pend
;------------
XIT_dirlp2	.proc
		sta direntry_limit_exp

		ldx #31+2
		lda direntry_datemode_exp
                beq mass
                ldx #41+2
                lda direntry_longmode_exp
                bne mass
                ldx #63+2
mass		#stx_l_a direntry_spc_exp

		.if SUPERCPU=1
		tsc
                .else
		tsx
		txa
                .fi
                sec
		sbc direntry_spc_exp
		sta direntry_spc_exp

                .cerror (DIRMODE_FAT | 1)!=DIRMODE_FAT2
		lda direntry_mode_exp
                lsr                     ;mask DIRMODE_FAT2
                cmp #DIRMODE_FAT >> 1
                bne nonfat
	        lda direntry_fat_dirsectors_exp
		jsr cluster2sector
nonfat
		jsr OP_read_cmd1
		jsr LWOP_reporterror_read_xit;-=> error

                ldx #2
-               lda direntry_backup_start_exp,x
		sta direntry_tmp_exp,x
                dex
                bpl -

		#lda0
                sta direntry_here_exp
                sta direntry_skip2_exp
		lda #15			;16 directory entrys
		sta direntry_number_exp
dirlp		lda direntry_skip_exp
		bmi nu
                php
                .if SUPERCPU=1
                dec a
                sta direntry_skip_exp
                .else
                dec direntry_skip_exp
                .fi
                lda start_zeropage+4
                ora #$20
                plp
                bne +
                and #$df
+               sta start_zeropage+4

nu		jsr dirnotlast

		#inc_l_a direntry_here_exp

                lda start_zeropage+4
                #andclc $20
                bne jw
		txa
                beq jw
		adc start_zeropage+5
                sta start_zeropage+5
		bcc jw
		inc start_zeropage+6
                lda start_zeropage+6
                cmp direntry_limit_exp
                bcc jw
                lda direntry_number_exp
                beq jw
                lda direntry_here_exp
                sta direntry_skip2_exp
                lda #$7f
                sta direntry_skip_exp
jw
		#dec_l_a direntry_number_exp
		bpl dirlp

		jsr wait_busy;jsr XIT_nomoreread	;-=> error
                jsr LWOP_reporterror_read_xit

                lda start_zeropage+4
                and #$20
                beq kok
                ldx #2
-               lda direntry_tmp_exp,x
		sta direntry_backup_start_exp,x
                dex
                bpl -
kok
		jsr next_sector0
		bne bzb

		jsr summary		;If buffer overflows here, that's no problem
		txa			;because the rest of the buffer won't be used
		clc			;as it's end of dir!
		adc start_zeropage+5
		sta start_zeropage+5
		bcc bzb
		inc start_zeropage+6
bzb     	rts
;-------------------------
summary 	ldx #255
		#stx_l_a direntry_first_exp	;end of directory
                inx

		lda direntry_raw_exp
                bmi bzb

		lda direntry_mode_exp
                bmi bzb
		eor #DIRMODE_PARTITION_END
		cmp #1
                bcs ce

                lda setup_atribut2_exp
                asl
                bcc ce
                #lda0
                pha
                pha
                ldy #1
                ldx #2
		jmp start_zeropage
ce

		.if SUPERCPU=1
                pea $0101
                .else
		lda #1
		pha			;1
		pha			;2
                .fi
ce2
		bcs se
                lda direntry_partnum_exp
                pha
                #lda0
                gcc se2

se
                lda setup_atribut2_exp
                and #CMOS_BLOCKSFREE
                adc #$7e                ;set v
                sec
		lda direntry_used_exp
		pha			;3
		lda direntry_used_exp+1
                bvc se2
                lsr
se2		pha			;4
  		ldx #12
dfsvcxc         lda prtused_text,x
                bcc vd
		lda blkfree_text,x	;17
                bvs vd
		lda blkused_text,x	;17
vd		pha
		dex
		bpl dfsvcxc
		ldx #12
gv		pha			;29
		dex
		bne gv
                .if SUPERCPU=1
                phx
                phx
                phx
                .else
		txa
		pha			;30
		pha			;31
		pha			;32
                .fi
		ldy #31
                ldx #32
		jmp start_zeropage
;----------------------------
dirnotlast	jsr OP_read_to_direntry

		lda direntry_first_exp
		beq notdirlabel

;		lda direntry_first_exp
		ldy #29
		lsr
		beq hnuhi		;load, skip $0401
					;lda #1
		pha			;1
		lda #4
		pha			;2
		ldy #31

hnuhi		#ldatax direntry_attrib_exp
		and #FTYPE_MASK | FATTR_CLOSED
		eor #FTYPE_DIR
                beq tip
		jsr flushdir
		sec
		gcs LWOP_reporterror_71_xit;-=> directory error

tip             sta direntry_first_exp

		txa
		and #FATTR_READ
                bne jo
                cpy #31
                bne ke
                pla
                pla
ke              jsr flushdir
		sec
                gcs LWOP_reporterror_27_xit;-=> no access
jo

                lda direntry_raw_exp
		bmi raw

		.if SUPERCPU=1
                pea $0101
                .else
		lda #1
		pha			;3
		pha			;4
                .fi

		lda direntry_partition_exp
		pha			;5 currentpartition number
		#lda0
		pha			;6
		#psr $1223		;7,8 rvson"
		ldx #240

dcb		lda direntry_name_exp-240,x
		pha			;24
		inx
		bne dcb
		ldx #7
dfsvcx		lda dirlabel_text,x
		pha			;32
		dex
		bpl dfsvcx
                #tyx_a
		inx
		jmp start_zeropage

raw		pla
		pla
                lda #"i"
                pha
                ldy #32
                .byte SKIP2
raw2		ldy #31
		ldx #$e0
luq     	lda direntry_exp-$e0,x
                pha
                inx
                bne luq
                #tyx_a
                inx
		jmp start_zeropage
;----------------------------
nomatch2	#dec_l_a direntry_partnum_exp
nomatch		ldx #0
		rts
;----------------------------
notdirlabel	lda direntry_raw_exp
		bmi raw2
		lda direntry_hidden_exp
		bmi nomatch		;hidden entry...
		lda direntry_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
		beq nomatch		;free entry...
		and #FTYPE_MASK
		eor #FTYPE_RELATIVE
		bne uie			;to report correct filesize
		sta direntry_recordsize_exp;for relative files...
uie		lda direntry_wildcard_exp
		bpl dirskip1
                pha
		jsr check_wildcards
                pla
		bcs nomatch2

dirskip1	lsr
		bcc dirskip2

		inc current_path+1
		jsr check_wildtype
		dec current_path+1
		bcs nomatch2

dirskip2	.if SUPERCPU=1
                pea $0101
                .else
		lda #1
		pha			;1
		pha			;2
                .fi

		.if DIRECTORY_BLOCKSIZE_256
		lda #255
		adc direntry_size_exp
		#lda0
		adc direntry_size_exp+1
		tay
		#lda0
		adc direntry_size_exp+2
		tax
		#lda0
		adc direntry_size_exp+3
		beq +
		ldx #255
                #txy_a
+		tya
                pha                     ;3
		.else
		lda #<511
		adc direntry_size_exp
		lda #>511
		adc direntry_size_exp+1
		tay
		#lda0
		adc direntry_size_exp+2
		tax
		#lda0
		adc direntry_size_exp+3
		lsr a
		beq +
		ldx #255
                #txy_a
+		txa
		ror a
		tax
		tya
		ror a
		pha			;3
		tay
		.fi
		clc
		adc direntry_used_exp
		sta direntry_used_exp
		txa
		adc direntry_used_exp+1
		bcc noovf		;In case of overflow 65535
		lda #255
		sta direntry_used_exp
noovf		sta direntry_used_exp+1

		txa
		pha			;4
		cpy #<10
		sbc #>10
		bcc sds
		txa
		cpy #<100
		sbc #>100
		bcc sds2
		txa
		cpy #<1000
		sbc #>1000
		bcc sds3
		gcs sds4

sds		lda #$20
		pha			;x
sds2		lda #$20
		pha			;x
sds3		lda #$20
		pha			;x

sds4    	#lda0
                sta direntry_name_exp+16
		lda #$22
		ldx #238
ncvbx		pha
		inx
		lda direntry_name_exp-239,x
		bne ncvbx
		lda #$22
		pha
                inx
                geq nnv
		lda #$20
ncvbx4		pha			;x
		inx
		bmi ncvbx4
nnv
		.if SUPERCPU=1
		ldx #$2a		;*
		lda direntry_attrib_exp	;closed?
		tay
		bpl +
		ldx #$20
+		phx			;x
                .else
                lda #$2a		;*
                ldy direntry_attrib_exp
                bpl +
		lda #$20
+		pha			;x
                .fi

                tya
		and #%01010000
		cmp #%01010000		;Deleteable & writeable?

		ldx #253
cbe		lda direntry_filetype_exp-253,x
		beq dirdata
		pha			;x
		inx
		bne cbe
                lda #$20
                gne js
dirdata 	lda #$20
hdh     	pha
                inx
                bne hdh
js      	bcs +
                lda #"<"
+               pha

		lda direntry_datemode_exp
                bne datemode

		ldy #31
		lda #$20
direxit
		.if SUPERCPU=1
                tax
                .byte SKIP1
jr              phx
		tsc
		eor direntry_spc_exp
		bne jr
		.else
                .byte SKIP1
jr              pha
		tsx
		cpx direntry_spc_exp
		bne jr
		lda #0
		.fi
		pha			;x EOL
                #tyx_a
                inx
		jmp start_zeropage

xe	     	jsr OP_read_to_direntry
flushdir        #dec_l_a direntry_number_exp
		bpl xe
		rts

datemode	lda direntry_longmode_exp
		bne mas3
                lda #$20
		pha
                gne mas4

mas3		pla
		pla
                pla
                pla
		tya
		and #%00000111
                tay
                lda type,y
                bne bd
                ldy #9
jdj2            ldx #2
                clc
jdj1            lda direntry_filetype_exp,x
                eor type2,y
                beq +
                sec
+               dey
                dex
                bpl jdj1
                bcc megvan
                tya
                bne jdj2
                dey
megvan          iny
		lda type2,y
bd              pha
		lda #$20
mas4		pha

                ldx #0
                jsr gd
                lsr
                jsr conv
                .if SUPERCPU=1
                phx
                pha
                .else
                pha
                tya
                pha
                .fi
                lda #"/"
                pha
                lda direntry_date_exp+3
                and #31
                jsr conv
                .if SUPERCPU=1
                phx
                pha
                .else
                pha
                tya
                pha
                .fi

                lda direntry_longmode_exp
                bne mas
                lda #"/"
                pha
                lda direntry_date_exp+2
                and #63
                sec
                sbc #20
                bcs +
                adc #100
+               jsr conv
                .if SUPERCPU=1
                phx
                pha
                pea $2020
                .else
                pha
                tya
                pha
		lda #" "
                pha
                pha
                .fi

mas		lda #" "
                pha

		jsr hour
                jsr conv
                .if SUPERCPU=1
                phx
                pha
                .else
                pha
                tya
                pha
                .fi
                lda #"."
                pha
                lda direntry_date_exp+1
                and #63
                jsr conv
                .if SUPERCPU=1
                phx
                pha
                .else
                pha
                tya
                pha
                .fi
                lda #" "
                pha
                jsr hour
                lda #"a"
                bcc +
                lda #"p"
+               pha
                ldy #41
                lda direntry_longmode_exp
                bne mas2
                lda #"m"
                pha
                ldy #63
mas2
                lda #1
		gne direxit

hour            ldx #2
                jsr gd
                cmp #12
                php
                tax
                bne +
                lda #12
+               cmp #13
                bcc xq
                sbc #12
xq              plp
		rts

gd              lda direntry_date_exp,x
                tay
                asl
                asl
                lda direntry_date_exp+1,x
                ror
                cpy #$80
                ror
                lsr
                lsr
                lsr
                rts

conv            sec
                ldx #47
ss              inx
                sbc #10
                bcs ss
                adc #58
                .if SUPERCPU=0
                tay
                txa
                .fi
                rts

type            .text "-",0,"rdl??"
type2		.text "?","usr","seq","prg"

dirlabel_text	.text $00,'46edi "'	;" IDE64
blkfree_text	.text " .eerf skcolb"
blkused_text	.text " .desu skcolb"
prtused_text	.text "  .snoititrap"
		.pend

;---------------------------------------
;Read 32 bytes to direntry_exp C=1 if error
;---------------------------------------
o_sys		= 11

OP_read_to_direntry .proc
		.if SUPERCPU=1
		lda direntry_mode_exp
                .else
		ldx direntry_mode_exp
                .fi
		bne dirp_ofs
;-----------------
;CFS filesystem is the simplest
;
dirp_cfs	.proc
		ldx #224
cb2		lda data_g		;DATA PORT
		#lda_low_reg		;0-7 BIT
		sta direntry_exp-224,x
		inx
		lda hig_reg		;8-15 BIT
		sta direntry_exp-224,x
		inx
		bne cb2
                lda direntry_number_exp
                lsr
                lsr
                tax
		lda direntry_nexts_exp
		asl
		asl
		asl
		rol next_sector,x
		asl
		rol next_sector,x
		rts
		.pend
;-----------------
;v0.02 filesystem
;
dirp_ofs        .proc
		#dec_xa
		bne dirp_partition
		lda direntry_number_exp
		bne kqj
		sta direntry_attrib_exp

		ldx #12			;Skip until end of sector
skip		bit data_g
		dex
		bne skip
		rts

t1		.text "dspurcdu",0,"cns"     ;SYS, NAT, 41, 71, 81, C81, PRN, FOR
t2		.text "eersebin",0,"fay"
t3		.text "lqgrlmrk",0,"sts"
t4		.byte FTYPE_DEL,FTYPE_NORMAL,FATTR_EXEC | FTYPE_NORMAL,FTYPE_NORMAL
		.byte FTYPE_NORMAL,FTYPE_NORMAL,FATTR_EXEC | FTYPE_DIR,FTYPE_NORMAL

kqj		eor #15
		bne notfirst		;Not first
		tax

		lda data_g		;($00)
                #lda_low_reg
                asl
                bmi last
                dex
		ror
                and #%00111111
                .byte SKIP1BMI
last		txa
		sta next_sector+1
                txa
		and hig_reg
		sta next_sector+2
                txa
                .if FAST_DATA_TRANSFERS
                and data_g
                .else
		bit data_g		;($02)
		and low_reg
                .fi
		and #15
		sta next_sector
                txa
		and hig_reg
		sta next_sector+3

		ldx #2
		jsr skip		;($06)
notfirst
		.if FAST_DATA_TRANSFERS
                #ldatax data_g          ;Attrib ($00)
                .else
		lda data_g
		#ldatax low_reg
                .fi
		and #3
		tay
		txa
		and #%00010000
		beq +			;DIR?
		ldy #6
+               jsr fillftype
		txa
		and #%11000000
		eor #%01100000
		asl
		bpl +			;Write protect?
		ora #%00100000
+		ror
		ora t4,y
		cpy #6			;DIR?
		bne ded3
		cpx #%10000000
		bcs ded3
		#lda0			;*DIR????
ded3
                cpx #$f0
                bcc +
                and #%01111111
+		sta direntry_attrib_exp

		jsr getname

		ldx #15
ktt		lda direntry_name_exp,x
		eor #$a0
		bne tre
                bcc +
                lda #$20
+		sta direntry_name_exp,x
		dex
		bpl ktt
tre
		ldx #direntry_size_exp-direntry_exp
		jsr getword		;($12)
		jsr getnextword		;($14)
		ldx #3
		jsr skip
                ldx #direntry_pointer_exp+1-direntry_exp
                jsr getword             ;($1c)
		lda data_g		;($1e)
		#lda_low_reg
		and #15
		sta direntry_pointer_exp
		lda hig_reg
		sta direntry_pointer_exp+3
		rts

onetype
                lda direntry_filetype_exp+1
                bne rrt2
                lda direntry_filetype_exp
                beq rrt2
                ldy #6
-               dey
                bmi rrt
                cmp t1,y
                bne -

fillftype	lda t3,y
		sta direntry_filetype_exp+2
		lda t2,y
		sta direntry_filetype_exp+1
		lda t1,y
                .byte SKIPLDA0
rrt             #lda0
                sta direntry_filetype_exp       ;one character type!?
rrt2            rts

getnextword	inx
		inx
getword		lda data_g		;($12)
		#lda_low_reg
		sta direntry_exp,x
		lda hig_reg
		sta direntry_exp+1,x
		rts

getname		ldx #240
cb23		lda data_g		;Get name ($02)
		#lda_low_reg
		sta direntry_name_exp-240,x
		inx
		lda hig_reg
		sta direntry_name_exp-240,x
		inx
		bne cb23
		rts
                .pend
;-----------------
;Partition table
;
dirp_partition	.proc
		#dec_xa
		bne partition2
                .if SUPERCPU=1
		sta direntry_attrib_exp
                .else
		stx direntry_attrib_exp
                .fi
		lda direntry_number_exp
		bne be
		lda #DIRMODE_PARTITION_END
		sta direntry_mode_exp
en2     	rts

be		eor #15
		bne en
                ldx #2
		bit data_g
		bit data_g
		lda hig_reg
		sta direntry_defpart_exp
		ldx #10
		jsr dirp_ofs.skip
		ldx #0
		jsr getnextsector
		ldx #2
		jsr getnextsector
;               ldx #2
		jsr dirp_ofs.skip
		jsr dirp_ofs.getname
		lda #FTYPE_DIR|FATTR_READ
		sta direntry_attrib_exp
		ldx #232
		gne dirp_ofs.skip

en		eor #1
		bne en2
                lda setup_atribut2_exp
                bpl en2
                ldx #3
                #lda0
lk              sta direntry_size_exp,x
                dex
                bpl lk
                sta direntry_pointer_exp
                lda #FATTR_CLOSED | FTYPE_NORMAL
                sta direntry_attrib_exp
                ldy #o_sys
                jsr dirp_ofs.fillftype
                ldx #6
lk2             lda koo2,x
                sta direntry_name_exp,x
                dex
                bpl lk2
                rts
koo2		.null "system"

getnextsector	lda data_g
		#lda_low_reg
		sta next_sector,x
		lda hig_reg
		sta next_sector+1,x
		rts

partition2     	#dec_xa
		bne isodir		;isodir
                .if SUPERCPU=0
		lda #0
                .fi
		ldx #3
vkf		sta next_sector,x
		sta direntry_size_exp,x
		dex
		bpl vkf
		sta direntry_pointer_exp
		sta direntry_filetype_exp
		lda #16
		sec
		sbc direntry_number_exp
		.if DIRECTORY_BLOCKSIZE_256=0
		asl
		.fi
		sta direntry_size_exp+1
		jsr dirp_ofs.getname
		lda direntry_number_exp
		eor #15
		eor direntry_defpart_exp
		cmp #1
		lda data_g
		#lda_low_reg
		bpl nn			;not valid
		and #%00110000
		beq nwritej
		and #%00100000
		bne nn			;hidden
		lda #FATTR_WRITE << 1
nwritej		ora #(FTYPE_NORMAL|FATTR_DELETE|FATTR_READ|FATTR_EXEC) << 1
		ror
                #inc_l_ direntry_partnum_exp
		.byte SKIPLDA0
nn		#lda0
		sta direntry_attrib_exp
		bit data_g
		lda data_g
		#lda_low_reg
		ldy #8
		and #%10110000
		beq bd			;unformatted
		cmp #%00010000
		beq bd2			;cfs
		dey			;unknown
		gne bd
bd2		iny
		lda setup_atribut2_exp
                bpl bd
                iny			;nat
bd		jsr dirp_ofs.fillftype
		ldx #5
		gne dirp_ofs.skip
;-----------------
;ISO directory
;
isodir  	#dec_xa
                beq isodir2
                #dec_xa
                beq isodir2
	        jsr fatdirectory
                #jmps dirp_ofs.onetype
isodir2
		lda direntry_number_exp
                eor #15
                bne ny
                sta direntry_iso_bytes_exp
                sta direntry_iso_bytes_exp+1

ny		lda data_g
                #lda_low_reg
                bne ny2

                clc
		sbc direntry_iso_bytes_exp
                tax
                beq nxp
                jsr dirp_ofs.skip
nxp             lda #>1024
                sbc direntry_iso_bytes_exp+1
                tay
                beq nxp2
-               jsr dirp_ofs.skip
                dey
                bne -
nxp2		tya
		sta direntry_number_exp
		sta direntry_attrib_exp
                lda direntry_skip2_exp
                beq jono
                sta direntry_skip_exp
                rts
jono
                clc
                #dec_l_a direntry_iso_dirsectors_exp
                bne ry	;A=0
                sec
ry		ldx #3
		.if SUPERCPU=1
                tdc
                .fi
ly		bcs tu
		inc next_sector,x
		bne lyy
tu              sta next_sector,x
                dex
                bpl ly
lyy		rts

ny2             lsr
                tax
                adc direntry_iso_bytes_exp
                sta direntry_iso_bytes_exp
                .if SUPERCPU=1
                #lda0
                adc direntry_iso_bytes_exp+1
                sta direntry_iso_bytes_exp+1
                .else
                bcc nx
                inc direntry_iso_bytes_exp+1
nx
		.fi
		txa
                cmp #17
                bcc ezmi
                pha
                lda data_g
                lda data_g
                ldx #direntry_pointer_exp-direntry_exp
                jsr dirp_ofs.getword
                lda direntry_pointer_exp
                ora #$40
                sta direntry_pointer_exp
                jsr dirp_ofs.getnextword
                ldx #direntry_date_exp-direntry_exp
                jsr dirp_ofs.getword
                jsr dirp_ofs.getnextword
                ldx #5
                jsr dirp_ofs.skip
                lda data_g
                lda hig_reg
                sta direntry_attrib_exp
                ldx #3
                jsr dirp_ofs.skip
                ldx #236
                ldy data_g
                .if !FAST_DATA_TRANSFERS
                ldy low_reg
                .fi
                lda direntry_mode_exp
                cmp #DIRMODE_ISO
-               beq bj
                tya
                lsr
                beq -
                tay

bj2u            pla
                #sbc1c
                pha
                lda data_g
                #lda_low_reg
                bmi ala
                cmp #32
                bcc ala
                cmp #":"
                beq ala
                cmp #"="
                beq ala
                cmp #65
                bcc jch
                cmp #91
                bcc nagy
                cmp #95
                beq ala
                cmp #97
                bcc jch
                cmp #123
                bcs jch
                sbc #32-1
                .byte SKIP2
ala		lda #164
		.byte SKIP2
nagy            ora #$80
jch             sta direntry_name_exp-236,x
                inx
                beq toolong
teb             dey
                bne bj2u
                tya
		sta direntry_name_exp-236,x
                geq txe

toolong 	cpy #1
		beq txe
		ldx #253
spx		lda direntry_name_exp-236,x
                cmp #","
                .if !CD_RELAX_FILETYPES
                beq tbj
                cmp #"."
                .fi
                bne ipx
tbj             #phy_a
		ldy #252
thi		lda direntry_name_exp-236,x
                #sta_l_y_ direntry_name_exp-236
                iny
                inx
                bne thi
                #tyx_a
                #ply_a
                gcs teb

ipx		inx
		bne spx

udyye		dey
                beq txe
		pla
                #sbc1c
                pha
                lda data_g
                #lda_low_reg
                cmp #","
                .if !CD_RELAX_FILETYPES
                beq tbj2
                cmp #"."
                .fi
                bne udyye
tbj2            ldx #252
                gne jch

txe
		ldx #236
rekq            lda direntry_name_exp-236,x
		beq ene
                cmp #";"
                bne xoq
                lda direntry_name_exp-235,x
		eor #"1"
                bne yfq
                cpx #254
                beq enei
                lda direntry_name_exp-234,x
                bne yfq
enei            sta direntry_name_exp-236,x
                geq ene

xoq             cmp #$5f	;_
		beq huzq
                cmp #","
                beq ftp
		cmp #"/"
                bne xfq
yfq             lda #"."
                .byte SKIP2
huzq            lda #$a4
                sta direntry_name_exp-236,x
xfq             inx
                bne rekq

ene             dex
		cpx #236
                bcc eni
		lda direntry_name_exp-236,x
                eor #"."
                beq enei
                inx
ukek            dex
		cpx #237
                bcc eni
		lda direntry_name_exp-236,x
                cmp #32
        	.if !CD_RELAX_FILETYPES
               	beq eni
                eor #"."
                .fi
                bne ukek
        	.if !CD_RELAX_FILETYPES
                geq ftp
                .else
                geq eni
                .fi
;-----------------
bj2             pla
                #sbc1c
                pha
                lda data_g
                #lda_low_reg
                jsr convertname
                beq udy
                dey
                beq udy2
bj              lda hig_reg
                jsr convertname
                beq udy
                dey
                bne bj2
udy2		tya
		sta direntry_name_exp-236,x
udy
		ldx #236
rek             lda direntry_name_exp-236,x
		beq eni
                cmp #";"
                bne xo
                lda direntry_name_exp-235,x
		cmp #"1"
                bne yf
                lda direntry_name_exp-234,x
                bne yf
                sta direntry_name_exp-236,x
                lda direntry_name_exp-237,x
                eor #"."
                bne eni
                sta direntry_name_exp-237,x
                geq eni

xo              cmp #","
                beq ftp
		.if !CD_RELAX_FILETYPES
                cmp #"."
                beq ftp
                .fi
		cmp #"/"
                bne xf
yf              lda #"."
                sta direntry_name_exp-236,x
xf              inx
                bne rek
eni             .if CD_RELAX_FILETYPES
       	        ldy #2
       	        jsr dirp_ofs.fillftype
                #jmps ent
                .else
		ldx #255        ;empty
                gne xt
                .fi

ftp		#lda0
		sta direntry_name_exp-236,x
xt		ldy #0
ftpl		inx
                beq ru
		lda direntry_name_exp-236,x
                cmp #","
                beq nd
                cmp #"<"
                beq nd
                cmp #">"
                beq nd
                cmp #32
                beq nd
                cmp #";"
                bne tkq
nd              ldx #0
ru              txa
                dex
tkq             #sta_l_y_ direntry_filetype_exp
                iny
                cpy #3
                bcc ftpl
                jsr dirp_ofs.onetype
ent
		pla
                sec
                sbc #17
ezmi            tax
                beq +
                jsr dirp_ofs.skip
+
		.if SUPERCPU=1
		lda direntry_name_exp
                .else
                ldx direntry_name_exp
                .fi
                bne noilabel
                lda direntry_date_exp+1
                lsr
                lsr
                lsr
                sta direntry_iso_dirsectors_exp
                ldx #16
                lda #32
li3             sta direntry_name_exp,x
                dex
                bpl li3
                lda #FTYPE_DIR|FATTR_READ|FATTR_EXEC
                gne nulldirin
noilabel
                #dec_xa
                bne noilabel2
                .if SUPERCPU=1
                sta direntry_name_exp+2
                .else
                stx direntry_name_exp+2
                .fi
                lda #"."
                sta direntry_name_exp
                sta direntry_name_exp+1

nulldir         ldx #3
                #lda0
xq              sta direntry_size_exp,x
                dex
                bpl xq
                lda #FTYPE_DIR|FATTR_READ|FATTR_EXEC|FATTR_CLOSED
nulldirin       jsr dl
		ldy #6
		jmp dirp_ofs.fillftype

noilabel2	lda direntry_attrib_exp
                and #%00000010
                bne nulldir
                ldx #3
ro              lda direntry_date_exp,x
                sta direntry_size_exp,x
                dex
                bpl ro
                lda #FTYPE_NORMAL|FATTR_READ|FATTR_EXEC|FATTR_CLOSED
dl              sta direntry_attrib_exp
		lda #8
                sta direntry_number_exp
		rts

convertname     cmp #95         ;_
                bne +
                lda #$a4
+               cmp #97         ;a
                blt +
		cmp #123        ;z+1
                bge +
                and #$df        ;to lowercase
+               sta direntry_name_exp-236,x
                inx
                rts
		.pend
