;
; dos.asm - Common dos functions
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;---------------------------------------
; misc settings for existing drives
;
; Also sets the drive identify message
;--------------------------------------
		#assert_cfg_open

OP_XIT_hdinit_2	.proc
		sta current_drive_exp
                jsr OP_get_current_drive_type
		beq OP_set_hdd_max.err

		#xit_init

		jsr set_power_management_and_features
                sec
		gcs LWOP_reporterror_73_xit
		.pend
;--------------------------------------
; Tell an ata drive the disk geometry
;
OP_set_hdd_max	.proc
		jsr OP_setgeometry
		bmi err			;for chs only
                tya
		jsr ata_select_drive_A
		bcs err			;error C=1
                lda next_sector+3
		#sta_low_reg
		sta nsector_g
		lda #init_d
		gne ata_command
err		rts
		.pend

OP_setgeometry	.proc
		sty next_sector+3
		tay
		jsr OP_get_current_drive_type
		bmi err			;for hdd only

		lda next_sector+3
		sta drive_maxsector_exp,x
		lda next_sector+2
		sta drive_maxcylinderlo_exp,x
		lda next_sector+1
		sta drive_maxcylinderhi_exp,x
		tya
		sta drive_maxhead_exp,x
		asl
err		rts
		.pend
;--------------------------------------
                .if SUPERCPU=0
ofs_fs_ident	.text "c64-ide v0.02-a "
cfs_fs_ident	.text "c64 cfs v 0.11b "
                .fi
;--------------------------------------
; Try to identify filesystem on disk
;
OP_XIT_init_filesystem_from_disk .proc

		jsr OP_get_current_drive_type
		and #<~FS_MASK
		sta drive_type_exp,x	;Unknown filesystem
                pha
		lda drive_hwproblems_exp,x
                bpl kfk
                lda #76+$80
                gne LWOP_err

kfk             #lda0			;First partition is active
		sta drive_workpartition_exp,x
                .if SMART_ALLOC
                .if SUPERCPU=1
                dec a
                .else
		lda #255
                .fi
                sta find_free_skip_drivepart_exp
                sta find_free_skipm_drivepart_exp
                .fi

                txa
                lsr
                ror			;*64
                ror
                sta partition_offset_exp
                tax
invali		#lda0
		sta partition_start_exp,x
		inx
                txa
		and #63
		bne invali

		pla
		bpl ata
		asl
                php

                .if ATAPI_TEST_UNIT_READY
		lda #ATAPI_TEST_UNIT_READY
                .else
                #lda0
                .fi
		jsr OP_atapi_packet
		jsr OP_atapi_command	;double ATAPI_SENSE_UNIT_ATTENTION after reset

                plp
		bpl isofs

		#lda0			;ZIP/LS120 drive, LBA sector 0
		.byte SKIP2
ata		lda #1			;ATA CHS (0/0/1)
		sta next_sector+3
		#stz_a next_sector
		#stz_aa next_sector+1
		#stz_aa next_sector+2

		lda #ram_exp_buffer0
		jsr XIT_read_a_sector

		ldx #15
-		lda boot_sector_ident_cfs_exp,x
		eor cfs_fs_ident,x
		bne inifi_nocfs
		dex
		bpl -

		lda boot_sector_default_p_cfs_exp
                pha
		#ldy_l_a boot_sector_maxsector_cfs_exp
		lda boot_sector_maxcyllow_cfs_exp
		sta next_sector+2
		lda boot_sector_maxcylhi_cfs_exp
		sta next_sector+1
		lda boot_sector_maxhead_cfs_exp
		and #%01001111
                pha
		jsr OP_set_hdd_max	;Tell disk the new geometry

		#cnext boot_sector_partdir1_cfs_exp , next_sector

		lda #ram_exp_buffer0
		jsr XIT_read_a_sector	;-=> error

		lda #ram_exp_buffer0
		sta pom2

		#ldx_l_a current_drive_exp;Store disk geometry
                pla
                and #64
                sta drive_lba_exp,x
                pla
		sta drive_workpartition_exp,x

		#ldx_l_a partition_offset_exp

loop		ldy #16
lp2		#lda_zp_y pom1
		sta partition_start_exp,x
                jsr inylda
		sta partition_end_exp,x
                jsr inylda
		sta partition_del_dir_exp,x
                jsr inylda
		sta partition_root_dir_exp,x
		sta partition_workdir_exp,x
		inx
		tya
		sec
		sbc #11
		tay
		and #3
		bne lp2
		lda pom1
		adc #32-1		;C=1
		sta pom1
		bne loop
		inc pom2
		txa
		and #63
		bne loop
;		#lda0
		jsr OP_check_partition_selection;Invalid? -=> error
                jsr LWOP_reporterror_77_xit
		lda #FS_CFS             ;CFS filesystem
                gne detectok

inylda		iny
		iny
		iny
		iny
		#lda_zp_y pom1
		rts
;----------------------------
inifi_nocfs
		ldx #15
-		lda boot_sector_ident_ofs_exp,x
		eor ofs_fs_ident,x
		bne inifi_noofs
		dex
		bpl -

		#ldy_l_a boot_sector_maxsector_ofs_exp
		lda boot_sector_maxcyllow_ofs_exp
		sta next_sector+2
		lda boot_sector_maxcylhi_ofs_exp
		sta next_sector+1
		lda boot_sector_maxhead_ofs_exp
		and #%00001111		;no lba here!
                jsr OP_set_hdd_max

		ldy #3
                lda #$0f
-               ldx fk,y
		and boot_sector_root_ofs_exp,x
                sta szam,y
                lda #$ff
                dey
                bpl -

		lda #%10000000		;Make partition valid, not hidden and readonly
                jsr szam_rootworkbig

		lda #FS_OFS             ;CFS filesystem
                gne detectok

fk		.byte 3,1,0,2
;----------------------------
                .if SUPERCPU=0
jotyp		.byte $01,$04,$06,$0b,$0c,$0e,$0f
                .fi
sectperclust	= pom4
rootdir		= pom5
datastart	= pom9

inifi_noofs	ldx #3
-		#stz_a datastart,x
                dex
                bpl -

		#ldx_l_a current_drive_exp
                #ldy_l_a drive_maxsector_exp,x
                lda drive_maxhead_exp,x
		jsr OP_set_hdd_max

ext		ldx #3
-		lda datastart,x
                sta szam+5,x
                dex
                bpl -

		lda #>boot_sector_sign_exp
		jsr check_s
                bne inifi_nopc		;invalid

nu              ldy #<boot_sector_type_exp
                #lda_zp_y pom1
                ldx #6
-               cmp jotyp,x
                beq fel
                dex
                bpl -

nextone		jsr incc
                bne nu

nu2             #lda_zp_y pom1
                eor #$5
                beq fel
                jsr incc
                bne nu2
                geq trydirect 		;none found

incc            lda pom1
                clc
                adc #16			;search for partition entry
                and #63
                sta pom1
                rts

fel		cmp #1
                lda pom1
                pha
		php

		ldx #3
                ldy #<boot_sector_startp_exp
                clc
-               #lda_zp_y pom1
		adc szam+5,x
		sta next_sector,x
                sta datastart,x
                iny
                dex
                bpl -
                ora #$40
                sta next_sector

                #ldx_l_a current_drive_exp
                lda drive_lba_exp,x
                bne rpu

                jsr chs2lba
                lda szam+3
                and #$0f
                sta szam+3
                jsr lba2chs

rpu             plp
                php
		lda #ram_exp_fat
                bcs +
		lda #ram_exp_buffer0
+
		jsr XIT_read_a_sector	;read boot sector

                plp
                pla
                bcc ext
                pha

                jsr detectfat
                beq nomore

                lda #>boot_sector_sign_exp
                jsr init_pom1_y
                pla
                sta pom1
                #jmps nextone

trydirect	ldx #0
-		lda ram_exp_buffer0_exp,x
                sta ram_fat_exp,x
		lda ram_exp_buffer0_exp+$100,x
                sta ram_fat_exp+$100,x
                inx
                bne -

                jsr detectfat

                .byte SKIP1
nomore		pla

		lda partition_offset_exp
                and #63
                beq inifi_nopc		;no partitions?

		lda #FS_FAT             ;FAT filesystem
detectok       	#ldx_l_ current_drive_exp
		ora drive_type_exp,x
		sta drive_type_exp,x
		rts			;-=> exit, OK
;----------------------------
inifi_nopc
unkn_fs		sec
		gcs LWOP_reporterror_75_xit;Unknown filesystem
;----------------------------
isofs           #stz_a current_path
                #stz_aa current_path+1

                lda #ATAPI_READ_TOC
                jsr OP_atapi_packet
                #pc 9,$40
                #pc 7,>$800
                jsr OP_atapi_datain_command
                bcs unsupp
                lda #ram_exp_buffer0
                jsr OP_read2048_here
                jsr wait_busy;		jsr ata_pi_nomore_data ;                bcs unsupp
                bcs unsupp
                lda toc_first_session_number_exp
                eor toc_last_session_number_exp
                beq unsupp
                ldx #3
                lda #15
-               adc toc_last_session_lba_exp,x
                sta next_sector,x
                #lda0
                dex
                bpl -
                gmi xi

unsupp          lda #15
		sta next_sector+3
		#stz_a next_sector
		#stz_aa next_sector+1
		#stz_aa next_sector+2

xi              inc current_path
		bmi ene
                ldx #3
-	      	inc next_sector,x
                bne xi2
                dex
		bpl -
ene		lda current_path+1
detectokx     	bne detectok
                geq unkn_fs ;filesystem not found

xi2     	lda #ram_exp_buffer0
		jsr XIT_read_a_sector

                ldx #4
-		lda cdstr,x
                cmp isofs_vd_id_exp,x
                bne xi
                lda isofs_vd_root_directory_exp,x
		sta szam,x
                dex
                bpl -

                .if SUPERCPU=1
                lda isofs_vd_type_exp
                .else
                ldx isofs_vd_type_exp
                .fi
                bmi ene		;0xff = end
                #dec_xa
                beq +
                #dec_xa
                bne xi
                lda isofs_vd_escape_exp
                eor #$25
                bne xi
                .if SUPERCPU=1
                inc a
                .else
                inx
                .fi
+               php

		lda #%10000000		;Make partition valid, not hidden and readonly
                jsr szam_rootworkbig

		lda #FS_JOLIET
                plp
		bne detectokx           ;Joliet filesystem

                lda #FS_ISO
		sta current_path+1	;ISO filesystem
                gne xi			;retry

                .if SUPERCPU=0
cdstr           .text "cd001"
                .fi
                .pend
;----------------------------
szam_rootworkbig .proc
		pha
		#ldx_l_a partition_offset_exp
		ldy #3
                sec
-		lda szam,y
		sta partition_root_dir_exp,x
		sta partition_workdir_exp,x
                inx
                dey
                bpl -
                dex
                pla
		sta partition_start_exp-3,x
		lda #%00010000
		sta partition_end_exp-3,x
                rts
		.pend
;----------------------------
check_s         .proc
		jsr init_pom1_y

                ldy #<boot_sector_sign_exp
                lda #$55
                #cmp_zp_y pom1
                bne +
                iny
                asl
                #cmp_zp_y pom1
+		rts
                .pend
;----------------------------
;result in Y
;
OP_check_partition_selection .proc
		sec
		sbc #1
		bcs fe
		#ldx_l_a current_drive_exp
		lda drive_workpartition_exp,x

fe		tay
		cpy #16
		bcs invalid
		lsr
		ror
		ror
		ror
		ora current_drive_exp
		ror
		ror
		ror
		tax
		lda partition_start_exp,x
		bpl invalid		;Must be valid
		lda partition_end_exp,x
		and #%10110000
		eor #%00010000		;CFS?
		beq valid
invalid		iny
		#tyx_a
		ldy #0
                sec
                rts
valid		clc
		rts
		.pend
;---------------------------------------
set_power_management_and_features .proc
		jsr OP_get_current_drive_type
                bpl ata
                asl
                bmi notcd+1

		lda #ATAPI_SET_SPEED
		jsr OP_atapi_packet

                ldy #9                  ;cdromspeed
		jsr afeat

		#pw 2                   ;255Kb/s should be enough for everyone
		#pc 3,$ff

		jsr OP_atapi_command
		bcc jo
		jsr OP_atapi_command	;retry
jo		jmp cdrom_idleoff

ata		jsr ata_select_drive_A
		lda #freeze_lock_d	;Freeze lock
		jsr ata_command

notcd		lda #$60
		jsr ata_select_drive_A
                lda #read_native_max_address_d
                jsr ata_command
                bcs non

                #lda0                   ;Disable HPA
                jsr setmax
non
                lda #4			;Set max freeze lock
                jsr setmax

		lda current_drive_exp
                lsr			;master/slave?
                .if DRIVES=4
                tax
                lda setup_powerman_exp,x
		.else
		lda setup_powerman_exp
                .fi
		bcc master
		lsr
		lsr
		lsr
		lsr
master		and #%00001111
		beq def
		tax
		lda powerconst-1,x
		#sta_low_reg
		sta nsector_g

		lda #idle1_d		;Power management feature
		jsr ata_command

		lda #idle2_d		;Power management feature
		jsr ata_command

def		ldy #0                  ;retry
		jsr set_features

                ldy #3                  ;writecache
		jsr set_features

                ldy #6                  ;readahead

set_features	jsr afeat
		#sta_low_reg
		sta feat_g
		lda #setfeatures_d
fdf		gne ata_command

                .if SUPERCPU=0
featc		.byte %10000000,$33,$99		;retry
		.byte %01000000,$82,$02		;writecache
                .byte %00100000,$55,$aa         ;readahead
                .byte %00010000,$ff,$01         ;cdromspeed
                .fi

afeat		lda current_drive_exp
		lsr	    		;master/slave?
                .if DRIVES=4
                tax
                .fi

		lda featc,y
		bcs nm
		lsr
                lsr
                lsr
                lsr
nm		.if DRIVES=4
		and setup_features_exp,x
                .else
		and setup_features_exp
                .fi
		beq dis
                iny
dis		lda featc+1,y
		rts

setmax		#sta_low_reg
		sta feat_g
		sta nsector_g
                lda #set_max_d
                gne fdf

                .if SUPERCPU=0
powerconst	.byte 0,12,24,36,48,60,72,84,96,120,180,240,241,242,244
                .fi
		.pend
