;
; ds1302.asm - DS1302 clock chip handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;--------------------------------------
; LWOP_get_cmos_setup (B0)
; LWOP_read_clock (B2)
;
;--------------------------------------
		#assert_cfg_open
LWOP_get_cmos_setup .proc
		lda #%11111111		;read ram burst command
		jsr cmos_start
		ldx #256-(setup_crc_exp-setup_params_exp+1)
-		pha
		jsr cmos_read_byte	;read byte from RAM
		sta setup_params_exp-256+(setup_crc_exp-setup_params_exp+1),x	;buffer
		pla
		eor setup_params_exp-256+(setup_crc_exp-setup_params_exp+1),x	;buffer
		inx
		bne -
		stx kill_port
		cmp #"i"
		beq cmos_default.ok	;C=1 ok
;---

cmos_default	.proc
		clc			;C=0 err
		ldx #setup_crc_exp-setup_params_exp-1
-		lda setup_default,x
		sta setup_params_exp,x
		dex
		bpl -
ok		rts

setup_default	.byte DRIVE0
		.if DOLPHINDOS_ACCEL=1
                .byte 14,CMOS_TURBO|CMOS_DOS_WEDGE,CMOS_FUNCTION_KEYS|CMOS_CONTROL
                .else
		.byte 14,CMOS_TURBO|CMOS_DOS_WEDGE,CMOS_FUNCTION_KEYS
                .fi
                .byte %11011101
                .if DRIVES=4
                .byte %11011101
                .fi
                .byte 127,127
                .byte 127,127
                .if DRIVES=4
                .byte 127,127
                .byte 127,127
                .fi
              	.byte $76 		;manager_selfilecolor_exp ,manager_bgcolor_exp
		.byte $06      		;manager_bar_color_exp ,manager_brcolor_exp
                .byte $cc		;manager_misc_color_exp ,manager_inactwincolor_exp
		.byte $77      		;manager_win_color_exp ,manager_actwincolor_exp
		.byte $1f		;manager_selbar_color_exp, manager_dircolor_exp
		.byte $3e		;manager_input_color_exp, manager_filecolor_exp
		.byte $07      		;manager_seldircolor_exp

                .byte $e6		;system colors

                .byte $e6       	;monitor text, background

                .byte $00		;Power management
                .if DRIVES=4
                .byte $00
                .fi

                .if DRIVES=2
                .byte DRIVE0-7+(FLOPPY8-7)*16
                .byte DRIVE1-7+(FLOPPY9-7)*16
                .byte LCD0-3+(FLOPPY10-7)*16
                .byte PC0-7+(FLOPPY11-7)*16
		.else
                .byte DRIVE0-7+(PC0-7)*16
                .byte DRIVE1-7+(FLOPPY8-7)*16
                .byte DRIVE2-7+(FLOPPY9-7)*16
                .byte DRIVE3-7+(FLOPPY10-7)*16
                .byte LCD0-3+(FLOPPY11-7)*16
                .fi
		.pend
;--------------------------------------
write_ram_burst	.proc
		clc
                jsr cmos_writeprotect
		lda #%11111110		;data write brust
		jsr cmos_start
		ldx #256-(setup_crc_exp-setup_params_exp)
		lda #"i"
-		tay
		lda setup_params_exp-256+(setup_crc_exp-setup_params_exp),x
		jsr cmos_send_byte
		tya
		eor setup_params_exp-256+(setup_crc_exp-setup_params_exp),x
		inx
		bne -
		jsr cmos_send_byte_last	;crc
		sec

cmos_writeprotect .proc
		php
		lda #%10001110  	;write control reg
		jsr cmos_start
                plp
                ror

cmos_send_byte_last .proc
		jsr cmos_send_byte

cmos_end	.proc
		ldx #0
                stx kill_port
                rts
                .pend
;--------------------------------------
LWOP_read_clock	.proc
		lda #%10111111		;burst read
		jsr cmos_start
                ldx #256-7
                sbc #$80                ;set v, c was set
it		bvc hy
		jsr cmos_read_byte
set_v           cmp date_max-256+7,x
                bcs err
                ldy date_default-256+7,x
                beq kt
                tay
		bne kt
err             ldx #256-7
                clv
hy		lda date_default-256+7,x
kt              sta sec_exp-256+7,x
                inx
                bne it
                geq cmos_end
                .pend

date_default	.proc
		.byte $00,$00,$00,$01,$01,$07,$00
                .pend
date_max	.proc
		.byte $60,$60,$24,$32,$13,$08,$9a
		.pend
;--------------------------------------
cmos_start	.proc
		ldx #$02
		stx kill_port		;UNreset
cmos_send_byte	.proc
		jsr +
+		jsr +
+		jsr +
+		sta clock_port		; bit 0
		lsr a
		rts
		.pend
;------------------
cmos_read_byte	.proc
		jsr +
+		jsr +
+		jsr +
+		lda clock_port		; bit 1
		lsr a
		tya
		ror a
		tay
		rts
		.pend
