;
; man.asm - File manager lower half
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
; starts in cfg_16
;

		#assert_cfg_16

DECORATION	= 1	   	;If 0 then old window decoration

manager		.proc
		pha
                .if SUPERCPU=1
                jsr scpu_no_opt		;no optimization, sprites!!!
                .fi

		lda #$37
		sta $01

                #sta_cfg_open
		jsr manager_data_swap	;cfg16
                #assert_cfg_16

		sec
                jsr restorescreen.save

		#sta_cfg_open
		ldx #7
ld              lda setup_mancolors_exp-1,x
		tay
                lsr
                lsr
                lsr
                lsr
                sta manager_colors+6,x
                tya
                and #15
                sta manager_colors-1,x
                dex			;set default colors
                bne ld
                stx $d418
		.if (SECOND_SID >> 8)=$d6
		lda $d030		;C128?
		lsr
		bcc +
		.fi
		stx SECOND_SID+$18	;Second SID
+               stx man_valid
                stx man_valid+1
                asl manager_dircolor
                sec
                ror manager_dircolor

		.if SERIAL_HANG_FIX
                jsr test_hang
                .fi

                pla
                cmp #"!"
                beq cler

		jsr man_check_sign	;cfg16
                #assert_cfg_16
                beq redraw

		.assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
cler		ldx #5
ldk             lda defaultp-1,x
                sta man_relpath-1+$000,x
                sta man_relpath-1+$100,x
                dex
                bne ldk
                txa
                ldx #2
rr      	sta man_bar-1,x
		sta man_el_l-1,x
		sta man_el_h-1,x
                sta man_partition-1,x
                dex
                bne rr
		stx man_selectedwin		;0
                stx man_jumpon
                stx man_jumppart
                inx
                stx man_parlist
                stx man_parlist+1
                lda #128
                sta man_font

                #sta_cfg_open
                ldx ActualDev
		lda setup_boot_drive_exp
                sta ActualDev
                sta man_plugin_drive
                cpx #31
                bcs we2
                cpx #8
                bcs we
we2             tax
we		#phx_a

                ldx #11
                stx ActualFNLength
ie              lda iu-1,x
                sta $200-1,x
                dex
                bne ie
                stx ActualFilename
                stx ActualSecAddr
                stx man_plugins
                stx man_redirect
                lda #2
                sta ActualFilename+1
                txa
                stx msgmode
                ldx #<man_plugins
                ldy #>man_plugins
                jsr newload_t
                #assert_cfg_16

                pla
		sta man_drive
		sta man_drive+1
                sta ActualDev

		ldx #0
		lda #<man_input
       		ldy #>man_input
               	jsr ident_drive.in
               	lda man_floppy
               	sta man_floppy+1
               	lda man_cmd
               	sta man_cmd+1

		clc
		jsr man_check_sign.set
                #assert_cfg_16
                gcc redraw

		#assert_8000romonly
iu              .text "1//:man,usr"

		#assert_cfg_16
restart		jsr manager_data_swap	;cfg16
                #assert_cfg_16
		.if SUPERCPU=1
                jsr scpu_no_opt		;no optimization, sprites!!!
                .fi

		.if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
		#assert_8000romioonly
redraw          sta cfg_open
                #assert_cfg_open
                .if SUPERCPU=1
                lda #1
                sta dos_accel_internal_exp
                .else
		sec
		rol dos_accel_internal_exp
                .fi
                sta cfg_16
                #assert_cfg_16
		.else
                #assert_cfg_16
redraw
                .fi
                jsr getcconv

		ldx #4
                ldy #8
                lda #8+48*4
                sec
ldf		pha
		lda manager_bar_color
		sta $d027,x
                lda #35
                sta $7f8,x
                pla
                sta $d000,y
                sbc #48
                dey
                dey
                dex
                bpl ldf
		lda #128
		sta 650
                sta 657
		asl
                sta $d015
		sta $d010
		sta 198
		sta msgmode
		sta $d017
		sta $d01c
		jsr bary
                tax
l		sta $800,x
		cpx #255-25
                bne +
                .if SUPERCPU=1
                dec a
                .else
                lda #255
                .fi
+		inx
		bne l
                lda #6
                sta man_savedwind
                .cerror <man_winstore
                stx man_savedwin
                lda #>man_winstore
                sta man_savedwin+1

-		lda $d011
		bpl -
-		lda $d011
		bmi -
		lda manager_brcolor
		sta $d020
		lda manager_bgcolor
		sta $d021
		lda #200
		sta $d016
		lda #27
		sta $d011
                lda man_font
                sta m_d018
                ldx #22
                asl
                bpl +
                dex
+		stx $d018
		lda #3
		sta $dd00
		lda #63
		sta $dd02
                lsr			;lda #%00011111
		sta $d01b
		sta $d01d
                lda manager_misc_color
                sta 646
                jsr clrscr

                ldx #22
ll              lda virtualcall-1,x
		sta man_virtual_c-1,x
                dex
                bne ll

		lda man_selectedwin
		pha
                jsr updateselectedwin
                jsr topdir	     ;cfg16
		jsr winselcolupdate	;cfg16

		#stz_a selectedwin
		#stz_aa man_oldsc_h
		#stz_aa man_oldsc_h+1
                #stz_aa man_virtual
                #stz_aa man_virtual+1
                #stz_aa man_redirect
                jsr killvirtualredrwincap
		inc selectedwin
                jsr killvirtualredrwincap

                .if SUPERCPU=1
                sei
                sta scpu_hwenable
                sta scpu_basic		;full optimization
                sta scpu_hwdisable
                cli
                .fi

iy		lda $dc01
		eor #$7f
		beq iy

		pla
                jsr updateselectedwin
		jsr changepanelforce
                lda man_jumpon
                pha
                #stz_a man_jumpon
		jsr main_reload2
                pla
                sta man_jumpon
		jsr scrollbar
                jsr changepanelforce
		jsr main_reload2
		jsr scrollbar
                jsr topdir

klp		lda #%00011111
		sta $d015
		jsr vbn
		#jmps klp

vbn		jsr placebar
		jsr scrollbar

kllp		lda 651
		lsr
		beq xf
                lda #255	;return hack
        	ldx $c5
		dex
                beq +
		lda #2
+		sta 651
xf
		lda 653
		lsr
		lsr
		bcs topmenu	;C= key
		lsr
		bcc u		;control
		jsr changepanel
		jsr changepanel2
		lda #4
u2		bit 653
		bne u2		;wait release

u		jsr $f142
		beq kllp
		ldx selectedwin

                cmp #"a"
                bcc nq
                cmp #"z"+1
uu              bcc main_quick
nq	        cmp #"A"
		bcc nq2
                cmp #"Z"+1
		bcc uu
nq2
		ldy #256-3
dh		iny
		iny
		iny
		cpy #funce-func
		beq nn
		cmp func,y
		bne dh

                cpy #funce2-func
                bcc bd
                #stz_a $d015
bd
		lda func+2,y
		pha
		lda func+1,y
		pha
                #lda0
nn		rts

		#assert_8000romonly
defaultp	.null "cd//"

		#assert_cfg_16
func		.byte 17		;down
		.rta main_down
		.byte 145		;up
		.rta main_up
		.byte 19		;home
		.rta main_home
		.byte 137		;f2
		.rta main_home
		.byte 133		;f1
		.rta main_pgup
		.byte 157		;left
		.rta main_pgup
		.byte 136		;f7
		.rta main_pgdn
		.byte 29		;right
		.rta main_pgdn
		.byte 140		;f8
		.rta main_end
		.byte 147		;clr
		.rta main_end
		.byte "*"		;*
		.rta main_invselect
		.byte 192		;*
		.rta main_invselect2
		.byte "+"		;+
		.rta main_allselect
		.byte 219		;+
		.rta main_allselect2
		.byte "-"		;-
		.rta main_noneselect
		.byte 221		;-
		.rta main_noneselect2
		.byte 20		;del
		.rta main_select
                .byte 94		;^
                .rta main_eject
funce2
		.byte "."		;.
		.rta main_parentdir
		.byte "/"		;/
		.rta main_rootdir
		.byte 95		;<-
		.rta main_exit
		.byte 13		;return
		.rta main_return
		.byte 141		;sh+return
		.rta main_shreturn
		.byte "1"		;1
		.rta main_reload
		.byte "2"		;2
		.rta main_menu
		.byte "3"		;3
		.rta main_view
		.byte "4"       	;4
		.rta main_edit
		.byte "5"		;5
		.rta main_copy
		.byte "6"		;6
		.rta main_rename
		.byte "7"		;7
		.rta main_mkdir
		.byte "8"		;8
		.rta main_delete
                .byte "9"		;9
                .rta main_command
funce

txtloading	.null 13,13,2,4,"loading"
txttoomany	.null 13,13,29,29,"too"," "+128,"many"," "+128,"files","!"+128

virtualcall    	sta man_virtual_at
		sei		;1
                sta cfg_stnd
		inc $01		;2
		jsr $d000	;3
                dec $01		;2
                sta bank_port2
                sta cfg_16
                cli		;1
                rts		;1
		.pend
;---------------------------
; Main quick search
;
main_quick	.proc
		sta man_jumpon
                #stz_a man_jumpon+1
                #stz_aa man_jumpontype
                jsr man_jumponfile2
                #jmps printdir
                .pend
;---------------------------
; Main (de)select file
;
main_select	.proc
		jsr calc_filepos_cursor
		jsr man_toggle_sel
		jsr printdirandstat
		ldx selectedwin
;---------------------------
; Main cursor down
;
main_down	.proc
		ldy man_el_h,x
		lda man_el_l,x
		sec
		adc man_bar,x
		bcc +
		iny
+		cmp man_files_l,x
		tya
		sbc man_files_h,x
		bcs x
		lda man_bar,x
		cmp #10
		beq o2
		inc man_bar,x
x		rts

o2		inc man_el_l,x
		bne y
		inc man_el_h,x
y		#jmps printdir
		.pend
;---------------------------
; Main cursor up
;
main_up		.proc
		lda man_bar,x
		beq o1
		dec man_bar,x
x		rts

o1		lda man_el_l,x
		bne py
		lda man_el_h,x
		beq x
		dec man_el_h,x
py		dec man_el_l,x
		#jmps printdir
		.pend

;---------------------------
; Main reload dir
;
main_reload2	.proc
		jsr ident_drive
sam             jsr same_dir
                bne main_reload
in              ldx selectedwin+1
                lda man_valid,x
                beq main_reload
                jsr man_copypanel
		ldy selectedwin+1
		ldx selectedwin
		lda man_dirend,y
		sta man_dirend,x
                lda man_files_l,y
		sta man_files_l,x
		lda man_files_h,y
		sta man_files_h,x
		lda man_fulllo,y
		sta man_fulllo,x
		lda man_fullhi,y
		sta man_fullhi,x
		lda man_fullfree,y
		sta man_fullfree,x
		lda man_partition,y
		sta man_partition,x
                jsr zeroblockfilesel
                #jmps main_reload.in

main_reload	.proc
		ldx #<$4cc
		ldy #>$4cc
		lda selectedwin
		beq +
		ldx #<$6ac
		ldy #>$6ac
+		jsr drawwin
                .word $0000
		.byte 18,4
		.word manager.txtloading

                ldx #2
lu		lda part,x
		sta man_input,x
                dex
                bpl lu

		ldx selectedwin
                lda man_parlist,x
                jsr setnameinput

		ldx selectedwin
                ldy dirm,x
		sty m_p2+1
                lda man_virtual,x
                ldx #$e0
                stx m_p2
                ora #0
                beq nonvirtual
                lda #VIRTUAL_LIST
                jsr man_virtual_c
                #jmps ti1

part		.text "$=p"

nonvirtual
;		#lda0
		sta ActualSecAddr
                sei
		jsr newload_t
ti1		pha
		php
		sei
		jsr mloaddir
                plp
                cli
                pla
                jsr basic_error
		bcs er
                jsr read_errchannel
er		jsr showerror
                sei
		jsr sortdir
                cli
in		jsr man_jumponfile
		ldx selectedwin
                lda #1
                sta man_valid,x
		txa
                lsr
		ldy #15
xu		ldx man_dirmem1-26,y
		bcc rp
                ldx man_dirmem2-26,y
rp              lda cconv,x
		eor #128
                bcs rp1
                sta $43f,y
                bcc rp2
rp1             sta $61f,y
rp2		dey
		bpl xu
		gmi printdirandstat
;---------------------------
; Main invert selection
;
main_invselect2	.proc
		eor #64^65
main_noneselect2 .proc
		eor #65^192
main_allselect2	.proc
		eor #192^128
main_allselect	.proc
		eor #128^1
main_noneselect	.proc
		eor #1
main_invselect	.proc
		sta m_ascii+3

		lda man_files_l,x
		sta m_ascii+1
                lda man_files_h,x
		sta m_ascii+2

lp		lda m_ascii+1
		bne sd
                lda m_ascii+2
                beq printdirandstat
                dec m_ascii+2
sd		dec m_ascii+1
		lda m_ascii+2
                lsr
		lda m_ascii+1
		jsr calc_filepos
                bit m_ascii+3
		bvs k1
                jsr man_dir
                bmi lp			;directory?
k1		lda m_ascii+3
		and #$81
		beq k2
		jsr man_sel
		bpl lp
k2		jsr man_toggle_sel
		#jmps lp
		.pend

;---------------------------
; Main go to parent dir
;
main_parentdir	.proc
		jsr man_cdpp
in		jsr showerror
		#jmps main_reload
		.pend
;---------------------------
; Main go to root dir
;
main_rootdir	.proc
		jsr man_cdroot
		#jmps main_parentdir.in
		.pend

err_is_exists   .proc
		lda #"6"
                ldx #"3"
in      	cmp man_errchannel
                bne noexists
                cpx man_errchannel+1
noexists
		rts
		.pend
;---------------------------
; Main viewer
;
plugin_exec     .proc
		lda man_plugin_drive
		sta ActualDev

		lda #>cconv
		sta ActualFilename+1
		sta ActualSecAddr

		#lda0
		sta ActualFilename
		jsr newload_t

		lda Status
		eor #$40
		bne pr
		bcc ui
pr		jsr read_errchannel
		jsr showerror
ie		jsr setdev
		#jmps getcconv

ui		cpy #$d0
		bcc nvirt
		ldx selectedwin+1
                lda man_virtual,x
                bne ie           ;1 virtual filesystem at a time!
                sta man_virtual_at;VIRTUAL_INIT
                jsr setdev
		ldx #<$201
		ldy #>$201
		lda $200
                pha
                jsr man_virtual_c+3
                pla
                jsr setname201
                jsr rel_path_m_p1s
                lda #":"
                dey
                sta (m_p1),y
                jsr append_actualfn_to_path
                jsr topdir
		ldx selectedwin
		lda #1
                sta man_virtual,x
                jsr ie
                #jmps main_reload

nvirt		pla
		pla

		jsr setdev
                #stz_a man_valid
                #stz_aa man_valid+1

                ldy #255
np              iny
		lda $201,y
                sta man_jumpon,y
                eor #","
                bne np
                sta man_jumpon,y
                ldx #0
np2             lda $202,y
		iny
                cpy $200
                bcc xj
                lda #32
                dey
xj		sta man_jumpontype,x
                inx
                cpx #3
                bcc np2

		jsr manager_data_swap2	;cfg16
                #assert_cfg_16
                .if SUPERCPU=1
		#psr manager.restart
		#psr gotobank0back
		#psr exit2basic
		#psr $1003
                jsr scpu_default_opt
                .else
                ldx #7
dk              lda st,x
                pha
                dex
                bpl dk
                .fi
		ldx #<$201
		ldy #>$201
		lda $200
		jmp exit2basic

		.if SUPERCPU=0
st              .rta $1003
                .rta exit2basic
                .rta gotobank0back
                .rta manager.restart
                .fi
		.pend
;---------------------------
; Main exit
;
main_exit	.proc
		jsr savedrawwin
                .word $576
		.byte 10,6
		.word txtexit

		jsr waitkey_restorewin

		cmp #"y"
		beq exit
		cmp #"1"
		bne setdev.rrts		;exit

exit		pla
		pla
                jsr manager_data_swap2andrestore	;cfg16
                #assert_cfg_16
		pla
		pla
		#psr $e37b
		jmp exit2basic

txtexit		.text 13,13,2,1,"exit","?"+128,13,13
		.null 2,2,"y","/"+128,"n"
		.pend
;---------------------------
; Set current device
;
setdev		.proc
		ldx selectedwin
		lda man_drive,x
		sta ActualDev
rrts		rts
                .pend
;---------------------------
; Main viewer
;
main_edit       .proc
                eor #%11101000^%10100000
main_menu       .proc
                eor #%10100000^%11010000
main_view	.proc
                eor #%11010000
                ldx #0
                geq main_shreturn.in
                .pend
;---------------------------
; Main return
;
;Virtual filesystem:
;$d000-$ffff
;$d000 init     (x,y filename, a namelength)
;$0334 select   (x,y file/dir name ends on 0)
;$0336 list dir (x,y where) return: x,y read bytes
;$0338 read     (x,y where a=size) return: x,y read bytes
;$033a write    (x,y where a=size) return: x,y bytes writen
;$033c mkdir    () return: a
;$033e rm       () return: a
;$0340 rmdir    () return: a

                .assert 0,0,0
skip_match      = man_recurs
                #assert_cfg_16

main_return	.proc
		ldx #0
                stx 198		;return hack
		.byte SKIP2
main_shreturn	.proc
		ldx #1
                #lda0
in		stx ActualSecAddr
                pha

                lda man_virtual,x
                bne rrts1
		jsr no_files
		beq rrts1			;empty dir
                jsr calc_filepos_cursor
                lda man_parlist,x
                lsr
                beq normal
                lsr
                tay
                jsr man_get
                sta man_input+2
                pla
                bne rrts2
                jsr changepart
                #jmps main_rootdir

rrts1           pla
rrts2           rts

normal	       	jsr man_dir
                bpl notdir
                pla
                bne rrts2
		jsr man_cd
uf		lda m_p1+1
		pha
		jsr showerror
                pla
		bcc failed
                bne failed

		jsr append_actualfn_plus3_to_path
		jsr topdir

failed		#jmps main_reload

notdir		jsr man_ext
		cmp #"c"
		bne nd2
		cpx #"b"
		bne nd2
		cpy #"m"
		bne nd2
                pla
                bne rrts2
		lda #"/"
		ldx #"0"
		jsr man_cd.in
		#jmps uf

nd2             jsr man_copyfname

		#sta_cfg_open
                ldx #255
tip             inx
                lda $201,x
                cmp #","
                bne ko
                #lda0
ko              sta direntry_name_exp,x
		bne tip
                #txy_a
                ldx #253
tip2            iny
                lda $201,y
                cpy $200
                bcc ko2
                #lda0
ko2		sta direntry_filetype_exp-253,x
		inx
		bne tip2
                lda #FTYPE_NORMAL | FATTR_CLOSED
                sta direntry_attrib_exp
                .if DIRMODE_CFS=(FTYPE_NORMAL | FATTR_CLOSED)
                lda #DIRMODE_CFS+1
                .fi
                sta direntry_mode_exp	;not cfs
                #stx_l_a wildcard_end_exp

                jsr OP_save_zeropage

		lda #<(man_plugins-1)
                sta current_path
		lda #>(man_plugins-1)
                sta current_path+1
                .if SUPERCPU=1
                stz current_path+2
                .fi

                pla
                sta skip_match

                jsr inc_cp_lda_ldy0
                beq nomx       		;no plugin list

		#sta_cfg_16

		ldx #0
nu2             sta cconv,x
                jsr inc_cp_lda_inx
                bne nu2
                stx m_p2

                #sta_cfg_open

		lda ActualSecAddr
		bne nom

retry           jsr inc_cp_lda
nomx            beq nom			;end of plugin list
                cmp #"="
                bne +
                asl skip_match
                bcs dm3
                gcc nom
+
		#sta_cfg_16
                ldx m_p2
dm		sta cconv,x
		jsr inc_cp_lda_inx
		bne dm
		stx ActualFNLength
                #sta_cfg_open

                jsr inc_cp_lda
                beq nom

                jsr check_wildcards

dm2		jsr inc_cp_lda_ldy0
		beq hi
                eor #","
                bne dm2
                jsr inc_cp_lda
                bcs dd

		pha
                jsr check_wildtype
                pla

dd		beq hi

dm3		jsr inc_cp_lda_ldy0
                bne dm3

hi		bcs retry	      ;no match
                lda skip_match
                bmi retry

		jsr OP_restore_zeropage
		#sta_cfg_16

		#jmps plugin_exec

		#assert_cfg_open
nom		jsr OP_restore_zeropage
                lda skip_match
                bne switch_16k
		lda $200
                sta ActualFNLength
                lda #2
                sta $7b
                lsr
		eor ActualSecAddr
		sta load_autostart_exp	;autostart
		pla
		pla
		#sta_cfg_16
                jsr manager_data_swap2andrestore	;cfg16
                #assert_cfg_16
		lda #128
		sta msgmode
		asl
		sta $0a
		pla
		pla
		#psr $e16f
		jmp exit2basic

		.assert CFG_8000_ROM,CFG_8000_ROM,CFG_8000_RAM

inc_cp_lda_ldy0 ldy #0
inc_cp_lda_inx  inx
inc_cp_lda     	inc current_path
		bne +
                inc current_path+1
+               lda (current_path),y
rrts            rts
		.pend
		#assert_cfg_16
;---------------------------
; Main go home
;
main_home	.proc
		jsr gotop
		#jmps printdir
		.pend

;---------------------------
; Main PgUp
;
main_pgup	.proc
		lda man_bar,x
		bne dpy2
		lda man_el_l,x
		sec
		sbc #11
                bcs ndn
		dec man_el_h,x
		bmi main_home
ndn		sta man_el_l,x
		#jmps printdir

dpy2		#stz_a man_bar,x
		rts
		.pend

;---------------------------
; Main PgDn
;
main_pgdn	.proc
		lda #10
		cmp man_bar,x
		bne dpy2
		lda #21-1
		adc man_el_l,x
		tay
                #lda0
		adc man_el_h,x
		pha
		tya
		cmp man_files_l,x
		pla
		sbc man_files_h,x
		bcs main_end
		lda man_el_l,x
		adc #11
                bcc main_pgup.ndn
		inc man_el_h,x
                gcs main_pgup.ndn

dpy2		ldy man_files_h,x
		bne u
		cmp man_files_l,x
		bcs main_end
u		sta man_bar,x
		rts
		.pend
;---------------------------
; Main end
;
main_end	.proc
		lda #10
		ldy man_files_h,x
		bne us
		cmp man_files_l,x
		bcc us
		lda man_files_l,x
		sbc #1
		bcs vf
		#lda0
vf		sta man_bar,x
		rts

us		sta man_bar,x
		lda man_files_l,x
		sec
		sbc #11
		sta man_el_l,x
		tya
		sbc #0
		sta man_el_h,x
		#jmps printdir
		.pend
;---------------------------
; Main eject
;
I_UI		= 6
I_RD		= 9
I_S		= 13
I_MD		= 16

main_eject	.proc
		ldy #0
putandexecinput .proc
		ldx #$60
le              lda uo,y
		bmi ji
                sta man_input-$60,x
                iny
                inx
                bne le
ji      	#jmps do_errchannel

uo              .text "u0>e1",0
		.text "ui",0
                .text "rd:",$80
                .text "s:",$80
                .text "md:",$80
		.pend
;---------------------------
; Identify drive
;
;73,CBM DOS V2.6 1541,00,00
;73,JIFFYDOS 5.0 1541,00,00
;73,CBM DOS V3.0 1570,00,00
;73,CBM DOS V3.0 1571,00,00
;73,JIFFYDOS 6.0 1571,00,00
;73,CBM DOS V3.1 1571,00,00
;73,COPYRIGHT CBM DOS V10 1581,00,00
;73,(C) 1989 JIFFYDOS 6.0 1581,00,00
;73,CMD FD DOS V1.40,00,00
;73,CMD RL DOS V2.06,00,00
;73,CMD HD DOS V1.90,00,00
;73,CMD HD DOS V1.92,00,00

ident_drive	.proc
		ldx selectedwin
                lda posl,x
                ldy posh,x

in		sta m_p5
		sty m_p5+1

		lda #160
		ldy #7
i		sta (m_p5),y
		dey
		bpl i

		#stz_a man_cmd,x
		lda #1
		sta man_floppy,x

		ldy #I_UI
		jsr putandexecinput

		lda man_errchannel
		cmp #"7"
		bne n5

		ldx #4
lk		lda man_errchannel+2,x
		cmp id,x
		bne kq
                dex
                bpl lk
		ldx selectedwin
                rol man_cmd,x
                lda #","
                sta man_errchannel+9
                ldx #2
                gne kxx

kq      	ldx #2
		lda #","
-		inx
		beq n5
		cmp man_errchannel,x
		bne -
		lda #" "
nn		dex
		beq n5
		cmp man_errchannel,x
		bne nn
kxx		ldy #0
n3		inx
		lda man_errchannel,x
		cmp #","
		beq n6
		and #63
		ora #128
		sta (m_p5),y
		iny
		cpy #8
		bcc n3

n6		ldy #0
		ldx #4
                jsr ccmp
                bne n5
                ldy #2
                dex
iwe		jsr ccmp
		beq n4
		dex
                bpl iwe
n5		ldx selectedwin	;no 1541/1571/1581/1570
		lsr man_floppy,x
n4		rts

ccmp		lda (m_p5),y
                cmp l1,x
		bne n7
                iny
		lda (m_p5),y
                dey
                cmp l2,x
n7		rts

id		.text ",cmd "
l1		.text "4"+128,"7"+128,"7"+128,"8"+128,"1"+128
l2              .text "1"+128,"0"+128,"1"+128,"1"+128,"5"+128

posl		.byte <$430
		.byte <$610
posh    	.byte >$430
		.byte >$610

		.pend
;---------------------------
; Do chdir
;
man_cd		.proc
		lda #"c"
		ldx #"d"
in		sta $201
		stx $202
		lda #":"
		sta $203
                jsr ispp
                beq man_cdpp

		ldx #3
		jsr man_copyfname3

                #lda0
                sta m_p1+1
		geq read_errchannel.inn
		.pend
;---------------------------
; Go to root
;
man_cdroot	.proc
		jsr rel_path_m_p1
                ldy #4
                #lda0
                sta (m_p1),y
                dey
nu              ldx selectedwin
                lda #1
                sta man_parlist,x
                lsr man_virtual,x
                bcc +
                jsr man_cdpp.exit
+		lda #"/"
                sta man_input+3
		ldx #4
		gne man_cdpp.in

in		jsr rel_path_m_p1
		ldy #3
		gne nu
;---------------------------
; Go to parent
;
man_cdpp	.proc
	    	jsr rel_path_m_p1s
                cpy #4
                bne kx
                jsr changepanel
                #ldatax selectedwin+1
                jsr updateselectedwin
                lda #3
                sta man_parlist,x
                lda man_drive,x
                sta ActualDev
                lda man_partition,x
                sta man_jumppart
inp             lda #"0"
                sta man_input
                rts

jumponstuff	ldx #0
nu              sta man_jumpon,x
                inx
                cpx #16
                bcs se
                iny
                lda (m_p1),y
                cmp #","
                beq se
                cmp #"/"
                bne nu
se              #lda0
                sta man_jumpon,x
		rts
vir
		jsr killvirtual
                txa
                jsr jumponstuff
                tax
eu              iny
                lda (m_p1),y
                cmp #"/"
                bne ke
                #lda0
ke              sta man_jumpontype,x
                inx
                cpx #3
                bcc eu

        	ldx selectedwin
                lsr man_virtual,x

                jsr inp
                jsr topdir
exit            lda #VIRTUAL_EXIT
                jmp man_virtual_c

kx              dey
jcn             dey
		lda (m_p1),y
                cmp #":"
                beq vir
                cpy #4
		bcc man_cdroot.in
                cmp #"/"
                bne jcn

		ldx #3
		lda #95
in      	sta man_input+2
		lda #"d"
		sta man_input+1
		lda #"c"
		sta man_input
		lda #"/"
		sta (m_p1),y
                iny
                lda (m_p1),y
                pha
		#lda0
		sta man_input,x
                sta (m_p1),y

                pla
                jsr jumponstuff
                jsr jumptypedir

                jsr topdir

;---------------------------
; Execute command, and print result
;
do_errchannel	.proc
                jsr setnameinput

		ldy #0
iy		lda (ActualFilename),y
		beq ie
		iny
		bne iy

ie
		.byte SKIP2
;---------------------------
; Check error channel
;
read_errchannel	.proc
		ldy #0
		sty ActualFNLength

inn		lda #15
		sta ActualSecAddr
                ldx man_redirect
                beq no
                txa
no		sta ActualLogical
                bne okp

		jsr newopen_t
ee		jsr basic_error
		bcs uy

okp		ldx ActualLogical
		jsr newchkout_t
		bcs ee
                lda man_redirect
                beq no3
                ldx ActualFNLength
                beq no3
                ldy #0
nk              lda (ActualFilename),y
                jsr newchrout_t
                iny
                dex
                bne nk
                lda #13
                jsr newchrout_t
                jsr newclrchn_t
no3		ldx ActualLogical
		jsr newchkin_t
		bcs ee
		ldx #0
di		jsr newgetin_t
		sta man_errchannel,x
		inx
		bmi uy
		lda Status
		beq di
uy
		jsr newclrchn_t
                lda man_redirect
                bne basic_error.rrts
no2		lda ActualLogical
		jmp newclose_t
		.pend
;---------------------------
; Set basic error msg from accu
;
basic_error	.proc
		bcc tul
		sta cfg_8
                #assert_cfg_8
		asl
                tay
                bne +
                ldy #$3c
+               lda $a326,y
                sta $ae
                lda $a327,y
                sta $af
                ldy #0
-		lda ($ae),y
                cmp #$80
                and #$7f
                beq ew
                sta man_errchannel,y
                iny
                bmi tul
                bcc -
ew              lda #13
                sta man_errchannel,y
                #assert_8000romioonly
tul     	sta cfg_16
                #assert_cfg_16
rrts            rts
		.pend

;---------------------------
; Show error channel C=0 on error
;
                #assert_cfg_16
putandexecinput_showerror .proc
                jsr putandexecinput
showerror	.proc
		lda man_errchannel
                lsr
		cmp #"0" >> 1 		;0/1
		beq v			;no error

		lda #15
                sta $d404+7
                sta $d404+14
                sta $d401
                sta $d418
                lsr
                sta $d405
                sta $d406
		ldx #$20
                stx $d404
                inx
                stx $d404
		jsr savedrawwin
                .word $593
		.byte 33,5
		.word txtdev

		ldx #$af
                sec
		lda ActualDev
dd		inx
                sbc #10
                bcs dd
                adc #$ba
                stx $5cb
                sta $5cc

		ldx #0
                #txy_a
u		stx m_p1
		ldx man_errchannel,y
		cpx #13
		beq u2
                lda cconv,x
                beq jk
                .enc screen
                cmp #"z"+1
                .enc none
                bcs jk
                eor #$c0
		eor m_d018
jk		eor #$80
                ldx m_p1
		sta $60c,x
                inx
		iny
                cpy #64
                beq u2
		cpy #32
		bne u
                ldx #40
                gne u
u2
		jsr waitkey_restorewin
                clc

v		rts

txtdev		.null 13,2,7,"device",2,2,"error",13,1
		.pend
;---------------------------
; Do partition change
;
changepart	.proc
                lda #"c"
                sta man_input
                lda #"P"
                sta man_input+1
                lda #3
                jsr setnameinput
		jsr read_errchannel.inn
                #jmps showerror
                .pend
;---------------------------
; Change panel (If C=0 then error)
;
changepanelforce .proc
		sec
		.byte SKIP1BCC
changepanel	.proc
		clc
		lda selectedwin+1
                tay
                jsr updateselectedwin
                tax
		lda man_drive,y
		sta ActualDev
                lda man_floppy,y
                bne masdrive	;Floppy, can't do anything
                bcs force
                lda ActualDev
                eor man_drive,x
		bne masdrive    ;different drives, nothing to do

                lda man_partition,y
                eor man_partition,x
                beq np		;same partition, no partition select

force		php
                ldx man_partition,y
                stx man_input+2
                inx
                beq illegal	;partition dir
                jsr changepart
                bcs illegal	;ok
                plp
		clc
err             rts		;error

illegal         plp
                bcc masdrive	;different partition, no directory change!
                gcs forc

np		jsr same_dir
		beq masdrive	;in same dir on same partiton

forc		lda selectedwin
                clc
                adc #>man_relpath
                sta ActualFilename+1
                lda #<man_relpath
                sta ActualFilename

                ldy #0
-               lda (ActualFilename),y
                beq +
                cmp #":"
                beq +
                iny
                bne -
+		sty ActualFNLength

                jsr read_errchannel.inn
                jsr showerror
                bcc err
masdrive	sec
		rts

changepanel2    .proc
-		lda $d011
		bpl -
		jsr winselcolupdate
                jsr topdir
;---------------------------
; Move fileselection
;
placebar	.proc
		ldx selectedwin
		lda man_bar,x
		cpx #0
		beq +
		adc #12-1
+               asl
		asl
		asl
		adc #53
;---------------------------
; Set file selector sprite position
;
bary		.proc
		ldx #9
-		sta $d000,x
                dex
                dex
                bpl -
		rts
		.pend

;---------------------------
; Set color of panels
;
winselcolupdate	.proc
		lda #<$d828
		ldy #>$d828
		ldx selectedwin+1
		jsr draw

		lda #<$da08
		ldy #>$da08
		ldx selectedwin

draw		sta m_p1
		sty m_p1+1
		lda manager_inactwincolor,x
		ldy #39
-		sta (m_p1),y
		dey
		bpl -
                tax

		ldy #68
-		txa
		sta (m_p1),y
		tya
		clc
		adc #40
		tay
		bcc +
		inc m_p1+1
+		cpy #252
		bne -
		rts
		.pend
;---------------------------
; Print panel statistics
;
man_printstat	.proc
		ldy selectedwin
                lda posl,y
		sta m_p2
                lda posh,y
		sta m_p2+1
                lda #8
                sta m_p1

		ldx man_block_l,y
		lda man_block_m,y
		pha
		lda man_block_h,y
		tay
		pla
		jsr num

		jsr man_printfullfree

		ldy selectedwin
		ldx man_filessel_l,y
		lda man_filessel_h,y
		jsr num2
		ldx man_fulllo,y
		lda man_fullhi,y
		jsr num2
                lda #3
                sta m_p1
		ldx man_drive,y
                jsr num3
		ldx man_partition,y
                lda m_p2
                sec
                sbc #74
                sta m_p2
num3		#lda0
num2		ldy #0
num		jsr numtoasc24
		lda #32
-		cpy m_p1
		bcs +
		sta (m_p2),y
		iny
		gne -
+
		lda m_p2
		adc #80-1
		sta m_p2
		bcc +
		inc m_p2+1
+               ldy selectedwin
		rts

posl    	.byte <$4bf
		.byte <$69f
posh            .byte >$4bf
		.byte >$69f
		.pend
;---------------------------
; Get parameter
;
getparm		.proc
		inc m_p1
		bne +
		inc m_p1+1
+		lda (m_p1),y
		rts
		.pend
;---------------------------
; Restore a window
;
; .word screen_position
; .byte width-1, height-1
;
waitkey_restorewin .proc
		#stz_a 198
-               jsr $f142
                beq -
                pha
                jsr restorewin
-		lda $c5
                eor #$40
                bne -
                sta 198
                pla
                rts

restorewin	.proc
-		lda $d011
		bpl -
                lda man_savedwind
                sec
                sbc #6
                sta man_savedwind
                clc
		adc #<(man_savedwin-5)
                sta m_p1
                #lda0
                tay
                adc #>(man_savedwin-5)
                sta m_p1+1
                jsr savewin.wp
		lda #1
		gne savewin.in
;---------------------------
; Save a window
;
; .word screen_position
; .byte width-1, height-1
;
savedrawwin    	.proc
		sec
                .byte SKIP1BCC
;---------------------------
; Save a window
;
; .word screen_position
; .byte width-1, height-1
;
savewin		.proc
		clc
		#plx_a
		stx m_p1
		pla
		sta m_p1+1
                bcc x
                pha
                #phx_a
x
		ldy #4
                ldx man_savedwind
-		lda (m_p1),y
                dex
                sta man_savedwin,x
		dey
                bne -

		php
		jsr wp
                plp
                bcs +

		#psz m_p1
+
		php
		#lda0
                jsr in
                #ldatax man_savedwind
                clc
                adc #6
                sta man_savedwind
		lda m_p1
		sta man_savedwin,x
		lda m_p1+1
		sta man_savedwin+1,x
                plp
                bcs drawwin
                rts

in		sta m_ascii+2
		ldx man_savedwind
		lda man_savedwin-6,x
		sta m_p1
		lda man_savedwin-5,x
		sta m_p1+1

lp		ldx m_ascii
		ldy m_ascii+2
		beq fy
		dey
-		lda (m_p1),y
		sta (m_p3),y
                jsr getparm
;		inc m_p1
;		bne *+4
;		inc m_p1+1
;		lda (m_p1),y
		sta (m_p4),y
		iny
		dex
		bpl -
		gmi rf

fy		lda (m_p3),y
		sta (m_p1),y
                jsr getparm
;       	inc m_p1
;       	bne *+4
;       	inc m_p1+1
		lda (m_p4),y
		sta (m_p1),y
		iny
		dex
		bpl fy

rf		tya
		clc
		adc m_p1
		sta m_p1
		bcc djx
                clc
		inc m_p1+1
djx
		lda m_p3
		adc #40
		sta m_p3
		sta m_p4
		bcc +
		inc m_p3+1
		inc m_p4+1
+
		dec m_ascii+1
		bpl lp
		rts

wp		jsr getparm
		sta m_p3
		sta m_p4
		jsr getparm
		sta m_p3+1
		eor #$d8^$04
		sta m_p4+1
		jsr getparm
		sta m_ascii
		jsr getparm
		sta m_ascii+1
                rts
		.pend
;---------------------------
; Draw a window
;
; .word screen_position		($00xx mean it's in X/Y)
; .byte width-1, height-1
; .word text_in_memory
;
drawwin_nowait	.proc
		sec
		.byte SKIP1BCC
drawwin       	.proc
		clc
		pla
		sta m_p1
		pla
		sta m_p1+1
                sty m_p4+1

		ldy #0
                jsr getparm
                sta m_p4
                jsr getparm
                bne vd2
                stx m_p4
                .byte SKIP2
vd2             sta m_p4+1
		jsr getparm
		sta m_ascii
		jsr getparm
		sta m_ascii+1
		jsr getparm
		sta m_p3
		jsr getparm
		sta m_p3+1

		#psz m_p1

		lda m_p4
		sta m_p1
		sta m_p2
		lda m_p4+1
		sta m_p1+1
		eor #$d8^$04
		sta m_p2+1

		ldx manager_win_color

		.if DECORATION=1
		lda #248
		sta m_ascii+2
		ldy #108
		lda #123
                .else
		lda #64+128
		sta m_ascii+2
		ldy #112+128
		lda #110+128
                .fi

		bcs now 	;faster copy window draw
-		bit $d011
		bpl -
now
		jsr windrln

lp		lda m_p1
		clc
		adc #40
		sta m_p1
		sta m_p2
		bcc ni
		inc m_p1+1
		inc m_p2+1
ni
		dec m_ascii+1
		beq ki

		lda #32+128
		sta m_ascii+2
                .if DECORATION=1
                ldy #244;245
                lda #234;246
                .else
		lda #93+128
                tay
                .fi
		jsr windrln
		#jmps lp
ki
		.if DECORATION=1
		lda #249
		sta m_ascii+2
		ldy #124
		lda #126
                .else
		lda #64+128
		sta m_ascii+2
		ldy #109+128
		lda #125+128
                .fi
		jsr windrln

drawtext	.proc
fr2		ldy #0
fr		lda (m_p3),y
		bmi ok
		beq vege
		cmp #13
		bne o1
rp		tya
		adc m_p3
		sta m_p3
		bcc +
		inc m_p3+1
+		lda m_p4
		clc
		adc #40
		sta m_p4
		bcc fr2
		inc m_p4+1
		gcs fr2

o1		cmp #29
		beq oo
                cmp #2
                bne vd
                iny
                lda (m_p3),y
                sta m_p1
                lda m_p3
                sbc m_p1
                sta m_p3
                bcs +
                dec m_p3+1
+               tya
                sec
                adc m_p1
                tay
                bne fr

vd		cmp #1
		bne okk
		tya
		clc
		adc m_p4
		sta m_p1
		sta m_p2
                #lda0
		adc m_p4+1
		sta m_p1+1
		eor #$d8^$04
		sta m_p2+1

		#phy_a
                .if DECORATION=1
		lda #247;64+128
		sta m_ascii+2
		ldy #244;245
		lda #234;246
                .else
		lda #64+128
		sta m_ascii+2
		ldy #107+128
		lda #115+128
                .fi
		jsr windrln
		#ply_a
		sec
		gcs rp

okk		eor m_d018
ok		sta (m_p4),y
oo		iny
		bne fr
vege		rts
		.pend
;---------------------------
; Draw 1 line of window
;
windrln		.proc
		sty m_ascii+3
		ldy m_ascii
		sta (m_p1),y
		txa
		sta (m_p2),y
		dey
-		lda m_ascii+2
		sta (m_p1),y
		txa
		sta (m_p2),y
		dey
		bne -
		lda m_ascii+3
		sta (m_p1),y
		txa
		sta (m_p2),y
		rts
		.pend
;---------------------------
; Draw panel
;
redrwincap	.proc
		ldx selectedwin
                lda posl,x
                ldy posh,x
		sta m_p4
		sty m_p4+1
                clc
                adc #231
                sta m_p2
                bcc +
                iny
+               sty m_p2+1

		ldy #68
		lda #30+128
-		sta (m_p4),y
		tya
		clc
		adc #40
		tay
		bcc +
		inc m_p4+1
+		lda #93+128
		cpy #212
		bne -
		lda #113+128
		sta (m_p4),y
                dec m_p4+1

                ldx #winbar2-winbar
                #stz_a m_p3
		ldy #39
lp3		lda m_p3
		bne ti
                dex
		lda winbar,x
                bmi dsc
                cmp #32
                bcs sc
                sta m_p3
ti              dec m_p3
		lda (m_p4),y
                ora #128
                gmi dsc
sc              eor m_d018
dsc		sta (m_p4),y
		dey
		bpl lp3

                asl m_d018
                lsr m_d018

                lda #<winbar2
                sta m_p3
                lda #>winbar2
                sta m_p3+1
		jsr drawtext
                asl m_d018
                sec
                ror m_d018
                #jmps man_printfullfree

posl		.byte <$0428
		.byte <$0608
posh		.byte >$0428
		.byte >$0608

winbar		.text " "+128,"device",":"+128
		.text 8," "+128
		.text "label",":"+128
		.text 16," "+128
winbar2         .text 13,13,2,28,"blocks"
		.text 13,13,2,28,"files"
                .text 13,13
		.null 13,13,2,28,"drive part"
		.pend
;---------------------------
; main copy
;
COPY_E_INPUT	= 15
COPY_E_OUTPUT	= 16
COPY_INPUT	= 17
COPY_OUTPUT	= 18

main_copy	.proc
		jsr no_files_or_partlist_or_virtual
		bne +			;empty panel?
		rts
+
		#stz_a man_overwrite	;no overwrite
		#stz_aa man_enterall 	;no direnter
                #stz_aa man_recurs_mode	;copy mode
                #stz_aa man_abort	;no abort

                ldy selectedwin+1
                lda #COPY_E_OUTPUT
		jsr open_error		;open error channels
                ldy selectedwin
                lda #COPY_E_INPUT
		jsr open_error

                jsr check_current

                jsr savewin		;directory copy
		.word $543
		.byte 33,10

		lda selectedwin+1
                jsr rel_path_m_p1.in
                jsr rel_path_m_p1s.in
                sty man_recurs_len

                tsx
                stx man_recurs_sp

		#stz_a man_multi
		#stz_aa man_multi+1

lp		ldy selectedwin
		#ldatax man_multi
		cmp man_files_l,y
		lda man_multi+1
		sbc man_files_h,y
		bcs ss2
		lda man_multi+1
                lsr
		txa
		jsr calc_filepos
                #lda0
		jsr man_sel
		bpl k
                jsr processfilename
                bcc k
		jsr man_toggle_sel
                #stz_a man_recurs

                jsr man_dir
                bpl reg
                sta man_ask		;no ask on dir
                jsr copyd
                #jmps tu

reg             jsr copyf
tu              jsr changepanelin

                jsr check_abort
                bne ss2

k		inc man_multi
		bne lp
		inc man_multi+1
		gne lp

ss2		jsr restorewin
		jsr printdirandstat
		jsr changepanelout

		jsr close_error
		jsr modifieddir
                jmp changepanel

ce		jsr close_error
		#jmps setdev

changepanelout	lda #COPY_E_OUTPUT
		.byte SKIP2
changepanelin	lda #COPY_E_INPUT
		sta man_redirect
		jmp changepanel

close_error	#stz_a man_redirect
		lda #COPY_E_INPUT
		jsr newclose_t
		lda #COPY_E_OUTPUT
		jmp newclose_t

open_error      ldx man_drive,y
                ldy #15
                jsr $ffba
                #stz_a ActualFNLength
                jmp newopen_t

		.pend
;-------------------------------------------
;Copy a dir (recursive)
;
copyd		.proc
		ldx #3
                jsr man_copyfname3
                ldy #0
		jsr processfilename.in2

		lda man_recurs_mode
                beq copy

                ldy #20
nou2    	cpy #17
		bcs tp
		ldx man_input+2,y
	       	bne +
tp	       	ldx #32
+	       	lda cconv,x
	        eor #128
        	sta $611,y
	       	dey
	       	bne nou2
                geq nocopy1

copy	  	jsr drawwin_nowait
                .word $543
		.byte 33,10
		.word copyf.txtcopy2

		ldy #20
ndu2    	cpy #17
		bcs tp2
		ldx $203,y
		bne +
tp2		ldx #32
+		lda cconv,x
                eor #128
		sta $5c3,y
		sta $613,y
		dey
		bne ndu2

                jsr main_copy.changepanelout  	;target panel

                jsr mkdir		;try to make dir
                jsr err_is_exists
                beq kku
                jsr showerror
                bcs nne
skipdirx        rts			;jmp skipdir

kku		lda man_enterall	;dir exists
		cmp #"s"
                beq skipdirx
                cmp #"a"
		beq nne

		jsr savedrawwin
                .word $571
		.byte 21,8
                .word txtexists

		ldy #3
-		ldx $201,y
		cpy $200
		bcs +
		lda cconv,x
                eor #$80
		sta $5ea-3,y
		iny
                cpy #19
		bcc -
+
                jsr waitkey_restorewin
                cmp #"s"
                bne +
                sta man_enterall
                lda #"n"
+		cmp #"a"
                bne +
                sta man_enterall
                lda #"y"
+		cmp #"y"
                bne skipdirx


nne             jsr chdir		;enter target dir
                bcc skipdirx

		jsr append_actualfn_plus3_to_path

                jsr main_copy.changepanelin		;change to source panel

nocopy1
		jsr chdir		;enter source directory
                bcc skipdir2

		jsr append_actualfn_plus3_to_path
                jsr topdir

		lda man_recurs_mode
                bne nb			;not copy

                jsr same_part
                bne nb			;not the same partition

                jsr rel_path_m_p1
                sta m_p2+1
                sty m_p2		;source
                lda selectedwin+1
                jsr rel_path_m_p1.in	;dest

		ldy #0
llk             lda (m_p2),y
                cmp (m_p1),y
                bne jo
                tax
                beq bajx
                iny
                gne llk
jo
		tax
                bne nb
                cpy man_recurs_len
                bcc nb
bajx		#jmps baj

nb		inc man_recurs

		lda man_recurs
                ldx ActualDev
                tay
                iny
                jsr $ffba
                lda #1
                ldx #<$ffd0
                ldy #>$ffd0
                jsr $ffbd
                jsr newopen_t	;open new source directory
err             jsr basic_error
                bcs tu
                jsr read_errchannel
tu              jsr showerror
		bcc skipd3

                ldx man_recurs
                #stz_a man_dirpos,x
                jsr newchkin_t
                bcs err

                jsr newgetin_t  ;skip dirlabel
                eor #"i"
                sta man_nonide64rec
                beq xm
                ldx #221
-		jsr newgetin_t
                dex
                bne -

xm              jsr readline

		lda Status
		pha
loop            pla
		beq jd
skipd3          #jmps skipdir3

jd		ldx man_recurs	;directory read loop
                jsr newchkin_t
                bcs err

                ldx man_recurs
                lda man_dirpos,x
                inc man_dirpos,x
		ldx man_nonide64rec
                beq twx
                and #7
                bne twx
                ldx #226
                .byte SKIP2
twx		ldx #224
		jsr readline2

                lda Status
                pha

                jsr newclrchn_t

                lda man_nonide64rec
		beq joi
                lda $206
                pha
                lda $222
                sta $206
                lda $223
                sta $207
                ldx #16
xt              dex
		beq kq
		lda $209,x
                cmp #$a0
                beq xt
kq		#stz_a $20a,x
                lda #6
                sta m_p1
                lda #2
                sta m_p1+1
                pla
                and #%10001111
                cmp #$81
		bcc kek
                cmp #$87
                bcs kek
                tax
                ldy #2
cek             lda typ-$81,x
                sta $21b,y
                .if SUPERCPU=1
                txa
                adc #6
                tax
                .else
                lda #$ff
                sbx #256-6
                .fi
                dey
                bpl cek

		cpx #$86+18
                beq thisd
                gne thisnd

typ             .text "qgrlmr"
                .text "ersebi"
                .text "spurcd"

joi		lda #255
		clc
		adc $214
                #lda0
                adc $215
                tax
                #lda0
                adc $216
                tay
                #lda0
                adc $217
                beq +
                ldx #255
                #txy_a
+               stx $201
                sty $202
                ldx #0    ;0123456789012345678901
                stx $214  ;LH"1234567890123456" PRG
                inx
		stx m_p1
		inx
                stx m_p1+1
-               lda $21d,x
		bne +
                lda #32
+               sta $216,x
                dex
                bpl -

                lda $21c
                and #FTYPE_MASK|FATTR_CLOSED
                beq kek				;deleted
                cmp #FTYPE_DIR|FATTR_CLOSED
                bne nio
thisd           jsr ispp
                beq loopx
                sec
                jsr ispp.point
loopx           beq loop
                jsr copyd
                #jmps ir

nio		ldy man_recurs_mode
		bne thisnd			;not copy
		cmp #FTYPE_NORMAL|FATTR_CLOSED
		bne kek
thisnd		ldy #0
		lda (m_p1),y
                pha
                iny
		lda (m_p1),y
                pha
		jsr man_copyfname
                #plx_a
                pla
                ldy #0
		jsr processfilename.in2
		jsr copyf
ir              lda man_recurs_mode
                bne kek				;not copy
                jsr main_copy.changepanelin		;change to source panel

kek             jsr check_abort
                beq loopx
                pla			;pop status

skipdir3	jsr newclrchn_t		;end of directory
		lda man_recurs
		jsr newclose_t		;close dir
                dec man_recurs
		jsr pont		;go into parent dir
                bcc exit

baj             jsr remove_last_path_part

		sec
skipdir2       	lda man_recurs_mode
                beq skipdir2x		;it's copy
                bcc skipdir		;could not cd, then no remove
                stx man_input+3
                ldx #255
n	        inx
                iny
                lda (m_p1),y
                sta man_input+4,x
                bne n
                sta man_input+3,x
		jsr topdir
                ldy #I_RD
                jmp putandexecinput_showerror

skipdir2x      	jsr topdir		;display dirname after leave
                jsr main_copy.changepanelout     	;enter destination panel

                jsr pont		;go into parent
                bcs remove_last_path_part

exit		ldx man_recurs_sp	;not recoverable error
		inx
                inx
                txs
                jsr newclall_t
                jmp manager.redraw

readline        ldx #224
readline2       jsr newgetin_t
		sta $204-224,x
                inx
                bne readline2
skipdir		rts

pont            ldx #3
                stx $200
                ldx #$5f
                .byte SKIP2
chdir           ldx #":"
		lda #"c"
                jsr chdu
		jmp showerror

mkdir           ldx #":"
		lda #"m"
chdu    	sta $201
                lda #"d"
                sta $202
		stx $203
                jsr setname200
		jmp read_errchannel.inn

txtexists       .text 13,2,4,"dir exists",13,1
		.text 13,13,2,6,"enter","?"+128,13,13
		.null 29,"yes","/"+128,"no","/"+128,"all","/"+128,"skip",160,"all","!"+128
		.pend
;-------------------------------------------
;Copy a file
;
copyf           .proc
		lda man_recurs_mode
                beq copy

                ldy #20
-	    	ldx $201-1,y
	       	bne +
	       	ldx #32
+	       	lda cconv,x
	        eor #128
        	sta $611,y
	       	dey
	       	bne -

                ldy #0
-	    	iny
		lda $200,y
                cmp #","
                bne +
                lda #"="
+		sta man_input+1,y
                tax
		bne -

		ldy #I_S
                jmp putandexecinput_showerror

copy		ldx #19
		clc
di	        ldy #3
kf              lda $201,x
		eor ill,y
                bne +
                lda #"?"
                sta $201,x
+		lda man_input,x
		eor ill,y
                bne +
                sec
+
		dey
                bpl kf
                dex
                bpl di
                bcs ajj

		lda man_ask
		bne noask
ajj
		jsr askname
                beq ss2
                sta man_ask
                gne copy

ss2		jsr check_abort.in
		beq ajj
		gne main_copy.changepanelout;change to destination panel

ill		.text ":*?="

noask
		jsr drawwin_nowait
                .word $543
		.byte 33,10
		.word txtcopy2			;big copy window

		ldy #20
-	    	ldx $200,y
		bne +
		ldx #32
+		lda cconv,x
                eor #128
		sta $5c3,y
		ldx man_input-1,y
		bne +
		ldx #32
+		lda cconv,x
                eor #128
		sta $613,y
		dey
		bne -

		lda #COPY_E_INPUT
                sta man_redirect

                lda #COPY_INPUT
		ldx ActualDev
;                ldy #0
                jsr $ffba
                lda ActualFNLength
                jsr setname201

		jsr newopen_t		;open source file
                jsr basic_error
                bcs xdx

		jsr inputto200

                jsr read_errchannel
xdx		jsr showerror
jh              php

                jsr main_copy.changepanelout;change to destination panel
                plp
                bcc skipfile2	  	;file open error

dfd             jsr copyoutput

                jsr setname200

                jsr newopen_t		;open destination file
                bcs noexistserr
                jsr read_errchannel
                lda #"3"
                ldx #"9"
                jsr err_is_exists.in
                bne fdf
                jsr close_output
                ldx #19
-		lda $201,x
                sta man_input,x
                dex
                bpl -
                jsr askname
                jsr inputto200
                geq dfd

fdf             jsr err_is_exists
                bne noexists

		lda man_overwrite
                cmp #"a"
                beq nnk
                cmp #"s"
                beq skipfilex

		jsr savedrawwin
                .word $571
		.byte 21,8
		.word txtexists

		ldy #19
ndu		ldx $201,y
		cpy $200
		bcc +
		ldx #32
+		lda cconv,x
                eor #$80
		sta $5ea,y
		dey
		bpl ndu

		jsr waitkey_restorewin
                cmp #"s"
                bne bb
                sta man_overwrite
                lda #"n"
bb              cmp #"a"
                bne xn
                sta man_overwrite
                lda #"y"
xn		cmp #"y"
                beq nnk
skipfilex       jmp skipfile

nnk             jsr close_output

                ldx $200
-		lda $201,x
                sta $204,x
                dex
                bpl -
                ldx #2
-		lda rew,x
                sta $201,x
                dex
                bpl -

                jsr copyoutput

		lda $200
                clc
                adc #3
                jsr setname201

                jsr newopen_t
noexistserr     jsr basic_error
                bcs noexists
                jsr read_errchannel

noexists        jsr showerror
		bcc skipfilex

                lda #64
                #sta_cfg_open
                sta manager_chkin_exp	;IDE64
                sta manager_chkout_exp	;IDE64
                #sta_cfg_16

		jsr selectoutput
		jsr selectinput

		ldx selectedwin
                ldy dirm,x
                iny
                sty m_p1+1
                #stz_a m_p1
                #stz_aa m_p3
                #stz_aa m_p3+1
                #stz_aa m_p4
                #stz_aa man_copybar+3
		#stz_aa m_p5
		#stz_aa m_p5+1
                #stz_aa m_p4+1
                #stz_aa man_offs
                lda #<$664
                sta m_p2
                lda #>$664
                sta m_p2+1

                ldx #31
-		lda man_dirmem2,x
                sta man_temp,x
                dex
                bpl -
                #sta_cfg_open
                #ldx_l_a manager_chkin_exp
                stx man_fast
                lda manager_chkout_exp
                sta man_fast2
                #sta_cfg_16
                beq re2		;destination serial!
;---
                txa
                bne bigcopyloop  	;source not serial!

                jsr man_fastload_init
                #jmps csc2
;---
re2		txa
                beq bigcopyloop          ;source serial!

                jsr man_fastsave_init

csc2            jsr selectoutput2
		jsr selectinput2

bigcopyloop     lda m_p1+1
                sta m_ascii+2
                bit man_fast	;fastload?
		bpl old
                jsr man_fastload
                #tyx_a
                lda m_p1+1
                sec
                sbc m_ascii+2
                tay
                #jmps kb2

old		ldx #0
                ldy #64
		lda man_fast2   ;fastsave?
                bpl tiu
                stx m_p1
                ldx #<8128
                ldy #>8128
tiu             jsr sizeeor
		bvc seri	;ide64?
                clc
                jsr readwrite
                cli
                bcc mv
                cmp #KERNAL_FILE_DATA
                beq mvy
                jsr selectoutput2
seri            jsr selectinput2
                lda man_offs
                tay
                clc
                adc m_ascii
                sta m_ascii
                bcc vf
                inc m_ascii+1
vf              #stz_a man_offs
nvx             inc m_ascii
		beq v1
vm		jsr newchrin_t
                sta (m_p1),y
                lda Status
                bne jo
                iny
                bne nvx
                jsr man_showbar1
                inc m_p1+1
                gne nvx

v1              inc m_ascii+1
                bne vm
kb
                #tyx_a
                ldy #0
kb2             jsr man_showbar2
                gne mvj

jo              cmp #$42
		beq kb
                iny
                bne kb
                inc m_p1+1
                gne kb

mvy             pha
                lda #COPY_E_INPUT
		gcs mvy3

mv		jsr man_showbar
mvj		lda Status
                pha

		jsr sizeeor

		bit man_fast2	;fastsave?
                bpl old2
                jsr man_fastsave
                #tyx_a
                lda m_p1+1
                sec
                sbc m_ascii+2
                tay
                #jmps hd

readwrite       #psr gotobank0back
                lda #m_p1
                sei
                bcc read_dt
                gcs write_dt	;newwrite_t

old2            bvc seri2	;ide64?
		sec
		jsr readwrite
                cli
                bcc mv2
                cmp #KERNAL_FILE_DATA
                beq mvy2
                jsr selectinput2
seri2           jsr selectoutput2
                lda m_p1+1
                sta m_ascii+2
                ldy #0
                jsr copy_upper

		#tyx_a
                ldy #0
hd              jsr man_showbar2
                gne mv2j

mvy2            lda #COPY_E_OUTPUT
mvy3            jsr redir_showerror
                bcc mv2
                lda #KERNAL_FILE_DATA
                jsr basic_error
                jsr showerror
mv2		jsr man_showbar
mv2j		lda Status
                bne baj
                tya
                clc
                adc m_p3
                sta m_p3
                bcc vxs
                inc m_p3+1
                bne vxs
                inc m_p4
vxs
                tax
		lda m_p3+1
                ldy m_p4
		jsr numtoasc24

                jsr check_abort
                bne baj

                pla
                beq bigcopyloop
                .byte SKIP1BEQ
baj		pla

baj2		ldx #31
cx2		lda man_temp,x
                sta man_dirmem2,x
                dex
                bpl cx2

skipfile3      	jsr newclrchn_t
skipfile        jsr close_output
		lda #COPY_E_OUTPUT
                jsr redir_showerror

skipfile2      	lda #COPY_INPUT
                jsr newclose_t
		lda #COPY_E_INPUT
                jmp redir_showerror

sizeeor		txa
                eor #$ff
                sta m_ascii
                tya
                eor #$ff
                sta m_ascii+1
nin             rts

copyoutput     	lda #COPY_OUTPUT
                ldx ActualDev
                ldy #1
                gne $ffba

close_output    lda #COPY_OUTPUT
		jmp newclose_t
;-----------------------
;Redirect and show error
;
redir_showerror sta man_redirect
                jsr read_errchannel
                #jmps showerror
;-----------------------
;Select input/output, if error then abort
;
selectoutput2	lda man_fast2
		bmi nin
selectoutput    ldx #COPY_OUTPUT
                jsr newchkout_t
                #jmps at
selectinput2	lda man_fast
		bmi nin
selectinput     ldx #COPY_INPUT
                jsr newchkin_t
at              bcc nin
		jsr basic_error
                jsr showerror
                pla
                pla
                gcc skipfile3

rew		.text "@0:"
txtcopy2       	.text 13,2,11,"copying",13,1
		.text 29,"from",":"+128,13,13
		.text 29,"to",":"+128,13,13
		.null 29,"blocks",":"+128,160,"0"+128

txtexists       .text 13,2,4,"file exists",13,1
		.text 13,13,2,4,"overwrite","?"+128,13,13
		.null 29,"yes","/"+128,"no","/"+128,"all","/"+128,"skip",160,"all","!"+128
                .pend

check_abort     .proc
		jsr $f142
                beq nos2
                cmp #3
                bne nos2

in		ldx #2
dd		lda $0,x
                sta man_temp+32,x
                inx
                bne dd
re		jsr savedrawwin
                .word $59f
		.byte 9,6
                .word txtabort
                jsr waitkey_restorewin
                cmp #3
                beq re
		cmp #"y"
                bne nos
                sta man_abort
nos		ldx #2
dd2		lda man_temp+32,x
                sta $0,x
                inx
                bne dd2
nos2		lda man_abort
		rts

txtabort        .text 13,13,29,29,"abort","?"+128,13,13
		.null 29,29,"yes","/"+128,"no"
		.pend
;---------------------------
; process filename for copy/delete
;
; C=0 on error
;
processfilename	.proc
		lda m_p1+1
                cmp #>man_dirmem1
                bcc err

		jsr man_copyfname
		ldy #1
                #ldatax (m_p1),y
                dey
                lda (m_p1),y
in2             asl
                sta man_copybar
                txa
                rol
                sta man_copybar+1
                tya
                rol
                sta man_copybar+2
nd		lda (ActualFilename),y
		cpy ActualFNLength
		bcc vf
		#lda0
                sta (ActualFilename),y
vf		sta man_input,y
		iny
		cpy #20
		bne nd
err             rts
		.pend
;---------------------------
; main mkdir
;
main_mkdir	.proc
                lda man_parlist,x
                lsr
                ora man_virtual,x
                bne xjx

		jsr savedrawwin
                .word $59b
		.byte 17,4
		.word txtmkdir

		ldx #<$614
		ldy #>$614
		lda #16
		jsr clrinput
		beq xji                 ;stop?

		ldx #16
-		lda man_input,x
		sta man_input+3,x
                sta man_jumpon,x
		dex
		bpl -
                jsr jumptypedir

		ldy #I_MD
                jsr putandexecinput_showerror

		clc
xji	        php
		jsr restorewin
                plp
		bcc modifieddir
xjx		rts

txtmkdir	.text 13,2,3,"make"," "+128
		.pend
dirtype		.null "dir",13,1

jumptypedir	.proc
                ldx #3
-		lda dirtype-1,x
                sta man_jumpontype-1,x
                dex
                bne -
rrts            rts
		.pend
;---------------------------
; main delete
;
main_delete	.proc
		jsr no_files_or_partlist_or_virtual
		beq jumptypedir.rrts    ;exit

		jsr check_current

                lda #"n"
                sta 631
                sta man_recurs_mode	;remove mode

                ldy selectedwin
                lda #COPY_E_INPUT
		jsr main_copy.open_error
		lda #COPY_E_INPUT
                sta man_redirect

		jsr savedrawwin
                .word $599
		.byte 21,4
		.word txtdelete2

		#stz_a man_multi
		#stz_aa man_multi+1

lp		ldy selectedwin
		#ldatax man_multi
		cmp man_files_l,y
		lda man_multi+1
		sbc man_files_h,y
		bcs ss2
		lda man_multi+1
                lsr
		txa
		jsr calc_filepos
                #lda0
		jsr man_sel
		bpl k
                jsr man_dir
                sta m_p5
                jsr processfilename
                bcc k
		jsr man_toggle_sel

                lda 631
                ldx #0
                stx 198
                cmp #"a"
                beq all

		#psz m_p1

		jsr savedrawwin
                .word $571
		.byte 21,6
		.word txtdelete

		ldy #19
ndu		ldx man_input,y
		bne +
		ldx #32
+		lda cconv,x
                eor #$80
		sta $5ea,y
		dey
		bpl ndu

		jsr waitkey_restorewin

		tax

		pla
                sta m_p1
                pla
                sta m_p1+1

                stx 631
                cpx #13
                beq all
                cpx #"y"
                beq all
                cpx #"a"
                beq all
                cpx #3
                bne no

ss2		jsr restorewin
		jsr main_copy.close_error
		#jmps modifieddir

all
		#stz_a man_recurs
                #stz_aa man_abort

		ldy m_p5
                bpl fil
                jsr copyd
                #jmps no
fil             jsr copyf

no
		lda man_abort
                bne ss2
k		inc man_multi
		bne lp
		inc man_multi+1
		gne lp

txtdelete      	.text 13,2,6,"delete",13,1,13,13
		.null 2,4,"yes","/"+128,"no","/"+128,"all"

txtdelete2     	.null 13,2,5,"deleting",13,1
		.pend
;---------------------------
; main rename
;
main_rename	.proc
		jsr no_files_or_partlist_or_virtual
		bne +
err             rts
+
	    	jsr calc_filepos_cursor
		jsr processfilename
                bcc err

		jsr savedrawwin
                .word $571
		.byte 21,8
		.word txtrename

                #stz_a $200+20
		ldy #19
-		lda man_input,y
		sta $200,y
		tax
		bne +
		ldx #32
+		lda cconv,x
		sta $612,y
                lda manager_input_color
                sta $da12,y
		dey
		bpl -

		ldx #<$68a
		ldy #>$68a
		lda #20
		jsr input
		beq xji                 ;stop?

		ldx #20
                #lda0
                beq kj
ul		lda man_input,x
kj		sta man_input+2,x
		lda rntxt,x
		sta man_input,x
		dex
		bpl ul

;                ldx #255
                lda #","
-		inx
		cmp $200,x
                bne -
                ldy #0
cc              inx
                lda $200,x
                bne +
                dex
                lda #32
+		sta man_jumpontype,y
                iny
                cpy #3
                bcc cc

                ldx #255
pi              inx
		cpx #16
                bcs en2
		lda man_input+2,x
                beq en
                sta man_jumpon,x
                eor #","
                bne pi
en		sta man_jumpon,x
en2

		ldy selectedwin
		lda man_floppy,y
                ora man_cmd,y		;CMD or FLOPPY
                lsr
                bcs nq

		lda man_input+2,x	;no type
                beq nq

                ldy #0
cc2             lda man_input+3,x
                bne knu
                lda #32
                dex
knu             inx
		sta man_jumpontype,y
                iny
                cpy #3
                bcc cc2
		clc

nq              inx
                lda man_input,x
                beq nx
                eor #","
                bne nq
                bcc nq
nx
                lda #"="
                sta man_input,x
                ldy #255
nq2             iny
                inx
                lda $200,y
                sta man_input,x
                beq nx2
                eor #","
                bne nq2
                bcc nq2
                sta man_input,x
nx2

		jsr do_errchannel
                jsr showerror

		clc
xji	        php
		jsr restorewin
                plp
		bcc modifieddir
		rts


rntxt   	.text "r:"
txtrename      	.text 13,2,6,"rename",13,1
		.text 29,"from",":"+128,13,13,13
		.null 29,"to",":"+128
		.pend
;---------------------------

man_showbar2    .proc
		jsr man_showbar
                lda m_p1+1
                sec
                sbc m_ascii+2
                tay
                lda m_ascii+2
                sta m_p1+1
                rts

man_showbar1	.proc
		#phx_a
                #phy_a
                inc m_p5+1
                bne man_showbar.in
                geq man_showbar.in2

man_showbar	.proc
		txa
                pha
                clc
                adc m_p5
                sta m_p5
                tya
                pha
                adc m_p5+1
                sta m_p5+1
                bcc in
in2             inc m_p4+1
in
                sec
xc		lda m_p5
                sbc man_copybar
                tay
		lda m_p5+1
		sbc man_copybar+1
                tax
                lda m_p4+1
                sbc man_copybar+2
                bcc we
                sta m_p4+1
                sty m_p5
                stx m_p5+1
                inc man_copybar+3
                bne xc
                dec man_copybar+3
we
		#ldatax man_copybar+3
                and #7
                tay
                txa
                lsr
                lsr
                lsr
                tax
                lda bar,y
jw              sta $6ac,x
                lda #32
                dex
                bpl jw
                #ply_a
                #plx_a
jooo            rts
bar		.byte 229,244,245,225,118,106,103,32
		.pend
;---------------------------
; reload other panel too if change happend in same dir
;
modifieddir     .proc
		jsr main_reload
                jsr same_dir
                bne man_showbar.jooo
                jsr changepanel
                jsr main_reload2.in
                jmp changepanel
		.pend
;---------------------------
; main command
;
main_command	.proc
		lda man_virtual,x
                bne func
                lda man_floppy,x
                beq x

		jsr savedrawwin
                .word $595
		.byte 29,5
		.word txtcmd

		ldx #<$60e
		ldy #>$60e
		lda #28
		jsr clrinput
		php                     ;stop?
		beq xji

tk		jsr do_errchannel

		clc
xji		php

		jsr restorewin

                plp
                bcs +
                jsr showerror
+
		plp
		bne modifieddir
x		rts

func            lda #VIRTUAL_FUNCTION
                jsr man_virtual_c
                bcc x
		jsr +
+               jsr changepanel
                jmp main_reload

txtcmd		.null 13,2,7,"disk"," "+128,"command",13,1
		.pend
;---------------------------
; Path of current window into m_p1, Y at end of name
;
rel_path_m_p1s 	.proc
		jsr rel_path_m_p1
in              ldy #0
nvfs            lda (m_p1),y
                beq kmv
                iny
                bne nvfs
kmv		rts
		.pend

rel_path_m_p1 	.proc
                lda selectedwin
in              clc
                adc #>man_relpath
                sta m_p1+1
                ldy #<man_relpath
                sty m_p1
                rts
                .pend
;---------------------------
; Handle top menu
;
topmenu       	.proc
ncs             lda 653
		lsr
                bcs bs2
                lsr
                bcs ncs
                gcc csv

bs		bcs nnf
bs2		lda $d011
                bmi bs2
-		lda $d011
                bpl -
                lda $d018
                eor #2
                sta $d018
                lda m_d018
                eor #64
                sta m_d018

		lda selectedwin
                pha
                #stz_a selectedwin
                #stz_aa man_oldsc_h
                #stz_aa man_oldsc_h+1
                jsr redrwincap
                jsr scrollbar
		inc selectedwin
                jsr redrwincap
                jsr scrollbar
                pla
                sta selectedwin
                sec

nnf	    	lda 653
                eor #3
                beq bs
                lsr
                lsr
                bcc nnf
                rts

csv		#stz_a $d015

                ldx #15
u               dex
                beq xb
jf              jsr wait_a_bit
                dey
                bne jf
		jsr $f142
		beq u

		jsr $7c
                bcs colkey
		and #$0f
                cmp #8
                bcs n
                adc #10
n               pha
		jsr changepanel
                #ldatax selectedwin+1
                jsr updateselectedwin
		pla
		sta man_drive,x
		sta ActualDev
                jsr man_cdroot
		jmp main_reload2

colkey      	ldy #9
lpp		cmp key,y
		bne oo
                tya
                adc #8-1
		gcc n
oo		dey
		bpl lpp
xb      	rts			;exit

key		.byte 155,41,95,129,149,150,151,152,153,154
		.pend
;---------------------------
; Display current path on top of screen
;
topdir		.proc
		lda m_p2
                pha

                jsr rel_path_m_p1s
                #lda0
                cpy #44
                bcc prin
                tya
                sbc #43
prin
		clv

                sta m_p2
		ldy #3
lp3		lda manager_inactwincolor
                sta $d800-3,y
                bvs kr
		#ldatax (m_p1),y
                bne vm
                #sev set_v
kr              ldx #32
vm		lda cconv,x
		eor #$80
		sta $400-3,y
                cpy #21
                bne tp
                lda m_p2
                beq tp
                clc
                adc m_p1
                sta m_p1
                bcc +
                inc m_p1+1
+               iny
                lda #"="+128
                sta $400-3,y
                sta $401-3,y
                lda manager_inactwincolor
                sta $d800-3,y
                sta $d801-3,y
                iny
                clv
tp              iny
                cpy #43
		bcc lp3

                pla
                sta m_p2
set_v		rts
		.pend
;---------------------------
; Place selector bar on file
;
man_jumponfile  .proc
		clc
		ldx selectedwin
		lda man_jumpon
                ora man_jumppart
                bne man_jumponfile2.kell
                .if SUPERCPU=1
                inc a
                .else
                lda #1
                .fi
                sta m_ascii
                gne man_jumponfile2.x

man_jumponfile2 .proc
		sec

kell		lda #2
                sta m_ascii
	        bcs xe

lp		jsr calc_filepos_cursor
		lda man_jumppart
                beq ke
                ldy #0
                jsr man_get
                bne xe
                geq ki

ke              ldy #3
lp2             lda man_jumpon-3,y
                beq jo
                jsr man_get
                bne xe
                iny
                cpy #19
                bcc lp2
jo              ldy #21
lp3             lda man_jumpontype-21,y
                beq ki
                jsr man_get
                bne xej
joo             iny
                cpy #24
                bcc lp3
		gcs ki

xej		cmp #$20^"<"
		bne xe
		lda man_jumpontype-21,y
		cmp #$20
                beq joo

xe		lda man_bar,x
		jsr chk
		bcs x
vv		lda man_bar,x
		cmp #10
		beq o2
		cmp #5
		bcs o1
ui		inc man_bar,x
                gne lp

x		jsr gotop
                dec m_ascii
                bne lp
ki		#stz_a man_jumpon
		#stz_aa man_jumppart
		rts

o1 		beq o3
		lda #10
		jsr chk
                bcs ui
                dec man_bar,x
		inc man_el_l,x
		bne vv
		inc man_el_h,x
                gne vv

o3		lda #10
		jsr chk
		bcs ui

o2		inc man_el_l,x
		bne lp
		inc man_el_h,x
                gne lp

chk     	ldy man_el_h,x
		sec
                adc man_el_l,x
		bcc +
		iny
+		cmp man_files_l,x
		tya
		sbc man_files_h,x
		rts
		.pend
;---------------------------
; Manager data swap
;
manager_data_swap2andrestore .proc
		jsr restorescreen
                .if SUPERCPU=1
                jsr scpu_default_opt
                .fi
		.if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                #sta_cfg_open
                #lda0
                sta dos_accel_internal_exp
                .fi
		.assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
manager_data_swap2 .proc
		lda selectedwin
                sta man_selectedwin
		lda m_d018
                sta man_font
manager_data_swap .proc
		#sta_cfg_open
                .cerror (<man_data) | (<manager_data_exp)
                .if SUPERCPU=1
                ldy #1
                sty m_p1+2
                dey
                .else
                ldy #0
                .fi
                sty m_p3
                sty m_p1
		lda #>man_data
                sta m_p3+1
		lda #>manager_data_exp
                sta m_p1+1
lp		#ldatax (m_p3),y
                #lda_zp_y m_p1
                sta (m_p3),y
                txa
                #sta_zp_y m_p1
                dey
                bne lp
                inc m_p1+1
                lda m_p3+1
                inc m_p3+1
                cmp #>man_data_end
                bne lp
		geq newclall_t		;cfg_16
		.pend

;---------------------------
; Check/set manager sign
;
		#assert_8000romioonly
man_check_sign	.proc
		sec
set		ldx #38
ldl             #sta_cfg_open
                lda ident_label_exp-1,x
                #sta_cfg_16
                bcc vdv
                cmp man_ident_label-1,x
                bne vs
vdv             sta man_ident_label-1,x
                dex
                bne ldl
vs              rts
                .pend
;---------------------------
; Same directory?
;
		#assert_cfg_16
same_dir	.proc
		jsr same_part
                bne no
                tax
nvs             lda man_relpath,x
                cmp man_relpath+$100,x
                bne no
                tay
                beq no
                inx
                bne nvs
no		rts
		.pend
;---------------------------
; Same partition?
;
same_part	.proc
                lda man_drive
                eor man_drive+1
                bne no
                lda man_partition
                eor man_partition+1
no		rts		;A=0 if Z=1!!!
		.pend
;---------------------------
; Z=1 if there are no files or it's a partition list
;
no_files_or_partlist_or_virtual .proc
		lda man_virtual,x
                cmp #1
                beq no_files.xke
no_files_or_partlist .proc
                lda man_parlist,x
		cmp #3
                beq no_files.xke
no_files 	.proc
		lda man_files_l,x
		ora man_files_h,x
xke             rts
                .pend
;---------------------------
; Append the filename to path
;
append_actualfn_plus3_to_path .proc
		lda ActualFilename
		clc
                adc #3
                sta ActualFilename
                bcc append_actualfn_to_path
                inc ActualFilename+2

append_actualfn_to_path .proc
                jsr rel_path_m_p1s

                ldx #0
dmf             lda (ActualFilename,x)
                beq mfd
                sta (m_p1),y
                inc ActualFilename
                bne +
                inc ActualFilename+1
+               iny
                bne dmf
mfd		lda #"/"
		sta (m_p1),y
                iny
                txa
                sta (m_p1),y
                rts
                .pend
;---------------------------
; Remove last part of path
;
remove_last_path_part .proc
		jsr rel_path_m_p1s
                cpy #4
                bne kx
                rts

ini		jsr rel_path_m_p1
		ldy #3
nu              lda #"/"
                sta man_input+3
		ldx #4
		gne in

kx              dey
                lda #"/"
jcn             dey
		cpy #5
		bcc ini
                cmp (m_p1),y
                bne jcn

in      	lda #"/"
		sta (m_p1),y
                iny
                #ldatax (m_p1),y
		#lda0
                sta (m_p1),y
                rts
                .pend

;---------------------------
; Select current file if none selected
;
check_current	.proc
                ldx selectedwin
		lda man_filessel_l,x
		ora man_filessel_h,x
		sta man_ask
		bne inputto200.rrts

	  	jsr calc_filepos_cursor
		jmp man_toggle_sel
;---------------------------
; Copy input to $200,$201-
;
inputto200	.proc
		ldx #0
vsd		lda man_input,x
                beq kv
                sta $201,x
                inx
                cpx #20
                bne vsd
kv		stx $200
rrts		rts		;Z=1
                .pend
;---------------------------
; Ask filename
;
askname		.proc
		jsr savedrawwin
                .word $571
		.byte 21,8
		.word txtcopy

		ldy #19
ndu		ldx $201,y
		bne +
		ldx #32
+		lda cconv,x
		sta $612,y
                lda manager_input_color
                sta $da12,y
		dey
		bpl ndu

		ldx #<$68a
		ldy #>$68a
		lda #20
		jsr input
                pha
		jsr restorewin
                pla
                eor #3
                rts

txtcopy		.text 13,2,7,"copy",13,1
		.text 29,"from",":"+128,13,13,13
		.null 29,"to",":"+128
                .pend
;---------------------------
; Go to the top
;
gotop           .proc
		#stz_a man_bar,x
                #stz_aa man_el_l,x
                #stz_aa man_el_h,x
                rts
                .pend
;---------------------------
; Filename is ".."?
;
ispp            .proc
		clc
point		ldy #3
                lda #"."
                bcs fg
                cmp (m_p1),y
                bne mvm
                iny
fg              cmp (m_p1),y
                bne mvm
                iny
                lda (m_p1),y
mvm		rts
		.pend
;---------------------------
; Restore screen settings
;
restorescreen	.proc
		clc
-		lda $d011
		bpl -
-		lda $d011
		bmi -
save		ldx #46
vlp2		ldy man_old646,x
		lda man_oldvic,x
		bcc ie
                ldy 646,x
                lda $d000,x
ie      	sta $d000,x
		sta man_oldvic,x
		tya
                sta 646,x
                sta man_old646,x
		dex
		bpl vlp2
;---------------------------
; clear screen
;
clrscr		.proc
                lda #147
		jmp $ffd2
		.pend
;---------------------------
; Kill virtual filesystem
;
killvirtualredrwincap .proc
		jsr redrwincap
killvirtual	.proc
                jsr rel_path_m_p1
                ldy #255
nx              iny
		lda (m_p1),y
                beq updateselectedwin.rrts
                eor #":"
                bne nx
                lda #"/"
                sta (m_p1),y
                gne remove_last_path_part
                .pend
;---------------------------
; Filename setup
;
setnameinput    .proc
		ldx #<man_input
		ldy #>man_input
                gne setname201.in

setname200	.proc
		lda $200
setname201      .proc
		ldx #<$201
                ldy #>$201
in              jmp $ffbd
		.pend

updateselectedwin .proc
		and #1
                sta selectedwin
                eor #1
                sta selectedwin+1
rrts            rts
                .pend


                .if SUPERCPU=1          ;from man_hi
mloaddir2       .proc
		ldy #31
-		lda zeroblockfilesel.pontpont,y
		sta man_pontpont,y
		dey
		bpl -

		lda manager_dircolor
		sta man_pontpont+25
		sta man_pontpont+26

zeroblockfilesel .proc
                #stz_a man_block_l,x
		#stz_aa man_block_m,x
		#stz_aa man_block_h,x
		#stz_aa man_filessel_l,x
		#stz_aa man_filessel_h,x
                rts

pontpont	.word 0
		.text '"',".",".",0,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32
		.text "dir<",32,32,32,0,1,1
                .pend
                .fi
