;
; messages.asm - error channel message handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
		#assert_cfg_open

invalidate_drive .proc
		jsr OP_get_current_drive_type
		and #<~FS_MASK
		sta drive_type_exp,x	;Unknown filesystem!

		ldx #MAX_BUFFERS-1
-		lda buffer_logical_exp,x;buffer used?
		beq +
		lda buffer_drive_exp,x	;this drive?
		eor current_drive_exp
		bne +
		lda buffer_info_exp,x
		eor #INFO_STATUS
		beq +			;status channel skipped
                lda buffer_type_exp,x
                eor #BUFFERMODE_DIRECT
                beq +			;direct channel skipped
		lda #INFO_IGNORE	;file invalidated
		sta buffer_info_exp,x
+		dex
		bpl -
		rts
		.pend

LWOP_reporterror_write_xit .proc
		bcc *+3
		cmp #ETIMEOUT		;rts
		beq LWOP_reporterror_28_xit
		jsr commonerror
		lda error_trans_write,x
                bne LWOP_err
                rts
		.pend

;---------------------------------------
;Nomore data, reporterror
;---------------------------------------
XIT_end_of_transfer_atapi .proc
		lda #EATAPIERROR
                .byte SKIP2
XIT_end_of_transfer_ata .proc
		lda #EATAERROR
                pha
		lda status_g
                #lda_low_reg
                lsr
                pla

LWOP_reporterror_read_xit .proc
		bcc *+3
		cmp #ETIMEOUT		;rts
		beq LWOP_reporterror_21_xit
                jsr commonerror
		lda error_trans_read,x
                bne LWOP_err
		rts
                .pend

LWOP_reporterror_01_xit .proc
		lda #1
                .byte SKIP2
                .pend
LWOP_reporterror_77_xit
		lda #77
LWOP_reportxy   stx next_sector
                sty next_sector+1
                .if SUPERCPU=1
                stz next_sector+2
                stz next_sector+3
                .else
                pha
                lda #0
                sta next_sector+2
                sta next_sector+3
                pla
                .fi
                .byte SKIP2
LWOP_reporterror_21_xit .proc
		lda #21
		.byte SKIP2
                .pend
LWOP_reporterror_28_xit .proc
		lda #28
		.byte SKIP2
		.pend
LWOP_reporterror_71_xit .proc
		lda #71
		.byte SKIP2
		.pend
LWOP_reporterror_26_xit .proc
		lda #$80+26
		.byte SKIP2
		.pend
LWOP_reporterror_27_xit .proc
		lda #$80+27
		.byte SKIP2
		.pend
LWOP_reporterror_29_xit .proc
		lda #$80+29
		.byte SKIP2
		.pend
LWOP_reporterror_30_xit .proc
		lda #$80+30
		.byte SKIP2
		.pend
LWOP_reporterror_31_xit .proc
		lda #$80+31
		.byte SKIP2
		.pend
LWOP_reporterror_32_xit .proc
		lda #$80+32
		.byte SKIP2
		.pend
LWOP_reporterror_33_xit .proc
		lda #$80+33
		.byte SKIP2
		.pend
LWOP_reporterror_34_xit .proc
		lda #$80+34
		.byte SKIP2
		.pend
LWOP_reporterror_39_xit .proc
		lda #$80+39
		.byte SKIP2
		.pend
LWOP_reporterror_60_xit .proc
		lda #$80+60
		.byte SKIP2
		.pend
LWOP_reporterror_62_xit .proc
		lda #$80+62
		.byte SKIP2
		.pend
LWOP_reporterror_63_xit .proc
		lda #$80+63
		.byte SKIP2
		.pend
LWOP_reporterror_64_xit .proc
		lda #$80+64
		.byte SKIP2
		.pend
LWOP_reporterror_70_xit .proc
		lda #$80+70
		.byte SKIP2
		.pend
LWOP_reporterror_72_xit .proc
		lda #$80+72
		.byte SKIP2
		.pend
LWOP_reporterror_73_xit .proc
		lda #$80+73
		.byte SKIP2
		.pend
LWOP_reporterror_74_xit .proc
		lda #$80+74
		.byte SKIP2
		.pend
LWOP_reporterror_75_xit lda #$80+75
		bcc noer
LWOP_err       	cmp #$80
		bcs cu
                jsr LWOP_disk_status
                #jmps piei

cu		jsr seterrorpos
		cmp #29+$80
		bne piei
		jsr invalidate_drive
piei		.if DRIVES=4
                bit errledon_g
                .fi
                #xit_exit
		lda xit_bk_exp
		lsr

		.if BANK=0
		bcs noer
		lsr
		bcs exitbank1
		lsr
		bne exitbank3
		geq exitbank2
		.fi

		.if BANK=1
		bcs exitbank0
		lsr
		bcs noer
		lsr
		beq exitbank2
		gne exitbank3
		.fi

		.if BANK=2
		bcs exitbank0
		lsr
		bcs exitbank1
		lsr
		bne exitbank3
		.fi

		.if BANK=3
		bcs exitbank0
		lsr
		bcs exitbank1
		lsr
		beq exitbank2
		.fi
noer            rts

LWOP_reporterror_xit .proc
		bcc *+3
		cmp #ETIMEOUT
		beq +
		cmp #EUNEXP
+		beq LWOP_reporterror_74_xit
		cmp #EATAERROR
		beq th
		cmp #EATAPIERROR
                clc
		bne noer
		lda error_g
		#lda_low_reg
                and #$f0
		cmp #ATAPI_SENSE_UNIT_ATTENTION
		beq LWOP_reporterror_29_xit

th		sec
		gcs LWOP_reporterror_31_xit
		.pend

commonerror	.proc
		cmp #EUNEXP
		beq LWOP_reporterror_74_xit
		bit error_g
		cmp #EATAERROR
		bne nata
		tax
                .if FAST_DATA_TRANSFERS
                lda error_g
                .else
		lda low_reg
                .fi
nk		dex
		asl
		bcs b
		gne nk

nata		cmp #EATAPIERROR
		bne ok
                .if FAST_DATA_TRANSFERS
                lda error_g
                .else
		lda low_reg
                .fi
		lsr
		lsr
		lsr
		lsr
		tax
b               .byte SKIP2

ok		ldx #0
		clc
		rts
                .pend

		#assert_cfg_open
LWOP_reporterror_ok .proc
		lda #$80
                .if DRIVES=4
                bit errledoff_g
                .fi

		#assert_cfg_open
seterrorpos     .proc
		#ldx_l_ current_drive_exp
		sta status_pos_exp,x
		rts
		.pend

		#assert_8000romonly

error_trans_write .proc
		.byte 0			;no sense
		.byte 0			;recovered error
		.byte 74+$80		;not ready
		.byte 25		;medium error
		.byte 76+$80    	;hardware error
		.byte 67		;illegal request
		.byte 29+$80		;unit attention
		.byte 26+$80		;data protect
		.byte 22		;blank check
		.byte 20		;vendor specific
		.byte 20		;copy aborted
		.byte 74+$80		;aborted command
		.byte 20		;obsolote
		.byte 67		;volume overflow
		.byte 25		;miscompare
		.byte 20		;reserved

		.byte 22		;amnf
		.byte 67		;tk0nf
		.byte 25		;abrt
		.byte 29+$80		;mcr ???
		.byte 67		;idnf
		.byte 29+$80		;mc
		.byte 20		;unc
		.byte 25		;bbk
                .pend

error_trans_read .proc
		.byte 0			;no sense
		.byte 0			;recovered error
		.byte 74+$80		;not ready
		.byte 23		;medium error
		.byte 76+$80    	;hardware error
		.byte 66		;illegal request
		.byte 29+$80		;unit attention
		.byte 27+$80		;data protect
		.byte 22		;blank check
		.byte 20		;vendor specific
		.byte 20		;copy aborted
		.byte 74+$80		;aborted command
		.byte 20		;obsolote
		.byte 66		;volume overflow
		.byte 23		;miscompare
		.byte 20		;reserved

		.byte 22		;amnf
		.byte 66		;tk0nf
		.byte 23		;abrt
		.byte 29+$80		;mcr ???
		.byte 66		;idnf
		.byte 29+$80		;mc
		.byte 20		;unc
		.byte 23		;bbk
                .pend
