;
; misc2.asm - misc stuff $8000
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;---------------------------------------
;Print imediate
;---------------------------------------
		.assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO,CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
LWNO_print_imediate .proc
		pla
		sta $22
		pla
		jsr printall
                #psz $22
		rts
		.pend
;---------------------------------------
;Print A/Y
;---------------------------------------
		.assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO,CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
LWNO_print_ay	.proc
		sta $22
		sty $23
		ldy #0
		geq printall.cbv
;---------------------------------------
;Printing common
;---------------------------------------
		.assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO,CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
printall	.proc
		sta $23
		sty $24
		ldy #0
-		inc $22
		bne cbv
		inc $23
cbv		lda ($22),y
		beq +
		jsr newchrout_t		; chrout
		gcc -

+		ldy $24
		rts
		.pend
;---------------------------------------
; hdinit
; C=1 if error then drive_type_exp is 00
;
; fills in drive_type_exp with 01/80/c0
;--------------------------------------
		#assert_cfg_open
LWOP_hdinit	.proc
		sta current_drive_exp

		jsr OP_select_common

lp2     	jsr wait_a_bit
		jsr wait_a_bit
                tya
                beq nw
                cpy #$e0
                bcs nw
		lda current_drive_exp
                lsr
                lda #$0a
                bcc +
                eor #$10
+               #sta_low_reg
                sta head_g
                .if FAST_DATA_TRANSFERS
                eor head_g
                .else
                bit head_g
                eor low_reg
                .fi
                and #$1f
                bne nosig
nw              jsr BACK_with_C1_on_timeout_or_C0_status
                .if FAST_DATA_TRANSFERS
                lda status_g
                .else
		lda low_reg
                .fi
                eor #$ff
                beq nosig
		bpl lp2                 ;Wait BSY==0

		ldy #3
jf              jsr wait_a_bit		;Wait for diagnostic code
                dex
                bne jf
                dey
                bne jf

                lda error_g
                #lda_low_reg
		asl
                cmp #$02		;Check diagnostic code ($01, $81)
                bne nosig		;not valid

		lda cyllow_g		;check ATA/ATAPI signature
		#lda_low_reg
		bne no_ata
		lda cylhi_g
		#lda_low_reg
		bne nosig

		lda status_g
		#lda_low_reg
                beq ok			;probably nothing here

		ldy #0
lp1     	dex
		bne jx
		dey
		beq ok
jx		jsr wait_a_bit
		lda status_g
		#lda_low_reg
                asl
                bpl lp1			;wait DRDY, but not forever
ok
		lda #ident_d
		jsr LWOP_identify_drive
		bcs nosig

yes_hdd		lda #$01		;*yes* drive=HDD
		.byte SKIP2
yes_cd		lda #$80		;*yes* drive=CDROM
		.byte SKIP2
yes_floppy	lda #$C0		;*yes* drive=ZIP

		#ldx_l_ current_drive_exp
                sta drive_allowwrite_exp,x
		sta drive_type_exp,x
                ora ident_capabilities_exp
                and #%10000010
                beq +
                lda #64
+               sta drive_lba_exp,x
                #ldy_l_a ident_maxsector_exp
                lda ident_maxhead_exp
                #sbc1c
                jsr OP_setgeometry
		clc
		rts

nosig		sec
		rts

no_ata		eor #$14
		bne nosig		;check atapi signature
		lda cylhi_g
		#lda_low_reg
		eor #$eb
		bne nosig
		lda #atapi_reset_d
		jsr ata_command
                bcs nosig
                .if FAST_DATA_TRANSFERS
                #stz_a feat_g
                .else
		#stz_a low_reg
		sta feat_g		;for some atapi drives
                .fi
		lda #atapi_ident_d
		jsr LWOP_identify_drive
		bcs nosig

		lda ident_config_exp
		and #31
		cmp #5			;CDROM?
		beq yes_cd
					;Floppy/Unknown, but it may work (if sector size=512)
		ldy #249
		ldx #215
j11		iny
		beq yes_cd		;Looks like an old CD-ROM
j1		inx
		beq yes_floppy		;It's a floppy
		lda ident_model_number_exp-216,x
		cmp cdtext-250,y
		beq j11
cc		cpy #250
		beq j1
		dex
		dey
		gne cc

cdtext		.text "cd-rom"
		.pend
;---------------------------------------
; Identify Drive
; C=1 if error
;
; reads indentify info from drive
;---------------------------------------
		#assert_cfg_open
LWOP_identify_drive .proc
		jsr ata_data_command
		bcs rrts		;error C=1

		lda #ram_exp_buffer0
		jsr OP_read512_here

		ldx #182		;fix ascii strings
cp		lda ident_serial_number_exp-182,x
		pha
		lda ident_serial_number_exp-181,x
		sta ident_serial_number_exp-182,x
		pla
		sta ident_serial_number_exp-181,x
		inx
		inx
		bne cp
                geq XIT_end_of_transfer_ata           ;geq ata_pi_nomore_data
rrts		rts
		.pend
;--------------------------------------
;make drive number to default
;--------------------------------------
		#assert_cfg_open
OP_hdinit_drivenumbers .proc
		ldx #(current_drivenumbers_end_exp-current_drivenumbers_exp)/2
lp		lda setup_drivenumbers_exp-1,x
		pha
                and #$0f
                beq jq
                cpx #DRIVES+1
                bne +
                sbc #4
+               clc
                adc #7
jq		sta current_drivenumbers_exp-1,x
                pla
                lsr
                lsr
                lsr
                lsr
                beq +
                clc
                adc #7
+		sta current_drivenumbers_exp-1+(current_drivenumbers_end_exp-current_drivenumbers_exp)/2,x
		dex
		bne lp
		lda setup_boot_drive_exp
		sta ActualDev
		rts
		.pend
;--------------------------------------
OP_kill_drives	.proc
		#lda0
		ldx #DRIVES-1
-		sta drive_type_exp,x
		dex
		bpl -
		rts
		.pend
;------------
; used by save and monitor too!
;
		#assert_8000romioonly
LWOP_findunusedchannel .proc
		sta cfg_open
		#assert_cfg_open
	     	lda #128
lp		#sbc1c
		jsr search_logical_A
		beq lp
		sta ActualLogical
                clc
err		rts
		.pend
;---------------------------------------
