;
; mon.asm - Monitor
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
		.assert 0,0,0

		.comment
a_acc		= 0			;A		6502		;fix 0
a_imp		= 1			;				;?
a_imm		= 2			;#$FF				;?
a_zpa		= 3			;$FF				;?
a_zpx		= 4			;$FF,x				;?
a_zpy		= 5			;$FF,y				;?
a_zxi		= 6			;($FF,x)			;fix 1
a_ziy		= 7			;($FF),y			;fix 1
a_abx		= 8			;$FFFF,x			;?
a_aby		= 9			;$FFFF,y			;?
a_abs		= 10			;$FFFF				;?
a_ind		= 11			;($FFFF)			;fix 0
a_rel		= 12			;$FFFF				;fix 0

a_lng   	= 13			;$FFFFFF		65816	;?
a_lnx		= 14    		;$FFFFFF,x                      ;fix 1
a_zsy		= 15    		;($FF,s),y			;fix 1
a_zps		= 16    		;$FF,s				;fix 1
a_zpi		= 17    		;($FF)				;fix 0
a_zpI		= 18			;[$FF]				;fix 1
a_zIy		= 19			;[$FF],y			;fix 1
a_xya   	= 20			;$FF,$FF			;fix 0
a_rll   	= 21			;$FFFF				;fix 0
a_axi   	= 22			;($FFFF,x)			;fix 0
		.endc
a_acc		= 0			;A		    		;fix 0
a_zxi		= 0			;($FF,x)			;fix 1
a_ind		= 1 			;($FFFF)			;fix 0
a_ziy		= 1     		;($FF),y			;fix 1
a_axi   	= 2 			;($FFFF,x)			;fix 0
a_lnx		= 2     		;$FFFFFF,x                      ;fix 1
a_zpi		= 3     		;($FF)				;fix 0
a_zsy		= 3     		;($FF,s),y			;fix 1
a_xya   	= 4 			;$FF,$FF			;fix 0
a_zpI		= 4 			;[$FF]				;fix 1
a_rll   	= 5 			;$FFFF				;fix 0
a_zIy		= 5 			;[$FF],y			;fix 1
a_rel		= 6 			;$FFFF				;fix 0
a_zps		= 6    			;$FF,s				;fix 1
a_imp		= 7			;				;?
a_imm		= 8			;#$FF				;?
a_zpa		= 9			;$FF				;?
a_zpx		= 10			;$FF,x				;?
a_zpy		= 11			;$FF,y				;?
a_abx		= 12    		;$FFFF,x			;?
a_aby		= 13    		;$FFFF,y			;?
a_abs		= 14			;$FFFF				;?
a_lng   	= 15			;$FFFFFF	             	;?

m_p1		= $fb
m_p2		= $fd
m_p3		= $13			;3 byte!
m_p4		= $22
m_pos		= $2
m_tmp		= $7
m_tmp2		= $8
m_tmp3		= $9			;word
m_prefetch	= $b
m_01		= $c
m_ascii		= $57
m_tsp		= $5f			;1 byte
m_tstack	= $60			;16 byte
m_inv		= $c7                   ;1 byte
mon_setprefetch	= $d9			;3 byte
mon_getbyte	= $dc                   ;3 byte
mon_storebyte	= $df                   ;3 byte
mon_exec	= $e2                   ;3 byte
last_cmd	= $e5

monitor_buffer	= $200

		#assert_cfg_open

monitor_start	.proc
		ldx #5
lf		pla
		sta monitor_freeze_exp,x
		dex
		bpl lf
		.if SUPERCPU=1
		tsc
		sta monitor_sp_exp
		.else
		tsx
		stx monitor_sp_exp
		.fi

		ldx #2
cpu		lda $00,x
		sta monitor_data_exp,x
		inx
		bne cpu
		ldx #63
cpu3		lda $d000,x
		sta monitor_vic_exp,x
		cpx #32
		bcs vy
		lda $d400,x
		sta monitor_sid_exp,x
		lda $dcf0,x
		sta monitor_cia1_exp,x
vy		dex
		bpl cpu3

		lda #$10
		sta $dc0e
		sta $dc0f
		sta $dd0e
		sta $dd0f

		ldx #4
iyfd		lda $dc04-1,x
		sta monitor_cia1_exp+4-1,x
		lda $dd04-1,x
		sta monitor_cia2_exp+4-1,x
		dex
		bne iyfd
		stx m_tsp
		stx $d01a
		#stx_l_a monitor_tod_exp
		lda #$fc
		sta $d030		;C128

		jsr mon_waitretrace

		dex
		stx $d019
		lda #1
		sta $d01a
                clc

dpu2		lda $d019
		bpl dpu1
		ldx $d012
		lda $d011
-		bit $d011
		bpl -
		gmi vok2
dpu3		lda $d011
		bmi dpu4
		sta $d011		;impossible interrupt
                sec
		gcs dpu2                ;but $d012 is still interresting

dpu1		bit $d011
		bpl dpu2

dpu4		lda $d019
		bpl dpu3
		ldx $d012
		lda $d011
vok2		bcs +
		asl
+		php
		lda monitor_vic_exp+$11
		asl
                plp
                ror
		sta monitor_vic_exp+$11
		#stx_l_a monitor_vic_exp+$12
-		lda $d011
		bmi -

		lda #11
		sta $d011
		lda setup_moncolors_exp
		sta $d020

                .if SUPERCPU=1
                lda scpu_optimv2
                sta monitor_d0b3_exp
                sta scpu_hwenable
                sta scpu_nooptim
                sta scpu_hwdisable
                .else
		lda monitor_data_exp
		eor #7
		ora monitor_data_exp+1
		and #7
		sta monitor_01_exp
                .fi

		ldy #1
		sty m_p1+1
		.if SUPERCPU
		sty m_p2+2
		.fi
		dey
		sty m_p1
		sty m_p2
		lda #>(monitor_data_exp+$100)
		sta m_p2+1
		ldx #6
vg		lda (m_p1),y
		#sta_zp_y m_p2
		iny
		bne vg
		inc m_p1+1
		inc m_p2+1
		dex
		bpl vg

		sty $dd03
		stx $dd02
		lda $dd00
		sta monitor_cia2_exp
		sty $dd02
		stx $dd03
		lda $dd01
		sta monitor_cia2_exp+1
		sty $dd03

		sty $dc03
		stx $dc02
		lda $dc00
		sta monitor_cia1_exp
		sty $dc02
		stx $dc03
		lda $dc01
		sta monitor_cia1_exp+1
		sty $dc03

		inx
cp		lda $d800,x
		asl
		asl
		asl
		asl
		sta m_pos
		lda $d900,x
		and #15
		ora m_pos
		sta monitor_d800_exp,x
		lda $da00,x
		asl
		asl
		asl
		asl
		sta m_pos
		lda $db00,x
		and #15
		ora m_pos
		sta monitor_d800_exp+$100,x
		inx
		bne cp

		inx
		stx $dc04
		stx $dc06
		stx $dd04
		stx $dd06
		dex
		stx $dc05
		stx $dc07
		stx $dd05
		stx $dd07

                ldx #$e0
-		lda #9
		sta $dc1e,x
                ldy #%00000000
		lda $dc1d,x
		bpl +
		ldy #%10000001
+               lda #9
		sta $dc1f,x
		lda $dc1d,x
                asl
                tya
		bcc +
		ora #%10000010
+		sta monitor_cia1_exp+$d-$e0,x
                txa
                clc
                adc #$10
                tax
                bcc -

		.if SUPERCPU=1
                lda monitor_oldbank_exp
                bpl nfreeze
                and #$7f
                sta monitor_oldbank_exp
                ldx #$1f
ll              lda $1d100,x
                sta monitor_vic_exp,x
                cpx #$10
                bcs sc
                lda $1d160,x
                sta monitor_cia1_exp,x
                lda $1d180,x
                sta monitor_cia2_exp,x
sc              dex
                bpl ll
                lda $d250
                sta monitor_data_exp+1
nfreeze
		lda monitor_data_exp+1
                and #7
                sta monitor_01_exp
		.fi

		lda monitor_sr_exp
		and #$10
		beq nbrk
                lda monitor_pc_exp+1
		sec
		sbc #2
                sta m_p4
                sta m_p2
		sta monitor_pc_exp+1
                lda monitor_pc_exp
                sbc #0
                sta monitor_pc_exp
                sta m_p4+1
                sta m_p2+1
                lda monitor_01_exp
                sta m_01
                lda #$37
                sta $01
                .if SUPERCPU=0
                lda #$2f
                sta $00
                .fi
		ldx #14
ie2		lda mon_peek,x
		sta $7e8,x
		dex
		bpl ie2
                stx m_p3
                stx m_p3+1
                ldy #2
sds             #sta_cfg_16
		jsr mon_getbyte_mem
                #sta_cfg_open
                jsr inc_m_p2
                sta monitor_buffer,y
                cmp instfp.fp,y
                beq +
                lsr m_p3
+               cmp instfp.fp2,y
                beq +
                lsr m_p3+1
+               dey
                bpl sds
aae		ldy monitor_buffer+2
                beq restore		;zero point
                ldy #3
                bit m_p3
                bmi restore2	    	;freeze1
                ldy #6
                bit m_p3+1
                bpl nopoint
restore2	#sty_l_a monitor_freezepoint_ab_exp
restore		jsr instfp.in
		#assert_cfg_16
		tya
                bmi ss
                jsr instfp.in
                #assert_cfg_16
                jsr instfp.in
                #assert_cfg_16
ss              #sta_cfg_open

nopoint
nbrk		#inc_l_a monitor_data_exp+$318;fix blocked nmi
nx

		lda #1
		sta monitor_ripper_exp
mon_rip_start	lda #11
		sta $d011

		jsr nmiblock

		lda setup_moncolors_exp
		sta $d020
                pha
		ldx #0
		stx $d015
		stx $d01a
		lda #$fc
		sta $d030		;C128
		.if (SECOND_SID >> 8)=$d6
		lda $d030		;C128?
		lsr
		bcc +
		.fi
		stx SECOND_SID+$18	;Second SID
+		stx $d01a
		stx $dc0e
		stx $dc0f
		stx $dd0e
		stx $dd0f
		dex
		stx $d019
		stx $d02f		;C128
		lda #20
		sta $d018
		lda #200
		sta $d016
		.if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                sta dos_accel_internal_exp
                .fi

		ldx #14
ie		lda mon_peek,x
		sta $7e8,x
		dex
		bpl ie

                pla
                lsr
                lsr
                lsr
                lsr
		#sta_cfg_16
		txs
                pha
		jsr $fda3
		#lda0
		tay
nn		sta $2,y
		sta $200,y
		iny
		bne nn

		ldy #$04
		sty $c2
		ldx #$00
		jsr $fd8c		;Old memory checker
		ldx #$3c
		ldy #$03
		stx $b2
		sty $b3
                stx $281
                sty $282

		jsr LWNO_set_new314vect
		#stz_a ActualInput
                #stz_aa $291
                #stz_aa $cf
                lda #10
                sta $289
                sta $28c
		lda #3
		sta ActualOutput
-               lda $d011
                bpl -
                pla
                sta $d021		;for kernal rev 2
		jsr $e536		;clear screen
                jsr $e453		;Init $0300 vectors
                lda #<derts
                sta $32c
                lda #>derts
                sta $32d
		jsr $e3bf		;Init basic
		lda $2b
		ldy $2c
                sta cfg_8
                #assert_cfg_8
		jsr $a408		;check memory overlap
		jsr $a644		;basic NEW
                sta cfg_open
                #assert_cfg_open

                lda $d020
		sta $d021
		lda #27
		sta $d011

		lda #","
                sta last_cmd
		ldx #0
		stx $dc0e
		stx m_inv
		stx m_prefetch
		.if SUPERCPU=1
		inx
		stx m_p2+2
		stx m_p4+2
		.fi
		bit $dc0d

		lda #128
		sta 650
                jsr LWOP_init_c128key
		lda monitor_dr_exp
		.byte SKIP2
eep		lda #8
		jsr OP_mon_chdrv
		bcs eep
		lda #<$450
		sta m_p1
		ldx #>$450
		stx m_p1+1

		lda monitor_sr_exp
		and #$10
                beq +
                ldx #txtbrk-txtmon+4
+
		ldy #12
leo		lda txtmon-4,x
		sta $400,y
                inx
		dey
		bpl leo

		lda monitor_01_exp
		pha

		jsr mon_printstat
		sty m_pos


		pla
		jsr u_b.in

		#sta_cfg_open
		lda monitor_tod_exp
		bmi xxf

		ldx #$80
		lda $dc08
		eor monitor_cia1_exp+8
		beq +
		inx

+		lda $dd08
		eor monitor_cia2_exp+8
		beq +
		inx
		inx
+		#stx_l_a monitor_tod_exp
xxf		#sta_cfg_16


loop		ldy m_pos
		lda 198
		bne hx
		lda (m_p1),y
		pha
		lda #15
		sta m_p3
d		ldy m_pos
		jsr mon_waitretrace
		inc m_p3
		lda m_p3
		and #15
		bne ig
		lda (m_p1),y
		eor #128
		sta (m_p1),y
ig		jsr $ea87
		.if SUPERCPU=0
                lda $28d
                sta $28e
                .fi
		lda 651
		lsr
		beq bd
		lda #2
		sta 651
bd		lda 198
		beq d
		pla
		ldy m_pos
		sta (m_p1),y
hx		dec 198

		#sta_cfg_open
		lda monitor_01_exp
		sta m_01
		#sta_cfg_16
		lda 631
		jsr xo
		#jmps loop

xo		#switch billc,billce

		jsr tosccode
		eor m_inv
		sta (m_p1),y
;----------------------
;right
;
b_right		.proc
		cpy #39
		bcs +
		inc m_pos
+		rts
		.pend
;--------------------------
;f2
;
b_f2		.proc
		jsr checkn
lp		jsr b_f3
		jsr pause2
                bne lp
                rts
                .pend
;--------------------------
;f8
;
b_f8		.proc
		jsr checkn
lp		jsr b_f5
		jsr pause2
                bne lp
                rts
                .pend
;--------------------------
;f1
;
b_f1		.proc
		lda #134
		.byte SKIP2
;--------------------------
;f7
;
b_f7		.proc
		lda #135
		sta 631
		ldx #12
		stx 198
                lda #1
                sta 651
		rts
		.pend
;----------------------
;down
;
b_down		.proc
		sec
		.byte SKIP1BCC
;--------------------------
;f5
;
b_f5		.proc
		clc
ll		lda m_p1
		ora m_p1+1
		eor #$c7
		beq le
		bcs s

ss2		jsr b_downin
		php
		jsr mon_scrolldown
		plp
		bcc ll
jmp_m_p4	jmp (m_p4)

le		jsr b_downin
		bcc br
re		jsr scrldn
		jmp (m_p4)

br		jsr mon_cprevline

		lda m_p1+1
		cmp #4
		bcs le
                jsr getlast
                clv
                jsr updown
                bcs re
scrldn		lda #<$7c0
		sta m_p1
		lda #>$7c0
		sta m_p1+1
s		#jmps mon_scrolldown
		.pend

getlast         .proc
		lda m_p2
                sta m_p3
                lda m_p2+1
                sta m_p3+1
                ldx last_cmd
                rts
                .pend

b_upin		.proc
		#sev b_f3.set_v
		.byte SKIP1BVC
		.pend

b_downin	.proc
		clv
		ldy #0
		#ldatax (m_p1),y
		iny
                php
		jsr getaddr
                plp

updown		.proc
                .enc screen
		ldy #0
		lda m_p3
		cpx #":"
		beq ne1
		iny
		cpx #"."
		beq ne2
		iny
		cpx #","
		beq ne3
		cpx #"a"
		beq ne3
		cpx #">"
		beq ne3
		iny
		cpx #"'"
		beq ne4
		iny
		cpx #"["
		beq ne5
		iny
		cpx #"]"
		beq ne6
		ldy #7
		cpx #"-"
		beq ne7
		clc
		rts
                .enc none

ne3		bvs mas
		sta m_p2
		lda m_p3+1
		sta m_p2+1
		jsr mon_getbyte
		jsr mon_getoplen
                sec
                gcs fe2

ne7
ne1		lda #8
		gne fe

ne4
ne2		lda #32
		gne fe

ne5		lda #1
		gne fe

mas		lda #16
		sta m_tmp

do2		lda m_p3
		sec
		sbc m_tmp
		jsr mon_copy_p3p2sbc
		lsr m_tmp2
do		jsr mon_getbyte
		pha
		tax
		lda opc,x
		tax
		lda o_table+1,x
                #orasta_a m_tmp2
		pla
		jsr mon_getoplen
		sta m_p4
		sec
		adc m_p2
		sta m_p2
		bcc +
		inc m_p2+1
+
		cmp m_p3
		lda m_p2+1
		sbc m_p3+1
		bcc do
		ldx m_tmp
		cpx #8
		bcc +
		ldx m_tmp2
		bmi nig
+		ldx m_p2
		cpx m_p3
		bne nig
		tax
		beq q
nig		dec m_tmp
		bne do2

q		lda m_p4
		clc
		gcc fe3

ne6		and #63
		bvs wee
		cmp #60
		#lda0
		adc #3

fe		bvs ww
                clc
fe2		adc m_p3
		sta m_p2
                #lda0
		adc m_p3+1
		sta m_p2+1
                #jmps mon_gprint

wee		cmp #3
		#lda0
		sbc #3
                gcc fe4

ww		sec
fe3		eor #$ff
fe4		adc m_p3
                sta m_p2
                lda m_p3+1
                sbc #0
                sta m_p2+1

mon_gprint	.proc
		lda mon_printlo,y
		sta m_p4
		lda mon_printhi,y
		sta m_p4+1
		sec
		rts
		.pend
;--------------------------
;left
;
b_left		.proc
		tya
		beq +
		dec m_pos
+		rts
		.pend
;--------------------------
;up
;
b_up		.proc
		sec
		.byte SKIP1BCC
;--------------------------
;f3
;
b_f3		.proc
		clc
ll		lda m_p1+1
		eor #4
		ora m_p1
		beq fel
		bcc ss2
		gcs mon_cprevline

ss2		jsr b_upin
		php
		jsr mon_cprevline
		plp
		bcc ll
		ldy #0
		jmp (m_p4)

fel		jsr b_upin
		bcc bty
re		ldy #0
		lda #<$3c0
		sta m_p1
		lda #>$3c0
		sta m_p1+1
		jsr b_f5.jmp_m_p4
bc		#jmps mon_scrollup

bty		jsr mon_cnextline

set_v		cmp #<$7e8
		lda m_p1+1
		sbc #>$7e8
		bcc fel
                jsr getlast
		#sev set_v
                jsr updown
                bcs re
		ldy #40
		lda #$20
ld		sta $3bf,y
		dey
		bne ld
		geq bc
		.pend

;--------------------------
;clr
;
b_clr		.proc
		lda #<($400-24)
                sta m_p1
                ldx #>($400-24)
                stx m_p1+1
		lda #32
                ldy #24
-               sta (m_p1),y
                iny
                bne -
                inc m_p1+1
                dex
                bpl -

;--------------------------
;home
;
b_home		.proc
		#stz_a m_p1
		#stz_aa m_pos
		lda #$04
		sta m_p1+1
;--------------------------
;stop
;
b_stop		.proc
		rts
		.pend
;--------------------------
;del
;
b_del		.proc
		tya
		beq b_stop
		dec m_pos
-		lda (m_p1),y
		dey
		sta (m_p1),y
		iny
		iny
		cpy #40
		bcc -
		dey
                gcs pspace
		.pend
;--------------------------
;insert
;
b_ins		.proc
		ldy #39
		lda (m_p1),y
		cmp #$20
		bne b_stop
-		cpy m_pos
		beq pspace
		dey
		lda (m_p1),y
		iny
		sta (m_p1),y
		dey
		bne -

		.assert CFG_8000_ROM, CFG_8000_ROM,CFG_8000_RAM
pspace		.proc
		lda #" "
		sta (m_p1),y
		iny
		rts
                .pend
;--------------------------
;shift return
;
		#assert_cfg_16
b_shret		.proc
		#stz_a m_pos
		#stz_aa m_inv
scrdown		#jmps mon_scrolldown
		.pend
;--------------------------
;rvsoff
;
b_rvsoff       	.proc
		#lda0
		.byte SKIP2
;--------------------------
;rvson
;
b_rvson		.proc
		lda #128
		sta m_inv
		rts
		.pend
;--------------------------
;return
;
b_ret		.proc
		sty m_tmp
		ldy #0
		sty m_pos
		sty m_inv

udu		lda (m_p1),y
		cmp #" "
		bne fo
		iny
		cpy #40
		bcc udu
                gcs b_shret.scrdown

fo		#switch utasc,utasce
		geq b_shret.scrdown

utasc		#jword 13,u_m		;m
		#jword 9,u_i		;i
		#jword 4,u_d		;d
		#jword 10,u_j		;j
		#jword 24,exit		;x
		#jword 18,u_r		;r
		#jword 7,u_g		;g
		#jword 58,u_kpont	;:
		#jword 45,u_minus	;-
		#jword 44,u_vesszo	;,
		#jword 62,u_vesszo	;>
		#jword 59,u_pvesszo
		#jword 39,u_ijel	;'
		#jword 6,u_f		;f
		#jword 20,u_t		;t
		#jword 2,u_b		;b
		#jword 15,u_o		;o
		#jword 17,u_q		;q
		#jword 3,u_c		;c
		#jword 8,u_h		;h
		#jword 19,u_s		;s
		#jword 12,u_l		;l
		#jword 22,u_v		;v
		#jword 1,u_a		;a
		#jword 0,u_at		;@
		#jword 46,u_pont	;.
		#jword 5,u_e		;e
		#jword 27,u_balk	;[
		#jword 29,u_jobbk	;]
		#jword 31,u_bnyil	;<-
		#jword 30,u_fnyil	;^
		#jword 11,u_k		;k
		#jword 23,u_w		;w
                #jword 42,u_star	;*
                #jword 14,u_n		;n
utasce
		.pend
;---------------------------
;Check keyboard (and stop)
;
                #assert_8000romioonly
checkkey2	.proc
		#lda0
		.byte SKIP2
checkkey1	.proc
		lda #$7f
                sta $dc00
-		lda $dc01
		cmp $dc01
		bne -
		cmp #$7f
		rts
		.pend
;---------------------------
;X exit
;
		#assert_8000romioonly
exit		.proc
		jsr mon_copyback
                #assert_cfg_open
		stx $dc0e
		stx $dc0f
		stx $dd0e
		stx $dd0f
		lda #127
		sta $dc0d
		sta $dd0d
		bit $dc0d
		bit $dd0d
		dex
		stx $d019
		ldx #$e0
dc		lda monitor_cia2_exp-$f0,x
		sta $dd00-$f0,x
		inx
		bne dc
                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                #stx_l_a dos_accel_internal_exp
                .fi
		lda monitor_tod_exp
		lsr
		bcc +
		ldx $dc08
		stx $dc08
+		lsr
		bcc +
		ldx $dd08
		stx $dd08
+
		jsr mon_vicback
nf		lda monitor_data_exp+$100,x
		sta $100,x
		inx
		bne nf

		.if SUPERCPU=1
		lda monitor_sp_exp
		tcs
		.else
		ldx monitor_sp_exp
                txs
		lda monitor_pc_exp
		eor #>$fce2
		bne nr
		lda monitor_pc_exp+1
		eor #<$fce2
		bne nr
                inx
		bne nr
		lda monitor_data_exp
		ora monitor_data_exp+1
		and #7
		bne nr
		#psr $fcec
		jmp gotobank0
nr
		.fi
		ldx #$fd
vyv		lda monitor_freeze_exp-$fd,x
		pha
		inx
		bne vyv

                .if SUPERCPU=1
                sta scpu_hwenable
                lda monitor_d0b3_exp
                sta scpu_optimv2
                sta scpu_hwdisable
                .fi

		.if SUPERCPU=1
                lda monitor_data_exp+1
                .else
		lda monitor_data_exp
		eor #7
		ora monitor_data_exp+1
                .fi
		and #7
		.if SUPERCPU=1
		cmp #7
		.else
		cmp #5
		.fi
		bcs sd

		.if SUPERCPU=1
		pea $0101
		.else
		lda #$01
		pha
		pha
		.fi
		lda #%00100100
		pha

		pha
		pha
		pha

		lda #$40		;rti
		pha
		lda monitor_ac_exp	;lda #xx
		pha
		.if SUPERCPU=1
		pea $a901		;sta $01
		.else
		lda #$a9
		pha
		lda #$00                ;sta $00
		pha
		.fi
		tsx
		lda #$85
		pha

		pla
		pla
		pla
		pla
		pla

		pla
		pla
		pla

		txa
		tsx
		inx
		inx
		sta $100,x

		.if SUPERCPU=1
		lda monitor_data_exp+1
                .else
		lda monitor_data_exp
                .fi
		gcc ivi

sd		.if SUPERCPU=1
		lda monitor_data_exp+1
		sta $01
                .else
		lda monitor_data_exp
		sta $00
                .fi
		lda monitor_ac_exp
ivi		.if SUPERCPU=1
		xba
		.else
		sta help_irqa_exp
		.fi
		#ldx_l_a monitor_xr_exp
		#ldy_l_a monitor_yr_exp
		#psr block_nmi
		lda #0		;no lda0!!! (xba)
		sta monitor_ripper_exp
		lda monitor_oldbank_exp
		cmp #3
		beq ey
		.if SUPERCPU=1
		xba
		.else
		lda help_irqa_exp
		.fi
		jmp exit2basic
ey		.if SUPERCPU=1
		xba
		.else
		lda help_irqa_exp
		.fi
		jmp switch_norm
		.pend

mon_vicback	.proc
		jsr mon_waitretrace
		lda monitor_vic_exp+$30
		and #%11111110
		sta $d030		;ide64 does not work with 2MHz
		ldx #$30
-		lda monitor_vic_exp-1,x
		sta $cfff,x
		dex
		bne -
		rts
		.pend

		#assert_8000romioonly
mon_copyback	.proc
		#sta_cfg_open
		lda #11
		sta $d011
		jsr mon_waitretrace
		.if SUPERCPU=1
                lda monitor_data_exp+1
                xba
                lda #$37
                jsr md
                .fi
		lda #2
		sta m_p1+1
		ldy #0
		sty m_p1
		sty m_p2
		lda #>(monitor_data_exp+$200)
		sta m_p2+1
		ldx #6
vy		#lda_zp_y m_p2
		sta (m_p1),y
		iny
		bne vy
		inc m_p1+1
		inc m_p2+1
		dex
		bne vy
                .if SUPERCPU=0
		txa
		geq ici
                .fi
ic		lda monitor_data_exp,x
ici		sta $00,x
		lda monitor_d800_exp,x
		sta $d900,x
		lsr
		lsr
		lsr
		lsr
		sta $d800,x
		lda monitor_d800_exp+$100,x
		sta $db00,x
		lsr
		lsr
		lsr
		lsr
		sta $da00,x
		inx
		bne ic
		.if SUPERCPU=1
                xba
md              sta monitor_data_exp+1
                .fi
		rts
		.pend

		.assert CFG_8000_ROM | CFG_D000_IO, CFG_8000_ROM,CFG_8000_RAM
nmiblock	.proc
		lda #<block_nmi
		sta $318
		lda #>block_nmi
		sta $319
                rts
                .pend
;---------------------------
;W view
;
		#assert_cfg_16
u_w		.proc
		jsr mon_copyback
                #assert_cfg_open
                jsr nmiblock
		jsr mon_vicback
nf		lda monitor_data_exp+$100,x
		sta $100,x
		inx
		bne nf
		lda monitor_cia2_exp+2
		eor #3
		ora monitor_cia2_exp
		sta $dd00
		jsr checkn
		jsr checkn2
		jsr mon_waitretrace
		jmp monitor_start.mon_rip_start
		.pend
;---------------------------
;R Registers
;
		#assert_cfg_16
u_r		.proc
		jsr mon_scrolldown
;---------------------------
;Print processor status
;
		#assert_8000romioonly
mon_printstat	.proc
		ldy #0
l		lda txtstat,y
		sta (m_p1),y
		iny
		cpy #32
		bcc l
		jsr clear_scrolldown
in		#sta_cfg_open
		lda monitor_pc_exp+1
		sta m_p2
		lda monitor_pc_exp
		sta m_p2+1

		lda #";"
		jsr mon_print_here
                ldx #4
vsc		#phx_a
		lda sor,x
                tax
                lda monitor_01_exp,x
		jsr mon_printhexsp
                #plx_a
                dex
                bpl vsc
		lda ActualDev
		jsr mon_printhexsp
                jsr pspace
		lda monitor_sr_exp
                jsr binary
		#sta_cfg_16
		#jmps clear_scrolldown

		.assert CFG_OPEN_RAM | CFG_8000_ROM,CFG_OPEN_RAM | CFG_8000_ROM, CFG_RAM | CFG_8000_RAM

sor		.byte monitor_01_exp-monitor_01_exp
		.byte monitor_sp_exp-monitor_01_exp
		.byte monitor_yr_exp-monitor_01_exp
		.byte monitor_xr_exp-monitor_01_exp
		.byte monitor_ac_exp-monitor_01_exp

		#assert_8000romonly
		.enc screen
txtstat		.text " addr ac xr yr sp bk dr nv-bdizc"
                .enc none
		.pend
;---------------------------
;G Goto
;
		#assert_cfg_16
u_g		.proc
		jsr get_addr2x
		jmp mon_exec
		.pend
;---------------------------
;Execute program in memory
;
mon_exec_mem	.proc
		jsr OP_m_p3_to_pc		;now cfg_open
                #assert_cfg_open
		lda monitor_data_exp
		eor #7
		ora monitor_data_exp+1
		and #2
		beq hy2
		#ldx_l_a monitor_sp_exp
		inx
		lda monitor_data_exp+$100,x
		cmp #<(__mydoit+1)
		bne xe
		inx
		lda monitor_data_exp+$100,x
		cmp #>(__mydoit+1)
		beq hy2
		dex
xe		dex
		lda #>(__mydoit+1)
		sta monitor_data_exp+$100,x
		dex
		lda #<(__mydoit+1)
		sta monitor_data_exp+$100,x
		dex
		#stx_l_a monitor_sp_exp
hy2		jmp exit
		.pend
                #assert_cfg_16
;---------------------------
;Send M-? command to drive
;
drive_memcmd	.proc
		pha
		lda m_01
		jsr ieclisten		;listen
		lda #$6f
		jsr iecsecond		;second
		lda #"m"
		jsr iecout
		lda #"-"
		jsr iecout
		pla
		jsr iecout
                lda $00,x
                jsr iecout
                lda $01,x
                jmp iecout
		.pend
;---------------------------
;Store a byte to drive
;
mon_storebyte_dr .proc
		pha
		lda #"w"
		ldx #m_p4
		jsr drive_memcmd
		lda #1
		jsr iecout
		pla
		jsr iecout
;---------------------------
;Send M-? command to drive
;
drive_endcmd	.proc
                lda #13
                jsr iecout
		jmp iecunlisten
                .pend
;---------------------------
;Execute program in drive
;
mon_exec_dr	.proc
		lda #"e"
                ldx #m_p3
		jsr drive_memcmd
		jsr drive_endcmd		;unlisten
		#jmps mon_scrolldown
		.pend
;---------------------------
;] Store sprite
;
u_jobbk		.proc
		.enc screen
		lda #"]"+128
                .enc none
		.byte SKIP2
;---------------------------
;[ Store character
;
u_balk		.proc
		.enc screen
		lda #"["
                .enc none
		jsr get_addr2store

		jsr get_binary2
		lda m_tmp2
		bpl io2
		bcs io
		jsr get_binary2
		bcs io
		jsr get_binary2
io		ldy #5
		gne sphr
io2		ldy #4
		gne sphr
		.pend
;---------------------------
;- Store hex io
;
u_minus       	.proc
		lda #5
                sta m_01
                jsr get_addr2store
                clc
		gcc u_kpont.hex
;---------------------------
;: Store hex
;
u_kpont		.proc
		lda #":"+128
		jsr get_addr2store
                lda #29
		cmp m_tmp
		bcs hex
                tay
                inc m_tmp2
                jsr u_ijel.in
                ldy #30
                gne u_vesszo.in

hex		php
		lda #8
		sta m_tmp
xx1		jsr get_addr2
		bne xi1
		lda m_p3
                plp
                php
                bcs d
                jsr mon_storebyte_mem
                jsr inc_m_p4
                #jmps we
d		jsr mon_storebyte_inc
we		dec m_tmp
		bne xx1

xi1		plp
		ldy #0
		bcs sphr
                ldy #7
                gne sphr
;---------------------------
;, Store disassembly
;
u_vesszo	.proc
		tya
		bne vq
		lda (m_p1),y
		cmp #","
		bne vq
		lda m_tmp
                .if SUPERCPU=1
                cmp #18
                .else
		cmp #15
                .fi
		bcc vq
                jsr u_a
                .if SUPERCPU=1
                ldy #8
                .else
		ldy #15
                .fi
in              gne topos6.in

vq
		lda #","
		jsr get_addr2store

		jsr get_addr2
		bne xi3
		lda m_p3
		jsr mon_getoplen
		sta m_tmp
		#jmps vr
xx2		dec m_tmp
		bmi xi2
		jsr get_addr2
		bne xi2
vr		lda m_p3
		jsr mon_storebyte_inc
		#jmps xx2

xi3		ldx m_01
		cpx #8
                bcs xi2
		lda (m_p1),y
		.enc screen
                cmp #"s"
                bne xi2
                iny
		lda (m_p1),y
                eor #"z"
                tay
                beq zz
                eor #"f"^"z"
                bne xi2
                ldy #3
                #sta_cfg_open
                adc monitor_freezepoint_ab_exp
                sta monitor_freezepoint_ab_exp
                #sta_cfg_16
                lsr
                bcc joo
                ldy #6
joo		jsr instfp
		jsr instfp
zz		jsr instfp
nem             ldy #0
		jsr get_addr2store

xi2		ldy #2
;---------------------------
;Print here + space + position 6
;
sphr		.proc
		jsr mon_gprint
		ldy #0
		jsr b_f5.jmp_m_p4
		jsr mon_scrolldown
		lda m_tmp2
		and #$7f
                .enc screen
                cmp #"a"
                .enc none
                beq +
		cmp (m_p1),y
		bne b
+		jsr mon_print_herex
		lda #" "
		sta (m_p1),y
		gne topos6
b		jsr b_f5.jmp_m_p4

topos6		.proc
		ldy #6
in		sty m_pos
		rts
		.pend

instfp  	.proc
		jsr mon_getbyte_mem
                jsr inc_m_p2
		clc
                gcc ss

		.assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
in              lda zp,y
		sec
ss
                .if SUPERCPU=1
                tyx
                xba
                lda monitor_freezepoint_stuff_exp,x
                pha
                xba
                sta monitor_freezepoint_stuff_exp,x
                pla
                .else
                sta cfg_open
                #assert_cfg_open
                ldx monitor_freezepoint_stuff_exp,y
                sta monitor_freezepoint_stuff_exp,y
                sta cfg_16
                #assert_cfg_16
                txa
                .fi
                bcs rw
                lda zp,y
rw              dey
                jsr mon_storebyte_mem
                jmp inc_m_p4

		.assert CFG_8000_ROM, CFG_8000_ROM,CFG_8000_RAM
zp		.byte $00
fp		.byte $de,$f7,$20
fp2		.byte $de,$fa,$20
		.pend
;---------------------------
;. Store screencode
;
		#assert_cfg_16
u_pont		.proc
		lda #"."
		.byte SKIP2
;---------------------------
;' Store ascii
;
u_ijel		.proc
		lda #"'"
		jsr get_addr2store

in
xf		iny
		lda m_tmp2
		lsr
		lda (m_p1),y
		bcc ik
		jsr fromsccode
ik		jsr mon_storebyte_inc
		cpy #37
		bcc xf

                ldy #0
		lda m_tmp2
                bmi nn
		lsr
		iny
		bcc nn
		ldy #3
nn		gne sphr
		.pend
;---------------------------
;; Change registers
;
u_pvesszo	.proc
		jsr get_addr2x
		jsr OP_m_p3_to_pc		;now cfg_open
                #assert_cfg_open
                ldx #$fc
jkq		jsr get_addr2ddb
		sta monitor_ac_exp-$fc,x
                inx
                bne jkq
		jsr get_addr2ddb
		jsr u_b.in
		#sta_cfg_open
		jsr get_addr2ddb
		jsr OP_mon_chdrv
		ldx #8
igu		iny
		cpy #40
		bcs ddb
		pha
		lda (m_p1),y
		lsr
		eor #24
		bne ddb2
		pla
		rol
		dex
		bne igu
		ora #%00100000
		sta monitor_sr_exp
		.byte SKIP2
xf		pla
ddb2		pla
ddb		#sta_cfg_16
		ldy #0
		geq mon_printstat.in

		#assert_cfg_open
get_addr2ddb	jsr get_addr2
		bne xf
		lda m_p3
		rts
		.pend
;---------------------------
;F Fill
;
                #assert_cfg_16
u_f		.proc
		jsr get_addr2x_cserep3p4
		jsr get_addr2x_cserep3p2
		jsr get_addr2x
                ldy m_p4

uh		lda m_p3
		jsr mon_storebyte
		cpy m_p2
		lda m_p4+1
		sbc m_p2+1
		iny
                sty m_p4
		bne +
		inc m_p4+1
+		bcc uh
		gcs u_t.sd
		.pend
;---------------------------
;T Transfer
;
u_t		.proc
		jsr get_addr2x_cserep3p2
		jsr get_addr2x_cserep3p4
		jsr get_addr2x_cserep3p4
		jsr in
sd		#jmps mon_scrolldown
					;p2 start p3 end p4 new
in		ldy m_p2
		cpy m_p4
		lda m_p2+1
		sbc m_p4+1
		bcs uh2

		lda m_p3
		pha
		sec
		sbc m_p2
		sta m_p3
		lda m_p3+1
		pha
		sbc m_p2+1
		sta m_p3+1

		php

		lda m_p3
		clc
		adc m_p4
		sta m_p4
		lda m_p3+1
		adc m_p4+1
		sta m_p4+1

		sty m_p3
		lda m_p2+1
		sta m_p3+1

		plp

		pla
		sta m_p2+1
		pla
		sta m_p2
		bcc enn

uh3		jsr mon_getbyte
		jsr mon_storebyte
		lda m_p3
		cmp m_p2
		lda m_p3+1
		sbc m_p2+1
		lda m_p2
		bne +
		dec m_p2+1
+		dec m_p2
		lda m_p4
		bne +
		dec m_p4+1
+		dec m_p4
		bcc uh3
enn		rts

uh2		jsr mon_getbyte
		jsr mon_storebyte
		cpy m_p3
		lda m_p2+1
		sbc m_p3+1
                iny
		sty m_p2
		bne +
		inc m_p2+1
+		inc m_p4
		bne +
		inc m_p4+1
+		bcc uh2
		rts
		.pend

get_addr2x_cserep3p4 .proc
		jsr get_addr2xx
cserep3p4	.proc
		ldx m_p3
		lda m_p4
		sta m_p3
		stx m_p4
		ldx m_p3+1
		lda m_p4+1
		sta m_p3+1
		stx m_p4+1
		rts
		.pend
;---------------------------
;BT Backtrace
;
		#assert_cfg_16
u_bt		.proc
		jsr mon_scrolldown
                #psz m_p2
		#stz_a m_p3
		#sta_cfg_open
		lda monitor_sp_exp
		sta m_tmp
                .assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
vwq		#sta_cfg_open
		inc m_tmp
		ldx m_tmp
		lda monitor_data_exp+$100,x
		sta m_tmp2
		sec
		sbc #2
		sta m_p2
		inx
		lda monitor_data_exp+$100,x
		sbc #0
		sta m_p2+1
		#sta_cfg_16
		jsr mon_getbyte
		cmp #$20		;jsr?
		bne fs
		inc m_tmp
		lda m_p3
		bne sc
		inc m_p3
		pla
		pla
                #psz m_p2
sc		lda m_p2+1
		jsr mon_printhex
		lda m_p2
		.byte SKIP2
fs		lda m_tmp2
		jsr mon_printhex
                jsr pspace
		cpy #36
		bcc vwq
		pla
		sta m_p2
		pla
		sta m_p2+1
		gcs u_b.sd
		.pend
;---------------------------
;B Select bank
;
		#assert_cfg_16
u_b		.proc
		iny
		cpy #40
		bcs xj

		lda (m_p1),y
		dey
                .enc screen
		cmp #"t"
		beq u_bt
                .enc none

		jsr get_addr2x
		lda m_p3
		jsr in
sd		#jmps mon_scrolldown

		.assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
in		#sta_cfg_open
		sta monitor_01_exp
                sta m_01
		#sta_cfg_16
		ldx #mem-type+11
		cmp #8
		bcc norm
		ldx #drive-type+11
		cmp #31
		bcc norm
		ldx #int-type+11
norm		#phy_a
		ldy #11
v		lda type,x
		dex
		sta mon_setprefetch,y
		dey
		bpl v
		#ply_a
xj		rts

type
drive		jmp mon_setprefetch_dr
		jmp mon_getbyte_dr
		jmp mon_storebyte_dr
		jmp mon_exec_dr
int		.byte $60,$60,$60
		jmp mon_getbyte_int
		.byte $60,$60,$60
mem		.byte $60,$60,$60
		jmp mon_getbyte_mem
		jmp mon_storebyte_mem
		jmp mon_exec_mem
		.pend
;---------------------------
;O Select device
;
		#assert_cfg_16
u_o		.proc
		jsr get_addr2x
		lda m_p3
		#sta_cfg_open
		jsr OP_mon_chdrv
		#sta_cfg_16
		bcs OP_mon_chdrv.err
		gcc mon_scrolldown
;---------------------------
;Change drive
;
		#assert_cfg_open
OP_mon_chdrv	.proc
		cmp #8
		bcc err
		cmp #31
		bcs err
		sta monitor_dr_exp
		sta ActualDev
		rts
                .assert CFG_8000_ROM, CFG_8000_ROM, CFG_8000_RAM
err		sec
		rts
		.pend
;---------------------------
;Q Quit
;
		#assert_cfg_16
u_q		.proc
		#sta_cfg_open
                ldx #2
jf              lda w0-1,x
		sta monitor_pc_exp-1,x
		lda w1-1,x
		sta monitor_data_exp-1,x
		lda w2-1,x
		sta monitor_data_exp+$314-1,x
                dex
                bne jf
		lda #$10
		sta $dc0e
                sta monitor_vic_exp+$1a
ls		lda cia-13,x
                cpx #13
                bcs uei
		lda $dc00,x
uei		sta monitor_cia1_exp,x
		inx
		cpx #16
		bne ls
		geq exit
cia		.byte $81,$01,$08
w0		.word $7be3
w1              .word $372f
w2              .word $ea31
		.pend
;---------------------------
;C Compare
;
		#assert_cfg_16
u_c		.proc
		jsr get_addr2x_cserep3p2
		jsr get_addr2x_cserep3p4
		jsr get_addr2x
		jsr mon_scrolldown

uh4		jsr mon_getbyte
		sta m_tmp
		jsr cserep3p2
		jsr mon_getbyte
		eor m_tmp
		cmp #1
		jsr cserep3p2

		bcc nodiff
                jsr mon_scrolldown_on_eol
		ldx #m_p2
		jsr mon_printhexaddrsp
		jsr checkkey1
		beq cserep3p2.rrts
nodiff
		lda m_p2
		cmp m_p4
		lda m_p2+1
		sbc m_p4+1
		inc m_p2
		bne +
		inc m_p2+1
+		inc m_p3
		bne +
		inc m_p3+1
+		bcc uh4
		tya
		beq cserep3p2.rrts
		gne mon_scrolldown
		.pend

get_addr2x_cserep3p2 .proc
		jsr get_addr2xx
cserep3p2	.proc
		ldx m_p2
		lda m_p3
		sta m_p2
		stx m_p3
		ldx m_p2+1
		lda m_p3+1
		sta m_p2+1
		stx m_p3+1
rrts		rts
		.pend
;---------------------------
;H Hunt
;
u_h		.proc
		jsr get_addr2x_cserep3p2
		jsr get_addr2x_cserep3p4

		ldx #0

xcc		jsr get_addr2
		cpy #40
		bcs hii
		cmp #32
		beq opo
		cmp #"?"
		beq kx
                cmp #"'"
                beq +
		cmp #'"'
		bne hii
+
                sta m_tmp
tcg		iny
		cpy #40
		bcs hii
		lda (m_p1),y
		cmp m_tmp
		beq xcc
                lsr m_tmp
                bcs +
		jsr fromsccode
                clc
+               rol m_tmp
		sta monitor_buffer,x
		#stz_a monitor_buffer+40,x
		inx
		gne tcg

opo		lda m_p3
		sta monitor_buffer,x
		#lda0
kx		sta monitor_buffer+40,x
		inx
		gcs xcc

hii		txa
		sta m_tmp
		bne uh5
xg		rts

uh5             ldx m_p2
		lda m_p2+1
uh5k		stx m_p3
		sta m_p3+1
		ldx #0
uh6		stx m_tmp2
		jsr mon_getbyte
		ldx m_tmp2
		cmp monitor_buffer,x
		beq ih4x
		lda monitor_buffer+40,x
		beq ih4
ih4x		inc m_p2
		bne +
		inc m_p2+1
+		inx
		cpx m_tmp
		bne uh6
                jsr mon_scrolldown_on_eol
		ldx #m_p3
		jsr mon_printhexaddrsp
		jsr checkkey1
		beq xg

ih4		ldx m_p3
		cpx m_p4
		lda m_p3+1
		sbc m_p4+1
		bcs kq
		cpx #255
		inx
		stx m_p2
		#lda0
		adc m_p3+1
		sta m_p2+1
		#jmps uh5k

kq		#jmps mon_scrolldown
		.pend
;---------------------------
;S Save
;
u_s		.proc
		jsr mon_getfname_ls
		bcs bi+1
		jsr get_addr2
		bne bi			;freeze!
		jsr cserep3p2
		jsr get_addr2x_cserep3p4
		jsr get_addr2
                beq start
                lda m_p2
                sta m_p3
                lda m_p2+1
                sta m_p3+1
                #jmps start

bi		lda #$60		;rts
		sta m_tmp

start           lda m_01
                cmp #8
                bcs bi+1

		lda #1
		jsr mon_open		;-=>exit
		jsr mon_chkout		;-=>exit
		lda m_tmp
                bmi uhu
		beq simplesave

		jsr mon_frzcomm
dy		#sta_cfg_open
		dey
		#lda_zp_y m_p2
		#sta_cfg_16
		jsr mon_chrout		;-=>exit
		tya
		bne dy
		dec m_p2+1
		dex
		bne dy
		stx m_01
		stx m_p2
		stx m_p2+1
		dex
		stx m_p4
		stx m_p4+1
                gmi uhu

simplesave    	lda m_p3
		jsr mon_chrout		;-=>exit
		lda m_p3+1
		jsr mon_chrout		;-=>exit

uhu		jsr writeb
kesz		jsr mon_close
		jsr mon_chrout.sii     	;-=>exit
pok		jsr mon_scrolldown
                .enc screen
six		lda #"o"
		sta (m_p1),y
		iny
		lda #"k"
		sta (m_p1),y
                iny
                .enc none
		gne clear_scrolldown

writeb		lda m_tmp
		bne dsj
                inc m_tmp

                lda m_01
                cmp #8
                bcs dsj
                lda m_p2+1
                cmp #>$0800
                bcc dsj
                ldy m_p4+1
                lda m_p4
                cpy #>$d000
                bcc kkd
                lda #<$cfff
                ldy #>$cfff
kkd             sec
		sbc m_p2
                tax
                tya
                sbc m_p2+1
                bcc dsj
                tay
                inx
                bne +
                iny
+               lda m_01
                and #3
                cmp #3
                lda #m_p2
                bcs al
                jsr newwrite_t_mon
                #jmps dd
al              jsr newwrite_t
dd              bcs dk
		txa
                bne +
                dey
+               dex
                txa
                adc m_p2
                sta m_p2
                tya
                adc m_p2+1
                sta m_p2+1
                #jmps dsj2

dk		jsr mon_chkout		;-=>exit
dsj		jsr mon_getbyte
		jsr mon_chrout		;-=>exit
dsj2		lda m_p2
		cmp m_p4
		lda m_p2+1
		sbc m_p4+1
                bcs mon_chrout.vege
		inc m_p2
		bne writeb
                lsr m_tmp
                inc m_p2+1
		jsr checkkey1
                bne writeb
                .byte SKIP2

sii2		pla
		pla
;---------------------------
;Close
;
mon_close	.proc
		jsr newclrchn_t
		lda ActualLogical
		#jmps newclose_t
                .pend
;---------------------------
;Chrin
;
mon_chrin	.proc
		lda Status
                bne u_s.sii2
                inc $d020
                dec $d020
                #jmps newchrin_t
;---------------------------
;Chrout
;
mon_chrout	.proc
		jsr newchrout_t
sii		bcs u_s.sii2
		inc $d020
		dec $d020
		lda Status
		bne u_s.sii2
vege		rts
;---------------------------
;Chkout
;
mon_chkout      .proc
		ldx ActualLogical
		jsr newchkout_t
		#jmps mon_chrout.sii
		.pend
;---------------------------
;Chkin
;
mon_chkin      .proc
		ldx ActualLogical
		jsr newchkin_t
		#jmps mon_chrout.sii
		.pend
;---------------------------
;Open
;
mon_open	.proc
		sta ActualSecAddr

		ldy ActualDev
                jsr newclall_t		;be safe!
		sty ActualDev

		jsr LWOP_findunusedchannel
		jsr newopen_t
                #jmps mon_chrout.sii
		.pend
;---------------------------
;K Defreeze
;
u_k		.proc
		jsr mon_getfname
                lda m_01
                ror
                cmp #4
                bcs u_v.eee

		#stz_a VerifyFlag
		#stz_aa m_tmp
                #stz_aa m_01
                sec
		gcs u_v.in
		.pend
;---------------------------
;L Load
;
u_l		.proc
		#lda0
		.byte SKIP2
;---------------------------
;V Verify
;
u_v		.proc
eee		lda #$60	;rts
		sta VerifyFlag
		jsr mon_getfname_ls
		bcs eee+1
		jsr get_addr2
		bne nh
		inc m_tmp
		jsr cserep3p4
                #jmps oj
nh
		lda m_tmp
                bmi eee+1

oj		clc
in		ror m_tmp2
		#lda0
		jsr mon_open		;-=>exit
		jsr mon_chkin	  	;-=>exit
		lda m_tmp2
		bpl simpleload

		jsr mon_frzcomm
dy		jsr mon_chrin
		dey
		#sta_cfg_open
		#sta_zp_y m_p2
		#sta_cfg_16
		bne dy
		dec m_p2+1
		dex
		bne dy
		stx m_p2
		stx m_p4
		stx m_p4+1
		ldy #>$800
		tya
		jsr read
		bcs br1
		lda #>$0fff
		jsr copy
		lda #>$0800
		ldy #>$c800
		jsr read
		lda #>$e000
		ldy #>$1000
		jsr read
		ldx #>$d000
		lda #>$efff
		jsr copy
		lda #>$e000
		ldy #>$2000
		jsr read
		stx m_p3
		stx m_p3+1
		dex
		stx m_p4
		stx m_p4+1
		#jmps dgu

copy		sta m_p3+1
		stx m_p4+1
		lda #$ff
		sta m_p3
		gne u_t.in

read		sta m_p2+1
		ldx #0
		lda #m_p2
		jmp newread_t

simpleload     	lda m_tmp
		bmi br2
		jsr mon_chrin
		sta m_p2
		jsr mon_chrin
		sta m_p2+1
                #jmps br2

br1             #stz_a m_p2+1
                jsr mon_chkin

br2     	ldx m_p4
	    	ldy m_p4+1
		lda m_tmp
		bne uhux
		ldx m_p2
		stx m_p4
		ldy m_p2+1
		sty m_p4+1
uhux		sty m_p3+1
		stx m_p3
		ldy #0

uhu2		lda VerifyFlag
		beq noverify
		lda m_p4
		sta m_p2
		lda m_p4+1
		sta m_p2+1
		jsr mon_getbyte
		sta m_tmp
		jsr mon_chrin
		cmp m_tmp
		beq dgu
		tya
		beq +
		cpy #40
		bcc ++
+		jsr mon_scrolldown
+		ldx #m_p2
		jsr mon_printhexaddrsp
		#jmps dgu

noverify        lda m_tmp2
                bmi nf
                lda m_01
                cmp #8
                bcs nf
                lda m_p4+1
                cmp #>$0800
                bcc nf
                cmp #>$d000
                bcc jo
                cmp #>$e000
                bcc nf
jo              #lda0
		sec
                sbc m_p4
                tax
                lda #>$d000
                sbc m_p4+1
                bcs +
                adc #$30
+               tay
                lda #m_p4
                sec
                ror m_tmp2
                jsr newread_t
                bcs nf2
                txa
                bne +
                dey
+               dex
                txa
                adc m_p4
                sta m_p4
                tya
                adc m_p4+1
                sta m_p4+1
                lda Status
                and #$fd
                #jmps dgua
nf2
		jsr mon_chkin
nf
		jsr mon_chrin
		jsr mon_storebyte
dgu		lda Status
dgua		eor #$40
		beq sixq
		inc m_p4
		bne uhu2x
		jsr checkkey1
		beq mon_close
                lsr m_tmp2
		inc m_p4+1
uhu2x		#jmps uhu2

sixq		sta Status
		jsr mon_close
		jsr mon_chrout.sii     	;-=>exit
		jsr mon_scrolldown
		ldx #m_p3
		jsr mon_printhexaddr
		lda #"-"
		ldx #m_p4
		jsr mon_print_hereaddr
                jsr pspace
		#jmps u_s.six		;print "OK"
		.pend
;---------------------------
;A Assembly
;
u_a		.proc

		.if SUPERCPU=1
offs		= 58
		.else
offs		= 55
		.fi

		jsr get_addr2x_cserep3p4
nj		iny
		cpy #37
		bcs vvv
		ldx #0
                stx m_tmp
sc		lda (m_p1),y
		beq nj
                .enc screen
		cmp #"z"+1
                .enc none
		bcs nj
                sta 4+monitor_buffer,x
		iny
		inx
                cpx #3
		bcc sc
		lda (m_p1),y
                and #$bf
		eor #$20
		beq spa
                dey
spa		lda #$20
		.byte SKIP2
dc		lda (m_p1),y
                ora #0
		jsr isnumnext2
		bcs oe
                .enc screen
                cmp #"a"
                .enc none
                bne ku
		lda (m_p1),y
                cmp #" "
                beq oe
ku		lda m_tmp
		bne ibbx
                jsr sml
                lda #"$"
		sta 4+monitor_buffer-1,x
                dey
oe		cmp #"$"
		bne ibbx
		lda m_tmp
ibbx		bne ibb
		inc m_tmp
		#phy_a
		jsr getaddr
		#ply_a
		jsr isnumnext
		bcs ibb
		jsr isnumnext
		bcc ibb2
		jsr sml
		lda #"0"
		sta 4+monitor_buffer-2,x
		inx
		gcs ibb

sml		lda 4+monitor_buffer-1,x
		sta 4+monitor_buffer,x
		lda 4+monitor_buffer-2,x
		sta 4+monitor_buffer-1,x
vvv		rts

sml2		jsr sml
		dex
                dex
                jsr sml
                inx
                inx
                rts

ibb2		jsr isnumnext
                .if SUPERCPU=1
                bcs ibb4
                .else
		bcs ibb
                .fi
	     	jsr isnumnext
                .if SUPERCPU=1
                bcc ibb3
                .else
		bcc ibb
		.fi
		jsr sml2
		lda #"0"
		sta 4+monitor_buffer-4,x
		inx
                .if SUPERCPU=1
                gcs ibb

ibb3		jsr isnumnext
		bcs ibb
		jsr isnumnext
		bcc ibb
		jsr sml2
		lda 4+monitor_buffer-5,x
		sta 4+monitor_buffer-4,x
		lda 4+monitor_buffer-6,x
		sta 4+monitor_buffer-5,x
		lda #"0"
		sta 4+monitor_buffer-6,x
		inx
                gcs ibb

ibb4		cmp #","
		bne ibb
                jsr isnumnext
                cmp #"$"
		bne ibb
		#phy_a
		jsr getaddrnc
		#ply_a
                jsr isnumnext
		.fi

ibb		cpy #40
		bcc dc

		#psz m_p1
		lda #$4c
		sta mon_getbyte
		lda #<mon_fake_getbyte
		sta mon_getbyte+1
		lda #>mon_fake_getbyte
		sta mon_getbyte+2
		lda #$60
		sta mon_setprefetch

		lda #<(monitor_buffer+40)
		sta m_p1
		lda #>(monitor_buffer+40)
		sta m_p1+1

		#stz_a $100
		lda m_p3
		sta $101
		lda m_p3+1
		sta $102
                .if SUPERCPU=1
                lda m_p3+2
		sta $103
                .fi

meg		#stz_a m_p2
		ldx $100
                ldy opc,x
                lda o_table,y
                and #31
                cmp 4+monitor_buffer
                bne bt

                .if SUPERCPU=1
                cpx #$80
                beq bbra
                cpx #$82
                beq bbra
                cpx #$62
                beq bbra
                .fi
                txa
		and #%00011111
		cmp #$10		;branch?
		bne nth
bbra		stx $104
		lda #4
		sta m_p2
nth
		lda #$01
		sta m_p2+1
		ldy #5
		jsr mon_printd2

		ldx #0
		lda m_p2
		cmp #6
		bcc xfv

		lda #128
                .if SUPERCPU=1
                ldy #23
                .else
		ldy #20
                .fi
		.byte SKIP1BEQ
xfv		txa
		sta (m_p1),y	          ;012345678901234567890
dd		lda monitor_buffer+offs,x ;,1234 11 22 33 LDA $1234
		clc
		bmi ok
		lda 4+monitor_buffer,x
		eor #" "
		bne hb
		lda monitor_buffer+offs,x
		beq ok
hb		lda monitor_buffer+offs,x
		cmp 4+monitor_buffer,x
		bne bt
		inx
		gpl dd
bt		inc $100
		bne meg
err		sec
ok
		php
		lda $100
                ldx m_p3
                .if SUPERCPU=1
                cmp #$80
                beq bbra2
                cmp #$82
                beq bbra3
                cmp #$62
                beq bbra3
                .else
		cmp #$1a
		bne nnop
                .enc screen
		lda #"e"
		sta monitor_buffer+offs-9
                .enc none
		lda #$ea
		sta $100
nnop
		.fi
		and #%00011111
		cmp #$10
		bne hi
bbra2
		lda #2-1
		adc m_p4
		sta m_tmp2
		#lda0
		adc m_p4+1
		sta m_tmp2+1

		lda m_p3
		sec
		sbc m_tmp2
		tax
		lda m_p3+1
		sbc m_tmp2+1
		bcc ms
		bne errc
		txa
		bmi errc
		gpl mege
ms		cmp #$ff
		bne errc
		txa
		bmi mege
errc		plp
		sec
		php
                .if SUPERCPU=1
                #jmps mege
bbra3
		lda #3-1
		adc m_p4
		sta m_tmp2
		#lda0
		adc m_p4+1
		sta m_tmp2+1

		lda m_p3
		sec
		sbc m_tmp2
		tax
		lda m_p3+1
		sbc m_tmp2+1
                pha
                ldy #9
		jsr mon_printhex
                ldy #23
                lda m_p3+1
		jsr mon_printhex
                pla
                sta m_p3+1
		#jmps tu
                .fi

mege		ldy #9
		jsr mon_printhex
                .if SUPERCPU=1
                ldy #23
                .else
		ldy #20
                .fi
		lda m_p3+1
		jsr mon_printhex
tu      	lda m_p3
		jsr mon_printhex
		stx m_p3
hi		plp

		pla
		sta m_p1
		pla
		sta m_p1+1
		bcs vvv2

                lda m_p4
                sta m_p2
                lda m_p4+1
                sta m_p2+1
		lda $100
		ldy #32
		jsr mon_storebyte_inc
                .if SUPERCPU=1
		cpy monitor_buffer+offs-9
		beq vv
		lda m_p3
		jsr mon_storebyte_inc
                .fi
		cpy monitor_buffer+offs-6
		beq vv
                .if SUPERCPU=1
                lda m_p3+1
                .else
		lda m_p3
                .fi
		jsr mon_storebyte_inc
		cpy monitor_buffer+offs-3
		beq vv
                .if SUPERCPU=1
                lda m_p3+2
                .else
		lda m_p3+1
                .fi
		jsr mon_storebyte_inc
vv
		jsr vvv2
                .if SUPERCPU=1
                lda (m_p1)
                .else
                ldy #0
                lda (m_p1),y
                .fi
                ldy #6
                sta m_tmp2
		cmp #","
                bne +
                ldy #2
+		gne sphr

vvv2		#sta_cfg_open
		lda monitor_01_exp
		#jmps u_b.in

		#assert_8000romonly
isnumnext	lda (m_p1),y
isnumnext2    	beq ib
                .enc screen
		cmp #"g"
		bcc ob
                .enc none
		cmp #"0"
		bcc ib
		cmp #":"
                bne ob
                lda #32
		.byte SKIP1BEQ
ib		sec
ob
		sta 4+monitor_buffer,x
		inx
		iny
		rts

		#assert_8000romonly
mon_fake_getbyte
		ldx m_p2
		lda $100,x
		rts
		.pend
;---------------------------
;@ Disk command
;
		#assert_cfg_16
u_at		.proc
		sty m_tmp
		jsr mon_getfname
		bcc lu
		#stz_a ActualFNLength
		lda #" "
		ldy #40
tr		dey
		cpy m_tmp
		beq lu
		cmp (m_p1),y
		beq tr
		sty m_tmp2
		ldx #0
		ldy m_tmp
ic		iny
		lda (m_p1),y
		jsr fromsccode
		sta monitor_buffer,x
		inx
		cpy m_tmp2
		bcc ic
		jsr mon_getfnameset

lu		lda ActualFNLength
		beq tii
		lda monitor_buffer
		eor #"$"
		bne tii
		.byte SKIP2
tii		lda #15

		jsr mon_open		;-=>exit
		jsr mon_chkin		;-=>exit
		lda ActualSecAddr
		and #15
		beq dir
		jsr mon_scrolldown
ti		jsr mon_chrin		;-=>exit
		ldx Status
		bne ti2
		jsr tosccode
		sta (m_p1),y
		iny
		cpy #40
		bcc ti
ti2		jsr mon_scrolldown
		jmp mon_close

dir		jsr mon_chrin		;-=>exit
		jsr mon_chrin		;-=>exit

		lda #128
dirlp		sta m_tmp
		jsr checkkey1
		beq ti2
		jsr mon_scrolldown
		lda m_p1
		sta m_p2
		lda m_p1+1
		sta m_p2+1
		jsr mon_chrin		;-=>exit
		jsr mon_chrin		;-=>exit
		jsr mon_chrin		;-=>exit
		tax
		jsr mon_chrin		;-=>exit
		jsr numtoasc16

		jsr mon_chrin
                tax
                jsr pspace
                txa
                cmp #18
                bne jfj
                geq xd

tii2		sta (m_p1),y
xd2		iny
xd		jsr mon_chrin
		beq dirlp
jfj     	jsr tosccode
		eor m_tmp
		cpy #40
		bcc tii2
		gcs ti2
		.pend
;---------------------------
;<- Remember position
;
u_bnyil		.proc
		jsr get_addr2
		beq xu
in		clc
xu		ldx m_tsp
		lda m_p3
		bcs +
		lda m_p2
+		sta m_tstack,x
		lda m_p3+1
		bcs +
		lda m_p2+1
+		sta m_tstack+8,x
		inx
                .if SUPERCPU=1
		txa
		and #7
		sta m_tsp
                .else
                lda #7
                sax m_tsp
                .fi
		jsr get_addr2
		beq xu
		gne u_fnyil.sd
		.pend
;---------------------------
;^ Restore position
;
u_fnyil		.proc
		jsr poptsp
		sta m_p2
		lda m_tstack+8,x
		sta m_p2+1
sd		#jmps mon_scrolldown
		.pend

;---------------------------
;* RAM/ROM
;
u_star	       	jsr mon_scrolldown
		#sta_cfg_open
		lda monitor_01_exp
		#sta_cfg_16
                and #$fb
                php
                beq lf
                lda #3
lf              eor #7
		jsr u_b.in

		.enc screen
		lda #"r"
		sta (m_p1),y
		iny
		lda #"a"
                plp
                bne +
                lda #"o"
+		sta (m_p1),y
		iny
		lda #"m"
		sta (m_p1),y
                .enc none
		gne u_fnyil.sd
;---------------------------
;N Numberconvert
;
u_n		.proc
	      	ldx #0
xf		iny
		lda (m_p1),y
		jsr fromsccode
		sta $200,x
                inx
		cpy #37
		bcc xf

		lda #>calc.b
                sta $301
		lda #<calc.b
                sta $300
		lda #<__myexpr
                sta $30a
		lda #>__myexpr
                sta $30b
                ldx #63
rp              lda calcf,x
                sta calc,x
                dex
                bpl rp
                jsr calc
		bcs clear_scrolldown.in
                jsr mon_scrolldown
		lda m_p1
		sta m_p2
		lda m_p1+1
		sta m_p2+1
                ldx $14
                lda $15
                jsr numtoasc16
                jsr pspace
                ldx #$14
		jsr mon_printhexaddr
                jsr pspace
                lda $15
                jsr binary
                lda $14
                jsr binary
                jsr pspace
                lda $14
                ldx #0
                jsr kcjjc3
                jsr kcjjc3
                jsr pspace
		jsr kcjjc
		jsr kcjjc2

		#assert_8000romioonly
clear_scrolldown .proc
                jsr mon_clearend
in              jmp mon_scrolldown
                .pend

		#assert_8000romonly
kcjjc           .proc
                lda $14
                ldx #0
kcjjc2          .proc
                jsr tosccode
kcjjc3          .proc
                sta (m_p1),y
                iny
		inx
                lda $14,x
                rts
                .pend

		#assert_8000romioonly
calcf
		.logical $2c0
calc		.proc
		sta cfg_stnd
                jsr $aaca		;trailer 0
                stx $7a
                sty $7b
                stx $3a
		tsx
                stx m_p2
                jsr $73
                jsr $a57c		;tokenize
                jsr $73
                jsr $ad8a		;get real
                jsr $b7f7		;to integer
		clc
                .byte SKIP1BCS
b               sec
		sta bank_port3
		sta cfg_16
                ldx m_p2
                txs
                rts
		.pend
                .here
;---------------------------
;Pop tsp
;
		#assert_8000romonly
poptsp		.proc
		ldx m_tsp
		dex
		txa
		and #7
		sta m_tsp
		tax
		lda m_tstack,x
		rts
		.pend
;---------------------------
;E Edit
;
		#assert_cfg_16
u_e		.proc
		iny
		cpy #40
		bcs xf

		lda (m_p1),y
                .enc screen
		cmp #"c"
		beq u_ec
		cmp #"s"
		beq u_es
                .enc none
xf		rts
;---------------------------
;IV vector reset
;
u_iv            .proc
                #sta_cfg_open
                ldx #$1f
-               lda $314,x
                sta monitor_data_exp+$314,x
                dex
                bpl -
                .if SUPERCPU=1
                lda #<__nmi
                sta monitor_data_exp+$318
                gra u_s.pok
                .else
                inc monitor_data_exp+$318
                #sta_cfg_16
                gmi u_s.pok
                .cerror (<__nmi) < $80
                .fi
                .pend
;---------------------------
;I Ascii
;
u_i		.proc
		cpy #39
		bcs up
		iny
		lda (m_p1),y
		cmp #"*"
		beq u_j
                .enc screen
                cmp #"v"
                beq u_iv
                cmp #"o"
                bne ds
                .enc none
                ldx #ds-lo-1
ie
		#phx_a
		lda lo,x
                php
                tax
                lsr
                and #$38
                sta m_p2
                txa
                and #$0f
                ora #$d0
                sta m_p2+1
                jsr mon_scrolldown
                jsr mon_printio
                plp
                bpl sw
                jsr mon_scrolldown
sw              #plx_a
		dex
                bpl ie
                rts

lo              .byte $9d
                .byte $0d
                .byte $9c
                .byte $0c
                .byte $d0
                .byte $40
                .byte $30
                .byte $20
                .byte $10
                .byte $00

ds		dey
up		ldx #3
		.byte SKIP2
;---------------------------
;D Disassembly
;
u_d		.proc
		ldx #2
		.byte SKIP2
;---------------------------
;ES Sprite
;
u_es		.proc
		ldx #5
		.byte SKIP2
;---------------------------
;EC binary
;
u_ec		.proc
		ldx #4
		.byte SKIP2
;---------------------------
;J Screencode
;
u_j		.proc
		ldx #1
		.byte SKIP2
;---------------------------
;M Hex
;
u_m		.proc
		ldx #0
		jsr checkn
		lda mon_printlo,x
		sta m_p4
		lda mon_printhi,x
		sta m_p4+1
		lda len,x
		sta m_tmp2
		clc
		adc m_p2
		sta m_p3
		#lda0
		adc m_p2+1
		sta m_p3+1
xp		iny
		lda (m_p1),y
                cmp #32
		beq xp2
                eor #"-"
                php
		bne ff
                sta m_tmp2
                iny
ff      	dey
		jsr get_addr2
                bne sd
sd2		pha
		lda m_p3
                sta m_p2
		clc
		adc m_tmp2
		sta m_p3
		lda m_p3+1
		sta m_p2+1
		adc #0
		sta m_p3+1
		pla
                plp
                bne nn
                #psr b_f2
                #jmps ll

sd		cmp #"-"
		beq sd2
		plp
		beq aa
                gne ll

nn		eor #32
		beq xi
		cmp #"-"^32
		bne ll
xi		tax
		jsr get_addr2
		beq ll
		txa
		beq ll
aa              #psr b_f8
		#jmps ll

xp2		cpy #39
		bcc xp
ll		jsr mon_scrolldown
		lda m_p2+1
                pha
		jsr b_f5.jmp_m_p4
		pla
                bpl rj
		lda m_p2+1
		bpl sss
rj		jsr pause
                beq sss
bu		lda m_p2
		cmp m_p3
		lda m_p2+1
		sbc m_p3+1
		bcc ll
sss		jmp topos6

len		.byte 32,128,8,128,8,63
		.pend

pause2		.proc
		jsr slowdown
pause	  	.proc
		jsr checkkey1
		beq sss
		cmp #$fb
		beq dd
		jsr checkkey2
		cmp #$ff
		beq dd
                jsr checkn
                jsr checkn2
                jsr checkkey1
                beq sss
                jsr checkn
dd              lda #1
sss		rts
		.pend

		#assert_8000romioonly
checkn		jsr checkkey2
		cmp #$ff
		bne checkn
		geq xc
checkn2		jsr checkkey2
		cmp #$ff
		beq checkn2
xc		jmp mon_waitretrace

		#assert_cfg_16
mon_printlo	.byte <mon_printm,<mon_printj,<mon_printd,<mon_printi,<mon_printec,<mon_printes,<mon_printa,<mon_printio
mon_printhi	.byte >mon_printm,>mon_printj,>mon_printd,>mon_printi,>mon_printec,>mon_printes,>mon_printa,>mon_printio
;---------------------------
		#assert_8000romonly
mon_print_herex	.proc
		sta last_cmd
                #jmps mon_print_here
;---------------------------
;Print hex number with space
;
		#assert_8000romonly
mon_printhexaddrsp .proc
		lda #32
;---------------------------
;Print address
;
		#assert_8000romonly
mon_print_hereaddr .proc
		.byte SKIP2
;---------------------------
;Print mode and address (:1000)
;
		#assert_8000romonly
mon_print_here	.proc
		ldx #m_p2
		sta (m_p1),y
		iny
;---------------------------
;Print hex number
;
		#assert_8000romonly
mon_printhexaddr .proc
		lda $01,x
		jsr mon_printhex
		lda $00,x
		#jmps mon_printhex

                #assert_cfg_16
mon_getbyte_inc_print .proc
		jsr mon_getbyte_inc
                tax

		#assert_8000romonly
mon_printhexsp	.proc
		pha
                jsr pspace
		pla
                #assert_8000romonly
mon_printhex	.proc
		pha
		lsr
		lsr
		lsr
		lsr
		jsr h
		pla
		and #15
h		cmp #10
		bcc +
		sbc #$39
+		adc #$30
		sta (m_p1),y
		iny
		rts
		.pend
;---------------------------
;Print binary
;
		#assert_8000romonly
binary  	.proc
		sec
		rol
fg		pha
		lda #"0" >> 1
                rol
		sta (m_p1),y
		iny
		pla
		asl
		bne fg
		rts
                .pend
;---------------------------
;Scroll downward
;
		#assert_8000romioonly
mon_scrolldown_on_eol .proc
		cpy #40
		bcc mon_scrolldown.x
                #assert_8000romioonly
mon_scrolldown	.proc
		lda m_p1
		ora m_p1+1
		eor #$c7
		bne eu

                jsr slowdown

tp		lda $d011
		bpl tp

		ldy #40
ni		lda $400,y
		sta $400-40,y
		iny
		lda $400,y
		sta $400-40,y
		iny
		bne ni
ni2		lda $500,y
		sta $500-40,y
		iny
		lda $500,y
		sta $500-40,y
		iny
		bne ni2
ni3		lda $600,y
		sta $600-40,y
		iny
		lda $600,y
		sta $600-40,y
		iny
		bne ni3
ni4		lda $700,y
		sta $700-40,y
		iny
		lda $700,y
		sta $700-40,y
		iny
		cpy #232
		bne ni4

		ldy #40
		lda #32
nn		sta $7bf,y
		dey
		bne nn
x		rts

eu		ldy #0

		#assert_8000romonly
mon_cnextline	.proc
		lda m_p1
		clc
		adc #40
		sta m_p1
		bcc +
		inc m_p1+1
+		rts
		.pend

		#assert_8000romioonly
slowdown	.proc
		ldx #20
iy		jsr checkkey1
		cmp #$fb
		bne tp
                #sta_cfg_open
                lda setup_atribut2_exp
                #sta_cfg_16
                #assert_8000romioonly
                and #CMOS_CONTROL
                bne iy
		jsr mon_waitretrace
		dex
		bne iy
tp              rts
		.pend
;---------------------------
;Scroll upward
;
		#assert_8000romonly
mon_scrollup	.proc
		#stz_a m_p1
		lda #>$400
		sta m_p1+1

-		lda $d011
		bpl -

		ldy #39
ni1
		lda $400+480,y
		sta $7c0,y

		lda $400+440,y
		sta $400+480,y

		lda $400+400,y
		sta $400+440,y

		lda $400+360,y
		sta $400+400,y

		lda $400+320,y
		sta $400+360,y

		lda $400+280,y
		sta $400+320,y

		lda $400+240,y
		sta $400+280,y

		lda $400+200,y
		sta $400+240,y

		lda $400+160,y
		sta $400+200,y

		lda $400+120,y
		sta $400+160,y

		lda $400+80,y
		sta $400+120,y

		lda $400+40,y
		sta $400+80,y

		lda $400,y
		sta $400+40,y

		lda $3c0,y
		sta $400,y
		dey
		bpl ni1

		ldy #39
ni2
		ldx $7c0,y

		lda $400+920,y
		sta $7c0,y

		lda $400+880,y
		sta $400+920,y

		lda $400+840,y
		sta $400+880,y

		lda $400+800,y
		sta $400+840,y

		lda $400+760,y
		sta $400+800,y

		lda $400+720,y
		sta $400+760,y

		lda $400+680,y
		sta $400+720,y

		lda $400+640,y
		sta $400+680,y

		lda $400+600,y
		sta $400+640,y

		lda $400+560,y
		sta $400+600,y

		lda $400+520,y
		sta $400+560,y

		txa
		sta $400+520,y

		dey
		bpl ni2
		iny
		rts
		.pend
;---------------------------
;Print memory as hex
;
		#assert_cfg_16
mon_printio	.proc
                lda #5
                sta m_01
		lda #"-"
                jsr mon_print_herex
dp2		jsr mon_getbyte_mem
		jsr inc_m_p2
		jsr mon_printhexsp
		cpy #29
		bcc dp2
                gcs mon_clearend
;---------------------------
;Print memory as hex
;
mon_printm	.proc
		lda #":"
		jsr mon_print_herex
		lda #8
		jsr mon_setprefetch

dp2		jsr mon_getbyte_inc_print
                #phx_a
		cpy #29
		bcc dp2
                jsr pspace
		ldy #37
dp		pla
		jsr tosccode
		sta (m_p1),y
		dey
		cpy #29
		bne dp
                ldy #38
		gne mon_clearend
		.pend
;---------------------------
;Print memory as screencode
;
mon_printj	.proc
		lda #"."
		.byte SKIP2
;---------------------------
;Print memory as ascii
;
mon_printi	.proc
		lda #"'"
		cmp #"."
		ror m_tmp
		jsr mon_print_herex
                jsr pspace
;		lda #32
		jsr mon_setprefetch

ny		jsr mon_getbyte_inc
		bit m_tmp
		bmi ua
		jsr tosccode
ua		sta (m_p1),y
		iny
		cpy #38
		bne ny
;---------------------------
;Clean till end of line
;
		#assert_8000romonly
mon_clearend	.proc
		lda #32
iy		sta (m_p1),y
		iny
		cpy #40
		bcc iy
		rts
		.pend
;---------------------------
;Print memory as character
;
		#assert_cfg_16
mon_printec	.proc
		.enc screen
		lda #"["
                .enc none
		jsr in
		#jmps mon_clearend

in		jsr mon_print_herex

		jsr pspace

in2		jsr mon_getbyte_inc

		sec
                rol
-		pha
		lda #"."
		bcc +
		lda #"#"
+		sta (m_p1),y
		iny
		pla
		asl
		bne -
		rts
		.pend
;---------------------------
;Print memory as sprite
;
mon_printes	.proc
		.enc screen
		lda #"]"
                .enc none
		jsr mon_printec.in
		jsr mon_printec.in2
		jsr mon_printec.in2
		lda m_p2
		and #63
		cmp #63
		bne mm
                jsr inc_m_p2
mm		#jmps mon_clearend
		.pend
;---------------------------
;Calculate opcode length
;
mon_getoplen	.proc
		lsr
		tax
		lda a_mod,x
		bcc +
		lsr
		lsr
		lsr
		lsr
                sec
+		and #15
		rol
		tax
		lda a_len,x
                and #3
		rts
		.pend
;---------------------------
;Print memory as disassembly
;
mon_printa	.proc
		.enc screen
		lda #"a"
                .enc none
                .byte SKIP2
mon_printd	.proc
		lda #","
		jsr mon_print_herex
		jsr mon_printd2
		#jmps mon_clearend

mon_printd2	.proc
		#psz m_p3

		jsr mon_getbyte_inc_print
		lda opc,x
		sta m_p3+1
		txa
		jsr mon_getoplen
		sta m_p3
                lda a_len,x
		pha
		dec m_p3
		bmi ip
		jsr mon_getbyte_inc_print
                stx monitor_buffer+1
		dec m_p3
		bmi ip
		jsr mon_getbyte_inc_print
		stx monitor_buffer+2
                .if SUPERCPU=1
		dec m_p3
		bmi ip
		jsr mon_getbyte_inc_print
		stx monitor_buffer+3
                .fi
ip
		lda #32
ii		sta (m_p1),y
		iny
                .if SUPERCPU=1
                cpy #18
                .else
		cpy #15
                .fi
		bne ii

		ldx m_p3+1
		lda o_table+1,x
		sta m_p3+1
		lda o_table,x
		jsr mon_printopc
                jsr pspace

		pla
                sta m_p3
                #andlsr $0c
                lsr
                beq +
                tax
                lda a_first-1,x
                sta (m_p1),y
                iny
+               lda m_p3
		cmp #$f0		;relative
		and #3
                beq n4i
                sta m_p3+1

		bcc n5
                sta m_p3

                .if SUPERCPU=1
                cmp #1
                beq nor
                lda monitor_buffer+1
                clc
                adc m_p2
                sta monitor_buffer+1
                lda monitor_buffer+2
                adc m_p2+1
                #jmps dc
nor
		.fi

                inc m_p3+1

		#ldatax monitor_buffer+1
		clc
		adc m_p2
		sta monitor_buffer+1
		txa
		bmi s
		#lda0
		adc m_p2+1
		#jmps dc

s		lda m_p2+1
		sbc #0
dc		sta monitor_buffer+2

n5		lda #"$"
		sta (m_p1),y
		iny

tt      	ldx m_p3+1
		lda monitor_buffer,x
		jsr mon_printhex
                dec m_p3+1
                bne tt

		lda m_p3
                lsr
                lsr
                lsr
                lsr
                beq n4
                .if SUPERCPU=1
                cmp #2
                bne nkn
                dey
                dey
                lda #","
		sta (m_p1),y
		iny
                lda #"$"
		sta (m_p1),y
		iny
                lda monitor_buffer+1
		jsr mon_printhex
                #jmps n4
                .fi
nkn             tax
tj              lda a_last-1,x
                lsr
cccu		sta (m_p1),y
                iny
                inx
		bcc tj

n4		pla
		sta m_p3
		pla
		sta m_p3+1
		rts

n4i		bcc n4
		.enc screen
                lda #"a"
                .enc none
                gcs cccu

mon_printopc	jsr in
		jsr +
+		lda m_p3
		ldx #5
br		lsr m_p3+1
		ror
		dex
		bne br
in		sta m_p3
		and #31
		sta (m_p1),y
		iny
		rts
		.pend
;---------------------------
;Store a byte to memory
;
mon_storebyte_mem .proc
		ldx m_p4+1
		cpx #$d0
		bcs uf3
                cpx #8
		bcc ntni
uf		.if SUPERCPU=1
		sta (m_p4)
		.else
		ldx #0
		sta (m_p4,x)
		.fi
		rts

uf3		cpx #$e0
		bcs uf
		ldx m_01
		cpx #5
		bcs uf2
		ldx #$33
		stx $01
		.if SUPERCPU=1
		sta (m_p4)
		.else
		sta (<(m_p4-$33),x)
		.fi
		lda #$37
		sta $01
		rts

ntni		tax
		lda m_p4+1
		pha
                #assert_cfg_open
		ora #>monitor_data_exp
		sta m_p4+1
		txa
		.if SUPERCPU=1
		sta [m_p4]
		.else
		ldx #0
		sta cfg_open
		.check CFG_OPEN_RAM, CFG_RAM
		sta (m_p4,x)
		sta cfg_16
		.fi
                #assert_cfg_16
		pla
		sta m_p4+1
		rts

uf2		#sta_cfg_open
		pha
		lda m_p4
		ldx m_p4+1
		cpx #$d4
		bcs n1
                .if SUPERCPU=1
                cpx #$d2
                .fi
		and #63
		tax
		pla
                .if SUPERCPU=1
                bcs n0
                .fi
		sta monitor_vic_exp,x
                .if SUPERCPU=0
                gcc jol
                .else
n0		rts
		.fi

		#assert_cfg_open
n1		beq c64
		cpx #$d8
		bcs n2
		pha
		lda $d030		;C128?
		lsr
		pla
		bcs c64
		cpx #$d6
		bcs n5

c64		and #31
		tax
		pla
		sta monitor_sid_exp,x
                .if SUPERCPU=0
                jmp jol
                .else
		rts
                .fi

		#assert_cfg_open
n2		cpx #$dc
		bcc color
		ora #$f0
		cpx #$dd
		beq cia2
		bcs n5
		eor #$10
cia2		tax
		pla
		sta monitor_cia1_exp-$e0,x
                .byte SKIP1
n5		pla
jol		#sta_cfg_16
		rts

		#assert_cfg_open
color		txa
                lsr
		pla
                clv
		bcs set_v
		asl
		asl
		asl
		asl
                #sev set_v
set_v           cpx #$da
		ldx m_p4
                bcs ii3
		eor monitor_d800_exp,x
		bvc ii2
                and #$f0
                .byte SKIP2
ii2		and #$0f
		eor monitor_d800_exp,x
		sta monitor_d800_exp,x
		#sta_cfg_16
		rts

		#assert_cfg_open
ii3		eor monitor_d800_exp+$100,x
		bvc ii5
                and #$f0
                .byte SKIP2
ii5		and #$0f
		eor monitor_d800_exp+$100,x
		sta monitor_d800_exp+$100,x
		#sta_cfg_16
		rts
		.pend
;---------------------------
;Prefetch bytes from drive
;
mon_setprefetch_dr .proc
		sta m_prefetch
		lda #"r"
		ldx #m_p2
		jsr drive_memcmd
		lda m_prefetch
		cmp #1
		beq nn
		jsr iecout
nn		jsr drive_endcmd       	;unlisten

		lda m_01
		jsr iectalk		;talk
		lda #$6f
		jmp iectksa		;tksa
		.pend
;---------------------------
;Get a byte from drive
;
mon_getbyte_dr	.proc
		#stz_a Status
vsz		dec m_prefetch
		bmi one
		bne more
		jsr iecin
		tax
sux		jsr iecuntalk		;untalk
		txa
		bit Status		;to fix to m xxf1 bug
		bvc ok
one		lda #1
		jsr mon_setprefetch
		dec m_prefetch
		jsr iecin
		tax
		jsr iecuntalk		;untalk
		txa
ok		rts

more		jsr iecin
		bit Status		;to fix the I 02c0 bug
		bvs sux
		rts
		.pend
;---------------------------
;Get a byte from memory
;
mon_getbyte_mem	.proc
		ldx #0
		lda m_p2+1
		bmi uk
		cmp #8
		bcc kji
                .if SUPERCPU=1
norm		lda (m_p2)
                .else
		lda (m_p2,x)
                .fi
		rts

kji		.if SUPERCPU=1
		tax
		ora #>monitor_data_exp
		sta m_p2+1
		lda [m_p2]
                stx m_p2+1
		.else
		pha
                #assert_cfg_open
		ora #>monitor_data_exp
		sta m_p2+1
		sta cfg_open
		.check CFG_OPEN_RAM, CFG_RAM
		lax (m_p2,x)
		sta cfg_16
                #assert_cfg_16
		pla
		sta m_p2+1
		txa
		.fi
		rts

tt		lda m_01
tt2		jmp $7e8

uk		cmp #$d0
		bcc tt
		cmp #$e0
		bcs tt
		lda m_01
		cmp #5
		bcc tt2

		#sta_cfg_open
		lda m_p2
		ldx m_p2+1
		cpx #$d4
		bcs n1
		.if SUPERCPU=1
                cpx #$d2
                bcs norm
		.fi
		and #63
		tax
		lda monitor_vic_exp,x
                .if SUPERCPU=0
                gcc jol
                .else
		rts
                .fi

		#assert_cfg_open
n1		cpx #$d8
		bcs n2
		cpx #$d4
		beq c64
		pha
		lda $d030		;C128?
		lsr
		pla
		bcc n5
c64		and #31
		tax
		lda monitor_sid_exp,x
                .if SUPERCPU=0
                gcs jol
                .else
		rts
                .fi

		#assert_cfg_open
n2		cpx #$dc
		bcs n3
		txa
		ldx m_p2
		#andlsr 3
		beq ct
		lda monitor_d800_exp+$100,x
		#jmps ib
ct		lda monitor_d800_exp,x
ib		bcs ed
		lsr
		lsr
		lsr
		lsr
ed		and #15
jol		#sta_cfg_16
		rts

		#assert_cfg_open
n3		ora #$f0
		cpx #$dd
		beq cia2
		bcs n5
		eor #$10
cia2		tax
		lda monitor_cia1_exp-$e0,x
		.byte SKIP2
                #assert_8000romioonly
n5		lda #$ff
		#sta_cfg_16
		rts
		.pend
;---------------------------
;Get a byte from internal
;
		#assert_cfg_16
mon_getbyte_int	.proc
		.if SUPERCPU=1
		lda m_p2+1
		cmp #$a0
		bcs mon_getbyte_mem.n5
		tax
		bmi ok
		cmp #$60
		bcs mon_getbyte_mem.n5
                cmp #$10
                bcc mon_getbyte_mem.n5
ok		lda [m_p2]
		.else
                lda #$f0
		bit m_p2+1
		bmi mon_getbyte_mem.n5
		beq mon_getbyte_mem.n5
		sta cfg_open
		#assert_cfg_open
		tax
		.check CFG_OPEN_RAM, CFG_RAM
		lda (m_p2-$f0,x)
		sta cfg_16
                #assert_cfg_16
		.fi
		rts
		.pend
;---------------------------
;Convert from ascii to screencode
;
                #assert_8000romonly
tosccode	.proc
		cmp #128
		bcs hi
		cmp #$20
		bcc inv
		cmp #$60
		bcc en
		and #$df
		bne hi
en		and #$3f
		rts

hi		and #$7f
		cmp #$7f
		bne +
		lda #$5e
+		cmp #$20
		ora #$40
		bcs +
inv		ora #$80
+		rts
		.pend
;---------------------------
;Convert from screencode to ascii
;
                #assert_8000romonly
fromsccode	.proc
		sta $d7
                cmp #$5e
                beq pi
		and #$3f
		asl $d7
		bit $d7
		bpl +
		ora #$80
+		bcs e
		bvs e
		ora #$40
e		rts

pi		lda #$ff
		rts
		.pend
;---------------------------
;Get a hex number from screen
;
                #assert_8000romonly
gethex		.proc
		sec
		lda (m_p1),y
		beq en
                .enc screen
		cmp #"9"+1
		bcs en
		cmp #"0"
		bcs ss
		cmp #"g"
		bcs en
		adc #9+"0"+1
ss		sbc #"0"
		.enc none
		asl m_p3
		rol m_p3+1
                .if SUPERCPU=1
                rol m_p3+2
                .fi
		asl m_p3
		rol m_p3+1
                .if SUPERCPU=1
                rol m_p3+2
                .fi
		asl m_p3
		rol m_p3+1
                .if SUPERCPU=1
                rol m_p3+2
                .fi
		asl m_p3
		rol m_p3+1
                .if SUPERCPU=1
                rol m_p3+2
                .fi
		iny
		cpy #41
                #orasta_a m_p3
en		rts
		.pend
;---------------------------
;Store common
;
                #assert_8000romonly
get_addr2store	.proc
		sta m_tmp2
		jsr get_addr2
		bne brt
		lda m_p3
		sta m_p4
		sta m_p2
		lda m_p3+1
		sta m_p4+1
		sta m_p2+1
		rts

brt2            pla
                pla
brt		pla
		pla
		rts
		.pend
;---------------------------
;Get a hex address from screen
;
                #assert_8000romonly
get_addr2xx	.proc
		jsr get_addr2
		bne get_addr2store.brt2
		rts

                #assert_8000romonly
get_addr2x	.proc
		jsr get_addr2
		bne get_addr2store.brt
                rts

                #assert_8000romonly
get_addr2	.proc
		lda #32
nx		iny
		cmp (m_p1),y
		bne getaddr
		cpy #39
		bcc nx
		iny
		rts

                #assert_8000romonly
getaddr		.proc
		#stz_a m_p3
		#stz_aa m_p3+1
                .if SUPERCPU=1
		#stz_aa m_p3+2
                .fi

                #assert_8000romonly
getaddrnc       .proc
		jsr gethex
		bcc aai
		cmp #30			;"^"?
		bne aa
		#phx_a
		jsr poptsp
		sta m_p3
		lda m_tstack+8,x
		sta m_p3+1
		#plx_a
		iny
		gcs aa
aai		jsr gethex
		bcs aa
		jsr gethex
		bcs aa
		jsr gethex
                .if SUPERCPU=1
		bcs aa
		jsr gethex
		bcs aa
		jsr gethex
                .fi
		cpy #40
		bne aa
		lda #32
		.byte SKIP2
aa		lda (m_p1),y
		cmp #32
		rts
		.pend
;---------------------------
;Get a filename from screen
;
                #assert_8000romonly
mon_getfname_ls .proc
		iny
                lda (m_p1),y
                .enc screen
                eor #"b"
                .enc none
                beq iko
                dey
                lda #$80
iko		eor #$80
		sta m_tmp
;---------------------------
;Get a filename from screen
;
                #assert_8000romonly
mon_getfname	.proc
xfn		cpy #39
		bcs mon_getfnameset.err
		lda (m_p1),y
		iny
		eor #'"'
		bne xfn
		tax
dtn		cpy #39
		beq mon_getfnameset.err
		lda (m_p1),y
		cmp #'"'
		beq mon_getfnameset
		iny
		jsr fromsccode
		sta monitor_buffer,x
		inx
		bne dtn

                #assert_8000romonly
mon_getfnameset	.proc
		stx ActualFNLength
		txa
		beq err
                .if <monitor_buffer
		lda #<monitor_buffer
		sta ActualFilename
                .else
                #stz_a ActualFilename
                .fi
		lda #>monitor_buffer
		sta ActualFilename+1
		clc
err		rts
		.pend
;---------------------------
;Store a byte and increment pointer
;
		#assert_cfg_16
mon_storebyte_inc .proc
		jsr mon_storebyte

                #assert_8000romonly
inc_m_p4	.proc
		inc m_p4
		bne +
		inc m_p4+1
+		rts
		.pend
;---------------------------
;Get a byte and increment pointer
;
		#assert_cfg_16
mon_getbyte_inc .proc
		jsr mon_getbyte

                #assert_8000romonly
inc_m_p2	.proc
		inc m_p2
		bne +
		inc m_p2+1
+               rts
                .pend
;---------------------------
;copy m_p3+1 to m_p2+1
;
                #assert_8000romonly
mon_copy_p3p2sbc .proc
		sta m_p2
		lda m_p3+1
		sbc #0
		sta m_p2+1
		rts
		.pend
;---------------------------
;Wait retrace
;
                #assert_8000romioonly
mon_waitretrace	.proc
-		bit $d011
		bpl -
-		bit $d011
		bmi -
		rts
		.pend
;---------------------------
;Calc prevline
;
                #assert_8000romonly
mon_cprevline	.proc
		lda m_p1
		sec
		sbc #40
		sta m_p1
		bcs +
		dec m_p1+1
+		rts
		.pend
;---------------------------
;Freeze common
;
		.assert CFG_OPEN_RAM | CFG_8000_ROM, CFG_8000_ROM,CFG_8000_RAM
mon_frzcomm	.proc
		lda #<(monitor_freeze_exp+512)
		sta m_p2
		lda #>(monitor_freeze_exp+512)
		sta m_p2+1

		ldy #<(monitor_freeze_end_exp-monitor_freeze_exp)
		ldx #3
		rts
		.pend
;---------------------------
;get binary data from screen (for ec and es)
;
                #assert_cfg_16
get_binary2	.proc
		ldx #8
igu		iny
		cpy #40
		beq ba
		lda (m_p1),y
		cmp #"."
		beq e
		cmp #32
		beq ba
		lda #1
e		lsr
		rol m_tmp
		dex
		bne igu
		lda m_tmp
		jsr mon_storebyte_inc
		clc
ba		rts
		.pend

;---------------------------
;Copy m_p3 to pc
;
                #assert_8000romioonly
OP_m_p3_to_pc	.proc
		#sta_cfg_open
		lda m_p3
		sta monitor_pc_exp+1
		lda m_p3+1
		sta monitor_pc_exp
                rts
                .pend
;---------------------------
;Addressing modes
;
		#assert_8000romonly

		.if SUPERCPU=1
a_mod		.byte a_imp+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;0

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpa+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_acc+a_imp*16
		.byte a_abs+a_abx*16
		.byte a_abx+a_lnx*16	;1

		.byte a_abs+a_zxi*16
		.byte a_lng+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;2

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_acc+a_imp*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_lnx*16	;3

		.byte a_imp+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_xya+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;4

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_xya+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_lng+a_abx*16
		.byte a_abx+a_lnx*16	;5

		.byte a_imp+a_zxi*16
		.byte a_rll+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imp*16
		.byte a_ind+a_abs*16
		.byte a_abs+a_lng*16	;6

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_axi+a_abx*16
		.byte a_abx+a_lnx*16	;7

		.byte a_rel+a_zxi*16
		.byte a_rll+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;8

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpy+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abx*16
		.byte a_abx+a_lnx*16	;9

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;a

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpy+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_abx+a_abx*16
		.byte a_aby+a_lnx*16	;b

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;c

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_zpi+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_ind+a_abx*16
		.byte a_abx+a_lnx*16	;d

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zps*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpI*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imp*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_lng*16	;e

		.byte a_rel+a_ziy*16
		.byte a_zpi+a_zsy*16
		.byte a_abs+a_zpx*16
		.byte a_zpx+a_zIy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_imp*16
		.byte a_axi+a_abx*16
		.byte a_abx+a_lnx*16	;f
		.else
a_mod		.byte a_imp+a_zxi*16
		.byte a_imp+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;0

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;1

		.byte a_abs+a_zxi*16
		.byte a_imp+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;2

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;3

		.byte a_imp+a_zxi*16
		.byte a_imp+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;4

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;5

		.byte a_imp+a_zxi*16
		.byte a_imp+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_acc+a_imm*16
		.byte a_ind+a_abs*16
		.byte a_abs+a_abs*16	;6

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;7

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;8

		.byte a_rel+a_ziy*16
		.byte a_imp+a_abx*16
		.byte a_zpx+a_zpx*16
		.byte a_zpy+a_zpy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_aby+a_aby*16	;9

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;a

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpy+a_zpy*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_aby+a_aby*16	;b

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;c

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;d

		.byte a_imm+a_zxi*16
		.byte a_imm+a_zxi*16
		.byte a_zpa+a_zpa*16
		.byte a_zpa+a_zpa*16
		.byte a_imp+a_imm*16
		.byte a_imp+a_imm*16
		.byte a_abs+a_abs*16
		.byte a_abs+a_abs*16	;e

		.byte a_rel+a_ziy*16
		.byte a_imp+a_ziy*16
		.byte a_zpx+a_zpx*16
		.byte a_zpx+a_zpx*16
		.byte a_imp+a_aby*16
		.byte a_imp+a_aby*16
		.byte a_abx+a_abx*16
		.byte a_abx+a_abx*16	;f
		.fi
;---------------------------
;Opcode index
;
		.if SUPERCPU=1
opc		.byte o_brk-o_table
		.byte o_ora-o_table
		.byte o_cop-o_table
		.byte o_ora-o_table
		.byte o_tsb-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_ora-o_table
		.byte o_php-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_phd-o_table
		.byte o_tsb-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_ora-o_table	;0

		.byte o_bpl-o_table
		.byte o_ora-o_table
		.byte o_ora-o_table
		.byte o_ora-o_table
		.byte o_trb-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_ora-o_table
		.byte o_clc-o_table
		.byte o_ora-o_table
		.byte o_inc-o_table
		.byte o_tcs-o_table
		.byte o_trb-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_ora-o_table	;1

		.byte o_jsr-o_table
		.byte o_and-o_table
		.byte o_jsr-o_table
		.byte o_and-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_and-o_table
		.byte o_plp-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_pld-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_and-o_table	;2

		.byte o_bmi-o_table
		.byte o_and-o_table
		.byte o_and-o_table
		.byte o_and-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_and-o_table
		.byte o_sec-o_table
		.byte o_and-o_table
		.byte o_dec-o_table
		.byte o_tsc-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_and-o_table	;3

		.byte o_rti-o_table
		.byte o_eor-o_table
		.byte o_wdm-o_table
		.byte o_eor-o_table
		.byte o_mvp-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_eor-o_table
		.byte o_pha-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_phk-o_table
		.byte o_jmp-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_eor-o_table	;4

		.byte o_bvc-o_table
		.byte o_eor-o_table
		.byte o_eor-o_table
		.byte o_eor-o_table
		.byte o_mvn-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_eor-o_table
		.byte o_cli-o_table
		.byte o_eor-o_table
		.byte o_phy-o_table
		.byte o_tcd-o_table
		.byte o_jmp-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_eor-o_table	;5

		.byte o_rts-o_table
		.byte o_adc-o_table
		.byte o_per-o_table
		.byte o_adc-o_table
		.byte o_stz-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_adc-o_table
		.byte o_pla-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rtl-o_table
		.byte o_jmp-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_adc-o_table	;6

		.byte o_bvs-o_table
		.byte o_adc-o_table
		.byte o_adc-o_table
		.byte o_adc-o_table
		.byte o_stz-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_adc-o_table
		.byte o_sei-o_table
		.byte o_adc-o_table
		.byte o_ply-o_table
		.byte o_tdc-o_table
		.byte o_jmp-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_adc-o_table	;7

		.byte o_bra-o_table
		.byte o_sta-o_table
		.byte o_brl-o_table
		.byte o_sta-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sta-o_table
		.byte o_dey-o_table
		.byte o_bit-o_table
		.byte o_txa-o_table
		.byte o_phb-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sta-o_table	;8

		.byte o_bcc-o_table
		.byte o_sta-o_table
		.byte o_sta-o_table
		.byte o_sta-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sta-o_table
		.byte o_tya-o_table
		.byte o_sta-o_table
		.byte o_txs-o_table
		.byte o_txy-o_table
		.byte o_stz-o_table
		.byte o_sta-o_table
		.byte o_stz-o_table
		.byte o_sta-o_table	;9

		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table
		.byte o_tay-o_table
		.byte o_lda-o_table
		.byte o_tax-o_table
		.byte o_plb-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table	;a

		.byte o_bcs-o_table
		.byte o_lda-o_table
		.byte o_lda-o_table
		.byte o_lda-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table
		.byte o_clv-o_table
		.byte o_lda-o_table
		.byte o_tsx-o_table
		.byte o_tyx-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lda-o_table	;b

		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_rep-o_table
		.byte o_cmp-o_table
		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_cmp-o_table
		.byte o_iny-o_table
		.byte o_cmp-o_table
		.byte o_dex-o_table
		.byte o_wai-o_table
		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_cmp-o_table	;c

		.byte o_bne-o_table
		.byte o_cmp-o_table
		.byte o_cmp-o_table
		.byte o_cmp-o_table
		.byte o_pei-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_cmp-o_table
		.byte o_cld-o_table
		.byte o_cmp-o_table
		.byte o_phx-o_table
		.byte o_stp-o_table
		.byte o_jmp-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_cmp-o_table	;d

		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_sep-o_table
		.byte o_sbc-o_table
		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_sbc-o_table
		.byte o_inx-o_table
		.byte o_sbc-o_table
		.byte o_nop-o_table
		.byte o_xba-o_table
		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_sbc-o_table	;e

		.byte o_beq-o_table
		.byte o_sbc-o_table
		.byte o_sbc-o_table
		.byte o_sbc-o_table
		.byte o_pea-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_sbc-o_table
		.byte o_sed-o_table
		.byte o_sbc-o_table
		.byte o_plx-o_table
		.byte o_xce-o_table
		.byte o_jsr-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_sbc-o_table	;f
		.else
opc		.byte o_brk-o_table
		.byte o_ora-o_table
		.byte o_jam-o_table
		.byte o_slo-o_table
		.byte o_no2-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_slo-o_table
		.byte o_php-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_anc-o_table
		.byte o_no2-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_slo-o_table	;0

		.byte o_bpl-o_table
		.byte o_ora-o_table
		.byte o_jam-o_table
		.byte o_slo-o_table
		.byte o_no2-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_slo-o_table
		.byte o_clc-o_table
		.byte o_ora-o_table
		.byte o_no2-o_table
		.byte o_slo-o_table
		.byte o_no2-o_table
		.byte o_ora-o_table
		.byte o_asl-o_table
		.byte o_slo-o_table	;1

		.byte o_jsr-o_table
		.byte o_and-o_table
		.byte o_jam-o_table
		.byte o_rla-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_rla-o_table
		.byte o_plp-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_anc-o_table
		.byte o_bit-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_rla-o_table	;2

		.byte o_bmi-o_table
		.byte o_and-o_table
		.byte o_jam-o_table
		.byte o_rla-o_table
		.byte o_no2-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_rla-o_table
		.byte o_sec-o_table
		.byte o_and-o_table
		.byte o_no2-o_table
		.byte o_rla-o_table
		.byte o_no2-o_table
		.byte o_and-o_table
		.byte o_rol-o_table
		.byte o_rla-o_table	;3

		.byte o_rti-o_table
		.byte o_eor-o_table
		.byte o_jam-o_table
		.byte o_sre-o_table
		.byte o_no2-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_sre-o_table
		.byte o_pha-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_asr-o_table
		.byte o_jmp-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_sre-o_table	;4

		.byte o_bvc-o_table
		.byte o_eor-o_table
		.byte o_jam-o_table
		.byte o_sre-o_table
		.byte o_no2-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_sre-o_table
		.byte o_cli-o_table
		.byte o_eor-o_table
		.byte o_no2-o_table
		.byte o_sre-o_table
		.byte o_no2-o_table
		.byte o_eor-o_table
		.byte o_lsr-o_table
		.byte o_sre-o_table	;5

		.byte o_rts-o_table
		.byte o_adc-o_table
		.byte o_jam-o_table
		.byte o_rra-o_table
		.byte o_no2-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rra-o_table
		.byte o_pla-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_arr-o_table
		.byte o_jmp-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rra-o_table	;6

		.byte o_bvs-o_table
		.byte o_adc-o_table
		.byte o_jam-o_table
		.byte o_rra-o_table
		.byte o_no2-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rra-o_table
		.byte o_sei-o_table
		.byte o_adc-o_table
		.byte o_no2-o_table
		.byte o_rra-o_table
		.byte o_no2-o_table
		.byte o_adc-o_table
		.byte o_ror-o_table
		.byte o_rra-o_table	;7

		.byte o_no2-o_table
		.byte o_sta-o_table
		.byte o_no2-o_table
		.byte o_sax-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sax-o_table
		.byte o_dey-o_table
		.byte o_no2-o_table
		.byte o_txa-o_table
		.byte o_ane-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_stx-o_table
		.byte o_sax-o_table	;8

		.byte o_bcc-o_table
		.byte o_sta-o_table
		.byte o_jam-o_table
		.byte o_sha-o_table
		.byte o_sty-o_table
		.byte o_sta-o_table
		.byte o_sax-o_table
		.byte o_stx-o_table
		.byte o_tya-o_table
		.byte o_sta-o_table
		.byte o_txs-o_table
		.byte o_shs-o_table
		.byte o_shy-o_table
		.byte o_sta-o_table
		.byte o_shx-o_table
		.byte o_sha-o_table	;9

		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table
		.byte o_tay-o_table
		.byte o_lda-o_table
		.byte o_tax-o_table
		.byte o_lxa-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table	;a

		.byte o_bcs-o_table
		.byte o_lda-o_table
		.byte o_jam-o_table
		.byte o_lax-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table
		.byte o_clv-o_table
		.byte o_lda-o_table
		.byte o_tsx-o_table
		.byte o_lae-o_table
		.byte o_ldy-o_table
		.byte o_lda-o_table
		.byte o_ldx-o_table
		.byte o_lax-o_table	;b

		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_no2-o_table
		.byte o_dcp-o_table
		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_dcp-o_table
		.byte o_iny-o_table
		.byte o_cmp-o_table
		.byte o_dex-o_table
		.byte o_sbx-o_table
		.byte o_cpy-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_dcp-o_table	;c

		.byte o_bne-o_table
		.byte o_cmp-o_table
		.byte o_jam-o_table
		.byte o_dcp-o_table
		.byte o_no2-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_dcp-o_table
		.byte o_cld-o_table
		.byte o_cmp-o_table
		.byte o_no2-o_table
		.byte o_dcp-o_table
		.byte o_no2-o_table
		.byte o_cmp-o_table
		.byte o_dec-o_table
		.byte o_dcp-o_table	;d

		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_no2-o_table
		.byte o_isb-o_table
		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_isb-o_table
		.byte o_inx-o_table
		.byte o_sbc-o_table
		.byte o_nop-o_table
		.byte o_sb2-o_table
		.byte o_cpx-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_isb-o_table	;e

		.byte o_beq-o_table
		.byte o_sbc-o_table
		.byte o_jam-o_table
		.byte o_isb-o_table
		.byte o_no2-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_isb-o_table
		.byte o_sed-o_table
		.byte o_sbc-o_table
		.byte o_no2-o_table
		.byte o_isb-o_table
		.byte o_no2-o_table
		.byte o_sbc-o_table
		.byte o_inc-o_table
		.byte o_isb-o_table	;f
                .fi
;---------------------------
;Opcodes
;
		.enc screen
		.if SUPERCPU=1
o_table
o_adc		.word "a"+"d"*32+"c"*1024
o_and		.word "a"+"n"*32+"d"*1024
o_asl		.word "a"+"s"*32+"l"*1024
o_bcc		.word "b"+"c"*32+"c"*1024
o_bcs		.word "b"+"c"*32+"s"*1024
o_beq		.word "b"+"e"*32+"q"*1024
o_bit		.word "b"+"i"*32+"t"*1024
o_bmi		.word "b"+"m"*32+"i"*1024
o_bne		.word "b"+"n"*32+"e"*1024
o_bpl		.word "b"+"p"*32+"l"*1024

o_bra		.word "b"+"r"*32+"a"*1024
o_brk		.word "b"+"r"*32+"k"*1024
o_brl		.word "b"+"r"*32+"l"*1024
o_bvc		.word "b"+"v"*32+"c"*1024
o_bvs		.word "b"+"v"*32+"s"*1024
o_clc		.word "c"+"l"*32+"c"*1024
o_cld		.word "c"+"l"*32+"d"*1024
o_cli		.word "c"+"l"*32+"i"*1024
o_clv		.word "c"+"l"*32+"v"*1024
o_cmp		.word "c"+"m"*32+"p"*1024

o_cop		.word "c"+"o"*32+"p"*1024
o_cpx		.word "c"+"p"*32+"x"*1024
o_cpy		.word "c"+"p"*32+"y"*1024
o_dec		.word "d"+"e"*32+"c"*1024
o_dex		.word "d"+"e"*32+"x"*1024
o_dey		.word "d"+"e"*32+"y"*1024
o_eor		.word "e"+"o"*32+"r"*1024
o_inc		.word "i"+"n"*32+"c"*1024
o_inx		.word "i"+"n"*32+"x"*1024
o_iny		.word "i"+"n"*32+"y"*1024

o_jmp		.word "j"+"m"*32+"p"*1024
o_jsr		.word "j"+"s"*32+"r"*1024
o_lda		.word "l"+"d"*32+"a"*1024
o_ldx		.word "l"+"d"*32+"x"*1024
o_ldy		.word "l"+"d"*32+"y"*1024
o_lsr		.word "l"+"s"*32+"r"*1024
o_mvn		.word "m"+"v"*32+"n"*1024
o_mvp		.word "m"+"v"*32+"p"*1024
o_nop		.word "n"+"o"*32+"p"*1024
o_ora		.word "o"+"r"*32+"a"*1024

o_pea		.word "p"+"e"*32+"a"*1024
o_pei		.word "p"+"e"*32+"i"*1024
o_per		.word "p"+"e"*32+"r"*1024
o_pha		.word "p"+"h"*32+"a"*1024
o_phb		.word "p"+"h"*32+"b"*1024
o_phd		.word "p"+"h"*32+"d"*1024
o_phk		.word "p"+"h"*32+"k"*1024
o_php		.word "p"+"h"*32+"p"*1024
o_phx		.word "p"+"h"*32+"x"*1024
o_phy		.word "p"+"h"*32+"y"*1024

o_pla		.word "p"+"l"*32+"a"*1024
o_plb		.word "p"+"l"*32+"b"*1024
o_pld		.word "p"+"l"*32+"d"*1024
o_plp		.word "p"+"l"*32+"p"*1024
o_plx   	.word "p"+"l"*32+"x"*1024
o_ply   	.word "p"+"l"*32+"y"*1024
o_rep   	.word "r"+"e"*32+"p"*1024
o_rol		.word "r"+"o"*32+"l"*1024
o_ror		.word "r"+"o"*32+"r"*1024
o_rti		.word "r"+"t"*32+"i"*1024

o_rtl		.word "r"+"t"*32+"l"*1024
o_rts		.word "r"+"t"*32+"s"*1024
o_sbc		.word "s"+"b"*32+"c"*1024
o_sec		.word "s"+"e"*32+"c"*1024
o_sed		.word "s"+"e"*32+"d"*1024
o_sei		.word "s"+"e"*32+"i"*1024
o_sep		.word "s"+"e"*32+"p"*1024
o_sta		.word "s"+"t"*32+"a"*1024
o_stp		.word "s"+"t"*32+"p"*1024
o_stx		.word "s"+"t"*32+"x"*1024

o_sty		.word "s"+"t"*32+"y"*1024
o_stz		.word "s"+"t"*32+"z"*1024
o_tax		.word "t"+"a"*32+"x"*1024
o_tay		.word "t"+"a"*32+"y"*1024
o_tcd		.word "t"+"c"*32+"d"*1024
o_tcs		.word "t"+"c"*32+"s"*1024
o_tdc		.word "t"+"d"*32+"c"*1024
o_trb		.word "t"+"r"*32+"b"*1024
o_tsb		.word "t"+"s"*32+"b"*1024
o_tsc		.word "t"+"s"*32+"c"*1024

o_tsx		.word "t"+"s"*32+"x"*1024
o_txa		.word "t"+"x"*32+"a"*1024
o_txs		.word "t"+"x"*32+"s"*1024
o_txy		.word "t"+"x"*32+"y"*1024
o_tya		.word "t"+"y"*32+"a"*1024
o_tyx		.word "t"+"y"*32+"x"*1024
o_wai		.word "w"+"a"*32+"i"*1024
o_wdm		.word "w"+"d"*32+"m"*1024+32768
o_xba		.word "x"+"b"*32+"a"*1024
o_xce		.word "x"+"c"*32+"e"*1024
                .else
o_table
o_adc		.word "a"+"d"*32+"c"*1024
o_anc		.word "a"+"n"*32+"c"*1024+32768
o_and		.word "a"+"n"*32+"d"*1024
o_ane		.word "a"+"n"*32+"e"*1024+32768
o_arr		.word "a"+"r"*32+"r"*1024+32768
o_asl		.word "a"+"s"*32+"l"*1024
o_asr		.word "a"+"s"*32+"r"*1024+32768
o_bcc		.word "b"+"c"*32+"c"*1024
o_bcs		.word "b"+"c"*32+"s"*1024
o_beq		.word "b"+"e"*32+"q"*1024

o_bit		.word "b"+"i"*32+"t"*1024
o_bmi		.word "b"+"m"*32+"i"*1024
o_bne		.word "b"+"n"*32+"e"*1024
o_bpl		.word "b"+"p"*32+"l"*1024
o_brk		.word "b"+"r"*32+"k"*1024
o_bvc		.word "b"+"v"*32+"c"*1024
o_bvs		.word "b"+"v"*32+"s"*1024
o_clc		.word "c"+"l"*32+"c"*1024
o_cld		.word "c"+"l"*32+"d"*1024
o_cli		.word "c"+"l"*32+"i"*1024

o_clv		.word "c"+"l"*32+"v"*1024
o_cmp		.word "c"+"m"*32+"p"*1024
o_cpx		.word "c"+"p"*32+"x"*1024
o_cpy		.word "c"+"p"*32+"y"*1024
o_dcp		.word "d"+"c"*32+"p"*1024+32768
o_dec		.word "d"+"e"*32+"c"*1024
o_dex		.word "d"+"e"*32+"x"*1024
o_dey		.word "d"+"e"*32+"y"*1024
o_eor		.word "e"+"o"*32+"r"*1024
o_inc		.word "i"+"n"*32+"c"*1024

o_inx		.word "i"+"n"*32+"x"*1024
o_iny		.word "i"+"n"*32+"y"*1024
o_isb		.word "i"+"s"*32+"b"*1024+32768
o_jam		.word "j"+"a"*32+"m"*1024+32768
o_jmp		.word "j"+"m"*32+"p"*1024
o_jsr		.word "j"+"s"*32+"r"*1024
o_lae		.word "l"+"a"*32+"e"*1024+32768
o_lax		.word "l"+"a"*32+"x"*1024+32768
o_lda		.word "l"+"d"*32+"a"*1024
o_ldx		.word "l"+"d"*32+"x"*1024

o_ldy		.word "l"+"d"*32+"y"*1024
o_lsr		.word "l"+"s"*32+"r"*1024
o_lxa		.word "l"+"x"*32+"a"*1024+32768
o_nop		.word "n"+"o"*32+"p"*1024
o_no2		.word "n"+"o"*32+"p"*1024+32768
o_ora		.word "o"+"r"*32+"a"*1024
o_pha		.word "p"+"h"*32+"a"*1024
o_php		.word "p"+"h"*32+"p"*1024
o_pla		.word "p"+"l"*32+"a"*1024
o_plp		.word "p"+"l"*32+"p"*1024

o_rla		.word "r"+"l"*32+"a"*1024+32768
o_rol		.word "r"+"o"*32+"l"*1024
o_ror		.word "r"+"o"*32+"r"*1024
o_rra		.word "r"+"r"*32+"a"*1024+32768
o_rti		.word "r"+"t"*32+"i"*1024
o_rts		.word "r"+"t"*32+"s"*1024
o_sax		.word "s"+"a"*32+"x"*1024+32768
o_sbc		.word "s"+"b"*32+"c"*1024
o_sb2		.word "s"+"b"*32+"c"*1024+32768
o_sbx		.word "s"+"b"*32+"x"*1024+32768

o_sec		.word "s"+"e"*32+"c"*1024
o_sed		.word "s"+"e"*32+"d"*1024
o_sei		.word "s"+"e"*32+"i"*1024
o_sha		.word "s"+"h"*32+"a"*1024+32768
o_shs		.word "s"+"h"*32+"s"*1024+32768
o_shx		.word "s"+"h"*32+"x"*1024+32768
o_shy		.word "s"+"h"*32+"y"*1024+32768
o_slo		.word "s"+"l"*32+"o"*1024+32768
o_sre		.word "s"+"r"*32+"e"*1024+32768
o_sta		.word "s"+"t"*32+"a"*1024

o_stx		.word "s"+"t"*32+"x"*1024
o_sty		.word "s"+"t"*32+"y"*1024
o_tax		.word "t"+"a"*32+"x"*1024
o_tay		.word "t"+"a"*32+"y"*1024
o_tsx		.word "t"+"s"*32+"x"*1024
o_txa		.word "t"+"x"*32+"a"*1024
o_txs		.word "t"+"x"*32+"s"*1024
o_tya		.word "t"+"y"*32+"a"*1024
		.fi
;---------------------------
;Normal and BRK start text
;
txtmon		.text "rotinom 46edi"
txtbrk		.text "noitpecxe krb"
		.enc none
