;
; path.asm - path resolver
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;----------------------------------
;Get filename to path_exp
;
		#assert_cfg_open

OP_save_zeropage_copy_filename_to_path .proc
		jsr OP_save_zeropage
		ldx #9
lxlx            lda pu-1,x
                sta start_zeropage-1,x
                dex
                bne lxlx
		txa
		ldy ActualFNLength
                .if SUPERCPU=1
                tyx
                .else
		ldx ActualFNLength
                .fi
		#jmps twb

xip		dey
		jsr start_zeropage
		dex
twb		sta path_exp,x
		bne xip
                rts

pu              sta cfg_stnd
                lda (ActualFilename),y
                sta cfg_open
                rts
		.pend
;----------------------------------
; cut off the beginning of path C not touched
;
OP_short_path	.proc
		ldx #0
be		lda path_exp,x
		beq ueji
uejilp		lda path_exp+1,x
		sta path_exp,x
		beq ueji
		inx
		bne uejilp
ueji		lda path_exp
		rts
		.pend
;----------------------------------
; set Z=1 if special character found
;
is_special_s	.proc
		cmp #"?"
		beq is_special.x
		cmp #"*"
		beq is_special.x
is_special	.proc
		cmp #0
		beq x
		cmp #","
		beq x
		cmp #"="
		beq x
		cmp #":"
x		rts
		.pend
;-----------------------------------
;This small routine takes care about that complex wildcard matching...
;
check_wildtype	.proc
		#lda0
		sta wildcard_csy_exp
		lda #direntry_filetype_exp-direntry_name_exp
		gne check_wildcards.wloop2

check_wildcards	.proc
		lda direntry_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
		cmp #FTYPE_DIR		;dirlabel as ".." and "."
		bne wilin
                lda direntry_mode_exp
                .if DIRMODE_CFS
                eor #DIRMODE_CFS
                .fi
		bne wilintt             ;Virtual "." and ".." only on CFS fs
		sta direntry_name_exp+2
		lda #"."
		sta direntry_name_exp
		jsr wilint		;check ".."
		bcc foun		;found
		ldx #3			;Current dir in filesize field
svx		lda direntry_size_exp,x
		sta direntry_pointer_exp,x
		dex
		bpl svx
wilintt		#lda0
wilint		sta direntry_name_exp+1	;check "."

wilin		#lda0
		sta wildcard_csy_exp
wloop2		sta wildcard_csx_exp
		tax
		#ldy_l_a wildcard_csy_exp

wloop1		#lda_zp_y current_path
		cmp #"?"
		bne wilany		;C=1
		lda direntry_name_exp,x
		bne wok
foun		rts			;-=> not found (c=1)

wilany		bcs wnspec		;mostly we compare only real text filenames...
		cmp #"*"
		bne wnstar		;C=1
		iny
		#sty_l_a wildcard_csy_exp
		#stx_l_a wildcard_csx_exp
		gcs wloop1

wnstar		jsr is_special		;_p
		bne wnspecx
tkkq		lda direntry_name_exp,x
		beq wiqq
		gne wki2

wnspecx         cmp wildcard_end_exp
		beq tkkq
wnspec		cmp direntry_name_exp,x
		bne wki

wok		inx
		iny
		cpx #direntry_filetype_exp-direntry_name_exp+3;exit on end of filetype
		beq gg
		cpx #16			;exit on end of filename
		bne wloop1
gg		#lda_zp_y current_path
		jsr is_special		;_p
		beq wiqq
                cmp wildcard_end_exp
                beq wiqq
		cmp #"*"
		bne wsad
		iny
		gcs gg
wiqq		clc
		rts			;-=> found (c=0)

wki		lda direntry_name_exp,x
		beq wsad
wki2		lda wildcard_csy_exp
		beq wsad
                sec
                #lda0
		adc wildcard_csx_exp
		cmp #direntry_filetype_exp-direntry_name_exp+3;exit on end of filetype
		beq gg
		cmp #16
		bne wloop2		;exit with err if end of name
		geq gg
wsad		sec
		rts			;-=> not found (c=1)
		.pend
;----------------------------------
; Search for special characters 0 , = :
;
searchspecial	.proc
		ldx #255
in		inx
		lda path_exp,x
		jsr is_special
		bne in
                tya
                eor path_exp,x
                rts
		.pend
;----------------------------------
OP_XIT_resolv_path_part_init .proc
		pha
                jsr XIT_init_drive
                jsr OP_get_partnum
                bcc colonfound
                pla
                pha
                ldx #0
                eor #RESOLV_PARTITION_MODE
                beq ee2
                eor #RESOLV_DIR_MODE^RESOLV_PARTITION_MODE
                bne curr
                dex		;"$<partition>"
ee              inx
                lda path_exp,x
                cmp #$30
                bcc ee3
                cmp #$3a
                bcc ee
ee3             sec
ee2             lda path_exp,x
                bne curr
                sta path_exp+1
                lda #"*"
                sta path_exp
                lda current_path	;specified
                .byte SKIPLDA0
curr            #lda0
colonfound      sta path_partition_exp

                pla
		tax
                and #RESOLV_L_DELIMITER_MASK
                beq jo
		jsr LWOP_reporterror_30_xit
jo		txa
		clv			;resolv from working dir
;----------------------------------
; Resolv path including links
;
; Valid path_partition_exp needed!
; clear V before call!
;
OP_XIT_resolv_path .proc
		sta resolv_mode_exp
                eor #RESOLV_PARTITION_MODE
                bne jrj
                sta path_partition_exp
jrj             ror
                sta path_resolv_allname_exp

		jsr OP_get_current_drive_type
                #andlsr FS_MASK
                sec
                beq LWOP_reporterror_75_xit
                tax
                lda dirmodeconst-1,x
		sta direntry_mode_exp

		lda #LINK_DEPTH
		sta resolv_loop_exp

resolv_pathloop	sec			;symlink resolv loop
		#dec_l_a resolv_loop_exp
		beq LWOP_reporterror_39_xit;symlink loop...

		php
		lda #<path_exp
		sta current_path	;for check_wildcard
		lda #>path_exp
		sta current_path+1
		.if SUPERCPU=1
		lda #1
		sta current_path+2
		.fi
		lda path_partition_exp
		jsr OP_check_partition_selection;-=> error
                jsr LWOP_reporterror_74_xit
                plp
		inx
		inx
		inx

		lda setup_atribut2_exp
                asl

		lda path_exp		;"/
                bcc ncmd
                .if SUPERCPU=1
                xba
                lda path_resolv_allname_exp
                bmi forcerel
                xba
                bra kek
                .else
		ldy path_resolv_allname_exp
                bmi forcerel
                gpl kek
                .fi

ncmd		cmp #":"
                bne kek
                lda path_exp+1		;":/
kek		eor #"/"
         	cmp #1			;C=1 relative path
forcerel       	ldy #3
bnmf		lda partition_root_dir_exp,x
		bcc rootd		;C=1 relative path
                #lda_l_y_ resolv_this_dir_exp
		bvs lastd		;Link file containing relative path...
		lda partition_workdir_exp,x
rootd		#sta_l_y_ resolv_this_dir_exp
lastd		sta next_sector,y
		dex
		dey
		bpl bnmf
		inx
		#stx_l_a partition_currentp_exp

resolv_next_dir				;directory resolv loop
		#lda0
                sta path_dirsectors_exp
                sta direntry_fat_dirsectors_exp
                sta direntry_skip2_exp	;hack
                sta resolv_empty_entry_found_exp;no empty entry so far (the dirscan just begins now...)

                lda resolv_mode_exp
                cmp #RESOLV_PARTITION_MODE
                bne npa
                lda path_resolv_allname_exp
                bmi lastpart
                lda path_exp
                gcs gh
he              jsr OP_short_path
gh              cmp #":"
                bne he
                geq lastpart2

npa		lda setup_atribut2_exp
		and path_resolv_allname_exp
                bmi lastpart
                lda path_exp
                #jmps sks

skipper		jsr OP_short_path	;skip /, //, /// etc
sks		cmp #"/"
		beq skipper

		cmp #":"		;":"?
                bne kqk
                lda setup_atribut2_exp
                bpl skipper
		gmi lastpart2		;last part, except there's nothing more left
kqk
		ldy #0
splp		#lda_zp_y current_path
		jsr is_special
		beq lastpart   		;last part of filename not resolved
		iny
		cmp #"/"
		bne splp
                sta wildcard_end_exp	;it's not the last part
                ldy #0
                geq resolv_nxt_sctr	;found a "xxxxxx/"

lastpart2	jsr OP_short_path
lastpart    	#lda0
                sta wildcard_end_exp    ;this is the last part we resolv!
					;found a "xxxx,e" or "xxxx=e" or "xxxx:e" or "xxxx"

                lda #"r"
                sta resolv_open_mode_exp
		lda resolv_mode_exp
		and #RESOLV_L_FILETYPE2_MASK | RESOLV_L_FILETYPE_MASK
		beq beginscan
                ldy #","      	;filetype ",xxx"
                and #RESOLV_L_FILETYPE_MASK     ;RESOLV_L_FILETYPE_MASK must have priority over RESOLV_L_FILETYPE2_MASK!
		bne +	  	;filetype "=xxx"
		ldy #"="
+		jsr searchspecial
		bne nonefound
                tay

crt2		inx
		lda path_exp,x
                cmp #":"
                beq zd
		jsr is_special
		beq en2
zd		cpy #15
		beq crt2
		#sta_l_y_ path_searchtype_exp
		iny
		gne crt2

en2		lda resolv_mode_exp
		cmp #RESOLV_DIR_MODE
                bne is
                #ldatax path_searchtype_exp
                clc
                eor #">"
                beq sd
                eor #"<"^">"
                beq sd
                eor #"l"^"<"
                beq cx
                eor #"n"^"l"
                bne sd2
cx		sec
		lda direntry_datemode_exp
                beq is
                tya
		eor #1
                bne is
sd		sta path_searchtype_exp+1
		ldy #1
                lda #"*"
                sta path_searchtype_exp
                txa
                bcs fk
sd2             ldx #255
ws		inx
		lda path_exp,x
                beq is
                eor #","
                bne ws
kr              sta path_exp,x
                lda path_exp+1,x
fk              eor #"n"
                bne df
                sta direntry_datemode_exp
df		eor #"n"^"l"
                sta direntry_longmode_exp

is		cpy #1
		bne ks

		lda resolv_mode_exp
                eor #RESOLV_OPENFIRST_MODE
                cmp #1
		lda path_searchtype_exp
                bcs nopen
                jsr checkopenmodes
                beq nonefound2
                cmp #"l"
                bne +
                lda #"r"
+
nopen		ldy #8*4
fl		cmp ve1-4,y
		beq vn
		dey
		dey
		dey
		dey
		bne fl
		iny
		gne ks

vn		ldx #2
se		lda ve1-1,y
		sta path_searchtype_exp,x
		dey
		dex
		bpl se
		ldy #3

ks		#lda0
		#sta_l_y_x path_searchtype_exp
                cpy #3
                tay
                blt typefound  	 ;filetype found, unknown
                ldy #4*4
vn2             ldx #2
		clc
se2             dey
		lda ve1,y
                eor path_searchtype_exp,x
                beq +
                sec
+               dex
                bpl se2
                bcc typefound
                dey
                bne vn2
                geq typefound	;unknown

nonefound2      tya
                bne nonefound   ;no write
                #sev set_v
                .byte SKIP1BVC
nonefound
                clv
		lda resolv_mode_exp
		cmp #RESOLV_OPENFIRST_MODE
                beq opn
             	and #RESOLV_L_NO_TYPE_TO_ANY_MASK
                gpl kkk
                .cerror RESOLV_L_NO_TYPE_TO_ANY_MASK & $80

opn		lda ActualSecAddr
		and #$0f
		eor #1
kkk		cmp #1                  ;* for 0,2-14 prg for 1
		ldx #3
                #lda0
                tay
-		sta path_searchtype_exp,x
                lda cprg,x
		bcc +
                lda cseq,x
                bvs +
		lda cany,x
+               dex
		bpl -

typefound      	tya
		clc
                adc #3
                lsr
                lsr
		sta resolv_filetype_exp
		lda resolv_mode_exp
		eor #RESOLV_OPENFIRST_MODE
                bne beginscan
                sta resolv_recordlen_exp

		ldy #","
		jsr searchspecial
                bne forceread
                sta path_exp,x
                jsr searchspecial.in
		bne forceread
                lda path_exp+1,x
                sta resolv_recordlen_exp
                jsr checkopenmodes
                beq ca
forceread       lda resolv_open_mode_exp;default
ca		tax
		lda ActualSecAddr
		#andlsr $0f
		bne jo
                ldx #"r"                ;read mode
		bcc jo
		ldx #"w"		;write mode
jo		#stx_l_a resolv_open_mode_exp
		cpx #"w"
                lda resolv_filetype_exp
                eor #RESOLV_FILETYPE_LNK
                php
                eor #RESOLV_FILETYPE_REL ^ RESOLV_FILETYPE_LNK
                beq ssd			;force create
		lda #RESOLV_OPEN_MODE
                bcc +
ssd             lda #RESOLV_CREAT_MODE
+               plp
                bne +
                and #<~RESOLV_L_FOLLOW_LAST_LINK_MASK
+               sta resolv_mode_exp
                .cerror (RESOLV_CREAT_MODE-RESOLV_CREAT_LINK_MODE)!=RESOLV_L_FOLLOW_LAST_LINK_MASK
                .cerror (RESOLV_OPEN_MODE-RESOLV_LINK_MODE)!=RESOLV_L_FOLLOW_LAST_LINK_MASK

beginscan      	#ldatax resolv_mode_exp
		and #RESOLV_L_STOREDIRE_MASK | RESOLV_L_STOREDIRF_MASK
                cmp #RESOLV_L_NORESOLV
                beq cdu			;dir, partition dir, r-h
                tay
                lda path_exp
		jsr is_special
                bne resolv_nxt_sctr
                txa
                eor #RESOLV_CD_MODE
                bne LWOP_reporterror_34_xit;no filename!
cdu             clc
		rts			;-=> exit, OK

resolv_nxt_sctr #sty_l_a resolv_store_exp
		#cnext next_sector , resolv_this_dir_exp
;------------------------
resolv_nxt_sclp	#cnext next_sector , resolv_this_sector_exp;sector level loop

                .cerror (DIRMODE_FAT | 1)!=DIRMODE_FAT2
		lda direntry_mode_exp
                lsr                     ;mask DIRMODE_FAT2
                cmp #DIRMODE_FAT >> 1
                bne nonfat
		lda direntry_fat_dirsectors_exp
		jsr cluster2sector
nonfat

		jsr OP_read_cmd1
		jsr LWOP_reporterror_read_xit;-=> error

                #inc_l_a path_dirsectors_exp

                #ldatax resolv_mode_exp
                and #RESOLV_L_STOREDIRE_MASK | RESOLV_L_STOREDIRF_MASK
                beq nxn
                and #RESOLV_L_STOREDIRE_MASK
                beq kq
                lda resolv_empty_entry_found_exp
                beq kq
                #lda0
                sta resolv_store_exp
kq
		lda #ram_exp_dir
		cpx #RESOLV_R2_MODE
                bne +
                lda #ram_exp_tree
+               jsr init_pom1_y

nxn      	lda #16			;16 directory entrys
;		sta resolv_anything_exp ;at least there's a valid filename
		sta direntry_number_exp
resolv_nxt_ntry	#dec_l_a direntry_number_exp	;entry level loop
		bpl resnotlast

		jsr wait_busy;jsr XIT_nomoreread       ;-=> error
                jsr LWOP_reporterror_read_xit

		jsr next_sector0
		bne resolv_nxt_sclp
		sec			;path not found...
		lda wildcard_end_exp
		bne LWOP_reporterror_39_xit;-=> path not found
                lda resolv_mode_exp
		and #RESOLV_L_FILENF_MASK
		bne LWOP_reporterror_62_xit;-=> file not found
		rts			;-=> exit, file not found

resnotlast	jsr OP_read_to_direntry2

		#ldy_l_a direntry_attrib_exp
		lda wildcard_end_exp
		bne np
					;last part of filename
		tya
		and #FATTR_CLOSED | FTYPE_MASK
                beq empty
                cmp #FTYPE_DIR
		bne reso
                #sty_l_a resolv_dirattrib_exp;remember this dir's attrib
                gcs reso

empty           lda resolv_empty_entry_found_exp
                bne resolv_nxt_ntry

		lda direntry_number_exp
		eor #15
		sta resolv_entry_offs_exp;remember entry position

		ldx #3			;Remember this empty entry position...
xkd2		lda resolv_this_sector_exp,x
		sta resolv_empty_entry_exp,x
		dex
		bpl xkd2
                #stx_l_a resolv_empty_entry_found_exp
		gmi resolv_nxt_ntry

np		tya
		and #FATTR_EXEC | FTYPE_MASK
		cmp #FATTR_EXEC | FTYPE_DIR;Executeable directory?
		beq reso
                tya
		and #FATTR_CLOSED | FTYPE_MASK
		cmp #FATTR_CLOSED | FTYPE_LNK;Closed link?
		bne resolv_nxtntry_

reso		jsr check_wildcards
		bcs resolv_nxtntry_	;no match

		lda wildcard_end_exp
		bne npjx
					;last part of filename
                #ldy_l_a direntry_attrib_exp
                lda resolv_mode_exp
                .cerror RESOLV_L_FOLLOW_LAST_LINK_MASK!=$80
		bpl vmwqe     		;resolv link? no RESOLV_L_FOLLOW_LAST_LINK_MASK
                tax
		tya
		and #FATTR_CLOSED | FTYPE_MASK
		eor #FATTR_CLOSED | FTYPE_LNK
		beq mds			;link! go resolv it
		txa

vmwqe   	cmp #RESOLV_OPEN_MODE
		beq vt1                 ;and must be executeable file if load
                cmp #RESOLV_S_MODE
		beq vt2
                cmp #RESOLV_LOAD_MODE
		beq vt3                 ;and must be executeable file if load
                and #RESOLV_L_FILETYPE_MASK
		beq mds2                ;filetype must match!
		gne mds                 ;and must be executeable file if load

vt3     	tya
		and #FATTR_CLOSED | FATTR_EXEC | FTYPE_MASK
		eor #FATTR_CLOSED | FATTR_EXEC | FTYPE_NORMAL
                gcs dd                	;not a closed executeable normal file, cannot load

vt2		tya
		and #FATTR_DELETE | FTYPE_MASK
		eor #FATTR_DELETE | FTYPE_LNK
                beq mds
		eor #FTYPE_NORMAL ^ FTYPE_LNK
		gcs ddd			;only normal deletable files can be removed

vt1		tya
		and #FATTR_CLOSED | FATTR_READ | FTYPE_MASK
		eor #FATTR_CLOSED | FATTR_READ | FTYPE_NORMAL
ddd             beq mds
                eor #FTYPE_NORMAL ^ FTYPE_RELATIVE
dd		bne resolv_nxtntry_	;currently only regular files can be opened

mds             inc current_path+1
                jsr check_wildtype
                dec current_path+1
                bcc npjx
resolv_nxtntry_	#jmps resolv_nxt_ntry	;filetype does not match

mds2		lda resolv_mode_exp
		cmp #RESOLV_MD_MODE
                beq vt4
		cmp #RESOLV_RD_MODE
                beq vt4
                cmp #RESOLV_CD_MODE
		bne npjx
                tya
		and #FATTR_EXEC | FTYPE_MASK
		eor #FATTR_EXEC | FTYPE_DIR
		gcs dd2			;not executeable dir, cannot CD

vt4		tya
		and #FTYPE_MASK
		eor #FTYPE_DIR
dd2             bne resolv_nxtntry_	;only directorys are interresting
;                geq npjx

npjx            ldx #31
-		lda direntry_exp,x
		sta direntry_f_exp,x
		dex
		bpl -
		lda direntry_number_exp
		eor #15
		sta resolv_entry_offs_exp;remember entry position
		#jmps flushdird

xe		jsr OP_read_to_direntry2
flushdird	#dec_l_a direntry_number_exp;skip rest of sector
		bpl xe

                jsr wait_busy
                jsr LWOP_reporterror_read_xit

		#cnext direntry_f_pointer_exp , next_sector

		lda resolv_mode_exp
                and #RESOLV_L_FOLLOW_LAST_LINK_MASK
		ora wildcard_end_exp
		bne trylink
endof           clc
                rts			;-=> exit, OK

trylink         lda direntry_f_attrib_exp
		and #FATTR_CLOSED | FTYPE_MASK
set_v		cmp #FATTR_CLOSED | FTYPE_LNK
		beq trylink2

		lda wildcard_end_exp
		beq endof

		jsr cutoff
		#jmps resolv_next_dir

trylink2       	jsr cutoff
		lda #ram_exp_buffer0	;read in link
		jsr XIT_read_a_sector	;-=> error

		ldx #0
lp		lda link_data_exp,x
		beq end
		inx
		bne lp
                dex

end		ldy #0
lp2		#lda_zp_y current_path
		beq end2
		iny
		sta link_data_exp,x
		inx
		bne lp2
		sec
		gcs LWOP_reporterror_39_xit;-=> buffer overflow

lp3		lda link_data_exp,x
end2		sta path_exp,x
		dex
		cpx #255
		bne lp3

		jsr OP_get_partnum
                bcs nfnd
                sta path_partition_exp	;update if specified
nfnd            #sev set_v		;resolv from last dir
		gvs resolv_pathloop

cutoff		inx
		lda path_exp,x
                beq kw
		eor #"/"
		bne cutoff
kw
		tay
                .byte SKIP2
jkjs2		inx
		iny
		lda path_exp,x
		#sta_zp_y current_path	;now kill "xxxx/" from "xxxx/sd/df" to get "sd/df"
		bne jkjs2
                rts

OP_read_to_direntry2
		.if SUPERCPU=1
		lda direntry_mode_exp
                .else
                ldy direntry_mode_exp
                .fi
		bne niu
                .if SUPERCPU=1
                tay
                .fi
		lda wildcard_end_exp
                bne niu
                lda resolv_store_exp
                beq niu

		ldx #224
cb2		lda data_g		;DATA PORT
		#lda_low_reg		;0-7 BIT
		sta direntry_exp-224,x
                #sta_zp_y pom1
                iny
		inx
		lda hig_reg		;8-15 BIT
		sta direntry_exp-224,x
                #sta_zp_y pom1
                iny
		inx
		bne cb2
                lda direntry_number_exp
                lsr
                lsr
                tax
		lda direntry_nexts_exp
		asl
		asl
		asl
		rol next_sector,x
		asl
		rol next_sector,x
                tya
                clc
                adc pom1
                sta pom1
                bcc aha
                inc pom2
aha		rts

niu		jmp OP_read_to_direntry

dirmodeconst 	.byte DIRMODE_OFS,DIRMODE_CFS,DIRMODE_FAT,DIRMODE_ISO,DIRMODE_JOLIET

checkopenmodes  ldy #3
-               cmp openmodes,y
                beq +
                dey
                bpl -
                rts
+               sta resolv_open_mode_exp
                rts

openmodes       .text "wmar"

ve1		.text "d","del"
		.text "r","rel"
		.text "b","dir"
		.text "j","lnk"
cseq		.text "s","seq"
cprg		.text "p","prg"
		.text "u","usr"
cany            .text "a","*",0      	;all

;                                     1=delimiter required
;                                    1=filetype ",xxx"
;                                   1=filetype "=xxx"
;                                  1=no filetype means "*"
;                                 1=remember first empry entry
;                                1=remember found entry
;                               1=report file not found
;                              1=follow if last part is a link
;                             1=dir
;                            1=md or rd
;                           1=load
;                          1=open
;                         1=scratch
;                        1=openfirst
;                       1=r-h
;                      1=chdir
;
;rsc		.word %0000000101001100	;RESOLV_DIR_MODE
;		.word %0000010011001010	;RESOLV_LOAD_MODE
;		.word %1000000010000000 ;RESOLV_CD_MODE
;		.word %0010000010000010 ;RESOLV_OPENFIRST_MODE
;RESOLV_OPEN_MODE_ =   %0000100011000010 ;RESOLV_OPEN_MODE
;RESOLV_CREAT_MODE_ =  %0000000010010010 ;RESOLV_CREAT_MODE
;		.word %0000001000010001 ;RESOLV_MD_MODE
;		.word %0001000000101101 ;RESOLV_S_MODE
;		.word %0000000000001011 ;RESOLV_R_MODE
;		.word %0000000001101010 ;RESOLV_R2_MODE
;		.word %0000001000100001 ;RESOLV_RD_MODE
;		.word %0000000011101011 ;RESOLV_L_MODE
;		.word %0100000001100001 ;RESOLV_RH_MODE
		.pend
;----------------------------------
;Get partition number from path C=1 if no ":"
;
OP_get_partnum .proc
		#lda0
                ldx #$ff
loop2		sta current_path
loop3           clv
loop            inx
		lda path_exp,x
		beq none
                cmp #$3a		;: (10)
                beq end
                bcs loop
                bvs loop
                sbc #$2f-1      	;/ (-1)
                bcc loop3
                beq cmd
                #sbc1
		asl current_path
                adc current_path
		asl current_path
		asl current_path
		adc current_path
                cmp #17
                bcc loop2
                lda #17
                gcs loop2

none		sec
		sta path_rewrite_exp	;no rewrite
		rts

cmd	        sbc #$80		;set v
		#txy_a
                gvs loop

end		dex
		bvc uh
                #tyx_a
uh		ldy #255

removepart	.proc
lp2             iny
                inx
                lda path_exp,x
                #sta_l_y path_exp
                bne lp2
                lda current_path
                clc
                rts
		.pend
