;
; save.asm - SAVE kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Save (A=start address on zeropage, X,Y last not saved byte)
;
; Used registers: A X Y
; Zeropage: $c1,$c2=start address
; Zeropage: $ae,$af=X and Y register before call
; Zeropage: $ac,$ad=the byte after the lastsaved
; Status: C
; C=1:
; A=0 break
; A=5 device not present
; A=8 missing filename
; A=9 illegal device number
; A=24 file data
; Status response:
; ST=$00 ok
; ST=$03 timeout
; ST=$80 device not present
;
; Notes:
; Can only save 65533 bytes+load address (1 tree sector)
;**************************************************
;--------------------------------------
; SAVE
;
;		stx $ae
;		sty $af
;		tax
;		lda $00,x
;		sta $c1
;		lda $01,x
;		sta $c2
;		jmp ($0332)
;--------------------------------------
		#assert_cfg_open

newsave		.proc

                .if DEBUGIO
                lda debug_io_exp
                beq kd
                #lda0
                sta debug_io_exp
                #psr $f5ed
                php
                lda $c2
                pha
                #phx_a
                #phy_a
                lda #3
                jmp monitor
kd		lda #1
		sta debug_io_exp
                .fi

		lda #KERNAL_MISSING_FILENAME
		ldx ActualFNLength
		beq exit_kernal_error

		lda ActualDev
		#devicehandler hard_save,exit_illegal_device,pclink_save,kernal_save
		.pend

kernal_save	.proc
		cpy #31
		bcs ill			;illegal serial device
		cpy #4
		bcs hard_save.in	;4-30
		#psr $f5ef
		tya
	   	jmp exit2basic		;Normal kernal rutin
ill		jmp exit_illegal_device
		.pend
;--------------------------------------
; SAVE to IDE
;--------------------------------------
pclink_save	.proc
hard_save	.proc
		ldy #0
in		sty $ac
		lda #$61
		sta ActualSecAddr
                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                sta dos_accel_internal_exp
                .fi
                #OP_scpu01up
		jsr LWOP_findunusedchannel;cfg_open after
                #OP_scpu01down
		jsr newopen_t
                bcs eee
                jsr dv

		bit msgmode
		bpl nomess
                jsr getfilename		;cfg_open after
                #assert_cfg_open
                sec
		jsr OP_loadingsavingmsg
nomess
		ldx ActualLogical
		jsr newchkout_t
eee             bcs eee2
                jsr dv

                ldx $ac

		jsr $fb8e		;$c1 to $ac
		jsr chroutdv
		lda $ad
		jsr chroutdv

                txa
                bne serial

		lda $ae
		sec
		sbc $ac
		tax
		lda $af
		sbc $ad
		tay
		bcc endd
                jsr write
eee2		bcs eee3
                jsr dv
		txa
		adc $ac
		sta $ac
		tya
		adc $ad
		sta $ad
endd		#lda0
		pha
		geq end

serial          #sta_cfg_open
		#OP_scpu01up
                jsr OP_save_zeropage

		ldy #10
di		lda read_byte_r3-1,y
		sta start_zeropage-1,y
		dey
		bne di

loop		lda $ac
		cmp $ae
		lda $ad
		sbc $af
		bcs ende
		jsr start_zeropage
                .if SUPERCPU=0
		tax
                .fi
		#jsr_OP_cfg16up
                #assert_cfg_16
                .if SUPERCPU=0
		txa
                .fi
		jsr iecout
		#jsr_OP_cfg16down
                #assert_cfg_open
		lda Status
		bne err
		inc $ac
		bne loop
		inc $ad
		jsr check_stop
		bne loop
		.byte SKIP1BNE
err		iny
		iny
ende
		#phy_a
                jsr OP_restore_zeropage_scpu01down

end		jsr newclrchn_t
		lda ActualLogical
		jsr newclose_t
                #ply_a
                jsr dv
		sec
                dey
		tya
		beq stop
                bpl dataerr

                #sta_cfg_open
		jsr OP_load_loading_end
		clc
stop
eee3
                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
		jsr clearaccel
                .fi
		jmp exit2basic

chroutdv       	jsr newchrout_t
dv		lda Status
                bne +
                rts

+               pla
                pla
dataerr
                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
		jsr clearaccel
                .fi
                lda #KERNAL_FILE_DATA
		gne exit_kernal_error	;error!

                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
clearaccel     	pha
                #lda0
                sta dos_accel_internal_exp
		pla
                .fi
                rts

write   	#psr switch_16k
		lda #$ac
		jmp write_dt		;newwrite_t
		.pend
;-----------------------------------
; Read a byte (for save)
;
read_byte_r3	.proc
		.logical start_zeropage
		sta cfg_stnd		;3
		lda ($ac),y		;2
		sta cfg_open		;3
		rts			;1
		.here
		.pend
;-----------------------------------
; Copy filename to path_exp, cfg_open after!
;
		.assert CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO, CFG_8000_RAM | CFG_A000_RAM | CFG_A000_BASIC | CFG_D000_ROM
getfilename     .proc
		#sta_cfg_open
                jsr OP_save_zeropage_copy_filename_to_path
                jmp OP_restore_zeropage
		.pend
