;
; trap.asm - IO space extension
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

		#assert_cfg_open

		.if BANK !=0

nmi_IDE_16	.proc
		cld
		.if DE32_EXISTS
		ora #$fc
		.fi
		sta nmi_oldconfig_exp
		#phx_a
		#phy_a

		ldy $dd0d		;Test if CIA#2 called this MNI
		bpl nmi_no_cia
		lda #$7f
		sta $dd0d
		ldy $dd0d
nmi_into_rom	#psr $fe64

		.if DE32_EXISTS
		lda nmi_oldconfig_exp
		lsr
		inc $318
		bcs switch_norm
		lsr
		bcc switch_16k
		gcs switch_8k
		.else
		lda nmi_oldconfig_exp
		cmp #$ff
		inc $318
		bcs switch_16k
		gcc switch_norm
		.fi

nmi_no_cia	lda $dc02
                pha
                ldy $dc03
                ldx #$ff
                stx $dc02
                .if SUPERCPU=1
                stz $dc03
                .else
		inx
                stx $dc03
                .fi
                lda $dc00
                ldx #$7f
                jsr keytest             ;check stop key
                sty $dc03
                pla
                sta $dc02

		.if BANK=3
		lda nmi_oldconfig_exp
		cmp #$ff
		.if DE32_EXISTS
		bcc ddp
		.else
		bcs ddp
		.fi			;DE32_EXISTS
		cpx #%11011111
		bne ddp
                .if SUPERCPU=0
		txa
		tsx
		inx			;y
		inx			;x
		inx			;a
		inx			;sr
		inx			;l
		inx			;h
		ldy $100,x
		tax
		cpy #>nmi_return
		beq ddp
                .fi			;SUPERCPU=0
		lda zp_valid_exp
		bne ddp
		lda #BANK
		jmp monitor
ddp
		.fi			;BANK=3

		txa
		bmi ht
		ldx #$bd
                jsr keytest
		inx
		bne ht2
ht		sta StopFlag
ht2
		lda StopFlag
		eor #$7f		;TEST "RUN/STOP"
		bne nmi_into_rom

		.if DE32_EXISTS
		#psr $a000
		.else
		#psr $a001
		.fi
		jmp gotobank0
		.pend

keytest         .proc
		stx $dc00

-		ldx $dc01
		cpx $dc01
		bne -
                sta $dc00
                rts
                .pend

		.fi			;Bank !=0

		#assert_cfg_open

		.if BANK=0

                .if SUPERCPU=1

syscall		.proc
		php
		sec
		xce
		bcc mon2       		;Yo lamer!
		xba
		phx
                lda 4,s
                eor #$de
                bne mon
                lda 3,s
		ldx #sysend-sysstart
-		dex
		cmp sysstart,x
		bcc -
		beq syscfound

mon		plx
                xba
mon2		pha
		phx
                phy
monitor2
		sta scpu_hwdisable
                jmp restore_no_cia.in

syscfound	txa
		beq mon
		lda syshi-1,x
                sta 4,s
		lda syslo-1,x
                sta 3,s
                plx
		xba
                plp
		rts

                .else

syscall		.proc
		php
		sta help_trap_accu_exp
		stx help_x_exp
                tsx
                inx
                inx
                inx
                lda $100,x
                eor #$de
                bne mon
                dex
                lda $100,x
		ldx #sysend-sysstart
-		dex
		cmp sysstart,x
		bcc -
		beq syscfound

mon		jsr ii
mon2		pha
                txa
                pha
                tya
                pha
monitor2
                jmp restore_no_cia.in

syscfound	txa
		beq mon
                plp
                pla
                pla
		lda syshi-1,x
		pha
		lda syslo-1,x
		pha
ii		ldx help_x_exp
		lda help_trap_accu_exp
		rts

                .fi

                .assert CFG_OPEN_RAM | CFG_8000_ROM | CFG_A000_ROM | CFG_D000_IO | CFG_E000_OPEN, CFG_A000_ROM, CFG_A000_RAM | CFG_A000_BASIC

sysstart	.byte 0
		.byte <(__brk+2)
		.byte <(__mytoken+2)
		.byte <(__clall+2)
		.byte <(__clrchn+2)
		.byte <(__load+2)
		.byte <(__save+2)
		.byte <(__open+2)
		.byte <(__close+2)
		.byte <(__chkin+2)
		.byte <(__chkout+2)
                .byte <(__mytprt+2)
                .byte <(__myexpr+2)
		.byte <(write_dt+2)
		.byte <(read_dt+2)
sysend
syslo		.byte <(monitor2-1)
		.byte <(LW_mytoken-1)
		.byte <(newclall-1)
		.byte <(newclrchn-1)
		.byte <(newload-1)
		.byte <(newsave-1)
		.byte <(newopen-1)
		.byte <(newclose-1)
		.byte <(newchkin-1)
		.byte <(newchkout-1)
		.byte <(OP_mylist-1)
		.byte <(OP_myexpr-1)
		.byte <(newwrite-1)
		.byte <(newread-1)

syshi		.byte >(monitor2-1)
		.byte >(LW_mytoken-1)
		.byte >(newclall-1)
		.byte >(newclrchn-1)
		.byte >(newload-1)
		.byte >(newsave-1)
		.byte >(newopen-1)
		.byte >(newclose-1)
		.byte >(newchkin-1)
		.byte >(newchkout-1)
		.byte >(OP_mylist-1)
		.byte >(OP_myexpr-1)
		.byte >(newwrite-1)
		.byte >(newread-1)
		.pend
		.fi			;Bank=0

                .if BANK=0
;-----------------------------------------------------------
; CHKIN
;
		#assert_a000romioonly
newchkin_t	.proc
		#psr switch_16k
		jmp __chkin
		.pend
;-----------------------------------------------------------
; CLRCHN
;
		#assert_a000romioonly
newclrchn_t	.proc
		#psr switch_16k
		jmp __clrchn
		.pend
;-----------------------------------------------------------
; LOAD
;
		#assert_a000romioonly
newload_t	.proc
		.if SUPERCPU=0
		sta $c3
		.fi
		#psr switch_16k
		.if SUPERCPU=0
		lda $c3
		.fi
		stx $c3
		sty $c4
		jmp __load
		.pend

		.fi			; BANK=0
;-----------------------------------------------------------
; LOAD
;
		#assert_a000romioonly
		.if BANK=2
newload_t	.proc
		.if SUPERCPU=0
		sta $c3
		.fi
		#psr gotobank0back
		.if SUPERCPU=0
		lda $c3
		.fi
		stx $c3
		sty $c4
		jmp __load
		.pend
		.fi			; BANK=2
;-----------------------------------------------------------
; READ
;
		#assert_a000romioonly
		.if (BANK=3)
newread_t	.proc
		#sta_cfg_open
		.if SUPERCPU=0
		sta help_trap_accu_exp
		.fi
		#psr gotobank0back
		.if SUPERCPU=0
		lda help_trap_accu_exp
		.fi
		jmp read_dt
		.pend
		.fi			; BANK=2

		#assert_a000romioonly
		.if (BANK=3)
newwrite_t_mon	.proc
		dec $01
                jsr newwrite_t
                inc $01
                rts

		#assert_a000romioonly
newwrite_t	.proc
		#sta_cfg_open
		.if SUPERCPU=0
		sta help_trap_accu_exp
		.fi
		#psr gotobank0back
		.if SUPERCPU=0
		lda help_trap_accu_exp
		.fi
		jmp write_dt
		.pend
		.fi			; BANK=3
;-----------------------------------------------------------
; OPEN
;
		#assert_a000romioonly
		.if (BANK=3) | (BANK=2) | (BANK=1)
newopen_t	.proc
		#psr gotobank0back
		jmp __open
		.pend
		.fi			; BANK=3

;-----------------------------------------------------------
; CLALL
;
		#assert_a000romioonly
		.if (BANK=2) | (BANK=3)
newclall_t	.proc
		#psr gotobank0back
		jmp __clall
		.pend
                .fi			; BANK=2
;-----------------------------------------------------------
; CLOSE
;
		#assert_a000romioonly
		.if (BANK=3) | (BANK=2) | (BANK=1)
newclose_t	.proc
		.if SUPERCPU=0
		tax
		.fi
		#psr gotobank0back
		.if SUPERCPU=0
		txa
		.fi
		jmp __close
		.pend
		.fi			; BANK=3

;-----------------------------------------------------------
; CHKIN
;
		#assert_a000romioonly
		.if (BANK=3) | (BANK=2)
newchkin_t	.proc
		#psr gotobank0back
		jmp __chkin
		.pend
		.fi			; BANK=3
;-----------------------------------------------------------
; CHKOUT
;
		#assert_a000romioonly
		.if (BANK=3) | (BANK=2) | (BANK=1)
newchkout_t	.proc
		#psr gotobank0back
		jmp __chkout
		.pend
		.fi			; BANK=3
;-----------------------------------------------------------
; CLRCHN
;
		#assert_a000romioonly
		.if (BANK=1) | (BANK=2) | (BANK=3)
newclrchn_t	.proc
		#psr gotobank0back
		jmp __clrchn
		.pend
		.fi			; BANK=1


		#assert_a000romioonly
                .if BANK!=0
newchrin_t      #psr gotobank0back
		jmp __chrin
		.fi			; BANK!=0

		#assert_a000romioonly
		.if (BANK=1) | (BANK=2)
newgetin_t	#psr gotobank0back
		#psr exit2basic
		jmp __getin
                .fi			; BANK=1

		#assert_a000romioonly
		.if BANK=1
newchrout_t
		#sta_cfg_open
		.if SUPERCPU=1
		xba
		.else
		sta help_trap_accu_exp
                lda #>(gotobank0back-1)
                pha
		.fi

		lda buffer_output_exp
		eor #STDOUT
		bne ueo
		lda buffer_input_exp
		cmp #STDIN
		bne ueo
		.if SUPERCPU=1
		#psr gotobank3back
		gcs ueo2

ueo		#psr gotobank0back
ueo2
		xba
		.else
		lda #<(gotobank3back-1)
                .byte SKIP2
ueo		lda #<(gotobank0back-1)
                pha
		lda help_trap_accu_exp
		.fi

		jmp __chrout
		.fi			; BANK==1

		#assert_a000romioonly
		.if BANK=2
newchrout_t
		.if SUPERCPU=0
		sta cfg_open
		#assert_cfg_open
		sta help_trap_accu_exp
		.fi
		#psr gotobank0back
		#psr exit2basic
		.if SUPERCPU=0
		lda help_trap_accu_exp
		.fi
		jmp __chrout
		.fi			; BANK==2

		#assert_a000romioonly
		.if BANK=3
newchrout_t
		sta cfg_open
		#assert_cfg_open
		.if SUPERCPU=1
		xba
		.else
		sta help_trap_accu_exp
                lda #>(gotobank0back-1)
                pha
		.fi

		lda buffer_output_exp
		eor #STDOUT
		bne ueo
		lda buffer_input_exp
		cmp #STDIN
		bne ueo
                .if SUPERCPU=1
		#psr switch_16k
		gcs ueo2

ueo		#psr gotobank0back
ueo2
		xba
		.else
		lda #<(switch_16k-1)
                .byte SKIP2
ueo		lda #<(gotobank0back-1)
                pha
		lda help_trap_accu_exp
		.fi

		jmp __chrout
		.fi			; BANK==3

;----------------------------------
; System vectors
;
		.if SUPERCPU=0
		#assert_cfg_open

		.cerror *>$bffa		;Overflow in range $a000-$bff9!

		*= $bffa		;$fffa
		.logical $fffa
debugzp		.word opencfg_nmi	;NMI
		.word $ffff
		.word opencfg_irqbrk	;IRQ/BRK
		.here

		.else			;SuperCPU

        	.cerror *>$c000		;Overflow in range $a000-$bfff!

		.fi			;SuperCPU
;----------------------------------
; $DE60-$DEFF stuff
;
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
		*= $9e60
		.logical $de60
		.text "ide"		;Soci: A way of detecting IDE64:

		.if BANK=0
		sta bank_port0
switch_norm
exit2basic	sta cfg_stnd		;$DE60="IDE"
		rts

		.else
exit2basic	sta bank_port0
switch_norm	sta cfg_stnd
		rts
		.fi			; BANK=0

		.if SUPERCPU=0
irq_call	sta cfg_stnd
		jmp ($fffe)

nmi_call	sta cfg_stnd
		jmp ($fffa)
                .fi

;------------------------------------------------------------
; Crossbank jmp
;
		.if BANK=0
gotobank1back
gotobank3	sta bank_port1
gotobank2back
gotobank2	sta bank_port1
gotobank3back
gotobank1	sta bank_port1
switch_16k
		sta cfg_16
derts		rts
;------------------
		.elsif BANK=1
gotobank2back
gotobank0	sta bank_port2
gotobank3back
gotobank3	sta bank_port2
gotobank0back
gotobank2	sta bank_port2
switch_16k
		sta cfg_16
derts   	rts
;------------------
		.elsif BANK=2
gotobank3back
gotobank1	sta bank_port3
gotobank0back
gotobank0	sta bank_port3
gotobank1back
gotobank3	sta bank_port3
switch_16k
		sta cfg_16
derts   	rts
;------------------
		.elsif BANK=3
gotobank0back
gotobank2	sta bank_port0
gotobank1back
gotobank1	sta bank_port0
gotobank2back
gotobank0	sta bank_port0
switch_16k
		sta cfg_16
derts   	rts
		.fi			; BANK=3

__brk		jsr sysc

nmi_return	pla
irq_return	sta cfg_open
block_nmi	rti			;must be right before!
__nmi		dec $318
		pha
					;Soci: Solved the cfg_16 vs. cfg_stnd
					;      problem!
		.if DE32_EXISTS
		lda ideconf_port
		.else
                #assert_a000romioonly    ;hack
		lda config16test
		.fi
		sta cfg_open
		#assert_cfg_open

		.if BANK=0
		jmp restore
		.else
		jmp nmi_IDE_16
		.fi

		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
;------------------------------------------------------------
; System calls from vector table
;
__mytoken	jsr sysc
__clall		jsr sysc
__clrchn	jsr sysc
__load		jsr sysc
__save		jsr sysc
__open		jsr sysc
__close		jsr sysc
__chkin		jsr sysc
__chkout	jsr sysc

;--------------------------------------
switch_8k0
		.if BANK=0
switch_8k	sta cfg_8
		rts
__chrin		sta cfg_open		;$3xx
		#assert_cfg_open
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jmp newchrin_ram_exp
		.else
		jmp newchrin
		.fi
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
__chrout	sta cfg_open		;$3xx
		#assert_cfg_open
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jmp newchrout_ram_exp
		.else
		jmp newchrout
		.fi
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
__getin		sta cfg_open		;$3xx
		#assert_cfg_open
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jmp newgetin_ram_exp
		.else
		jmp newgetin
		.fi
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
		.byte $ff
;--------------------------------------
		.elsif BANK=1
		.byte $ff
__chrin		sta bank_port0
		#assert_a000romioonly
		jmp newchrin_t		;$3xx
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
__chrout	sta bank_port0
		#assert_a000romioonly
		jmp newchrout_t		;$3xx
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
__getin		sta bank_port0
		#assert_a000romioonly
		jmp newgetin_t		;$3xx
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
switch_8k	sta cfg_8
		rts
;--------------------------------------
		.elsif BANK=2
		.byte $ff
__chrin		sta bank_port0
		#assert_a000romioonly
		jmp newchrin_t		;$3xx
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
__chrout	sta bank_port0
		#assert_a000romioonly
		jmp newchrout_t		;$3xx
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
__getin		sta bank_port0
		#assert_a000romioonly
		jmp newgetin_t		;$3xx
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
switch_8k	sta cfg_8
		rts
;--------------------------------------
		.elsif BANK=3
		.byte $ff
__chrin		sta bank_port0
		jmp __chrin    		;$3xx
__chrout	sta bank_port0
		sta cfg_open		;$3xx
		#assert_cfg_open
__getin		jmp newchrout		;__getin should not be called from bank3!
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
		jmp $f142		;$3xx No more raster bugs with getin from keyboard...
switch_8k	sta cfg_8
		rts
		.fi
;--------------------------------------
		.if (DE32_EXISTS=1) & (SUPERCPU=0)
__keyb		lda ideconf_port
		sta bank_port1
                .if BANK=1
		sta cfg_open
		#assert_cfg_open
                jmp c128keys
		.assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
		.fi
		.fi
;-----------------------------------------------------------
; Basic stuff
;
		.if (BANK=0) | (BANK=3)
__mytprt	jsr sysc
__myexpr       	jsr sysc
__mydoit	jsr $73
		sta cfg_16
                #assert_cfg_16
		jsr mydoit		;This already does exit2basic...
                .assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
		jmp $a7ae
basexeccmd	sta cfg_stnd		;Some optimization for basic...
		jmp $73
basnottoken	sta cfg_stnd
		jsr $b0a1
		jmp $a9a8
		.fi			; BANK=0
;------------------

		.here

		.cerror *>$9ef1		;I/O block $9e60 - $9ef0 overflow!

;--------------------------------------------------------------------
; Kernal extension ($9ef1-)
;

		*= $9ef1
		.logical $def1

write_dt	jsr sysc		;Write    ($def1)
read_dt		jsr sysc		;Read     ($def4)
		.if BANK!=0
sysc		sta bank_port0		;def7
		jmp sysc
		.else
		jmp sysc		;def7 freezepoint
sysc		sta cfg_open		;defa
		#assert_cfg_open
		jmp syscall		;defd
                .assert CFG_D000_IO, CFG_D000_IO, CFG_D000_ROM
		.fi			; BANK!=0

		.here

;-----------------------------------------------------------------

		.cerror *>$9f00		;Kernal extension block $9ef1 - $9eff overflow!

		.if SUPERCPU=0


		*=$9f00
;--------------------------------------
; Handle NMI in opencfg
;
		#assert_cfg_open
opencfg_nmi	pha			;Soci: only have to save A

		lda #>nmi_return
		pha
		lda #<nmi_return
		pha
		.cerror nmi_return & %00011000;BUG, B or D flag set in nmi!
		.cerror !(nmi_return & %00000100);BUG, I flag not set in nmi!
		pha
		jmp nmi_call		;Standard cfg, JMP ($FFFA)
;--------------------------------------
; Handle IRQ in opencfg
;
		#assert_cfg_open
opencfg_irqbrk
		.if DEBUG
		sta help_irqa_exp
		pla
		pha
		and #$10
		bne jump_break		;brk?

		lda #>irq_return
		pha
		lda #<irq_return
		pha
		.cerror irq_return & %00011000;BUG, B or D flag set in irq!
		.cerror !(irq_return & %00000100);BUG, I flag not set in irq!
		pha

		lda help_irqa_exp
		jmp irq_call		;Standard cfg, JMP ($FFFE)
		.else			;DEBUG
		pha			;3

		lda #>nmi_return	;5
		pha			;8
		lda #<nmi_return	;10
		pha			;13
		.cerror nmi_return & %00011000;BUG, B or D flag set in irq!
		.cerror !(nmi_return & %00000100);BUG, I flag not set in irq!
		pha			;16

		jmp irq_call		;Standard cfg, JMP ($FFFE) 19, 23, 28 cycle delay
		.fi			;DEBUG

;--------------------------------------
; Debugger (C64 screen of death?)
;
		#assert_cfg_open
		.if DEBUG
jump_break	.proc
		cld
		txa
		pha
		lda #$7f
		sta $dd0d
		bit $dd0d
		ldx #0
		stx $d015
		stx $dd02
		lda #27
		sta $d011
		lda #8
		sta $d016
		lsr
		sta $d018
		lsr
		sta ident_label_exp
		sta $d021
vw		sta $d800,x
		sta $d900,x
		sta $da00,x
		sta $db00,x
		inx
		bne vw
		ldx #debugtxte-debugtxt
vw2		lda debugtxt-1,x
		sta $28,x
		lda #15
		sta $d828,x
		dex
		bne vw2
		sta bank_port0
		.if BANK=0
		lda help_irqa_exp
		jsr debughex		;ac
		pla
;		jsr debughex		;xr
		jsr debughexy		;yr
		pla
		jsr debughex		;sr
		pla
		sec
		sbc #2
		tay
		pla
		sbc #0
;		jsr debughex
		jsr debughexy
		tsx
		txa
		ldx #2+2+2+2+2+2
		jsr debughex		;sp
		lda ActualDev
		jsr debughex		;dr
		lda zp_valid_exp
		jsr debughex3
		lda $00
		eor #255
		ora $01
		jsr debughex3		;01
		inx
		ldy #10
vw3		pla
		jsr debughex		;stack
		dey
		bne vw3
		sty $00

		lda #$e2
		sta 792
		lda #$fc
		sta 793
		sta cfg_8
					;Split effekt... (PAL only)
break_loop	sty $d020
		cmp (0,x)
		sta $d020
		nop
		lda #7
		bne break_loop

debughexy	jsr debughex
		tya
debughex	pha
		lsr
		lsr
		lsr
		lsr
		jsr debughex2
		pla
debughex3	and #15
debughex2	cmp #10
		bcc +
		adc #$26
+		eor #$30
		sta $51,x
		lda #1
		sta $d851,x
		inx
		rts
		.fi			; BANK=0

		.enc screen
debugtxt	.text "acxryrsrpc$",BANK+$30,"spdrz1 stack"
		.enc none
debugtxte
		.pend

		.cerror *>$a000;Debugger code overflow $9f00 - $9fff !

		.fi			;DEBUG

		.fi			;SuperCPU
