;
; write.asm - BLKWRITE kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Write (A=start address on zeropage, X,Y number of bytes to write)
;
; Used registers: A X Y
; Status: C
; C=1:
; A=5 device not present
; A=7 not output file
; A=9 illegal device number
; Status response:
; ST=$00 ok
; ST=$80 device not present
;**************************************************
;----------------------------------------
;IDE write
;
		#assert_cfg_open

newwrite	.proc
		jsr read_write_st	;store number of bytes and start address and save zeropage
		#ldatax buffer_output_exp
		jsr read_write_st2	;store current puffer and drive and check state
		asl
		bcc write_exerr		;writeable?
		lda buffer_type_exp,x
		cmp #BUFFERMODE_FILE
		beq write_file
		cmp #BUFFERMODE_PCLINK
		beq pclink_write2
		#lda0
write_exerr	tay
		jsr OP_restore_zeropage
		tya
                asl
		bcs full
                bmi exit_not_output_file;readable otherwise
		gpl exit_illegal_device

full		lda #$83
		gne newread.commonend

write_file	ldy #0
		tsx
fg		cpx #$03+44		;enough stack space?
		bcs ook
		pla
		sta sp_exp,x		;no, let's backup
		iny
		inx
		gne fg
ook		#phy_a

		#OP_scpu01up
		jsr XIT_write_file

		#jmps read_write_end

pclink_write2	#lda0
		pha
		#OP_scpu01up
;		sec
;		.if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
;		php			;not needed, one byte received only
;                sei
;                .fi
		jsr pclink_readwrite
;                .if (PCLINK_TYPE=3) | (PCLINK_TYPE=4) | (PCLINK_TYPE=5)
;                tax
;                ror
;                plp			;not needed, one byte received only
;                asl
;                txa
;                .fi
		#jmps read_write_end

		.pend
;----------------------------------------
;file write
;
;BLK_WRITE_DIRTY, BLK_RW_DIRTY, BLK_RW_EOF_DIRTY, BLK_RW_EOF, BLK_WRITE, BLK_RW

XIT_write_file	.proc
bytes_in_buffer	= current_path

		#xit_init		;remember stack and bank

		#ldx_l_a buffer_currentp_exp
		lda buffer_info_exp,x
                pha			;remember status
		lda #INFO_IGNORE	;be safe
		sta buffer_info_exp,x

		lda buffer_address_end_hi_exp,x
		beq middle		;no need to copy into buffer

		ldy #write_wr_one_re-write_wr_one_r
-       	lda write_wr_one_r-1,y
		sta start_zeropage-1,y	;install zeropage stuff
		dey
		bne -   		;now Y=0
		lda pom9		;source
		sta write_wr_one_r.at1+1
		lda pom10
		sta write_wr_one_r.at1+2

		sec
                tya			;Y=0
		adc buffer_address_low_exp,x
		sta write_wr_one_r.at2+1
		lda buffer_address_hi_exp,x
		and #$7f
		adc #0
		sta write_wr_one_r.at2+2;Position

		tya			;Y=0
		sec
		sbc write_wr_one_r.at2+1;How many bytes left in buffer?
		sta bytes_in_buffer
		lda buffer_address_max_hi_exp,x
		sbc write_wr_one_r.at2+2
		#andclc 7
		sta bytes_in_buffer+1

		#ldatax write_size
		sbc bytes_in_buffer
		lda write_size+1
		sbc bytes_in_buffer+1
		bcs +
		stx bytes_in_buffer
                lda write_size+1
		sta bytes_in_buffer+1   ;Have to write less or equal bytes then in buffer!
+       	php

                jsr copy_bytes_to_buffer

		txa
		clc
		adc write_wr_one_r.at1+1
		sta pom9
		#lda0
		adc write_wr_one_r.at1+2
		sta pom10		;adjust destination address

		jsr read_write_adj	;count write size and adjust buffer position

		plp
		bcc vege		;we wrote less bytes then in buffer, so we are ready

		#lda0
		sta buffer_address_end_hi_exp,x
		sta buffer_address_end_low_exp,x
		lda buffer_start,x	;mark it as unloaded sector
		#sbc1
		sta buffer_address_hi_exp,x;buffer_address_low_exp is already at 255

		lda #2
		gne nu                  ;do buffer flush!
;
;	Now we have to save full sectors, buffer is empty
;
middle		lda write_size
		ora write_size+1
		beq vegeur		;nothing to write?

		lda write_size+1
                cmp #2
                bcc lessthen512b

		#lda0			;no buffer flush
nu		sta write_p1_exp

		ldx #save_lot_cfs_re-save_lot_cfs_r-1
-       	lda save_lot_cfs_r,x
		pha
		dex
		bpl -   		;install stack stuff

		tsx
		lda pom9
		sta save_lot_cfs_r.at1+1,x
		sta save_lot_cfs_r.at3+1,x
		clc
		adc #$80
		sta save_lot_cfs_r.at2+1,x;init pointers
		sta save_lot_cfs_r.at4+1,x
		lda pom10
		sta save_lot_cfs_r.at1+2,x
		adc #0
		sta save_lot_cfs_r.at2+2,x
		#adc1c
		sta save_lot_cfs_r.at4+2,x
		lda pom10
		#adc1c
		sta save_lot_cfs_r.at3+2,x

		jsr savesize
tnt2		jsr XIT_save_tree	;-=> error

tnt		jsr find_free_sectors_blockwrite
                jsr restoresize

		lda write_sectors_exp
		jsr OP_write_cmd
		bcs handle_error

		lda status_g
		#lda_low_reg
write_loop	and #%00001000
		beq handle_error2	;DRQ=1?

		#ldy_l_a write_p1_exp	;flush buffer first?
                beq kvxx

		#ldy_l_a buffer_currentp_exp
		ldx buffer_start,y
                jsr savesize
		jsr write_buffer_cfs	;write out the content of buffer
                jsr restoresize
		#lda0
		sta write_p1_exp	;ok, buffer flushed
		geq fww

handle_error2	sec
		lda #EUNEXP
handle_error	jmp LWOP_reporterror_read_xit;-=> error

execwrite       lda #1
		pha
		#phx_a
                .if SUPERCPU=1
                #jmps OP_scpu01down
                .else
		rts
                .fi

kvxx		tsx
		jsr execwrite

		sec
		#lda0
		adc save_lot_cfs_r.at3+2,x
		sta save_lot_cfs_r.at1+2,x
		#adc1c
		sta save_lot_cfs_r.at3+2,x
		lda save_lot_cfs_r.at4+2,x
		#adc1c
		sta save_lot_cfs_r.at2+2,x
		#adc1c
		sta save_lot_cfs_r.at4+2,x

		dec write_size+1		;beware on memory top...
		dec write_size+1

                #OP_scpu01up

fww		#ldx_l_a buffer_currentp_exp
                .if SUPERCPU=1
		lda buffer_filepos2_exp,x
                inc a
                inc a
		sta buffer_filepos2_exp,x
                .else
                inc buffer_filepos2_exp,x
                inc buffer_filepos2_exp,x
                .fi
                bne fw
                jsr incfileposhi
fw
he2		lda status_g
		#lda_low_reg
		bpl okbsy
he		jsr wait_busy
		bcc he2
		gcs handle_error

okbsy		ror
		bcs he			;error?

		rol

		#dec_l write_sectors_exp
		bne write_loop

		jsr XIT_end_of_transfer_ata

		lda write_size
		ora write_size+1
		beq vegeur		;0 bytes left

                lda write_size+1
		cmp #2
		bcc lessthen512

		jsr savesize
		#ldx_l_a buffer_currentp_exp
		lda buffer_filepos2_exp,x
		bne tnt			;end of a tree sector?
		geq tnt2
;
; Now have to load the rest to buffer
;
lessthen512b	#psz pom9	;store pom10, pom9
                jsr XIT_save_tree
                #ply_a
                #jmps vsp

lessthen512	tsx
		ldy save_lot_cfs_r.at1+1,x
                lda save_lot_cfs_r.at1+2,x
		pha
vsp		ldx #write_wr_one_re-write_wr_one_r
le2		lda write_wr_one_r-1,x
		sta start_zeropage-1,x	;install zeropage stuff
		dex
		bne le2
                pla
		sty write_wr_one_r.at1+1
		sta write_wr_one_r.at1+2

                jsr OP_buffer_start
		sta write_wr_one_r.at2+2

                lda write_size
                sta bytes_in_buffer
                lda write_size+1
                sta bytes_in_buffer+1

                jsr copy_bytes_to_buffer
		#phx_a

		#ldx_l_a buffer_currentp_exp
		lda buffer_filesize1_exp,x
		sec
		sbc #1
		tay
		lda buffer_filesize2_exp,x
		sbc buffer_filepos2_exp,x
		sta pom2
		lda buffer_filesize3_exp,x
		sbc buffer_filepos3_exp,x
		bne more
		lda buffer_filesize4_exp,x
		sbc buffer_filepos4_exp,x
		bcc more
		bne more
                lsr pom2
                bne more

		adc buffer_start,x
                gcc fer

more		ldy #0
		lda buffer_address_max_hi_exp,x
fer     	sta buffer_address_end_hi_exp,x
		tya
		sta buffer_address_end_low_exp,x

		pla
		sec
		sbc #1
		sta buffer_address_low_exp,x
                lda write_wr_one_r.at2+2
		sbc #0
		cmp buffer_address_end_hi_exp,x
		bne krk
                tay
                lda buffer_address_low_exp,x
                cmp buffer_address_end_low_exp,x
                tya
                bcs krk
		ora #$80
krk		sta buffer_address_hi_exp,x

                #stz_a write_size
                #stz_aa write_size+1

vege            #sev set_v
                .byte SKIP1BVC
vegeur          clv
		.if SUPERCPU=1
		lda xit_sp_exp
                dec a
                tcs
                .else
		ldx xit_sp_exp
                dex
		txs
                .fi

		#ldx_l_a buffer_currentp_exp
                bvs +
                jsr update_filelen
                clv
+		pla			;be safe
                cmp #INFO_BLK_WRITE_DIRTY
                beq md
                cmp #INFO_BLK_RW_DIRTY
                beq md
                cmp #INFO_BLK_RW_EOF_DIRTY
                bne nmd
md              bvs ss
                sbc #INFO_BLK_WRITE_DIRTY-INFO_BLK_WRITE
                .cerror (INFO_BLK_WRITE_DIRTY-INFO_BLK_WRITE)!=(INFO_BLK_RW_DIRTY-INFO_BLK_RW)
                .cerror (INFO_BLK_RW_DIRTY-INFO_BLK_RW)!=(INFO_BLK_RW_EOF_DIRTY-INFO_BLK_RW_EOF)
                .cerror (INFO_BLK_WRITE_DIRTY^INFO_BLK_WRITE) & $80
                .cerror (INFO_BLK_RW_DIRTY^INFO_BLK_RW) & $80
                .cerror (INFO_BLK_RW_EOF_DIRTY^INFO_BLK_RW_EOF) & $80
nmd             bvc ss
                clc
                adc #INFO_BLK_WRITE_DIRTY-INFO_BLK_WRITE
ss		sta buffer_info_exp,x

		#lda0
		clc
set_v		rts			;-=> ok
                .pend
;-----------------------------------
; Update filelength
;
update_filelen  .proc
		sec
		#lda0
		adc buffer_address_low_exp,x
		sta pom4
                #lda0
		adc buffer_address_hi_exp,x
		sec
		sbc buffer_start,x
                #andclc $7f
		adc buffer_filepos2_exp,x
		sta pom3
		#lda0
		adc buffer_filepos3_exp,x
		sta pom2
		#lda0
		adc buffer_filepos4_exp,x
		sta pom1
		lda buffer_filesize1_exp,x
		cmp pom4
		lda buffer_filesize2_exp,x
		sbc pom3
		lda buffer_filesize3_exp,x
		sbc pom2
		lda buffer_filesize4_exp,x
		sbc pom1
		bcs +
                ldy #3
-               lda pom1,y
                sta buffer_filesize1_exp,x
                txa
                adc #MAX_BUFFERS
                tax
                dey
                bpl -
                sbc #MAX_BUFFERS*4-1
                tax
+               rts

                .cerror (buffer_filesize2_exp-buffer_filesize1_exp)!=MAX_BUFFERS
                .cerror (buffer_filesize3_exp-buffer_filesize2_exp)!=MAX_BUFFERS
                .cerror (buffer_filesize4_exp-buffer_filesize3_exp)!=MAX_BUFFERS
                .pend
;-----------------------------------
; CFS saving 512 bytes normal
;
save_lot_cfs_r	.proc
		.logical $101
		sta cfg_stnd		;3
lp
at2		lda $0000,y		;12
		sta hig_reg		;15

at1		lda $0000,y		;6
		#sta_low_reg		;9

		sta data_g		;18

at4		lda $0000,y		;27
		sta hig_reg		;30

at3		lda $0000,y		;21
		#sta_low_reg		;24

		sta data_g		;33

		iny			;34
		bpl lp			;36
		sta cfg_open		;39
		rts			;40
		.here
		.pend
save_lot_cfs_re

write_wr_one_r	.proc
		.logical start_zeropage
		.if SUPERCPU=1
		sta cfg_stnd		;3
at1		lda $ffff,x		;3
at2		sta $01ff00,x		;4
		inx			;1
at3		bne at1         	;2 16
                nop
                jmp switch_16k		;SCPU bug!
		.else
lp      	sta cfg_stnd		;3
at1		lda $ffff,x		;3
		sta cfg_open		;3
at2		sta $ff00,x		;3
		inx			;1
at3		bne lp          	;2
		rts			;1
		rts			;1 18 max
		.fi
		.here
		.pend
write_wr_one_re
